
package gov.va.med.via.audit;

import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;

import java.io.Serializable;
import java.util.Date;

/**
 * Implementing class of the AuditRequest - data object for audit requests - access methods only
 *  @author ZZZZZZTALBOM
 */

public class AuditRequestImpl implements AuditRequest,Serializable{
	
	private Long id;
	String siteId;				// User/Login Site identifier and/or record site id
	String userId;				//User (provider) identifier 
	String userNpi;				//user (provider) national provider identifier
	String userName;			//user (provider) name 
	String patientId;			//patient ien (local pid/dfn)
	String category;			//audit category
	String requestingApp;		//requesting application 
	String queryStartDate;		//query start date
	String queryEndDate; 		//query end date
	String itemId;				//used to store an ien or identifier information of a record/data object
	String recordSiteCode;		//used with itemId to determine the record and site combination
	String target;				//used as a search string - such as a match string
	String direction;			//used as a search string
	String criteria;			//used as a search string
	String maxRecords;			//used to limit the size of response, number of records returned
	String type;				//used as a search string
	String status;				//used as a search string
	
	//write specific audit elements - notes
	String noteIen;				//note identifier
	String titleIen;			//note type/title identifier
	String encounterString;		//note encounter string
	String noteText;			//note text
	
	Date  requestReceivedTime;	//Timestamp of when the request was received by system
	Date  responseSentTime;		//Timestamp of when the response was returned by system
	long duration;				//milliseconds - processing duration startTime less endTime
	String success;				//indicates whether response was successful - does not contain exceptions/faults
	String requestContent; 		//request message elements - what is appropriate to store
	
	private static final long serialVersionUID = 1L;
	
	public String getSiteId() {
		return siteId;
	}
	public void setSiteId(String siteId) {
		this.siteId = siteId;
	}
	public String getUserId() {
		return userId;
	}
	public void setUserId(String userId) {
		this.userId = userId;
	}
	public String getUserNpi() {
		return userNpi;
	}
	public void setUserNpi(String userNpi) {
		this.userNpi = userNpi;
	}
	public String getUserName() {
		return userName;
	}
	public void setUserName(String userName) {
		this.userName = userName;
	}
	public String getPatientId() {
		return patientId;
	}
	public void setPatientId(String patientId) {
		this.patientId = patientId;
	}
	public String getCategory() {
		return category;
	}
	public void setCategory(String category) {
		this.category = category;
	}
	public String getRequestingApp() {
		return requestingApp;
	}
	public void setRequestingApp(String requestingApp) {
		this.requestingApp = requestingApp;
	}
	public String getQueryStartDate() {
		return queryStartDate;
	}
	public void setQueryStartDate(String queryStartDate) {
		this.queryStartDate = queryStartDate;
	}
	public String getQueryEndDate() {
		return queryEndDate;
	}
	public void setQueryEndDate(String queryEndDate) {
		this.queryEndDate = queryEndDate;
	}
	public Date getRequestReceivedTime() {
		return requestReceivedTime;
	}
	public void setRequestReceivedTime(Date requestReceivedTime) {
		this.requestReceivedTime = requestReceivedTime;
	}
	public void setResponseSentTime(Date responseSentTime) {
		this.responseSentTime = responseSentTime;	
	}
	public Date getResponseSentTime() {
		return this.responseSentTime;
	}
	public void setDuration(long duration) {
		this.duration = duration;
	}
	public long getDuration() {
		return this.duration;
	}
	public String getSuccess() {
		return success;
	}
	public void setSuccess(String success) {
		this.success = success;
	}
	public String getRequestContent() {
		return requestContent;
	}
	public void setRequestContent(String requestContent) {
		this.requestContent = requestContent;
	}

	public Long getId( )
	{
	    return id;
	}
	@Override
	public String getItemId() {
		return itemId;
	}
	@Override
	public void setItemId(String itemId) {
		this.itemId = itemId;
	}
	@Override
	public String getTarget() {
		return target;
	}
	@Override
	public void setTarget(String target) {
		this.target = target;
		
	}
	@Override
	public String getDirection() {
		return this.direction;
	}
	@Override
	public void setDirection(String direction) {
		this.direction = direction;
	}
	@Override
	public String getNoteIen() {
		return noteIen;
	}
	@Override
	public void setNoteIen(String noteIen) {
		this.noteIen = noteIen;
	}
	@Override
	public String getTitleIen() {
		return titleIen;
	}
	@Override
	public void setTitleIen(String titleIen) {
		this.titleIen = titleIen;
	}
	@Override
	public String getEncounterString() {
		return encounterString;
	}
	@Override
	public void setEncounterString(String encounter) {
		this.encounterString = encounter;
	}
	@Override
	public String getNoteText() {
		return noteText;
	}
	@Override
	public void setNoteText(String noteText) {
		this.noteText = noteText;
	}
	@Override
	public void setRecordSiteCode(String recordSiteCode) {
		this.recordSiteCode = recordSiteCode;
	}
	@Override
	public String getRecordSiteCode() {
		return this.recordSiteCode;
	}
	@Override
	public void setCriteria(String criteria) {
		this.criteria = criteria;
	}
	@Override
	public String getCriteria() {
		return this.criteria;
	}
	@Override
	public void setMaxRecords(String maxRecords) {
		this.maxRecords = maxRecords;
	}
	@Override
	public String getMaxRecords() {
		return this.maxRecords;
	}
	@Override
	public void setType(String type) {
		this.type = type;
	}
	@Override
	public String getType() {
		return this.type;
	}
	@Override
	public void setStatus(String status) {
		this.status = status;
	}
	@Override
	public String getStatus() {
		return this.status;
	}
	
	@Override
	/**
	 * Constructs the detail content to store in the audit record - builds content primarily in Name:Value pairs and sets the content to be logged
	 * 
	 * @param webserviceAndWebserviceMethodName - String name of the webservice and the method called/processed
	 * @param queryBean - QueryBean object containing query elements
	 */
	public void buildRequestContent(String webserviceAndWebserviceMethodName, QueryBean queryBean) {
		//Buildup a string with basic top level content and then the full name/value pairs of all other parameters and queryBean properties name:value
		String newLine = "\r\n";
		StringBuilder contentBuilder = new StringBuilder();
		contentBuilder.append("Webservice and Webservice Method: " + webserviceAndWebserviceMethodName).append(newLine);
		contentBuilder.append("Request received and logged at: "  + this.getRequestReceivedTime().toString()).append(newLine);
		contentBuilder.append(newLine);
				
		//Top level parameter content name:value
		if(this.getRequestingApp()!=null)contentBuilder.append("REQUESTING APP: ").append(this.getRequestingApp()).append(newLine);
		if(this.getPatientId()!=null)contentBuilder.append("DFN: ").append(this.getPatientId()).append(newLine);
		if(this.getUserId()!=null)contentBuilder.append("DUZ: ").append(this.getUserId()).append(newLine);
		if(this.getSiteId() !=null)contentBuilder.append("SITE CODE: ").append(this.getSiteId()).append(newLine);
		if(this.getNoteIen()!=null)contentBuilder.append("NOTE IEN: ").append(this.getNoteIen()).append(newLine);
		contentBuilder.append(newLine);
				
		//Query Bean properties name:value
		if(queryBean.getRequestingApp() != null || queryBean.getRecordSiteCode() !=null ||queryBean.getItemId() !=null || queryBean.getStartDate() != null || queryBean.getEndDate() != null)
		{
			contentBuilder.append("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< Query Bean Properties >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>").append(newLine);		
			if(queryBean.getRecordSiteCode() != null) contentBuilder.append("QueryBean.recordSiteCode: ").append(queryBean.getRecordSiteCode()).append(newLine);
			if(queryBean.getItemId() != null) contentBuilder.append("QueryBean.itemId: ").append(queryBean.getItemId()).append(newLine);
			if(queryBean.getRequestingApp() != null) contentBuilder.append("QueryBean.requestingApp: ").append(queryBean.getRequestingApp()).append(newLine);
			if(queryBean.getStartDate() != null) contentBuilder.append("QueryBean.startDate: ").append(queryBean.getStartDate()).append(newLine);
			if(queryBean.getEndDate() != null) contentBuilder.append("QueryBean.endDate: ").append(queryBean.getEndDate()).append(newLine);
			if(queryBean.getTarget() != null)contentBuilder.append("QueryBean.target: ").append(queryBean.getTarget()).append(newLine);
			if(queryBean.getDirection() != null)contentBuilder.append("QueryBean.direction: ").append(queryBean.getDirection()).append(newLine);
			if(queryBean.getCriteria() != null)contentBuilder.append("QueryBean.criteria: ").append(queryBean.getCriteria()).append(newLine);
			if(queryBean.getMaxRecords() != null)contentBuilder.append("QueryBean.maxRecords: ").append(queryBean.getMaxRecords()).append(newLine);
			if(queryBean.getType() != null)contentBuilder.append("QueryBean.type: ").append(queryBean.getType()).append(newLine);
			if(queryBean.getStatus() != null)contentBuilder.append("QueryBean.status: ").append(queryBean.getStatus()).append(newLine);
			
			//QueryBean.provider
			Provider provider = queryBean.getProvider();
			if(provider != null){
				if(provider.getProviderName() != null) contentBuilder.append("QueryBean.provider.providerName: ").append(provider.getProviderName()).append(newLine);
				if(provider.getUserId() != null) contentBuilder.append("QueryBean.provider.userId: ").append(provider.getUserId()).append(newLine);
				if(provider.getProviderType() != null) contentBuilder.append("QueryBean.provider.providerType: ").append(provider.getProviderType()).append(newLine);
				if(provider.getProviderFlag() != null) contentBuilder.append("QueryBean.provider.providerFlag: ").append(provider.getProviderFlag()).append(newLine);
				if(provider.getUserClass() != null) contentBuilder.append("QueryBean.provider.userClass: ").append(provider.getUserClass()).append(newLine);
				if(provider.getLoginSiteCode() != null) contentBuilder.append("QueryBean.provider.loginSiteCode: ").append(provider.getLoginSiteCode()).append(newLine);
				if(provider.getLoginSiteName() != null) contentBuilder.append("QueryBean.provider.loginSiteName: ").append(provider.getLoginSiteName()).append(newLine);
				if(provider.getTitle() != null) contentBuilder.append("QueryBean.provider.title: ").append(provider.getTitle()).append(newLine);
				if(provider.getPhone() != null) contentBuilder.append("QueryBean.provider.phone: ").append(provider.getPhone()).append(newLine);
				if(provider.getEmailAddress() != null) contentBuilder.append("QueryBean.provider.emailAddress: ").append(provider.getEmailAddress()).append(newLine);			
			}		
			//QueryBean.patient
			Patient patient = queryBean.getPatient();
			if(patient != null){
				if(patient.getLocalPid() != null) contentBuilder.append("QueryBean.patient.localPid: ").append(patient.getLocalPid()).append(newLine);
				if(patient.getLocalSiteId() != null) contentBuilder.append("QueryBean.patient.localSiteId: ").append(patient.getLocalSiteId()).append(newLine);
				if(patient.getMpiPid() != null) contentBuilder.append("QueryBean.patient.mpiPid: ").append(patient.getMpiPid()).append(newLine);
				if(patient.getPatientType() != null) contentBuilder.append("QueryBean.patient.patientType: ").append(patient.getPatientType()).append(newLine);
				if(patient.isVeteran() != null) contentBuilder.append("QueryBean.patient.isVeteran: ").append(patient.isVeteran()).append(newLine);
				String[] locations = patient.getVistaLocations();
				if(locations != null)
				{
					StringBuilder vistaLocations = new StringBuilder();
					vistaLocations.append("( ");
					for(String location:locations){
						vistaLocations.append(location + "  ");
					}
					vistaLocations.append(")");
					contentBuilder.append("QueryBean.patient.vistaLocations: ").append(vistaLocations.toString()).append(newLine);
				}
			}
		}
		
		//write audit elements - notes ( write note )
		if (this.getTitleIen() !=null)
		{
			contentBuilder.append(newLine);
			contentBuilder.append("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< Write Audit (Write Notes) >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>").append(newLine);
			if(this.getTitleIen()!=null)contentBuilder.append("TITLE IEN: ").append(this.getTitleIen()).append(newLine);
			if(this.getEncounterString()!=null)contentBuilder.append("ENCOUNTER STRING: ").append(this.getEncounterString()).append(newLine);
			if(this.getNoteText()!=null)contentBuilder.append("NOTE TEXT: ").append(this.getNoteText()).append(newLine);
		}
		
		this.setRequestContent(contentBuilder.toString());

	}
	
	
	
}
