
package gov.va.med.via.audit;

import gov.va.med.via.audit.persistence.AuditLogPersistenceManager;
import gov.va.med.via.exception.ErrorCodeEnum;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *  RequestAutiodLogger implementing class - manages application logic, ensures request is correctly configured, populated and delgates storing/auditing
 *  of the request data to an AudtiLogPeristenceManager delegate
 *  
 *  @author ZZZZZZTALBOM
 */

public class RequestAuditLoggerImpl implements RequestAuditLogger{

	private static Log logger = LogFactory.getLog( RequestAuditLoggerImpl.class );
	private AuditLogPersistenceManager auditLogPersistenceManager;
	
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.audit.RequestAuditLogger#auditRequest(gov.va.med.via.audit.AuditRequest)
	 */
	public void auditRequest(AuditRequest auditRequest) {
		//TODO: TBD - determine if times can remain strings or if they need to be refactored into dates.
		 try{
	         String queryStart = auditRequest.getQueryStartDate();
	         String queryEnd = auditRequest.getQueryEndDate();
	         if(queryStart != null && queryEnd == null){
	        	 auditRequest.setQueryEndDate(System.currentTimeMillis()+"" );
	         }
	         if(queryStart == null && queryEnd == null){
	        	 auditRequest.setQueryStartDate(System.currentTimeMillis()+"" );
	        	 auditRequest.setQueryEndDate(System.currentTimeMillis()+"" );
	         }
			 
			 auditLogPersistenceManager.storeAuditLog( auditRequest );
	     }
	     catch ( Exception e ){
	    	 StringBuilder sb = new StringBuilder();
	         sb.append( ErrorCodeEnum.VIA_AUDIT_LOG_ERROR.getClientMessage() );
	         sb.append( " " );
	         sb.append( e.getMessage() );

	         if ( logger.isErrorEnabled() ){
	           logger.error( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( auditRequest.getCategory(), auditRequest.getRequestingApp(), sb.toString(), e ) );
	         }
	           
	     }	
	}

	/* (non-Javadoc)
	 * @see gov.va.med.via.audit.RequestAuditLogger#setAuditLogPersistenceManager(gov.va.med.via.audit.persistence.AuditLogPersistenceManager)
	 */
	public void setAuditLogPersistenceManager(AuditLogPersistenceManager auditLogPersistenceManager) {
		this.auditLogPersistenceManager = auditLogPersistenceManager;
	}

}
