
package gov.va.med.via.audit.persistence.hibernate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.orm.hibernate3.HibernateTemplate;

import gov.va.med.via.audit.AuditRequest;
import gov.va.med.via.audit.persistence.AuditLogPersistenceManager;

/**
 *  Implementing AuditLogPersistenceManager which will persist an AuditRequest by using a Hibernate Template
 *  persistence of the Audit is synchronous
 * 
 *  @author ZZZZZZTALBOM
 */

public class HibernateAuditLogPersistenceManagerImpl implements AuditLogPersistenceManager{

	 private static final Log logger = LogFactory.getLog( HibernateAuditLogPersistenceManagerImpl.class );
	 private HibernateTemplate hibernateTemplate;
	 
	 
	/* (non-Javadoc)
	 * @see gov.va.med.via.audit.persistence.AuditLogPersistenceManager#storeAuditLog(gov.va.med.via.audit.AuditRequest)
	 */
	@Override
	public void storeAuditLog(AuditRequest auditRequest) {
		try{
			hibernateTemplate.save( auditRequest );
            hibernateTemplate.flush();
        }
        catch ( RuntimeException ex ){
            logger.error( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( auditRequest.getCategory(), auditRequest.getRequestingApp(), "Error Auditing Request. ", ex ), ex);
        }
    }
	
	/**
	 * Configuration method - allows Spring injection of a Hibernate template used in persistence of an AuditRequest
	 * @param loggerHibernateTemplate
	 */
	public void setHibernateTemplate( HibernateTemplate loggerHibernateTemplate )
    {
        this.hibernateTemplate = loggerHibernateTemplate;
    }

		
}
