
package gov.va.med.via.audit.persistence.jms;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.jms.core.MessageCreator;

import gov.va.med.via.audit.AuditRequest;
import gov.va.med.via.audit.persistence.AuditLogPersistenceManager;

import javax.annotation.Resource;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;

/**
 *  JmsAuditLogPersistenceManager writes audit information from Requests asynchronously to RequestAuditQueue.
 *  An increase in performance and throughput is expected decoupling the audit path from the Request path
 * 
 *  @author ZZZZZZTALBOM
 *
 */

public class JmsAuditLogPersistenceManagerImpl implements AuditLogPersistenceManager{
	private static final Log logger = LogFactory.getLog( JmsAuditLogPersistenceManagerImpl.class );
	private JmsTemplate requestAuditMessageJmsTemplate = null;
	  
	/* (non-Javadoc)
	 * @see gov.va.med.via.audit.persistence.AuditLogPersistenceManager#storeAuditLog(gov.va.med.via.audit.AuditRequest)
	 */
	public void storeAuditLog( final AuditRequest auditRequest ){
		try{
			requestAuditMessageJmsTemplate.send( new MessageCreator()
				{public Message createMessage( Session session )throws JMSException
					{
					ObjectMessage objectMessage = session.createObjectMessage();
	                    objectMessage.setObject( (Serializable)auditRequest );
	                    return objectMessage;
	                }
	            } );
	      }
	      catch ( Exception exception ){
	    	  logger.error( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( auditRequest.getCategory(), auditRequest.getRequestingApp(), "Error Auditing Request message.", exception ), exception );
	      }
	}
	
	/**
	 * Configuration method that allows injection of a JmsTemplate delegate that will store/send the AuditRequest as a JMS message to a Jms Queue
	 * @param requestAuditMessageJmsTemplate
	 */
	@Resource
	public void setRequestAuditMessageJmsTemplate(JmsTemplate requestAuditMessageJmsTemplate) {
		this.requestAuditMessageJmsTemplate = requestAuditMessageJmsTemplate;
	}

}
