package gov.va.med.via.business;

import gov.va.med.via.databeans.DataBean;

import java.util.List;
import java.util.Map;

public interface ViaResponse {
	
	
	
	/**
	 * Accessor to the Map (keyed by DAO) of maps (keyed by siteCode) of lists of DataBean objects
	 * 
	 * @return Map<String,Map<String,List<DataBean>>>
	 */
	public Map<String,Map<String,List<DataBean>>> getResponseMap();
	
	
	/**
	 * Accessor to the Map of site specific DataBean lists
	 * 
	 * @param daoName
	 * @return Map<String,List<DataBean>>
	 */
	public Map<String,List<DataBean>> getSiteDataBeanMapForDao(String daoName);
	
	
	/**
	 * 
	 * Accessor to a List of DataBeans for a site and dao
	 * 
	 * @param daoName
	 * @param siteCode
	 * @return List<DataBean>
	 */
	public List<DataBean> getDomainDataBeansForSite(String daoName, String siteCode);


	/**
	 * This utility method is designed to simplify the extraction of the DataBean objects in the 
	 * case where a single Dao is used, a single site is used, and a single Databean object is expected to be returned. 
	 * 
	 * @return
	 * @throws UnsupportedOperationException if the response map has multiple daos; if the response map has data from multiple sites; or the List of DataBean contains more than one element 
	 * 
	 * 
	 */
	public DataBean getSingleResponseDataBeanFromSingleDaoAndSite();

	/**
	 * This utility method is designed to simplify the extraction of the DataBean List objects in the 
	 * case where a single Dao is used and a single site is used. 
	 * 
	 * @return
	 * @throws UnsupportedOperationException if the response map has multiple daos or if the response map has data from multiple sites 
	 * 
	 * 
	 */
	public List<DataBean> getResponseListFromSingleDaoAndSite();
	

}
