package gov.va.med.via.business;

import java.util.Map;


/**
 * Business interface for the Via business vista read processing
 * @author ZZZZZZTALBOM
 *
 */
public interface VistaService {
	
	/**
	 * 
	 * Generic processing method of service/business core processing
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse processVistaRead(Map<String,Object>requestParameterMap);
	
	
	/**
	 * @return boolean indicating processing is active - accessible 
	 */
	public boolean isAlive();
	
	
	/**
	 * Logs Provider User into the Via Application - authenticates with login site
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse loginProvider(Map<String,Object>requestParameterMap);
	
	/**
	 * Matches a patient from the search strings provided
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse matchPatient(Map<String,Object>requestParameterMap);
	
	
	/**
	 * Selects a patient from criteria provided by calling application
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse selectPatient(Map<String,Object>requestParameterMap);

}
