package gov.va.med.via.common.taglet;

import com.sun.tools.doclets.Taglet; 
import com.sun.javadoc.*;
import java.util.Map;

/**
 * A Taglet representing @autorule. This tag can be used in any kind of method. 
 * 
 */
public class AutoRuleTaglet implements Taglet {

    private static final String NAME = "autorule";
    private static final String HEADER = "Validation Rules:";

    /**
     * Return the name of this custom tag.
     */
    public String getName() {
        return NAME;
    }

    public boolean inField() {
        return false;
    }

    public boolean inConstructor() {
        return false;
    }

    public boolean inMethod() {
        return true;
    }

    public boolean inOverview() {
        return false;
    }

    public boolean inPackage() {
        return false;
    }

    public boolean inType() {
        return false;
    }

    /**
     * Will return false since <code>@rule</code>
     * is not an inline tag.
     * @return false since <code>@rule</code>
     * is not an inline tag.
     */
    public boolean isInlineTag() {
        return false;
    }

    /**
     * Register this Taglet.
     * @param tagletMap  the map to register this tag to.
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
	public static void register(Map tagletMap) {
       AutoRuleTaglet tag = new AutoRuleTaglet();
       Taglet t = (Taglet) tagletMap.get(tag.getName());
       if (t != null) {
           tagletMap.remove(tag.getName());
       }
       tagletMap.put(tag.getName(), tag);
    }

    private String toHtmlLine(String text)
    {
    	if ( text==null || text.trim().equals("")) return "&nbsp;";
    	return wrapCodeTagAroundVariables(text);
    }
    
    private String wrapCodeTagAroundVariables(String text)
    {
    	String[] parts = text.split("\\s");
    	StringBuilder builder = new StringBuilder();
    	
    	for (String part: parts )
    	{
    		if ( part.toLowerCase().startsWith("querybean") || part.toLowerCase().indexOf("querybean")!=-1)
    		{
    			part = "<code>"+part+"</code>";
    		}    		
    		builder.append(part+" ");
    	}
    	return builder.toString().trim();
    }
    
    
    /**
     * Given the <code>Tag</code> representation of this custom
     * tag, return its string representation.
     * @param tag   the <code>Tag</code> representation of this custom tag.
     */
    public String toString(Tag tag) 
    {
    	String text = tag.text();
    	text = toHtmlLine(text);
    	
    	return "<p><DT><B>" + HEADER + "</B><DD>"
               + "<table cellpadding=2 cellspacing=0><tr><td>"
               + "- " +text
               + "</td></tr></table></DD>";
    }

    /**
     * Given an array of <code>Tag</code>s representing this custom
     * tag, return its string representation.
     * @param tags  the array of <code>Tag</code>s representing of this custom tag.
     */
    public String toString(Tag[] tags) 
    {
        if (tags.length == 0) {
            return null;
        }
        String result = "<p><DT><B>" + HEADER + "</B><DD>";
        result += "<table cellpadding=2 cellspacing=0><tr><td>";
        for (int i = 0; i < tags.length; i++) {
            if (i > 0) 
            {
                result+= "<br/>";
            }
            result += "- "+toHtmlLine(tags[i].text());
        }
        return result + "</td></tr></table></DD>";
    }
}
