package gov.va.med.via.common.validator;

import java.text.SimpleDateFormat;

import gov.va.med.via.rpc.RPCStringUtils;

public class AccessionNumberStringValidator extends StringNotEmptyValidator 
{

	protected static SimpleDateFormat sdf = new SimpleDateFormat("MMddyy");
	
	public AccessionNumberStringValidator(Rule rule, Object webMethodParameter) {
		super(rule, webMethodParameter);
	}

	@Override
	public void validate() throws ValidationException {
		super.validate();

		String accessionNumber = targetValue.trim();
		String message = rule.target + " (AccessionNumber) does not match 'MMddyy-case#' (e.g. 042070-123) format.";
		
		int dashIndex = accessionNumber.indexOf('-');
		if ( dashIndex==-1)
		{
			throw new ValidationException(message);
		}
		if ( accessionNumber.endsWith("-"))
		{
			throw new ValidationException(message);
		}
		if ( accessionNumber.startsWith("-"))
		{
			throw new ValidationException(message);
		}

		String caseNumber = accessionNumber.substring(dashIndex+1);
		if ( caseNumber.trim().isEmpty())
		{
			throw new ValidationException(message);
		}
		
		/// the first part appears to be a vistaProcedureDateTime
		String dateString = accessionNumber.substring(0,dashIndex);
		
		/// assume DateFormat.VISTA
		if (RPCStringUtils.isNullOrEmpty(dateString))
        {
			throw new ValidationException(message);
        }
		
		try
		{
			sdf.parse(dateString);
		}
		catch ( Exception e)
		{
			throw new ValidationException(message);
		}
		
		
		/// convert  a vista timestamp like 3011225 to 12 25 01    
		
		/// also should be able to handle a date in ISO format ( 20101225 )
		
		
/*
        if (dateFormat == DateFormat.VISTA)
        {
            if (RPCStringUtils.isNullOrEmpty(vistaProcedureDateTime) || vistaProcedureDateTime.length() < 7)
            {
                //throw new exceptions.MdoException(exceptions.MdoExceptionCode.ARGUMENT_DATE_FORMAT, "Expected a Vista format timestamp (e.g. 3011225 or 3011225.120005)");
            }
            return RPCStringUtils.CSubstring(vistaProcedureDateTime, 3, 2) +
            		RPCStringUtils.CSubstring(vistaProcedureDateTime, 5, 2) + 
            		RPCStringUtils.CSubstring(vistaProcedureDateTime, 1, 2) + "-" + caseNumber;
        }
        if (dateFormat == DateFormat.ISO)
        {
            if (RPCStringUtils.isNullOrEmpty(vistaProcedureDateTime) || vistaProcedureDateTime.length() < 8)
            {
                //throw new exceptions.MdoException(exceptions.MdoExceptionCode.ARGUMENT_DATE_FORMAT, "Expected a ISO format timestamp (e.g. 20101225)");

            }
            return RPCStringUtils.CSubstring(vistaProcedureDateTime, 4, 2) + 
            	   RPCStringUtils.CSubstring(vistaProcedureDateTime, 6, 2) +
            	   RPCStringUtils.CSubstring(vistaProcedureDateTime, 2, 2) + "-" + caseNumber;
        }
*/
		
	}

	@Override
	public String toJavadocString() 
	{
		String message = "queryBean.criteria (AccessionNumber) does not match 'MMddyy-<case#>' (e.g. 042070-123) format.";
		return message;
	}

	
	
}
