package gov.va.med.via.common.validator;

import gov.va.med.via.util.VistaTimestamp;

public class EncounterStringValidator extends StringNotEmptyValidator
{
	
	public EncounterStringValidator(Rule rule, Object webMethodParameter) {
		super(rule, webMethodParameter);
	}

	/*
	 * This methods checks if String target value is null
	 * @exception the ValidationException related to the curent rule  
	 */
	public void validate() throws ValidationException
	{
		super.validate();
		
		String[] flds = targetValue.split(";");
		if (flds.length != 3)
		{
			throw new ValidationException("Invalid encounter string: The encounter string does not contain 3 parts separated by semicolons");
		}

		String locationIEN=flds[0];
		if ( locationIEN==null || locationIEN.trim().equals(""))
		{
			throw new ValidationException("Invalid encounter string: non-numeric location IEN.  The first part of the 3 part, semicolon-separated encounter string is a null or empty string");
		}
		if (!isNumeric(locationIEN))
		{
			throw new ValidationException("Invalid encounter string: non-numeric location IEN.  The first part of the 3 part encounter string does not contain a numeric location ien");
		}

		String vistaTimestamp = flds[1];
		if ( !"NOW".equals(vistaTimestamp) && !VistaTimestamp.isValidVistaTimestamp(vistaTimestamp))
		{
			throw new ValidationException("Invalid vista timestamp: The 2nd part of the 3 part, semicolon-separated encounter string " +
					" must have a format of [yyyy -1700]MMdd.HHmmss.  For example, 3130715.163242 represents the date and time of JUL 15, 2013 at 16:32:42");
		}


		String type = flds[2];
		if (!type.equals("A") && !type.equals("H") && !type.equals("E"))
		{
			throw new ValidationException("Invalid encounter string: type must be A, H or E. The type (3rd part of the 3 part encounter string separated by semicolons) must be A, H, or E");
		}

	}
	
	
	
	@Override
	public String toJavadocString()
	{
		return rule.target + " must be a valid encounter string";
	}

}
