package gov.va.med.via.common.validator;

import java.text.SimpleDateFormat;
import java.util.Date;

public class GetNotesWithTextDateRangeValidator extends Value1Value2Validator {

	

	public static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
	
	public GetNotesWithTextDateRangeValidator(Rule rule,
			Object webMethodParameter) {
		super(rule, webMethodParameter);
	}

	@Override
	public void validate() throws ValidationException 
	{
		super.validate();
		
		String sStartDate=value1;
		String sEndDate = value2; 
		Date startDate=null;
		// startDate can be null, can be 0, can be a yyyyMMdd value
		if ( sStartDate==null || sStartDate.trim().isEmpty() || sStartDate.trim().equals("0"))
		{
			// ignore
		}
		else
		{
			try
			{
				startDate =sdf.parse(sStartDate);
			}
			catch ( Exception e)
			{				
				throw new ValidationException("queryBean.startDate cannot be '"+sStartDate+"'. Accepted values are '0' or date in 'yyyyMMdd' format");
			}
		}
		
		Date endDate=null;
		if ( sEndDate==null || sEndDate.trim().equals("") || sEndDate.trim().equals("0"))
		{
			// ignore
		}
		else
		{
			try
			{
				
				endDate =sdf.parse(sEndDate);
			}
			catch ( Exception e)
			{				
				throw new ValidationException("queryBean.endDate cannot be '"+sEndDate+"'. Accepted values are '0' or date in 'yyyyMMdd' format");
			}
		}
		
		if ( startDate!=null && endDate!=null)
		{
			/// check the date ranges for validity
			if (endDate.before(startDate))
			{
				throw new ValidationException("Invalid date range. queryBean.endDate '"+ sEndDate+"' cannot occur before queryBean.startDate '"+sStartDate+"'");
			}
		}
		


	}
	
	@Override
	public String toJavadocString() 
	{
		return rule.value1 + " and " + rule.value2 +" must be a valid date range. Accepted values are '0' (for unlimited) or a date in 'yyyyMMdd' format" ;
	}
	

}
