package gov.va.med.via.common.validator;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;

/**
 * RequiredParameterComparisonValidator is used when one of two parameters is required based
 * upon the value of a third parameter - the compareValue
 * Branching values are accessed and used to determine the value of the compareValue and then
 * a determination can be made which of two parameters is required.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class RequiredParameterComparisonValidator extends RuleValidator {

	public RequiredParameterComparisonValidator(Rule rule, Object webMethodParameter) {
		super(rule, webMethodParameter);
	}


	@Override
	public void validate() throws ValidationException {
		
		
		String compareValue;
		String ifCompare="";
		String elseCompare="";
		
		try {
			compareValue = (String)PropertyUtils.getProperty(webMethodParameter, rule.getCompareValue());
			ifCompare = rule.getIfCompare();
			elseCompare= rule.getElseCompare();
			
			if(compareValue == null){
				throw new Exception();
			}
		} catch (Exception e) {
			throw new ValidationException(rule.getCompareValue() + " not in input or not a String");
		}
		
		if(compareValue.equalsIgnoreCase(ifCompare))
		{
			try {
				String value1 = (String)PropertyUtils.getProperty(webMethodParameter, rule.getValue1());
				if(value1 == null){
					throw new ValidationException(rule.getValue1() + " is required when queryBean.type is '"+ifCompare+"'");
				}
				else if (!StringUtils.isNumeric(value1)){
					throw new ValidationException(rule.getValue1() + " must be a numeric id value");
				}
			} catch (Exception e) {
				throw new ValidationException(rule.getValue1() + " is required when queryBean.type is '"+ifCompare+"'");
			}	
		}
		else if(compareValue.equalsIgnoreCase(elseCompare))
		{
			try {
				String value2 = (String)PropertyUtils.getProperty(webMethodParameter, rule.getValue2());
				if(value2 == null){
					throw new ValidationException(rule.getValue2() + " is required when queryBean.type is '"+elseCompare+"'");
				}
				else if (!StringUtils.isNumeric(value2)){
					throw new ValidationException(rule.getValue2() + " must be a numeric id value");
				}
			} catch (Exception e) {
				throw new ValidationException(rule.getValue2() + " is required when queryBean.type is '"+elseCompare+"'");
			}	
		}

	}

	@Override
	public String toJavadocString()
	{
		if ( rule.getErrMsg()!=null && !rule.getErrMsg().trim().isEmpty())
		{
			return rule.getErrMsg();
		}

		return "The comparison value will determine which one of two different parameters is required.  Check with the System Administrator how to construct your request appropriately for this validation rule.";
	}


	
}
