package gov.va.med.via.common.validator;

/* Rule is the class that encapsulates the Validation Rule
 * It defines the attributes of a Rule
 * @author      Sridhar Alluri
 * @version     %I%, %G%
 * @since		1.0
 */
public class Rule {
	public static String DEFAULT_EXPRESSION_SEPARATOR=";";
	/**
	 * the Rule's default 'parts' separator
	 */
	private String separator=DEFAULT_EXPRESSION_SEPARATOR;
	
	/*
	 * Hold the name of the rule. For eg, equals, between
	 */
	public String name;
	
	/*
	 * Hold the target of the rule. It is the property on which the rule is applied
	 */
	public String target;
	
	/*
	 * Hold the data type of the target of the rule. For eg, String, Date etc
	 */
	public String targetType;
	
	/*
	 * Hold the first value against which target needs to be checked. It is used in all the rules
	 */
	public String value1;
	
	/*
	 * Hold the second value against which target needs to be checked. It is used in some of the rules
	 */
	public String value2;
	
	/*
	 * Holds a comparison value - used rarely - when a parameter (compareValue) is accessed and then value one of two values must be present (value1 or value2)
	 */
	public String compareValue;
	
	/*
	 * Holds a branching/decision value used to compare against the compareValue
	 */
	public String ifCompare;
	
	/*
	 * Holds a branching/decision value for and if then comparison -  used to compare against the compareValue
	 */
	public String elseCompare;
	
	
	
	
	/** 
	 * holds the 'short-form' expression that contains the above attributes
	 */
	public String expression; 
	
	
	private String javadoc;
	
	
	
	public String errMsg;
	
	public Boolean nullAllowed=Boolean.FALSE;  /// false by default
	
	public Boolean getNullAllowed() {
		return nullAllowed;
	}

	public void setNullAllowed(Boolean nullAllowed) {
		this.nullAllowed = nullAllowed;
	}

	/*
	 * Returns the name of the rule
	 */
	public String getName() {
		return name;
	}
	
	/*
	 * Returns the target type of the rule
	 */
	public String getTargetType() {
		return targetType;
	}
	
	/*
	 * Sets the data type of the target
	 * @param  targetType  data type of the target
	 */
	public void setTargetType(String targetType) {
		this.targetType = targetType;
	}
	
	
	/*
	 * Sets the name of the rule
	 * @param  name  Name of the rule
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/*
	 * Returns the target property that is validated
	 */
	public String getTarget() {
		return target;
	}
	
	/*
	 * Sets the target of the rule
	 * @param  target  Property that is the target of the rule
	 */
	public void setTarget(String target) {
		this.target = target;
	}
	
	/*
	 * Returns the value1 property
	 */
	public String getValue1() {
		return value1;
	}
	
	/*
	 * Sets the value1 property of the rule
	 * @param  value1  Value against which the target is compared
	 */
	public void setValue1(String value1) {
		this.value1 = value1;
	}
	
	/*
	 * Returns the value2 property
	 */
	public String getValue2() {
		return value2;
	}
	
	/*
	 * Sets the value2 property of the rule
	 * @param  value2 Second Value against which the target is compared.
	 */
	public void setValue2(String value2) {
		this.value2 = value2;
	}

	public String getExpression() {
		return expression;
	}

	
	public void setExpression(String expression) 
	{
		this.expression = expression;
		new RuleExpressionParser().parseRuleExpression(this);
	}


	public String getSeparator() {
		return separator;
	}

	public void setSeparator(String separator) {
		this.separator = separator;
	}

	public String getErrMsg() {
		return errMsg;
	}

	public void setErrMsg(String errMsg) {
		this.errMsg = errMsg;
	}

	public String getJavadoc() {
		return javadoc;
	}

	public void setJavadoc(String javadoc) {
		this.javadoc = javadoc;
	}

	public String getCompareValue() {
		return compareValue;
	}

	public void setCompareValue(String compareValue) {
		this.compareValue = compareValue;
	}

	public String getIfCompare() {
		return ifCompare;
	}

	public void setIfCompare(String ifCompare) {
		this.ifCompare = ifCompare;
	}

	public String getElseCompare() {
		return elseCompare;
	}

	public void setElseCompare(String elseCompare) {
		this.elseCompare = elseCompare;
	}
	
	
}
