package gov.va.med.via.common.validator;

public class StringBetweenLengthValidator extends StringNotEmptyValidator 
{
	public StringBetweenLengthValidator(Rule rule, Object webMethodParameter) {
		super(rule, webMethodParameter);
	}

	/*
	 * checks if String target value length is between minimum and maximum length
	 * @exception the ValidationException related to the curent rule  
	 */
	@Override
	public void validate() throws ValidationException
	{
		super.validate();
		
		int targetLength = targetValue.length();

		if (rule.value1==null ){
			throw new ValidationException("Value1 cannot be null");
		}					
		if (rule.value2==null ){
			throw new ValidationException("Value2 cannot be null");
		}	

		int minLength = -1;
		try
		{		
			minLength =  Integer.parseInt(rule.value1);
		}
		catch ( NumberFormatException e)
		{
			throw new ValidationException("Value1 '"+rule.value1+"' must be a positive integer");
		}
		if ( minLength<0)
		{
			throw new ValidationException("Value1 '"+rule.value1+"' must be greater than or equal to zero");
		}
		
		
		int maxLength = -1;
		try
		{		
			maxLength =  Integer.parseInt(rule.value2);
		}
		catch ( NumberFormatException e)
		{
			throw new ValidationException("Value2 '"+rule.value2+"' must be a positive integer");
		}
		if ( minLength<1)
		{
			throw new ValidationException("Value2 '"+rule.value2+"' must be greater than zero");
		}
		
		if (minLength>maxLength)
		{
			throw new ValidationException("Value1 '"+rule.value1+"' cannot be greater than Value 2 '"+rule.value2+"'");
		}
		
		if (targetLength<minLength || targetLength>maxLength){
			throw new ValidationException(rule.target + " needs to have minlength of " + minLength + 
					" and maxlength of " + maxLength);
		}    
		
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.common.validator.RuleValidator#toJavadocString()
	 */
	@Override
	public String toJavadocString()
	{
		return rule.target + " must have a minimum length of "+rule.value1+" and a maximum length of " + rule.value2;
	}

}
