package gov.va.med.via.common.validator;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/* ValidationErrors is the class that holds the collection of Validation errors
 * @author      Sridhar Alluri
 * @version     %I%, %G%
 * @since		1.0
 */
public class ValidationErrors {
	/*
	 * Logger
	 */
	private static Log log = LogFactory.getLog(ValidationErrors.class);
	
	/*
	 * Holds the collection of exceptions
	 */
	private ArrayList<Exception> errors;
	
	/*
	 * Sole constructor.
	 * Instantiates the list that holds the errors
	 */
	public ValidationErrors(){
		errors = new ArrayList<Exception>();
	}
	
	/*
	 * This method takes an exception and adds it to the list of exceptions
	 * @param  ex  the exception that needs to be added to the list of exceptions
	 */
	public void addException(Exception ex){
		errors.add(ex);
	}
	
	/*
	 * This method returns the number of exceptions so that the Validator client
	 * can interrogate if there were any errors during validation
	 * @return  the number of errors found during validation
	 */
	public int getNumberOfErrors(){
		return errors.size();
	}
	
	/*
	 * This method returns the collection of exceptions that were found during validation
	 * @return  the collection of errors
	 */
	public Collection<Exception> getErrors(){
		return errors;
	}
	
	/*
	 * This method logs the Validation errors
	 */
	public void logErrors(){
		for(Exception error: errors){
			log.debug(error.toString());
		}
	}
	
	/*
	 * This method creates a String of all the concatenated errors.
	 */
	public String getConcatenatedErrors()
	{
    	StringBuilder str = new StringBuilder(); /// string builder is faster
    	boolean first =true;
		for(Exception error: errors)
		{
			if ( first ) first=false;
			else str.append(", ");
       		str.append(error.toString());
		}
		
		return str.toString();
	}	
}
