package gov.va.med.via.common.validator;

import gov.va.med.via.exception.ErrorCodeEnum;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/*
 * Validator implements the Validation of Rules on targets
 */
public class Validator 
{
	private static Log log = LogFactory.getLog(Validator.class);

	private List<Rule> rules=new ArrayList<Rule>();	

	private List<Validator> validators=new ArrayList<Validator>();

	public List<Rule> getRules() {
		return rules;
	}

	public void setRules(List<Rule> rules) {
		this.rules = rules;
	}


	public List<Validator> getValidators() {
		return validators;
	}

	public void setValidators(List<Validator> validators) {
		this.validators = validators;
	}


	/*
	 * This methods validates the properties against the rules 
	 * @param webMethodParameter the object that holds the properties that need to be validated 
	 * @return the list of errors(could be empty)
	 */
	public ValidationErrors validate(Object webMethodParameter)
	{
		ValidationErrors errors = new ValidationErrors();
		if ( rules != null)
		{
			for (Rule rule: rules)
			{
				String targetType = rule.getTargetType();   	

				RuleNames ruleName =  RuleNames.valueOf(rule.getName());  
				try 
				{
					validateRule(webMethodParameter, rule, targetType, ruleName);
				} 
				catch (Exception e) 
				{
					errors.addException(e);
				}
			}
		}

		if ( validators!= null)
		{
			for (Validator validator: validators)
			{
				ValidationErrors currentErrors = validator.validate(webMethodParameter);
				Collection<Exception> exceptions = currentErrors.getErrors();
				for(Exception exception: exceptions)
				{
					errors.addException(exception);
				}
			}
		}
		return errors;
	}

	private void validateRule(Object webMethodParameter, Rule rule,
			String targetType, RuleNames ruleName) {
		switch ( ruleName)
		{

		case notnull:
		{
			if ( targetType==null || targetType.equalsIgnoreCase("String"))
			{
				new StringNotNullValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("Object"))
			{
				new ObjectNotNullValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("StringArray"))
			{
				new StringArrayNotNullValidator(rule,webMethodParameter).validate();
			}
			else
			{
				throw new ValidationException("targetType " + targetType + " has no defined 'notnull' validator");
			}
			break;
		}
		case notempty:
		{
			new StringNotEmptyValidator(rule,webMethodParameter).validate();
			break;
		}
		case equals:
		{
			if ( targetType==null || targetType.equalsIgnoreCase("String"))
			{
				new StringEqualsValue1Validator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("Date"))
			{
				new DateEqualsValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("int"))
			{
				new IntegerEqualsValue1Validator(rule,webMethodParameter).validate();
			}
			else
			{
				throw new ValidationException("targetType " + targetType + " has no defined 'equals' validator");
			}
			break;
		}
		case minlength:
		{
			new StringMinLengthValidator(rule,webMethodParameter).validate();
			break;
		}
		case maxlength:
		{
			new StringMaxLengthValidator(rule,webMethodParameter).validate();
			break;
		}
		case between:
		{
			if ( targetType==null || targetType.equalsIgnoreCase("String"))
			{
				new StringBetweenLengthValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("Date"))
			{
				new DateBetweenValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("int"))
			{
				new IntegerBetweenValue1AndValue2Validator(rule,webMethodParameter).validate();
			}
			else
			{
				throw new ValidationException("targetType " + targetType + " has no defined 'between' validator");
			}
			break;					
		}
		case pattern:
		{
			new StringPatternValidator(rule,webMethodParameter).validate();
			break;
		}
		case optional_pattern:
		{
			new OptionalPatternValidator(rule,webMethodParameter).validate();
			break;
		}		
		case lessthan:
		{
			new IntegerLessThanValue1Validator(rule,webMethodParameter).validate();
			break;
		}
		case mandatory_pattern:
		{
			new MandatoryPatternValidator(rule,webMethodParameter).validate();
			break;
		}
		case morethan:
		{
			new IntegerMoreThanValue1Validator(rule,webMethodParameter).validate();
			break;
		}
		case before:
		{
			new DateBeforeValidator(rule,webMethodParameter).validate();
			break;
		}
		case after:
		{
			new DateAfterValidator(rule,webMethodParameter).validate();
			break;
		}
		case valid_encounterstring:
		{
			new EncounterStringValidator(rule,webMethodParameter).validate();
			break;
		}
		case valid_ien:
		{
			new IENStringValidator(rule,webMethodParameter).validate();
			break;					
		}
		case valid_optional_ien:
		{
			new OptionalIENStringValidator(rule,webMethodParameter).validate();
			break;					
		}
		case maxRecords:
		{
			new MaxRecordsStringValidator(rule,webMethodParameter).validate();
			break;	
		}
		case valid_duz:
		{
			new DuzStringValidator(rule,webMethodParameter).validate();
			break;	
		}
		case valid_ssn:
		{
			new SocialSecurityNumberStringValidator(rule,webMethodParameter).validate();
			break;	
		}
		case valid_accessionnumber:
		{
			new AccessionNumberStringValidator(rule,webMethodParameter).validate();
			break;	
		}
		case get_notes_with_text_date_range:
		{
			new GetNotesWithTextDateRangeValidator(rule,webMethodParameter).validate();
			break;	
		}
		case null_or_negative_one_or_valid_govt_date:
		{
			new NullorNegativeOneorYYYYMMDDValidator(rule,webMethodParameter).validate();
			break;	
		}
		case compareValue1Value2:
		{
			new Value1EqualsValue2Validator(rule,webMethodParameter).validate();
			break;
		}

		case null_or_valid_utcDate:
		{
			new NullOrUtcDateStringValidator(rule, webMethodParameter).validate();
			break;
		}

		case dateRangeNoTarget:
		{
			new DateRangeValidator(rule, webMethodParameter).validate();
			break;
		}
		case null_or_negative_one:
		{
			if ( targetType==null || targetType.equalsIgnoreCase("String"))
			{
				new NullorNegativeOneorYYYYMMDDValidator(rule,webMethodParameter).validate();
			}
			else if ( targetType.equalsIgnoreCase("int"))
			{
				new NullOrNegativeOneIntegerValidator(rule,webMethodParameter).validate();
			}
			else
			{
				throw new ValidationException("targetType " + targetType + " has no defined 'null_or_negative_one' validator");
			}
			break;
		}
		case value1BasedOnOptValue2:
		{
			new Value1RequiredIfValue2PresentValidator(rule,webMethodParameter).validate();
			break;
		}
		case valid_utcDate:
		{
			new UtcDateStringValidator(rule,webMethodParameter).validate();
			break;
		}
		case optional_string_not_emtpy:
		{
			new OptionalStringNotEmptyValidator(rule,webMethodParameter).validate();
			break;			
		}
		case null_or_valid_integer:
		{
			new NullOrUtcDateStringValidator(rule, webMethodParameter).validate();
			break;
		}
		case state_abbr:
		{
			new StateAbbreviationValidator(rule,webMethodParameter).validate();
			break;
		}
		case city:
		{
			new CityValidator(rule,webMethodParameter).validate();
			break;
		}
		case parameter_required_based_on_compare_value:
		{
			new RequiredParameterComparisonValidator(rule, webMethodParameter).validate();
			break;
		}
		case valid_visitstring:
		{
			new VisitStringValidator(rule,webMethodParameter).validate();
			break;
		}		
		default:
		{
			throw new ValidationException(ErrorCodeEnum.VALIDATION_EXCEPTION, " Invalid rulename: " + rule.getName());
		}
		}
	}


	/*
	 * This method is a debug used to log the rules
	 */
	public void logRules(){
		for (Rule rule: rules){
			log.debug("rule Name: " +rule.getName());
			log.debug("rule target: " +rule.getTarget());
		}
	}



}
