package gov.va.med.via.common.validator;

import gov.va.med.via.util.VistaTimestamp;

/**
 * The Class VisitStringValidator.
 * 
 * @author DOMAINROMERJ
 */
public class VisitStringValidator extends StringNotEmptyValidator
{
	
	/**
	 * Instantiates a new visit string validator.
	 *
	 * @param rule the rule
	 * @param webMethodParameter the web method parameter
	 */
	public VisitStringValidator(Rule rule, Object webMethodParameter) {
		super(rule, webMethodParameter);
	}

	/*
	 * This methods checks if String target value is null
	 * @exception the ValidationException related to the curent rule  
	 */
	/* (non-Javadoc)
	 * @see gov.va.med.via.common.validator.StringNotEmptyValidator#validate()
	 */
	public void validate() throws ValidationException
	{
		super.validate();
		
		String[] flds = targetValue.split(";");
		if (flds.length != 3)
		{
			throw new ValidationException("Invalid visit string: The visit string does not contain 3 parts separated by semicolons");
		}

		String type = flds[0];
		if (!type.equals("A") && !type.equals("H") && !type.equals("E"))
		{
			throw new ValidationException("Invalid visit string: type must be A, H or E. The type (1st part of the 3 part visit string separated by semicolons) must be A, H, or E");
		}

		String vistaTimestamp = flds[1];
		if ( !"NOW".equals(vistaTimestamp) && !VistaTimestamp.isValidVistaTimestamp(vistaTimestamp))
		{
			throw new ValidationException("Invalid vista timestamp: The 2nd part of the 3 part, semicolon-separated visit string " +
					" must have a format of [yyyy -1700]MMdd.HHmmss.  For example, 3130715.163242 represents the date and time of JUL 15, 2013 at 16:32:42");
		}

		String locationIEN=flds[2];
		if ( locationIEN==null || locationIEN.trim().equals(""))
		{
			throw new ValidationException("Invalid visit string: non-numeric location IEN.  The third part of the 3 part, semicolon-separated visit string is a null or empty string");
		}
		if (!isNumeric(locationIEN))
		{
			throw new ValidationException("Invalid visit string: non-numeric location IEN.  The third part of the 3 part visit string does not contain a numeric location ien");
		}

	}
	
	
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.common.validator.StringNotEmptyValidator#toJavadocString()
	 */
	@Override
	public String toJavadocString()
	{
		return rule.target + " must be a valid visit string";
	}

}
