package gov.va.med.via.databeans;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;

public class Encounter {

	String id;
	String patientId;
	String providerId;
	String timestamp;
	String locationId;
	String type;
	KeyValuePair<String, String> service;
	String purpose;
	String description;

	public Encounter() {
	}

	/** parses an Encounter from an EncounterString. This method has been migrated from C# to Java 
	 *  and was originally located in the MDWS 2.5 NoteLib:getEncounterFromString method.
	 *  
	 * 
	 * @param encounterString
	 * @return
	 * @throws ValidationException 
	 */
	public static Encounter parseFromEncounterString(String encounterString) throws ValidationException
	{
		String[] flds = encounterString.split(";");
		if (flds.length != 3)
		{
			throw new ValidationException("Invalid encounter string: The encounter string does not contain 3 parts separated by semicolons (locationIEN;vistaTimestamp;A or H or E");
		}

		String locationIEN=flds[0];
		if ( locationIEN==null || locationIEN.trim().equals(""))
		{
			throw new ValidationException("Invalid encounter string: non-numeric location IEN.  The first part of the 3 part, semicolon-separated encounter string is a null or empty string");
		}
		if (!StringUtil.isNumeric(locationIEN))
		{
			throw new ValidationException("Invalid encounter string: non-numeric location IEN.  The first part of the 3 part encounter string does not contain a numeric location ien");
		}
		String type = flds[2];

		String timestampString = flds[1];
		if (!"NOW".equals(timestampString) && !VistaTimestamp.isValidVistaTimestamp(timestampString))
		{
			throw new ValidationException("Invalid vista timestamp");
		}
		
		//TBD: how to test for valid VistA timestamp (fld[1])
		if (!type.equals("A") && !type.equals("H") && !type.equals("E"))
		{
			throw new ValidationException("Invalid encounter string: type must be A, H or E. The type (3rd part of the 3 part encounter string) must be A, H, or E");
		}

		Encounter encounter = new Encounter();
		encounter.setLocationId(locationIEN);
		encounter.setTimestamp(timestampString);
		encounter.setType(type);
		return encounter;
	}
	
	
	
	
	
	
	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getPatientId() {
		return patientId;
	}

	public void setPatientId(String patientId) {
		this.patientId = patientId;
	}

	public String getProviderId() {
		return providerId;
	}

	public void setProviderId(String providerId) {
		this.providerId = providerId;
	}

	public String getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(String timestamp) {
		this.timestamp = timestamp;
	}

	public String getLocationId() {
		return locationId;
	}

	public void setLocationId(String locationId) {
		this.locationId = locationId;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public KeyValuePair<String, String> getService() {
		return service;
	}

	public void setService(KeyValuePair<String, String> service) {
		this.service = service;
	}

	public String getPurpose() {
		return purpose;
	}

	public void setPurpose(String purpose) {
		this.purpose = purpose;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	

}
