
package gov.va.med.via.databeans;

import java.util.Calendar;

/**
 * The LabOrder class represents lab orders for a patient.
 */
public class LabOrder extends BaseDataBean
{
  /**
   * Date the sample was collected.
   */
  private Calendar collectionDate;
  /**
   * Name of test ordered.
   */
  private String testName;
  /**
   * The sample collected from the patient for this lab order.
   */
  private String collectionSample;
  /**
   * Internal Id of the order, used to retrieve results for an order.
   */
  private String internalId;
  /**
   * Order Id of the order.
   */
  private String orderId;
  /**
   * Subscript of the test associated with the lab order.  Subscripts indicate the 
   * type of results to be returned.  The getLabOrderResults method is used for CH
   * Subscripts.
   * 
   * Other Subscripts: MI - MicroBiology, SP - Surgical Pathology, CY - Cytology,
   * AU - Autopsy, BB-BloodBank, EM - Electron Microscopy
   */
  private String subscript;
  private Calendar orderDate;
  private String requestingProvider;
  private String status;

    /**
     * Lab results associated with lab order -
     * this is used if data source returns lab results along with order data (BHIE).
     */
  private LabResult[] labResults;
  
  public LabOrder()
  {
  }

  
  
  /**
   * Calendar patient sample was collected.
   */
  public Calendar getCollectionDate()
  {
    return collectionDate;
  }

  public void setCollectionDate(Calendar newCollectionDate)
  {
    collectionDate = newCollectionDate;
  }

  /**
   * Lab test name for this lab order.
   */
  public String getTestName()
  {
    return testName;
  }

  public void setTestName(String newTestName)
  {
    testName = newTestName;
  }

  /**
   * Collection sample for the lab order.
   */
  public String getCollectionSample()
  {
    return collectionSample;
  }

  public void setCollectionSample(String newCollectionSample)
  {
    collectionSample = newCollectionSample;
  }

  /**
   * Internal ID of lab order.  Used to retrieve results for order.
   */
  public String getInternalId()
  {
    return internalId;
  }

  public void setInternalId(String newInternalId)
  {
    internalId = newInternalId;
  }

  /**
   * Order ID of this lab order.
   */
  public String getOrderId()
  {
    return orderId;
  }

  public void setOrderId(String newOrderId)
  {
    orderId = newOrderId;
  }

  /**
   * Subscript for the test associated with the lab order.
   * 
   * Values: CH - CLinical Chemistry
   * MI - Microbiology
   * SP - Surgical Pathology
   * AU - Autopsy
   * BB - BloodBank
   * EM - Electron Microscopy
   */
  public String getSubscript()
  {
    return subscript;
  }

  public void setSubscript(String newSubscript)
  {
    subscript = newSubscript;
  }

  public Calendar getOrderDate()
  {
    return orderDate;
  }

  public void setOrderDate(Calendar newOrderDate)
  {
    orderDate = newOrderDate;
  }

  public String getRequestingProvider()
  {
    return requestingProvider;
  }

  public void setRequestingProvider(String newRequestingProvider)
  {
    requestingProvider = newRequestingProvider;
  }

  public String getStatus()
  {
    return status;
  }

  public void setStatus(String status)
  {
    this.status = status;
  }

    public LabResult[] getLabResults() {
        return labResults;
    }

    public void setLabResults(LabResult[] labResults) {
        this.labResults = labResults;
    }
}