 
package gov.va.med.via.databeans;
import java.util.Date;

/**
 * Demographic information regarding a patient. This class extends the Patient class.
 */
public class PatientDemographics extends Patient 
{
 
  /**
   * Line 1 of address of patient.
   */
  private String address1;
  /**
   * Line 2 of the patient's address.
   */
  private String address2;
  /**
   * City of the patient's address.
   */
  private String city;
  /**
   * State where the patient resides.
   */
  private String state;
  /**
   * Zip code of the patient's address.
   */
  private String zipCode;
  /**
   * Primary Care Manage start and end dates.  Not populated for vista patients.
   */
  private Date pcmDates;
  /**
   * Primary Provider assigned to the patient.
   */
  private String primaryProvider;
  /**
   * Clinic name the patient is assigned to.
   */
  private String clinic;
  /**
   * Phone number of the patient's Primary Care Manager.
   */
  private String pcmPhone;
  /**
   * Primary phone number for the patient.
   */
  private String phone1;
  /**
   * Secondary phone number for patient.
   */
  private String phone2;
  /**
   * Internal if of the admission if the patient is an inpatient.
   */
  private String admissionId;
  /**
   * Admission date if the patient is an inpatient.
   */
  private String admissionDate;
  /**
   * Ward the patient is assigned to if an inpatient.
   */
  private String ward;
  /**
   * Internal id of ward if the patient is an inpatient.
   */
  private String wardId;

  /**
   * Internal lab id of the patient.  Only defined for vista patients.
   */
  private String labId;

  /**
   * Code Green Alert
   */
  private String codeGreen;

  private String percentServiceConnected;

  private String siteCode;
  private String siteName;
  private String siteMoniker;
  private String sourcePlatform;
  private String siteAgency;

  public PatientDemographics()
  {
  }

  public PatientDemographics(String siteCode)
  {
    this.siteCode = siteCode;
  }


  public String getSiteCode() {
      return siteCode;
  }

  public void setSiteCode(String SiteCode) {
    this.siteCode = SiteCode;
  }

  public String getSiteName() {
    return siteName;
  }

  public void setSiteName(String SiteName) {
    this.siteName = SiteName;
  }

  public String getSiteMoniker()
  {
    return siteMoniker;
  }

  public void setSiteMoniker(String siteMoniker)
  {
    this.siteMoniker = siteMoniker;
  }
  
  /**
   * Line 1 of the patient's address.
   */
  public String getAddress1()
  {
    return address1;
  }

  public void setAddress1(String newAddress1)
  {
    address1 = newAddress1;
  }

  /**
   * Line 2 of the patient address.
   */
  public String getAddress2()
  {
    return address2;
  }

  public void setAddress2(String newAddress2)
  {
    address2 = newAddress2;
  }

  /**
   * City of the patient's address.
   */
  public String getCity()
  {
    return city;
  }

  public void setCity(String newCity)
  {
    city = newCity;
  }

  /**
   * State of the patient's address.
   */
  public String getState()
  {
    return state;
  }

  public void setState(String newState)
  {
    state = newState;
  }

  /**
   * Zip Code of the patient's address.
   */
  public String getZipCode()
  {
    return zipCode;
  }

  public void setZipCode(String newZipCode)
  {
    zipCode = newZipCode;
  }

  /**
   * Start and end dates of PCM assignment. Not defined for Vista data.
   */
  public Date getPcmDates()
  {
    return pcmDates;
  }

  public void setPcmDates(Date newPcmDates)
  {
    pcmDates = newPcmDates;
  }

  /**
   * Primary Provider name assigned to the patient.
   */
  public String getPrimaryProvider()
  {
    return primaryProvider;
  }

  public void setPrimaryProvider(String newPrimaryProvider)
  {
    primaryProvider = newPrimaryProvider;
  }

  /**
   * Clinic name the patient is assigned to.
   */
  public String getClinic()
  {
    return clinic;
  }

  public void setClinic(String newClinic)
  {
    clinic = newClinic;
  }

  /**
   * Provider's phone number.
   */
   public String getPcmPhone()
  {
    return pcmPhone;
  }

  public void setPcmPhone(String newpcmPhone)
  {
    pcmPhone = newpcmPhone;
  }

  /**
   * Primary phone number of the patient.
   */
  public String getPhone1()
  {
    return phone1;
  }

  /**
   * Primary Phone number of the patient.
   */
  public void setPhone1(String newPhone1)
  {
    phone1 = newPhone1;
  }

  /**
   * Secondary phone number for the patient.
   */
  public String getPhone2()
  {
    return phone2;
  }

  public void setPhone2(String newPhone2)
  {
    phone2 = newPhone2;
  }

  /**
   * Internal id of the admission if the patient is an inpatient.
   */
  public String getAdmissionId()
  {
    return admissionId;
  }

  public void setAdmissionId(String newAdmissionId)
  {
    admissionId = newAdmissionId;
  }

  /**
   * Date of admission if the patient is an inpatient.
   */
  public String getAdmissionDate()
  {
    return admissionDate;
  }

  public void setAdmissionDate(String newAdmissionDate)
  {
    admissionDate = newAdmissionDate;
  }

  /**
   * Ward name if the patient is an inpatient.
   */
  public String getWard()
  {
    return ward;
  }

  public void setWard(String newWard)
  {
    ward = newWard;
  }

  /**
   * Internal id of the patient's ward if an inpatient.
   */
  public String getWardId()
  {
    return wardId;
  }

  public void setWardId(String newWardId)
  {
    wardId = newWardId;
  }

  /**
   * Internal lab id of the patient.  Only set for Vista data.
   */
  public String getLabId()
  {
    return labId;
  }

  public void setLabId(String newLabId)
  {
    labId = newLabId;
  }

  /**
   * Code Green Alert
   */
   public String getCodeGreen()
   {
     return codeGreen;
   }

   public void setCodeGreen(String newCodeGreen)
   {
     codeGreen = newCodeGreen;
   }

  public String getPercentServiceConnected()
  {
    return percentServiceConnected;
  }

  public void setPercentServiceConnected(String newPercentServiceConnected)
  {
    percentServiceConnected = newPercentServiceConnected;
  }

    public String getSourcePlatform() {
        return sourcePlatform;
    }

    public void setSourcePlatform(String sourcePlatform) {
        this.sourcePlatform = sourcePlatform;
    }

    public String getSiteAgency() {
        return siteAgency;
    }

    public void setSiteAgency(String siteAgency) {
        this.siteAgency = siteAgency;
    }
}