package gov.va.med.via.databeans;

import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.UserSecurityKey;

/**
 * Class represents a provider/user.
 */
public class Provider extends Person 
{
	
    private String nickName;
    private String pwd;
    //Team team;
    private String office; 
    private String voicePager;
    private String digitalPager;
    private String title;
    private String phone;
    private String esig;
    private String userClass;
    private String orderRole;
    private String greeting;
    private String emailAddress;
    private String currentContext;
    private Boolean defunct;
    private Boolean canWriteMedOrders;
    private String inactiveDate;
    private Boolean isInactive;
    private String providerType;
    private Boolean requiresCosigner;
    private Provider usualCosigner;
    /*Dev Note - legacy code used a Service databean object - but the only thing transfered is the service name and a simple string will suffice*/
    //Service service;
    private String service;
        
	/*
	 * Provider's user id. Set by a call to the XUS AV CODE RPC in RPC Client
	 */
	private String userId;

	/**
	 * set as the context value "OR CPRS GUI CHART" in VistaAccountDao
	 */
	private String permissionString;

	/**
	 * Site code associated with Provider's userId. This value is set in the
	 * VistaAccountDao.callRpc() method
	 */
	private String loginSiteCode;

	/**
	 * Name of provider. This value is set in the VistaAccountDao.callRpc()
	 * method with rpc call to ORWU USERINFO
	 */
	private String providerName;

	/**
	 * Site code name.
	 */
	private String loginSiteName;

	/**
	 * National Provider Ide
	 */
	private String NPI;

	/**
	 * Security Phrase - old -BSE related feature
	 */
	private String securityPhrase;

	/**
	 * Provider class of provider.
	 */
	private String providerClass;

	/**
	 * Office phone number for provider.
	 */
	private String dutyPhone;
	/**
	 * Provider's pager number.
	 */
	private String pager;
	/**
	 * Flag to indicate the user is a provider. (1=provider)
	 */
	private String providerFlag;
	/**
	 * Signature class of the provider.
	 */
	private String sigText;

	private KeyValuePair[] sitePermissions;

	private UserSecurityKey[] securityKeys;

	private KeyValuePair primaryPermission;

	private String sigClass;

	private KeyValuePair[] locationIdMappings;

	// // Stores the access and verify code in encrypted form.*/
	// / the VIA application no longer needs to store an encrypted AV code. When
	// a VIA web service makes a login call,
	// / then the DUZ is established. The DUZ is the userID attribute. This DUZ
	// is used in subsequent VIA calls
	// / and followup login calls are not necessary
	// / private String encryptedAvCode;

	private String cfg;

	private Boolean dodProvider;

	public Provider() {
	}

	/**
	 * Name of provider.
	 */
	public String getProviderName() {
		return providerName;
	}

	public void setProviderName(String newName) {
		providerName = newName;
		this.name = newName;
	}

	/**
	 * Provider class of provider.
	 */
	public String getProviderClass() {
		return providerClass;
	}

	public void setProviderClass(String newProviderClass) {
		providerClass = newProviderClass;
	}


	/**
	 * Office phone of provider.
	 */
	public String getDutyPhone() {
		return dutyPhone;
	}

	public void setDutyPhone(String newDutyPhone) {
		dutyPhone = newDutyPhone;
	}

	/**
	 * Provider's pager number.
	 */
	public String getPager() {
		return pager;
	}

	public void setPager(String newPager) {
		pager = newPager;
	}

	/**
	 * Flag indicating provider has provider status.
	 */
	public String getProviderFlag() {
		return providerFlag;
	}

	public void setProviderFlag(String newProviderFlag) {
		providerFlag = newProviderFlag;
	}

	/**
	 * Signature class of provider.
	 */
	public String getSigText() {
		return sigText;
	}

	public void setSigText(String newSigClass) {
		sigText = newSigClass;
	}

	public UserSecurityKey[] getSecurityKeys() {
		return securityKeys;
	}

	public void setSecurityKeys(UserSecurityKey[] securityKeys) {
		this.securityKeys = securityKeys;
	}

	public KeyValuePair[] getSitePermissions() {
		return sitePermissions;
	}

	public void setSitePermissions(KeyValuePair[] sitePermissions) {
		this.sitePermissions = sitePermissions;
	}

	public String getPermissionString() {
		return permissionString;
	}

	public void setPermissionString(String permissionString) {
		this.permissionString = permissionString;
	}

	public String getUserId() {
		return userId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}

	public KeyValuePair getPrimaryPermission() {
		return primaryPermission;
	}

	public void setPrimaryPermission(KeyValuePair primaryPermission) {
		this.primaryPermission = primaryPermission;
	}

	public String getSigClass() {
		return sigClass;
	}

	public void setSigClass(String sigClass) {
		this.sigClass = sigClass;
	}

	public KeyValuePair[] getLocationIdMappings() {
		return locationIdMappings;
	}

	public void setLocationIdMappings(KeyValuePair[] locationIdMappings) {
		this.locationIdMappings = locationIdMappings;
	}

	public String getLoginSiteCode() {
		return loginSiteCode;
	}

	public void setLoginSiteCode(String loginSiteCode) {
		this.loginSiteCode = loginSiteCode;
	}

	public String getLoginSiteName() {
		return loginSiteName;
	}

	public void setLoginSiteName(String loginSiteName) {
		this.loginSiteName = loginSiteName;
	}

	// public String getEncryptedAvCode()
	// {
	// return encryptedAvCode;
	// }
	//
	// public void setEncryptedAvCode(String encryptedAvCode)
	// {
	// this.encryptedAvCode = encryptedAvCode;
	// }

	public String getCfg() {
		return cfg;
	}

	public void setCfg(String cfg) {
		this.cfg = cfg;
	}

	public Boolean isDodProvider() {
		return dodProvider;
	}

	public void setDodProvider(boolean dodProvider) {
		this.dodProvider = dodProvider;
	}

	public String getNPI() {
		return NPI;
	}

	public void setNPI(String NPI) {
		this.NPI = NPI;
	}

	public String getSecurityPhrase() {
		return securityPhrase;
	}

	public void setSecurityPhrase(String securityPhrase) {
		this.securityPhrase = securityPhrase;
	}

	public String getNickName() {
		return nickName;
	}

	public void setNickName(String nickName) {
		this.nickName = nickName;
	}

	public String getPwd() {
		return pwd;
	}

	public void setPwd(String pwd) {
		this.pwd = pwd;
	}

	public String getOffice() {
		return office;
	}

	public void setOffice(String office) {
		this.office = office;
	}

	public String getVoicePager() {
		return voicePager;
	}

	public void setVoicePager(String voicePager) {
		this.voicePager = voicePager;
	}

	public String getDigitalPager() {
		return digitalPager;
	}

	public void setDigitalPager(String digitalPager) {
		this.digitalPager = digitalPager;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public String getEsig() {
		return esig;
	}

	public void setEsig(String esig) {
		this.esig = esig;
	}

	public String getUserClass() {
		return userClass;
	}

	public void setUserClass(String userClass) {
		this.userClass = userClass;
	}

	public String getOrderRole() {
		return orderRole;
	}

	public void setOrderRole(String orderRole) {
		this.orderRole = orderRole;
	}

	public String getGreeting() {
		return greeting;
	}

	public void setGreeting(String greeting) {
		this.greeting = greeting;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public String getCurrentContext() {
		return currentContext;
	}

	public void setCurrentContext(String currentContext) {
		this.currentContext = currentContext;
	}

	public String getInactiveDate() {
		return inactiveDate;
	}

	public void setInactiveDate(String inactiveDate) {
		this.inactiveDate = inactiveDate;
	}

	public String getProviderType() {
		return providerType;
	}

	public void setProviderType(String providerType) {
		this.providerType = providerType;
	}

	public Provider getUsualCosigner() {
		return usualCosigner;
	}

	public void setUsualCosigner(Provider usualCosigner) {
		this.usualCosigner = usualCosigner;
	}

	public Boolean getDodProvider() {
		return dodProvider;
	}

	public void setDodProvider(Boolean dodProvider) {
		this.dodProvider = dodProvider;
	}

	public Boolean isDefunct() {
		return defunct;
	}

	public void setDefunct(Boolean defunct) {
		this.defunct = defunct;
	}

	public Boolean canWriteMedOrders() {
		return canWriteMedOrders;
	}

	public void setCanWriteMedOrders(Boolean canWriteMedOrders) {
		this.canWriteMedOrders = canWriteMedOrders;
	}

	public Boolean isInactive() {
		return isInactive;
	}

	public void setIsInactive(Boolean isInactive) {
		this.isInactive = isInactive;
	}

	public Boolean requiresCosigner() {
		return requiresCosigner;
	}

	public void setRequiresCosigner(Boolean requiresCosigner) {
		this.requiresCosigner = requiresCosigner;
	}
	
	@Override
	public void setName(String name){
		this.name = name;
		this.providerName = name;
	}
	
	@Override
	public String getName(){
		return this.providerName;
	}
	
	/*Dev Note - legacy code used a Service databean object - but the only thing transfered is the service name and a simple string will suffice*/
	public void setService(String serviceName){
		this.service = serviceName;
	}
	
	
	public String getService(){
		return this.service;
	}
}