 
package gov.va.med.via.databeans;

import gov.va.med.via.util.StringUtil;

import java.util.HashMap;
import java.util.Map;

/**
 *
 */
public class QueryBean
{
	
    private Provider provider;
    private Patient patient;
    private String sortBy;

    private String recordSiteCode;
    private String requestingApp;
    
    private boolean multiSiteQuery = true;//default true, but can be switched to limit sites processed.
    
    private Map<String,String>additionalQueryParams = new HashMap<String,String>();//used internally for the GenericRpc - development centric
    
    private String[] supplementalParameters;
    

    /**
    * Set to 1 if requesting only active data (appointments, prescriptions, clinical
    * reminders)
    */
    private String active;
    /**
    * End date for the query.
    */
    private String endDate;
    /**
    * Start date for the query.
    */
    private String startDate;
    /**
    * Maximum number of items to return.  This is usually a required attribute when
    * the return is an array of items.
    */
    private String maxRecords;

    /**
    * Status of data being requested.  This could be Inpatient vs outpatient (procedures,
    * prescriptions/medications)
    */
    private String status;

    /**
    * This is the internal is of the data item when detailed informations is being
    * requested.
    */
    private String itemId;
    
	/**
	* This is a query value used by several RPC queries - it has a generic use, however many client apps of the
	* legacy service methods VIAS replaces will recognize this query parameter by name
	* replacing legacy services wil
	* requested.
	*/
	private String target;
	 
	/**
	* This is a query value, used by several RPC queries - it has a generic use across methods
	* requested.
	*/
	private String criteria;
	  
	/**
	* This is a specific named query parameter
	* requested.
	*/
	private String type;
	   
	/**
	* This is a specific named query parameter
	* requested.
	*/
	private String direction;
	
	/**
     * This is a specific named query parameter
     * requested.
     */
     private String name;

    public Patient getPatient() {
        return patient;
    }

    public void setPatient(Patient patient) {
        this.patient = patient;
    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public String getSortBy() {
        return sortBy;
    }

    public void setSortBy(String sortBy) {
        this.sortBy = sortBy;
    }

    public String getActive() {
        return active;
    }

    public void setActive(String active) {
        this.active = active;
    }

    public String getEndDate() {
        return endDate;
    }

    public void setEndDate(String endDate) {
        this.endDate = endDate;
    }

    public String getItemId() {
        return itemId;
    }

    public void setItemId(String itemId) {
        this.itemId = itemId;
    }

    public String getMaxRecords() {
        return maxRecords;
    }

    public void setMaxRecords(String max) {
        this.maxRecords = max;
    }

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getRecordSiteCode() {
        return recordSiteCode;
    }

    public void setRecordSiteCode(String recordSiteCode) {
        this.recordSiteCode = recordSiteCode;
    }

    public String getRequestingApp() {
        return requestingApp;
    }

    public void setRequestingApp(String requestingApp) {
        this.requestingApp = requestingApp;
    }
    
    /**
     * Determines the correct siteCode/siteCodes the processing should connect to and process the webservice request
     * It might be the set of sites a patient has been seen at - the treatment facility site - OR if those are empty in the patient
     * it would be the patient's loginSite and/or the recordSiteCode set on the queryBean - various different application logic will be used
     * at different times.
     * 
     * @return a String[] containing a targeted set of siteCodes - which may consist of a single value
     */
    public String[] getSiteCodes() {
    	   	
		return this.getSiteCodes(this.multiSiteQuery);
	}
    
    public String[] getSiteCodes(String webservice) {
    	
    	//TODO use a configured map of services to determine if it is by default a multi site service
    	//for now use the default of true
    	return getSiteCodes(this.multiSiteQuery);
	}
    
    public String[] getSiteCodes(boolean multiSite) {
    	
    	Patient patient = this.getPatient();
    	Provider provider = this.getProvider();
    	String[] patientLocations = {};
    	String siteCode = null;
    	String itemId = this.getItemId();
    	if(itemId != null){	
    		siteCode = this.getRecordSiteCode();
    		if(siteCode == null && provider != null){
    			siteCode = provider.getLoginSiteCode();
    		}
    	}
    	else if(patient != null && multiSite){
    		patientLocations = patient.getVistaLocations();
    		if(patientLocations == null || patientLocations.length < 1)
    		{
    			siteCode = (patient.getLocalSiteId() != null)?patient.getLocalSiteId():this.getRecordSiteCode();
    		}
    	}
    	else if (provider != null){
    		siteCode = (provider.getLoginSiteCode() != null)?provider.getLoginSiteCode():this.getRecordSiteCode();
    	}
    	else{
    		//QueryBean not yet populated with a patient or a provider 
    		siteCode = this.getRecordSiteCode();    		
    	}
    	
    	if(!StringUtil.isEmpty(siteCode))
    	{
    		patientLocations = new String[1];
    		patientLocations[0]= siteCode;
    	}
    	
		return patientLocations;
	}
    
    public String getLoginSiteCode() {
    	
    	String siteCode = null;
    	Provider provider = this.getProvider();
    	if (provider != null){
    		siteCode = (provider.getLoginSiteCode() != null)?provider.getLoginSiteCode():this.getRecordSiteCode();
    	}
    	else{
    		//QueryBean not yet populated with a provider 
    		siteCode = this.getRecordSiteCode();    		
    	}
    	
    	return siteCode;
	}

	public boolean isMultiSiteQuery() {
		return multiSiteQuery;
	}

	public void setMultiSiteQuery(boolean multiSiteQuery) {
		this.multiSiteQuery = multiSiteQuery;
	}
	
	public void addAdditionalQueryParam(String key, String param){
		if(!this.additionalQueryParams.containsKey(key)){
			this.additionalQueryParams.put(key, param);
		}
	}
	
	public String getAdditionalQueryParam(String key){
		return this.additionalQueryParams.get(key);
	}
	
	public void clearAdditionalQueryParam()
	{
		this.additionalQueryParams.clear();
	}

	public Map<String, String> getAdditionalQueryParams() {
		return additionalQueryParams;
	}

	public void setAdditionalQueryParams(Map<String, String> additionalQueryParams) {
		this.additionalQueryParams = additionalQueryParams;
	}
	
	
	public String[] getSupplementalParameters() {
		return this.supplementalParameters;
	}

	public void setSupplementalParameters(String[] supplementalParameters) {
		this.supplementalParameters = supplementalParameters;
	}
	
	public String getTarget() {
		return target;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	public String getCriteria() {
		return criteria;
	}

	public void setCriteria(String criteria) {
		this.criteria = criteria;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getDirection() {
		return direction;
	}

	public void setDirection(String direction) {
		this.direction = direction;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	

}
