

package gov.va.med.via.exception;

import java.text.MessageFormat;


/**
 *  Base exception class of all the VIA core project exceptions.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public abstract class AbstractViaBaseException
    extends
        RuntimeException
{

    private static final long serialVersionUID = -5240505020001244412L;

    private ErrorCodeEnum errorCode = null;

    private String[] customMsgValues = null;
    
    //indicates to exception handling processes whether the exception has been previously logged - used to avoid duplicate logging of rethrown exceptions
    private boolean logged = false;


    /**
     * Constructor
     * 
     * @param errorCode
     */
    public AbstractViaBaseException( ErrorCodeEnum errorCode )
    {
        this( errorCode, null, ( String[] )null );
    }


    /**
     *  Constructor
     * 
     * @param errorCode
     * @param customMsgValues
     */
    public AbstractViaBaseException( ErrorCodeEnum errorCode, String... customMsgValues )
    {
        this( errorCode, null, customMsgValues );
    }


    /**
     *  Constructor
     * 
     * @param errorCode
     * @param cause
     */
    public AbstractViaBaseException( ErrorCodeEnum errorCode, Throwable cause )
    {
        this( errorCode, cause, ( String[] )null );
    }


    /**
     *  Constructor
     * 
     * @param errorCode
     * @param cause
     * @param customMsgValues
     */
    public AbstractViaBaseException( ErrorCodeEnum errorCode, Throwable cause, String... customMsgValues )
    {
    	super( createErrorMessage(errorCode, customMsgValues), cause );

        setErrorCode( errorCode );
        setCustomMsgValues( customMsgValues );

    }


    private static java.lang.String createErrorMessage(ErrorCodeEnum errorCode, java.lang.String[] customMsgValues) {
    	String errorMessage = errorCode.getClientMessage();
    	if ( customMsgValues != null && customMsgValues.length > 0 )
        {
            MessageFormat messageFormat = new MessageFormat( errorMessage );
            errorMessage = messageFormat.format( customMsgValues );
        }
        return errorMessage;
	}


	

	/**
     * Gets the errorcode that has the error message.
     */
    public ErrorCodeEnum getErrorCode( )
    {
        return errorCode;
    }


    /**
     * Sets the errorcode that has the error message.
     */
    private void setErrorCode( ErrorCodeEnum errorCodeName )
    {
        this.errorCode = errorCodeName;
    }


    /**
     * Gets the runtime data values to be embedded in the error message.
     */
    public String[] getCustomMsgValues( )
    {
        return customMsgValues;
    }


    /**
     * Sets the runtime data values to be embedded in the error message.
     */
    public void setCustomMsgValues( String[] customMsgValues )
    {

        this.customMsgValues = customMsgValues;

    }


	/**
	 * @return boolean indicating already logged
	 */
	public boolean isLogged() {
		return logged;
	}


	/**
	 * sets the already logged flag
	 * @param logged
	 */
	public void setLogged(boolean logged) {
		this.logged = logged;
	}
}
