

package gov.va.med.via.exceptionframework;

import gov.va.med.via.exception.ExceptionUtil;

import java.util.Date;


/**
 * Utility that concatenates the error logger message with the application processing area and the Webservice method name; the Requesting Application and an Error Message
 */
public class LogMessageUtil
{
    private static final String MESSAGE_DELIMETER = "#";
    private static final String EMPTY_STRING = "";
    private static final String EMPTY_STACK_TRACE = "";
    
    
    public static String buildMessage( String category, String requestingApp, String errorMessage, Exception exception )
    {
    	
    	String errorStackTrace = EMPTY_STACK_TRACE;
    	   	
    	if ( category == null )
        {
        	category = EMPTY_STRING;
        }
        if ( requestingApp == null )
        {
        	requestingApp = EMPTY_STRING;
        }
        if ( errorMessage == null )
        {
            errorMessage = EMPTY_STRING;
        }
        if(exception != null)
        {
        	errorStackTrace = ExceptionUtil.buildErrorStackTraceString(exception);
        }
        
        String messageAndStackTrace = "EXCEPTION MESSAGE: " + errorMessage + "\n\r" + errorStackTrace;
        
        StringBuilder message = new StringBuilder();

        message.append( MESSAGE_DELIMETER );
        message.append( category ).append( MESSAGE_DELIMETER );
        message.append( requestingApp ).append( MESSAGE_DELIMETER );
        message.append( messageAndStackTrace );

        return quotedString( message.toString() );
    }
    
    public static String buildMessageInfo( String msg )
    {
        
        if ( msg == null )
        {
        	msg = EMPTY_STRING;
        }

        StringBuilder message = new StringBuilder();

        message.append( msg );

        return quotedString( message.toString() );
    }


    /**
     * Creates a SQL-quoted string. Any single-quotes ['] are doubled [''] in order to make the string "SQL compatible".
     *
     * @param source Description of Parameter
     * @return "SQL Compatible" string, after replacing single-quotes ['] with double quotes ['']
     */
    private static String quotedString( String source )
    {
        if ( source == null )
        {
            return EMPTY_STRING;
        }
        else
        {
            // ???? why isn't this using the " char?  I would think two singles would be screwey-er. SJB
            source = replace( source, "'", "''" );
        }

        return source;
    }


    /**
     * String-replacer
     *
     * @param source Description of Parameter
     * @param find Description of Parameter
     * @param replacement Description of Parameter
     * @return Description of the Returned Value
     */
    public static String replace( String source, String find, String replacement )
    {
        int i = 0;
        int j;
        int k = find.length();
        int m = replacement.length();

        while ( i < source.length() )
        {
            j = source.indexOf( find, i );

            if ( j == -1 )
            {
                break;
            }

            source = replace( source, j, j + k, replacement );

            i = j + m;
        }

        return source;
    }


    /**
     * String-replacer
     *
     * @param source Description of Parameter
     * @param start Description of Parameter
     * @param end Description of Parameter
     * @param replacement Description of Parameter
     * @return String after replacing provided character set
     */
    public static String replace( String source, int start, int end, String replacement )
    {
        if ( start == 0 )
        {
            source = replacement + source.substring( end );
        }
        else if ( end == source.length() )
        {
            source = source.substring( 0, start ) + replacement;
        }
        else
        {
            source = source.substring( 0, start ) + replacement + source.substring( end );
        }

        return source;
    }


    /**
     * int-replacer
     *
     * @param source Description of Parameter
     * @param find Description of Parameter
     * @param replacement Description of Parameter
     * @return String after replacing provided character set
     */
    public static String replace( String source, String find, int replacement )
    {
        String sReplacement = Integer.toString( replacement );

        return replace( source, find, sReplacement );
    }
}
