package gov.va.med.via.hl7.mpi;

public class EncodingCharacters {

	public static final int CMP_SEP_CHAR = 0;
	public static final  int REP_SEP_CHAR = 1;
	public static final  int ESCAPE_CHAR = 2;
	public static final  int SUB_SEP_CHAR = 3;

	char fieldSep;
	char[] encChars;

	public EncodingCharacters() {
		setFieldSeparator('|');
		encChars = new char[4];
		setComponentSeparator('^');
		setRepetitionSeparator('~');
		setEscapeCharacter('\\');
		setSubcomponentSeparator('&');
	}

	public EncodingCharacters(char fieldSeparator, String encodingCharacters)
	{
		setFieldSeparator( fieldSeparator == '\0' ? '|' : fieldSeparator);
		encChars = new char[4];
		if (encodingCharacters == null)
		{
			setComponentSeparator('^');
			setRepetitionSeparator('~');
			setEscapeCharacter('\\');
			setSubcomponentSeparator('&');
		}
		else
		{
			for (int i = 0; i < 4; i++)
			{
				encChars[i] = encodingCharacters.charAt(i);
			}
		}
	}

	public EncodingCharacters(
			char fieldSeparator,
			char componentSeparator,
			char repetitionSeparator,
			char escapeCharacter,
			char subcomponentSeparator)
	{
		setFieldSeparator( fieldSeparator);
		setComponentSeparator(componentSeparator);
		setRepetitionSeparator(repetitionSeparator);
		setEscapeCharacter( escapeCharacter);
		setSubcomponentSeparator(subcomponentSeparator);
	}

	/** copies contents of "other" */
    public EncodingCharacters(EncodingCharacters other)
    {
        setFieldSeparator( other.getFieldSeparator());
        encChars = new char[4];
        setComponentSeparator(other.getComponentSeparator());
        setRepetitionSeparator(other.getRepetitionSeparator());
        setEscapeCharacter(other.getEscapeCharacter());
        setSubcomponentSeparator( other.getSubcomponentSeparator());
    }
	

	public char getFieldSeparator() {
		return fieldSep;
	}

	public void setFieldSeparator(char fieldSep) {
		this.fieldSep = fieldSep;
	}

	public char[] getEncChars() 
	{
		return encChars;
	}

	public void setEncChars(char[] encChars) 
	{
		this.encChars = encChars;
	}

	public char getComponentSeparator() 
	{
		return encChars[CMP_SEP_CHAR];
	}

	public void setComponentSeparator(char componentSeparator) {
		encChars[CMP_SEP_CHAR]= componentSeparator;
	}

	public char getRepetitionSeparator() {
		return encChars[REP_SEP_CHAR];
	}

	public void setRepetitionSeparator(char repetitionSeparator) {
		encChars[REP_SEP_CHAR] = repetitionSeparator;
	}

	public char getEscapeCharacter() {
		return encChars[ESCAPE_CHAR];
	}

	public void setEscapeCharacter(char escapeCharacter) {
		encChars[ESCAPE_CHAR] = escapeCharacter;
	}

	public char getSubcomponentSeparator() {
		return encChars[SUB_SEP_CHAR];
	}

	public void setSubcomponentSeparator(char subcomponentSeparator) {
		encChars[SUB_SEP_CHAR] = subcomponentSeparator;
	}

	
	 /**
     * Returns the encoding characters (not including field separator)
     * as a string.
     */
    public String toString()
    {
        String result = "";
        for (int i = 0; i < this.encChars.length; i++)
        {
            result += this.encChars[i];
        }
        return result;
    }
    
    public Object clone()
    {
        return new EncodingCharacters(this);
    }
	
}
