package gov.va.med.via.hl7.mpi;

import gov.va.med.via.util.StringUtil;

public class MsaSegment {

	 EncodingCharacters encChars = new EncodingCharacters();
     String ackCode = "";
     String msgCtlId = "";
     String textMessage = "";
     String expectedSequenceNumber = "";
     String delayedAckType = "";
     String errorId = "";
     String errorText = "";
	
	public MsaSegment() {
	}

	public MsaSegment(String rawSegmentString) 
    {
	    parse(rawSegmentString);
    }
	
    public void parse(String rawSegmentString)
    {
        String[] flds = StringUtil.split(rawSegmentString, getEncodingChars().getFieldSeparator());

        if (!"MSA".endsWith(flds[0]))
        {
            throw new RuntimeException("Invalid MSA segment: incorrect header");
        }

        if ("".equals(flds[1]))
        {
            throw new RuntimeException("Invalid MSA segment: missing acknowledgement code");
        }
        setAckCode(flds[1]);

        if ("".equals(flds[2]))
        {
            throw new RuntimeException("Invalid MSA segment: missing message control ID");
        }
        setMessageControlId(flds[2]);

        if (flds.length > 3)
        {
            setTextMessage( flds[3]);
        }
        if (flds.length > 4)
        {
            setExpectedSequenceNumber(flds[4]);
        }
        if (flds.length > 5)
        {
            setDelayedAckType(flds[5]);
        }
        if (flds.length > 6)
        {
            String[] components = StringUtil.split(flds[6], getEncodingChars().getComponentSeparator());
            setErrorId( components[0]);
            setErrorText( components[1]);
        }
    }


	
	public EncodingCharacters getEncodingChars() {
		return encChars;
	}

	public void setEncodingChars(EncodingCharacters encChars) {
		this.encChars = encChars;
	}

	public String getAckCode() {
		return ackCode;
	}

	public void setAckCode(String ackCode) {
		this.ackCode = ackCode;
	}

	public String getMessageControlId() {
		return msgCtlId;
	}

	public void setMessageControlId(String msgCtlId) {
		this.msgCtlId = msgCtlId;
	}

	public String getTextMessage() {
		return textMessage;
	}

	public void setTextMessage(String textMessage) {
		this.textMessage = textMessage;
	}

	public String getExpectedSequenceNumber() {
		return expectedSequenceNumber;
	}

	public void setExpectedSequenceNumber(String expectedSequenceNumber) {
		this.expectedSequenceNumber = expectedSequenceNumber;
	}

	public String getDelayedAckType() {
		return delayedAckType;
	}

	public void setDelayedAckType(String delayedAckType) {
		this.delayedAckType = delayedAckType;
	}

	public String getErrorId() {
		return errorId;
	}

	public void setErrorId(String errorId) {
		this.errorId = errorId;
	}

	public String getErrorText() {
		return errorText;
	}

	public void setErrorText(String errorText) {
		this.errorText = errorText;
	}

}
