package gov.va.med.via.hl7.mpi;

import java.text.SimpleDateFormat;
import java.util.Date;

import gov.va.med.via.util.StringUtil;

public class MshSegment {
	private static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddhhmmss");
	
	EncodingCharacters encChars = new EncodingCharacters();
	String sendingApp = "";
	String sendingFacility = "";
	String receivingApp = "";
	String receivingFacility = "";
	String timestamp = "";
	String security = "";
	String msgCode = "";
	String eventTrigger = "";
	String msgCtlId = "";
	String processingId = "";
	String versionId = "";
	String sequenceNumber = "";
	String continuationPointer = "";
	String acceptAckType = "";
	String appAckType = "";
	String countryCode = "";


	public MshSegment() {
	}

	public MshSegment(String rawSegmentString) 
	{
		parse(rawSegmentString);
	}
	
	public String toSegment()
    {
        String result = "MSH" +
            getEncodingChars().getFieldSeparator() + getEncodingChars().toString() +
            getEncodingChars().getFieldSeparator() + getSendingApplication() +
            getEncodingChars().getFieldSeparator() + getSendingFacility() +
            getEncodingChars().getFieldSeparator() + getReceivingApplication() +
            getEncodingChars().getFieldSeparator() + getReceivingFacility() +
            getEncodingChars().getFieldSeparator() + sdf.format(new Date()) + //DateTime.Now.ToString("yyyyMMddhhmmss") +
            getEncodingChars().getFieldSeparator() + getSecurity() +
            getEncodingChars().getFieldSeparator() + getMessageCode() + getEncodingChars().getComponentSeparator() + getEventTrigger() +
            getEncodingChars().getFieldSeparator() + getMessageControlId() +
            getEncodingChars().getFieldSeparator() + getProcessingId() +
            getEncodingChars().getFieldSeparator() + getVersionId();
        return result + '\r';
    }
	
	private void setSeparators(String s)
    {
        encChars = new EncodingCharacters(s.charAt(3), s.substring(4, 4+8)); // s.Substring(4, 8));
    }

	public void parse(String rawSegmentString)
	{
		if ( !rawSegmentString.substring(0,3).equals("MSH"))
		{
			throw new RuntimeException("Invalid MSH segment: incorrect header");
		}

		setSeparators(rawSegmentString);

		String[] flds = StringUtil.split(rawSegmentString,getEncodingChars().getFieldSeparator());

		if (flds.length < 12)
		{
			throw new RuntimeException("Invalid MSH segment: incorrect number of fields");
		}

		setSendingApplication(flds[2]);
		setSendingFacility( flds[3]);
		setReceivingApplication( flds[4]);
		setReceivingFacility( flds[5]);

		setTimestamp( flds[6]);
		// TODO - Validate UTC timestamp

		setSecurity( flds[7]);

		if (StringUtil.isEmpty(flds[8])) 
		{
			throw new RuntimeException("Invalid MSH segment: missing message type");
		}

		String[] components = StringUtil.split(flds[8],getEncodingChars().getComponentSeparator());

		if (StringUtil.isEmpty(components[0])) 
		{
			throw new RuntimeException("Invalid MSH segment: incorrect message type fields");
		}

		setMessageCode(components[0]);

		if (components.length > 1)
		{
			setEventTrigger( components[1]);
		}

		if (StringUtil.isEmpty(flds[9]))
		{
			throw new RuntimeException("Invalid MSH segment: missing message control ID");
		}

		setMessageControlId( flds[9]);

		if (StringUtil.isEmpty(flds[10]))
		{
			throw new RuntimeException("Invalid MSH segment: missing processing ID");
		}

		setProcessingId( flds[10]);

		if (StringUtil.isEmpty(flds[11]))
		{
			throw new RuntimeException("Invalid MSH segment: missing version ID");
		}

		setVersionId( flds[11]);

		if (flds.length > 12)
		{
			setSequenceNumber( flds[12]);
		}
		if (flds.length > 13)
		{
			setContinuationPointer( flds[13]);
		}
		if (flds.length > 14)
		{
			setAcceptAckType( flds[14]);
		}
		if (flds.length > 15)
		{
			setApplicationAckType( flds[15]);
		}
		if (flds.length > 16)
		{
			setCountryCode( flds[16]);
		}
	}


	public EncodingCharacters getEncodingChars() {
		return encChars;
	}


	public void setEncodingChars(EncodingCharacters encChars) {
		this.encChars = encChars;
	}


	public String getSendingApplication() {
		return sendingApp;
	}


	public void setSendingApplication(String sendingApp) {
		this.sendingApp = sendingApp;
	}


	public String getSendingFacility() {
		return sendingFacility;
	}


	public void setSendingFacility(String sendingFacility) {
		this.sendingFacility = sendingFacility;
	}


	public String getReceivingApplication() {
		return receivingApp;
	}


	public void setReceivingApplication(String receivingApp) {
		this.receivingApp = receivingApp;
	}


	public String getReceivingFacility() {
		return receivingFacility;
	}


	public void setReceivingFacility(String receivingFacility) {
		this.receivingFacility = receivingFacility;
	}


	public String getTimestamp() {
		return timestamp;
	}


	public void setTimestamp(String timestamp) {
		this.timestamp = timestamp;
	}


	public String getSecurity() {
		return security;
	}


	public void setSecurity(String security) {
		this.security = security;
	}


	public String getMessageCode() {
		return msgCode;
	}


	public void setMessageCode(String msgCode) {
		this.msgCode = msgCode;
	}


	public String getEventTrigger() {
		return eventTrigger;
	}


	public void setEventTrigger(String eventTrigger) {
		this.eventTrigger = eventTrigger;
	}


	public String getMessageControlId() {
		return msgCtlId;
	}


	public void setMessageControlId(String msgCtlId) {
		this.msgCtlId = msgCtlId;
	}


	public String getProcessingId() {
		return processingId;
	}


	public void setProcessingId(String processingId) {
		this.processingId = processingId;
	}


	public String getVersionId() {
		return versionId;
	}


	public void setVersionId(String versionId) {
		this.versionId = versionId;
	}


	public String getSequenceNumber() {
		return sequenceNumber;
	}


	public void setSequenceNumber(String sequenceNumber) {
		this.sequenceNumber = sequenceNumber;
	}


	public String getContinuationPointer() {
		return continuationPointer;
	}


	public void setContinuationPointer(String continuationPointer) {
		this.continuationPointer = continuationPointer;
	}


	public String getAcceptAckType() {
		return acceptAckType;
	}


	public void setAcceptAckType(String acceptAckType) {
		this.acceptAckType = acceptAckType;
	}


	public String getApplicationAckType() {
		return appAckType;
	}


	public void setApplicationAckType(String appAckType) {
		this.appAckType = appAckType;
	}


	public String getCountryCode() {
		return countryCode;
	}


	public void setCountryCode(String countryCode) {
		this.countryCode = countryCode;
	}

}
