package gov.va.med.via.hl7.mpi;

import gov.va.med.via.util.StringUtil;

public class RdfSegment {

	EncodingCharacters encChars = new EncodingCharacters();
    int ncols = 0;
    RdfColumn[] cols;
	
	public RdfSegment() {

	}
	
	public RdfSegment(String rawSeg)
    {
        parse(rawSeg);
    }
	
	RdfColumn parseColumn(String rawColumn)
    {
        String[] subcomponents = StringUtil.split(rawColumn, getEncChars().getComponentSeparator());
	    if (subcomponents.length < 3 || "".equals(subcomponents[0]))
	    {
		    throw new RuntimeException("Invalid RDF column: " + rawColumn);
	    }
        int fldLth = Short.parseShort(subcomponents[2]);//Convert.ToInt16(subcomponents[2]);
	    return new RdfColumn(new ColumnDescription(subcomponents[0],subcomponents[1],fldLth));
    }
	
    void parse(String rawSeg)
    {
        String[] flds = StringUtil.split(rawSeg, getEncChars().getFieldSeparator());

        if (flds.length < 3)
        {
            throw new RuntimeException("Invalid RDF segment: less than 3 fields");
        }

        if (!"RDF".equals(flds[0]))
        {
            throw new RuntimeException("Invalid RDF segment: incorrect header");
        }

        String[] rawColumns = StringUtil.split(flds[2], getEncChars().getRepetitionSeparator());
        setColumns( new RdfColumn[rawColumns.length]);

        for (int i = 0; i < rawColumns.length; i++)
        {
        	getColumns()[i] = parseColumn(rawColumns[i]);
        }
    }
    
    public String toSegment()
    {
        String result = "RDF" + getEncChars().getFieldSeparator() + getNcols() + getEncChars().getFieldSeparator();
        
        for (int i = 0; i < getColumns().length-1; i++)
        {
            result += getColumns()[i].getDescription().toComponent() + getEncChars().getRepetitionSeparator();
        }
        result += getColumns()[getColumns().length - 1].getDescription().toComponent();
        return result + '\r';
    }

    

	public EncodingCharacters getEncChars() {
		return encChars;
	}

	public void setEncChars(EncodingCharacters encChars) {
		this.encChars = encChars;
	}

	public int getNcols() {
		return ncols;
	}

	public void setNcols(int ncols) {
		this.ncols = ncols;
	}

	public RdfColumn[] getColumns() {
		return cols;
	}

	public void setColumns(RdfColumn[] cols) {
		this.cols = cols;
	}

	
}
