package gov.va.med.via.hl7.mpi;

import gov.va.med.via.util.StringUtil;

public class RdtSegment 
{
	
	 private EncodingCharacters encodingChars = new EncodingCharacters();
     private RdtColumn[] columns;
	

	public RdtSegment() {

	}

	public RdtSegment(RdfSegment rdfSeg, String rawSeg) throws Exception
    {
        parse(rdfSeg, rawSeg);
    }

    void parse(RdfSegment rdfSeg, String rawSeg) throws Exception
    {
        String[] flds = StringUtil.split(rawSeg, getEncodingChars().getFieldSeparator());

        if (flds.length != rdfSeg.getColumns().length + 1)
        {
            throw new Exception("Invalid RDT segment: incorrect number of columns");
        }

        if (!"RDT".equals(flds[0]))
        {
            throw new RuntimeException("Invalid RDT segment: incorrect header");
        }

        columns = new RdtColumn[rdfSeg.getColumns().length];
        for (int i = 0; i < rdfSeg.getColumns().length; i++)
        {
            String[] columnValues = getColumnValues(flds[i+1]);
            if (columnValues == null || columnValues.length == 0)
            {
                columns[i] = new RdtColumn(rdfSeg.getColumns()[i].getDescription());
            }
            else if (columnValues.length == 1)
            {
                columns[i] = new RdtColumn(rdfSeg.getColumns()[i].getDescription(), columnValues[0]);
            }
            else
            {
                columns[i] = new RdtColumn(rdfSeg.getColumns()[i].getDescription(), columnValues);
            }
        }
    }

    String[] getColumnValues(String rawColumn)
    {
        if (rawColumn == null || rawColumn.trim().equals(""))
        {
            return null;
        }
        String[] result = StringUtil.split(rawColumn, getEncodingChars().getRepetitionSeparator());
        return result;
    }

    
    public RdtColumn getColumn(String colName)
    {
        for (int i = 0; i < getColumns().length; i++)
        {
            if (getColumns()[i].getDescription().getFieldName().equals(colName))
            {
                return getColumns()[i];
            }
        }
        return null;
    }


	
	
	
	public EncodingCharacters getEncodingChars() {
		return encodingChars;
	}


	public void setEncodingChars(EncodingCharacters encodingChars) {
		this.encodingChars = encodingChars;
	}


	public RdtColumn[] getColumns() {
		return columns;
	}


	public void setColumns(RdtColumn[] columns) {
		this.columns = columns;
	}

}
