package gov.va.med.via.util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;

import org.joda.time.DateTime;
import org.joda.time.Period;

//TODO: Auto-generated Javadoc
/**
* The Class DateTimeIterator.
*/
public class DateTimeIteratorUtil
{	
	/** The start date. */
	private DateTime startDate;
	
	/** The end date. */
	private DateTime endDate;
	
	/** The iter start date. */
	private DateTime iterStartDate;
	
	/** The iter end date. */
	private DateTime iterEndDate;
	
	/** The iter length. */
	private Period iterLength; 
	
	/** The iter length2. */
	private Period iterLength2; 
	
	/** The precision. */
	private int precision;
	
	// Period(int years, int months, int weeks, int days, int hours, int minutes, int seconds, int millis)
	// MINIMUM_ITERATION set to ONE day
	public Period MINIMUM_ITERATION = new Period(0, 0, 0, 1, 0, 0, 0, 0);
    public static Period DAY_ITERATION    = new Period(0, 0, 0, 1, 0, 0, 0, 0);		// One day
    public static Period HOUR_ITERATION   = new Period(0, 0, 0, 0, 1, 0, 0, 0);		// One Hour
    public static Period MINUTE_ITERATION = new Period(0, 0, 0, 0, 0, 1, 0, 0);		// One Minute	
    public static Period SECOND_ITERATION = new Period(0, 0, 0, 0, 0, 0, 1, 0);		// One Second
	

	public DateTimeIteratorUtil()
	{		
	}
	
    public DateTimeIteratorUtil(Calendar startDate, Calendar endDate, String iterationLength)
    {
        this.startDate = new DateTime(startDate);
        this.endDate = new DateTime(endDate);
        iterStartDate =  new DateTime(startDate);
        this.iterLength = IterationTimeSpanFromString(iterationLength);
        setPrecision();         // initialize the iterator
    }
    
	/**
	 * Gets the start date.
	 *
	 * @return the start date
	 */
	public DateTime getStartDate()
	{
		return startDate;
	}
	
	/**
	 * Sets the start date.
	 *
	 * @param startDate the new start date
	 */
	public void setStartDate(DateTime startDate)
	{
		this.startDate = startDate;
	}
	
	/**
	 * Gets the end date.
	 *
	 * @return the end date
	 */
	public DateTime getEndDate()
	{
		return endDate;
	}
	
	/**
	 * Sets the end date.
	 *
	 * @param endDate the new end date
	 */
	public void setEndDate(DateTime endDate)
	{
		this.endDate = endDate;
	}
	
	/**
	 * Gets the iter start date.
	 *
	 * @return the iter start date
	 */
	public DateTime getIterStartDate()
	{
		return iterStartDate;
	}
	
	/**
	 * Gets the iter start date.
	 *
	 * @return the iter start date
	 */
	public String getIterStartDateString()
	{
		return iterStartDate.toString(DateTimeUtil.FORMAT_DATE_TIME);
	}
	
	/**
	 * Sets the iter start date.
	 *
	 * @param iterStartDate the new iter start date
	 */
	public void setIterStartDate(DateTime iterStartDate)
	{
		this.iterStartDate = iterStartDate;
	}
	
	/**
	 * Gets the iter end date.
	 *
	 * @return the iter end date
	 */
	public DateTime getIterEndDate()
	{
		return iterEndDate;
	}
	
	/**
	 * Gets the iter start date.
	 *
	 * @return the iter start date
	 */
	public String getIterEndDateString()
	{
		return iterEndDate.toString(DateTimeUtil.FORMAT_DATE_TIME);
	}
	
	/**
	 * Sets the iter end date.
	 *
	 * @param iterEndDate the new iter end date
	 */
	public void setIterEndDate(DateTime iterEndDate)
	{
		this.iterEndDate = iterEndDate;
	}
	
	/**
	 * Gets the iter length.
	 *
	 * @return the iter length
	 */
	public Period getIterLength()
	{
		return iterLength;
	}
	
	/**
	 * Sets the iter length.
	 *
	 * @param iterLength the new iter length
	 */
	public void setIterLength(Period iterLength)
	{
		this.iterLength = iterLength;
		setPrecision();
	}
	
	/**
	 * Gets the iter length2.
	 *
	 * @return the iter length2
	 */
	public Period getIterLength2()
	{
		return iterLength2;
	}
	
	/**
	 * Sets the iter length2.
	 *
	 * @param iterLength2 the new iter length2
	 */
	public void setIterLength2(Period iterLength2)
	{
		this.iterLength2 = iterLength2;
	}
	
	/**
	 * Gets the precision.
	 *
	 * @return the precision
	 */
	public int getPrecision()
	{
		return precision;
	}
	
	/**
	 * Sets the precision.
	 *
	 * @param precision the new precision
	 */
	public void setPrecision(int precision)
	{
		this.precision = precision;
	}
	

	 /// <summary>Set the precision of the DdrPart based on the precision of the
	 /// iteration length.
	 /// </summary>
	 void setPrecision()
	 {
	     if (iterLength.getMillis() != 0 || iterLength.getSeconds() != 0)
	     {
	         MINIMUM_ITERATION = SECOND_ITERATION;
	         precision = Precision.SECOND.getValue();
	         return;
	     }
	     if (iterLength.getMinutes() != 0)
	     {
	         MINIMUM_ITERATION = MINUTE_ITERATION;
	         precision = Precision.MINUTE.getValue();
	         return;
	     }
	     if (iterLength.getHours() != 0)
	     {
	         MINIMUM_ITERATION = HOUR_ITERATION;
	         precision = Precision.HOUR.getValue();
	         return;
	     }
	     // otherwise just make it days
	     // could extend this to larger intervals at some point in time but
	     // iteration could get trickier...maybe?
	     MINIMUM_ITERATION = DAY_ITERATION;
	     precision = Precision.DAY.getValue();
	     return;
	 }
		
	 /// <summary>Makes the iteration timespan from a string.
	 /// </summary>
	 /// <param name="iterationLength">dd.hhmmss</param>
	 /// <returns></returns>
	 static private Period IterationTimeSpanFromString(String iterationLength)
	 {
		 Period period;
	     int days = IterationDays(iterationLength);
	     int hours = IterationHours(iterationLength);
	     int minutes = IterationMinutes(iterationLength);
	     int seconds = IterationSeconds(iterationLength);
	    
	     // Period(int years, int months, int weeks, int days, int hours, int minutes, int seconds, int millis)
	     if (days == 0 && hours == 0 && minutes == 0 && seconds == 0)
	     {
	    	 // If not interval was defined, default to one day
	    	period =  new Period(0, 0, 0, 1, 0, 0, 0, 0);
	     }
	     else
	     {
	    	 period = new Period(0, 0, 0, days, hours, minutes, seconds, 0);
	     }
	     
	      return period;
	 }
	
	 /// <summary>
	 /// find the separator and get the value to the left of it.
	 /// </summary>
	 /// <param name="iterationLength"></param>
	 /// <returns></returns>
	 static private int IterationDays(String iterationLength)
	 {
	     Integer retVal = new Integer(0);
	     int position = iterationLength.indexOf('.');
	     
	     if (position > 0)
	     {
	         int days = Integer.parseInt(RPCStringUtils.CSubstring(iterationLength, 0, position));
	         retVal  = new Integer(days);
	     }
	      
	     return retVal;
	 }
	
	 static private int IterationHours(String iterationLength)
	 {
		 Integer retVal = new Integer(0);
	     int position = iterationLength.indexOf('.');
	     if (iterationLength.length() > position + 2)	         
	     {
	    	 int hours = Integer.parseInt(RPCStringUtils.CSubstring(iterationLength, position + 1, 2));
	    	 retVal  = new Integer(hours);
	     }
	     
	     return retVal;
	 }
	
	 static private int IterationMinutes(String iterationLength)
	 {
		 Integer retVal = new Integer(0);
	     int position = iterationLength.indexOf('.');
	     if (iterationLength.length() > position + 4)
	     {
	    	 int minutes = Integer.parseInt(RPCStringUtils.CSubstring(iterationLength, position + 3, 2));
	    	 retVal  = new Integer(minutes);
	     }
	     
	     return retVal;
	 }
	
	 static private int IterationSeconds(String iterationLength)
	 {
	     int retVal = 0;
	     int position = iterationLength.indexOf('.');
	     if (iterationLength.length() > position + 6)
	     {
	    	 int seconds = Integer.parseInt(RPCStringUtils.CSubstring(iterationLength, position + 5, 2));
	    	 retVal  = new Integer(seconds);
	     }
	     
	     return retVal;
	 }
	
	 public String GetSCListerPart()
	 {
	     return RPCStringUtils.CSubstring(Timestamp.fromDateTime(iterStartDate.toGregorianCalendar()), 0, precision);
	 }
	 
	 public void AdvanceIterStartDate()
     {
         iterStartDate = iterEndDate;
     }

     public boolean IsDone()
     {
         return (iterStartDate.isAfter(endDate) || iterStartDate.isEqual(endDate));
     }
     
     public void SetIterEndDate()
     {
         if (iterStartDate.plus(iterLength).isBefore(endDate) || iterStartDate.plus(iterLength).isEqual(endDate))
         {
             iterEndDate = iterStartDate.plus(iterLength);
         }
         else
         {
             iterEndDate = endDate;
         }
     }

	 /// <summary>
	 /// Also use as lengths of substrings in the VistaTimeStamp string which has format of
	 /// yyyMMdd.HHmmss for DdrLister.Part()
	 /// </summary>
	 public enum Precision
	 {
	     YEAR (3),
	     MONTH (5),
	     DAY (7),  
	     DOT (8),
	     HOUR (10),
	     MINUTE (12),
	     SECOND (14),
	     MILLISECOND (17);
	     
	     /** The value. */
	  	private final int value;
	 	      	
	  	Precision(int value)
	  	{
	  		this.value =value; 
	  	}

		public int getValue()
		{
			return value;
		}  
  	
	 }

	 
     public static String[] trimToInterval(String[] fields, String fromTS, String toTS)
     {
    	 if (fields != null && fields.length >1){
    		 List<String> lst = new ArrayList<String>(fields.length);
    		 fromTS =Timestamp.fromUtcString(fromTS);
    		 toTS = Timestamp.fromUtcString(toTS);
    		 Calendar fromDT = Timestamp.toDateTime(fromTS);//THIS is in TimeStamp
    		 Calendar toDT = Timestamp.toDateTime(toTS);
    		 for (int i = 0; i < fields.length; i++)
    		 {
    			 //String[] flds = StringUtils.split(response[i], StringUtils.CARET);
    			 Calendar thisDT = Timestamp.toDateTime(fields[1]);
    			 if (thisDT.getTimeInMillis() >= fromDT.getTimeInMillis() && thisDT.getTimeInMillis() < toDT.getTimeInMillis())	
    			 {
    				 lst.add(fields[2]);
    			 }
    		 }
    		 return (String[])lst.toArray(new String[0]);
    	 }
    	 else{
    		 return fields;
    	 }
	}
	
}