

package gov.va.med.via.util;


import java.util.Arrays;


/**
 * Strips out new lines in a given string.
 * 
 * 
 */
public class StringUtil
{
	
	public static final String CRLF = "\r\n";
    public static final String CARET = "^";
    public static final String STICK = "|";
    public static final String COLON = ":";
    public static final String SEMICOLON = ";";
    public static final String COMMA = ",";
    public static final String PERIOD = ".";
    public static final String SLASH = "/";
    public static final String SPACE = " ";
    public static final String EQUALS = "=";
    public static final String AMPERSAND = "&";
    public static final String ATSIGN = "@";
	
	
	
    /**
     * Strips out new lines in a given string.
     * 
     * @param source String with new lines.
     * @return String with no new lines.
     */
    public static String stripNewlines( String source )
    {
        if ( source == null )
        {
            return null;
        }
        StringBuilder buff = new StringBuilder( source.length() );
        char c;
        for ( int i = 0; i < source.length(); i++ )
        {
            c = source.charAt( i );
            if ( c != '\n' )
            {
                buff.append( c );
            }
        }

        return buff.toString();
    }


    public static String[] concat( String[] a, String[] b )
    {
        if ( a == null )
        {
            return b;
        }
        else if ( b == null )
        {
            return a;
        }
        else
        {
            String[] result = new String[a.length + b.length];

            int i = 0;
            for ( String element : a )
            {
                result[i++ ] = element;
            }

            for ( String element : b )
            {
                result[i++ ] = element;
            }

            return result;

        }
    }


    
    public static String getLastStringAfterTheDelimiter( String stringData, char searchDelimiter )
    {
        return stringData.substring( stringData.lastIndexOf( searchDelimiter ) + 1, stringData.length() );
    }


    /**
     * Compare the two string while ignoring any whitespace found.
     * 
     * @param str1 String 1 to compare.
     * @param str2 String 2 to compare.
     * @return True if the two string don't differ or differ by white space only; false otherwise.
     */
    public static boolean equalsIgnoreWhiteSpace( String str1, String str2 )
    {
        boolean result = false;
        //first remove any ending white space if file/strings
        char[] c1 = str1.trim().toCharArray();
        char[] c2 = str2.trim().toCharArray();
        int c1Length = c1.length;
        int c2Length = c2.length;

        for ( int c1i = 0, c2i = 0; c1i < c1Length && c2i < c2Length; c1i++ , c2i++ )
        {
            while ( c1i < c1Length && Character.isWhitespace( c1[c1i] ) )
            {
                c1i++ ;
            }

            while ( c2i < c2Length && Character.isWhitespace( c2[c2i] ) )
            {
                c2i++ ;
            }

            if ( c1i == c1Length - 1 && c2i == c2Length - 1 )
            {
                result = true;
            }

            if ( c1[c1i] != c2[c2i] )
            {
                break;
            }
        }

        return result;
    }


    /**
     * Split and sort the split substrings and return a newly constructed <String> from the sorted order of split
     * substrings.
     * 
     * @param stringToSort <String> needing sorted
     * @param splitterExp <RegExp> used to split the substrings into <Array>
     * @return <String> newly constructed from sorted substrings
     */
    public static String splitAndSortsubStrings( String stringToSort, String splitterExp )
    {
        String list[] = stringToSort.split( splitterExp );
        Arrays.sort( list );
        StringBuilder sbuff = new StringBuilder();
        for ( int i = 0; i < list.length; i++ )
        {
            if ( i > 0 )
            {
                sbuff.append( splitterExp );
            }
            sbuff.append( list[i] );
        }

        return sbuff.toString();

    }


    public static String splitAndSortsubStrings( String stringToSort, String splitterExp, String innerSplitter )
    {
        String list[] = stringToSort.split( splitterExp );
        Arrays.sort( list );
        StringBuilder sbuff = new StringBuilder();
        for ( int i = 0; i < list.length; i++ )
        {
            if ( i > 0 )
            {
                sbuff.append( splitterExp );
            }
            sbuff.append( splitAndSortsubStrings( list[i], innerSplitter ) );
        }

        return sbuff.toString();
    }
    
    public static boolean isNullOrEmpty(String s)
    {
        return s == null || s.equals("");
    }

    public static String[] split(String s, char delimiter)
    {
        return split(s, "" + delimiter);
    }

    public static String[] split(String s, String delimiter)
    {
        return s.split(escapeRegExMetaChars(delimiter),-1);
    }

    private static String escapeRegExMetaChars(String str)
    {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < str.length(); i++)
        {
            if (isRegExMetaChar(str.charAt(i)))
            {
                sb.append("\\" + str.charAt(i));
            }
            else sb.append(str.charAt(i));
        }

        return sb.toString();
    }

    private static boolean isRegExMetaChar(char c)
    {
        switch(c)
        {
            case '^': return true;
            case '.': return true;
            case '$': return true;
            case '|': return true;
            case '*': return true;
            case '+': return true;
            case '?': return true;
            case '-': return true;
            default: return false;
        }
    }

    public static String[] trimArray(String[] s)
    {
        int i;
        for (i = s.length - 1; i >= 0; i--)
        {
            if (!s[i].equals(""))
            {
                break;
            }
        }
        String[] result = new String[i + 1];
        int len = i+1;
        for(int j = 0; j < len; j++)
        {
            result[j] = s[j];
        }

        return result;
    }

    public static boolean isNumeric(String s)
        {
        if (s.equals(""))
        {
            return false;
        }
        for (int i=0; i<s.length(); i++)
        {
            if (s.charAt(i) < '0' || s.charAt(i) > '9')
            {
                return false;
            }
        }
        return true;
    }

    public static boolean isAlpha(String s)
    {
        if (s.equals(""))
        {
            return false;
        }
        for (int i = 0; i<s.length(); i++)
        {
            if (!isAlphaChar(s.charAt(i)))
            {
                return false;
            }
        }
        return true;
    }

    public static boolean isNumericChar(char c)
    {
        return (c >= '0' && c <= '9');
    }

    public static boolean isAlphaChar(char c)
    {
        return ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'));
    }

    public static boolean isAlphaNumericChar(char c)
    {
        return (isAlphaChar(c) || isNumericChar(c));
    }

    public static boolean isWhiteSpace(char c)
    {
        return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
    }

    public static String removeNonNumericChars(String s)
    {
        if (s == null)
        {
            return null;
        }
        String rtn = "";
        for (int i=0; i<s.length(); i++)
        {
            if (s.charAt(i) >= '0' && s.charAt(i) <= '9')
            {
                rtn += s.charAt(i);
            }
        }
        return rtn;
    }

    public static String piece(String s, String delimiter, int pieceNum)
    {
        String[] flds = split(s,delimiter);
        if (pieceNum > flds.length)
        {
            return null;
        }
        return flds[pieceNum-1];
    }

    public static String piece(String s, int pieceNum)
    {
        return piece(s, "^", pieceNum);
    }

    public static boolean isEmpty(String s)
    {
        return s == null || s.length() == 0;
    }

    public static String strPack(String s, int n)
    {
        String result = ""+s.length();
        while (result.length() < n)
        {
                result = '0' + result;
        }
        return result + s;
    }

    public static int getFirstWhiteSpaceAfter(String s, int idx)
    {
        int i = idx;
        while (i >= 0 && !isWhiteSpace(s.charAt(i)))
        {
            i--;
        }
        return i;
    }

    public static int getFirstWhiteSpacePreceding(String s, int startIndex)
    {
        int i = startIndex;
        while (i >= 0 && !isWhiteSpace(s.charAt(i)))
        {
            i--;
        }
        return i;
    }
    
    public static String filteredString(String s)
    {
        String result = "";
        for (int i=0; i<s.length(); i++)
        {
            int c = asciiAt(s,i);
            if (c == 9)
            {
                    result += "        ";
            }
            else if ((c >= 32 && c <=127) || (c >= 161 && c <=255))
            {
                    result += s.charAt(i);
            }
            else if (c >= 128 && c <= 159)
            {
                    result += "?";
            }
            else if (c == 10 || c == 13 || c == 160)
            {
                    result += " ";
            }
        }
        return result;
    }

    public static int asciiAt(String s, int idx)
    {
        char c = s.charAt(idx);
        return (int)c;
    }

    public static String prependChars(String s, char c, int sLen)
    {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < sLen; i++)
        {
            sb.append(c);
        }

        String chars = sb.toString();
        return CSubstring(chars, 0, sLen - s.length()) + s;
    }

    public static int getIdx(String[] lines, String target, int startingIdx)
    {
        for (int i = startingIdx; i < lines.length; i++)
        {
            if (lines[i].startsWith(target))
            {
                return i;
            }
        }
        return -1;
    }

    public static String trimTrailingZeroes(String s)
    {
        int i = s.length() - 1;
        while (s.charAt(i) == '0' && i > 0)
        {
            i--;
        }

        return CSubstring(s, 0, i+1);
    }

    public static String reverse(String s)
    {
        StringBuilder result = new StringBuilder("");
        for (int i = s.length() - 1; i >= 0; i--)
        {
            result.append(s.charAt(i));
        }

        return result.toString();
    }

    /**
     * Return a string with all invalid XML 1.0 characters
     * (per W3C standard: http://www.w3.org/TR/REC-xml/#charsets) removed
     */
    public static String stripInvalidXmlCharacters(String s)
    {
        if (s == null || s.equals(""))
        {
            return "";
        }

        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++)
        {
            char current = s.charAt(i);
            if (current == 0x9 || current == 0xA || current == 0xD ||
                ((current >= 0x20) && (current <= 0xD7FF)) ||
                ((current >= 0xE000) && (current <= 0xFFFD)) ||
                ((current >= 0x10000) && (current <= 0x10FFFF)))
            {
                sb.append(current);
            }
        }
        return sb.toString();
    }

    /**
     * Returns a new string that is a substring of str.
     * Function is a C# porting utility.
     * @param str Source string
     * @param beginIndex The beginning index, inclusive.
     * @param length The number of characters in the substring. 
     * @return a new string that is a substring of str.
     */
    public static String CSubstring(String str, int beginIndex, int length)
    {
        return str.substring(beginIndex, beginIndex+length);
    }


	public static boolean isDecimal(String s) 
	{
		return (s.indexOf(".")!=-1) && isNumeric(s.replaceAll("\\.", ""));
	}
	
	/**
	 * trims the trailing blank spaces from the inputString
	 * @param inputString
	 * @return the inputString with the blank spaces removed
	 */
	public static String trimEnd( String inputString )
	{
		return inputString.replaceAll("\\s+$", "");
	}
	
	/**
	 * trims the trailing regex characters from the inputString
	 * @param inputString
	 * @param regex
	 * @return the inputString with the blank spaces removed
	 */
	public static String trimEnd( String s, String regex )
	{
		return s.replaceAll("[" + regex + "]+$", "");
	}
	
	/**
	 * trims the starting regex characters from the inputString
	 * @param inputString
	 * @param regex
	 * @return the inputString with the blank spaces removed
	 */
	public static String trimStart( String s, String regex )
	{
		return s.replaceAll("^[" + regex + "]+", "");
	}
	
	
}
