package gov.va.med.via.util;

import gov.va.med.via.rpc.DateUtils;

import java.util.Calendar;
import java.util.Date;

public class VistaTimestamp
{
	public static final int yrIdx = 0;
	public static final int yrLth = 3;
	public static final int moIdx = 3;
	public static final int moLth = 2;
	public static final int dyIdx = 5;
	public static final int dyLth = 2;
	public static final int dotIdx = 7;
	public static final int dotLth = 1;
	public static final int hrIdx = 8;
	public static final int hrLth = 2;
	public static final int mnIdx = 10;
	public static final int mnLth = 2;
	public static final int scIdx = 12;
	public static final int scLth = 2;

	public static String getDatePart(String ts)
	{
		return padZeroes(ts).substring(yrIdx,yrIdx+yrLth);
	}

	public static String getTimePart(String ts)
	{
		return padZeroes(ts).substring(hrIdx);
	}

	public static int getYear(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(yrIdx,	yrIdx+ 	yrLth)) + 1700;
		// return Convert.ToInt32(padZeroes(ts).SubString(yrIdx,yrLth)) + 1700;
	}

	public static int getMonth(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(moIdx,	moIdx+	moLth));
		///return Convert.ToInt32(padZeroes(ts).SubString(moIdx,moLth));
	}

	public static int getDay(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(dyIdx,	dyIdx+	dyLth));
		//return Convert.ToInt32(padZeroes(ts).SubString(dyIdx,dyLth));
	}

	public static int getHour(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(hrIdx,	hrIdx+	hrLth));
		//return Convert.ToInt32(padZeroes(ts).SubString(hrIdx,hrLth));
	}

	public static int getMinute(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(mnIdx,	mnIdx+	mnLth));
		//return Convert.ToInt32(padZeroes(ts).SubString(mnIdx,mnLth));
	}

	public static int getSecond(String ts)
	{
		return Integer.parseInt(padZeroes(ts).substring(scIdx));
		//return Convert.ToInt32(padZeroes(ts).SubString(scIdx));
	}

	public static String padZeroes(String ts)
	{
		if (ts == "*SENSITIVE*")
		{
			return ts;
		}
		if (ts.indexOf("@") != -1)
		{
			ts = ts.replace('@','.');
		}
		if (ts.indexOf(".") != -1)
		{
			String[] parts = ts.split("\\.");

			ts = parts[0] + "0000000".substring(0,	0+ 7 - parts[0].length()) + '.' +
					parts[1] + "000000".substring(0, 	0+ 6 - parts[1].length());
		}
		else
		{
			ts = ts + "0000000".substring(0,	0+ 7-ts.length()) + ".000000";
		}
		return ts;
	}

	public static boolean isValid(String ts)
	{
		try
		{
			ts = padZeroes(ts);
			int yr = getYear(ts);
			if (yr < 1900)
			{
				return false;
			}
			int mo = getMonth(ts);
			if (mo > 12)
			{
				return false;
			}
			int dy = getDay(ts);
			if (mo > 0 && dy > 0 && !DateUtils.isValidDay(yr, mo, dy))
			{
				return false;
			}
			if (getHour(ts) > 23)
			{
				return false;
			}
			if (getMinute(ts) > 59)
			{
				return false;
			}
			if (getSecond(ts) > 59)
			{
				return false;
			}
		}
		catch ( Exception e)
		{
			return false;
		}
		return true;
	}

	public static String corrected(String ts)
	{
		if (ts == "*SENSITIVE*")
		{
			return ts;
		}
		ts = padZeroes(ts);
		int yr = getYear(ts);
		if (yr < 1900)
		{
			return "";
		}
		int mo = getMonth(ts);
		if (mo == -1 || mo > 12)
		{
			//ts = ts.SubString(yrIdx,yrLth) + "00" + ts.SubString(dyIdx);
			ts = ts.substring(yrIdx,	yrIdx+	yrLth) + "00" + ts.substring(dyIdx);
			mo = 0;
		}
		int dy = getDay(ts);
		if (dy == -1 || !DateUtils.isValidDay(yr,mo,dy))
		{
			//ts = ts.substring(yrIdx,yrLth+moLth) + "00" + ts.SubString(dotIdx);
			ts = ts.substring(yrIdx,	yrIdx +yrLth+moLth) + "00" + ts.substring(dotIdx);
			dy = 0;
		}
		int hr = getHour(ts);
		if (hr == -1 || hr > 24) // SMW believe it or not I found one with timestamp of 24
		{
			//ts = ts.SubString(yrIdx,yrLth+moLth+dyLth+dotLth) + "00" + ts.SubString(mnIdx);
			ts = ts.substring(yrIdx,	yrIdx+	yrLth+moLth+dyLth+dotLth) + "00" + ts.substring(mnIdx);
		}
		int mn = getMinute(ts);
		if (mn == -1 || mn > 59)
		{
			ts = ts.substring(yrIdx,	yrIdx+	yrLth+moLth+dyLth+dotLth+hrLth) + "00" + ts.substring(scIdx);
		}
		int sc = getSecond(ts);
		if (sc == -1 || sc > 59)
		{
			ts = ts.substring(yrIdx,	yrIdx+	yrLth+moLth+dyLth+dotLth+hrLth+mnLth) + "00";
		}
		return ts;
	}

	public static String toUtcString(String vistaTS)
	{
		if (vistaTS == "")
		{
			return "";
		}
		if (vistaTS == "*SENSITIVE*")
		{
			return vistaTS;
		}
		String correctedTS = corrected(vistaTS);
		if (correctedTS == "")
		{
			return "";
		}
		int yr = getYear(correctedTS);
		String result = ""+yr;
		result += correctedTS.substring(moIdx);
		return result;
	}

	/**
    public static DateTime toDateTimeFromRdv(String rdvTS)
    {
        String[] parts =rdvTS.split( StringUtil.SPACE );
        String[] dateFlds = parts[0].split( StringUtil.SLASH);

        // TBD - should we do this? Technically '00' is an invalid value for the day and should be fixed at that VistA
        // sometimes we get '00' as the day value from VistA - we change this to '01' to create our DateTime object
        if (dateFlds[1].equals("0") || dateFlds[1].equals("00"))
        {
            dateFlds[1] = "01";
        }

        if (parts.length == 1)
        {
            return new DateTime(Short.parseShort(dateFlds[2]),
            					Short.parseShort(dateFlds[0]),
            					Short.parseShort(dateFlds[1]));
        }
        else
        {
            String[] timeFlds = parts[1].split(StringUtil.COLON);
            try
            {
                return new DateTime(Short.parseShort(dateFlds[2]),
                		Short.parseShort(dateFlds[0]),
                		Short.parseShort(dateFlds[1]),
                		Short.parseShort(timeFlds[0]),
                		Short.parseShort(timeFlds[1]),
                                    0);
            }
            catch (Exception e)
            {
                if (timeFlds[0] == "24" && timeFlds[1] == "00")
                {
                    timeFlds[0] = "00";
                    return new DateTime(Short.parseShort(dateFlds[2]),
                    		Short.parseShort(dateFlds[0]),
                    		Short.parseShort(dateFlds[1]),
                    		Short.parseShort(timeFlds[0]),
                    		Short.parseShort(timeFlds[1]),
                                        0);
                }
                throw e;
            }
        }
    }
	 */

	/// <summary>
	/// Pad a month or day String with zeroes
	/// </summary>
	/// <param name="part">Month or day as String (e.g. "03" or "2" or "28" etc.)</param>
	/// <returns>Padded String using zero. Always returns a String of length 2</returns>
	private static String padDatePartWithZeroes(String part)
	{
		if (part==null || part.isEmpty())
		{
			return "00";
		}
		part = part.trim();
		if (part.length() == 1)
		{
			return "0" + part;
		}
		return part;
	}
	public static String toUtcFromRdv(String rdvTS)
	{
		if (rdvTS==null || rdvTS.isEmpty())
		{
			return "";
		}
		String[] parts = rdvTS.split( StringUtil.SPACE);
		String[] dateFlds = parts[0].split( StringUtil.SLASH);
		String datePartyyyyMMdd = dateFlds[2] + padDatePartWithZeroes(dateFlds[0]) + padDatePartWithZeroes(dateFlds[1]);
		if ( parts.length == 1)
		{
			return datePartyyyyMMdd + ".000000";
		}
		else
		{
			String[] timeFlds = parts[1].split( StringUtil.COLON);
			return datePartyyyyMMdd + "." + timeFlds[0] + timeFlds[1] + "00"; 
			//return new DateTime(Convert.ToInt16(dateFlds[2]),
			//                    Convert.ToInt16(dateFlds[0]),
			//                    Convert.ToInt16(dateFlds[1]),
			//                    Convert.ToInt16(timeFlds[0]),
			//                    Convert.ToInt16(timeFlds[1]),
			//                    0);
		}
	}
	
	/**
	 * determines whether a VistaTimestamp is valid.     
	 * 
	 * @param vistaTimestamp
	 * @return java.util.Date associated with the vistaTimestamp
	 */
	public static boolean isValidVistaTimestamp(String vistaTimestamp)
    {
		if (vistaTimestamp.indexOf("SENSITIVE") != -1)
	    {
			return true;
	    }
	    if (vistaTimestamp.equals("TODAY"))
	    {
	    	return true;
	    }
	    if (vistaTimestamp.equals(""))
	    {
            return true; ///not too sure about this
	    }
        try
        {
            String correctedTS = corrected(vistaTimestamp);
            int year = getYear(correctedTS);
            getMonth(correctedTS)/* -1 */;  /// the minus 1 is for java dates (not needed in c# code)
            getDay(correctedTS);
            getHour(correctedTS);
            getMinute(correctedTS);
            getSecond(correctedTS);
            if ( year<1900) return false; 
            return true;
        }
        catch (Exception e)
        {
            // VISTA has some timestamps like NOW000.000000 - WTF are we supposed to do with these - they aren't valid timestamps anyways
            return false;
        }
    }
	
	
	/**
	 * converts a VistaTimestamp to a java.util.Date.  A new Date() is returned if the vistaTimestamp contains the "SENSITIVE" keyword.
	 * If "TODAY" is the VistaTimestamp string value, then a new Date() is also returns.  Also, if the vistaTimestamp is an empty string,
	 * then a new Date() is returned.  If an Exception is caught or unusual vistaTimestamp strings are entered like 'NOW000.000000', 
	 * then a new Date() will also be returned.  In all other cases, the vistaTimestamp string will be parsed into a corresponding java.util.Date object.   
	 * 
	 * @param vistaTimestamp
	 * @return java.util.Date associated with the vistaTimestamp
	 */
	public static Date convertVistaTimestampToDate(String vistaTimestamp)
    {
		if (vistaTimestamp.indexOf("SENSITIVE") != -1)
	    {
		    return new Date();
	    }
	    if (vistaTimestamp.equals("TODAY"))
	    {
		    return new Date();//DateTime.Now;  <-- does NOT take advantage of the Zeroed out hours minutes seconds of C# DateTime.TODAY
	    }
	    if (vistaTimestamp.equals(""))
	    {
            return new Date();
	    }
        try
        {
            String correctedTS = corrected(vistaTimestamp);
            int year = getYear(correctedTS);
            int month = getMonth(correctedTS)-1;  /// the minus 1 is for java dates (not needed in c# code)
            int day = getDay(correctedTS);
            int hour = getHour(correctedTS);
            int minute = getMinute(correctedTS);
            int second = getSecond(correctedTS);
            
            
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.YEAR, year);
            calendar.set(Calendar.MONTH, month);
            calendar.set(Calendar.DATE, day);
            calendar.set(Calendar.HOUR_OF_DAY, hour);
            calendar.set(Calendar.MINUTE, minute);
            calendar.set(Calendar.SECOND, second);
            calendar.set(Calendar.MILLISECOND, 0);
            return calendar.getTime();
            
        }
        catch (Exception e)
        {
            // VISTA has some timestamps like NOW000.000000 - WTF are we supposed to do with these - they aren't valid timestamps anyways
            return new Date();
        }
    }

	public static Date toDate(String vistaTimestamp)
    {
	    return convertVistaTimestampToDate(vistaTimestamp);
    }

	
    public static Date toDateTime(String vistaTimestamp)
    {
    	return convertVistaTimestampToDate(vistaTimestamp);
    }
	
	
	/// <summary>Turns UTC String into VistA timestamp (year - 1700)
	/// </summary>
	/// <param name="ts"></param>
	/// <returns></returns>
	public static String fromUtcString(String ts)
	{
		if ("NOW".equals(ts)) return ts;
		
		//ts = corrected(ts);
		//ts = ts.SubString(0,ts.Length-3);
		int yr = Integer.parseInt(ts.substring(0,	0+	4));
		String result = "" +(yr - 1700);
		result += ts.substring(4);
		return result;
	}


	/** Converts Date into VistA format.
	 * A call to ORWU DT returns the above expected result where the last character 4
	 * actually corresponds to 40 seconds.  Further research needed to see if this is 
	 * perhaps a bug in the MDWS code, but for now, we'll replicate their algorithm.
	 * 
	 * @param date
	 * @return yyyMMdd.HHmmss with year - 1700
	 */
	public static String convertDateToVistAFormat(Date date)
	{
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		String vistaYear = ""+(calendar.get(Calendar.YEAR)-1700);
		int month = calendar.get(Calendar.MONTH)+1;  /// January is zero in java and 1 in vista and .NET, so add one to the result
		String vistaMonth = StringUtil.prependChars((""+month),'0',2);
		String vistaDay = StringUtil.prependChars((""+(calendar.get(Calendar.DATE))),'0',2);
		String result = vistaYear + vistaMonth + vistaDay;
		result += '.';
		String vistaHour = StringUtil.prependChars((""+(calendar.get(Calendar.HOUR_OF_DAY))), '0', 2);
		String vistaMin = StringUtil.prependChars((""+(calendar.get(Calendar.MINUTE))), '0', 2);
		String vistaSec = StringUtil.prependChars((""+(calendar.get(Calendar.SECOND))), '0', 2);
		result += vistaHour + vistaMin + vistaSec;
		return result;
	}

	/** Converts Date into VistA format.  This method was renamed to convertDateToVistAFormat. 
	 * A call to ORWU DT returns the above expected result where the last character 4
	 * actually corresponds to 40 seconds.  Further research needed to see if this is 
	 * perhaps a bug in the MDWS code, but for now, we'll replicate their algorithm.
	 * 
	 * @param date
	 * @return yyyMMdd.HHmmss with year - 1700
	 * @see VistaTimestamp.convertDateToVistAFormat
	 */
	public static String fromDateTime(Date date)
	{
		return convertDateToVistAFormat(date);
	}

	/*


    /// <summary>
    /// Convert DateTime into VistA format
    /// </summary>
    /// <param name="t">DateTime</param>
    /// <returns>yyyMMdd with year - 1700</returns>
    public static String fromDateTimeShortString(DateTime t)
    {
        String sYear = ""+(t.Year - 1700);
        String sMonth = StringUtil.prependChars( (""+(t.Month)), '0', 2);
        //String sDay = StringUtil.prependChars(Convert.ToString(t.Day), '0', 2);
        String sDay = StringUtil.prependChars((""+(t.Day)), '0', 2);
        return sYear + sMonth + sDay;
    }
	 */
	public static String noSeconds(String vistaTS)
	{
		if (vistaTS.indexOf('.') == -1)
		{
			return vistaTS;
		}
		vistaTS = padTimeToSeconds(vistaTS);
		String[] flds = vistaTS.split("\\.");
		return flds[0] + '.' + flds[1].substring(0,	0+	4);
	}

	public static String padTimeToSeconds(String vistaTS) 
	{
		String result = vistaTS;
		while (result.length() < 12)
		{
			result += '0';
		}
		return result;
	}

	public static String fromUtcFromDate(String utcFrom)
	{
		if (utcFrom.indexOf('.') == -1)
		{
			utcFrom += ".000000";
		}
		return fromUtcString(utcFrom);
	}

	public static String fromUtcToDate(String utcTo)
	{
		if (utcTo.indexOf('.') == -1)
		{
			utcTo += ".235959";
		}
		return fromUtcString(utcTo);
	}

	public static String invertTimestamp(String ts)
	{
		Double dTs = Double.parseDouble(ts);
		Double newTs = 9999999 - dTs;
		return newTs.toString();
	}
	
	/**
	 * converts the time now to a Vista format using the local timezone
	 * 
	 * @return
	 */
	public static String now()
	{
		return fromDateTime(new Date()); 
	}

	public static String trimZeroes(String ts)
	{
		int idx = ts.length() - 1;
		while (ts.charAt(idx) == '0')
		{
			idx--;
		}
		return ts.substring(0, 0+	idx + 1);
	}
}
