package gov.va.med.via.util;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.databeans.Encounter;

public class VistaUtils {

	private VistaUtils() {
	}
	
	public static String getVisitString(Encounter encounter)
    {
        return encounter.getLocationId() + ';' +
            VistaTimestamp.fromUtcString(encounter.getTimestamp()) + ';' +
            encounter.getType();
    }

	public static void checkVisitString(String s) throws ValidationException
    {
        if (s==null || s.equals(""))
        {
            throw new ValidationException("Visit String is " + ((s==null)?"null":"empty"));
        }
        String[] pieces = s.split(";");
        if (pieces.length != 3)
        {
            throw new ValidationException("Invalid visit string (need 3 semi-colon delimited pieces): " + s);
        }
        if (!isWellFormedIen(pieces[0]))
        {
            throw new ValidationException("Invalid visit string (invalid location IEN): " + s);
        }            
        if (!VistaTimestamp.isValid(pieces[1]))
        {
            throw new ValidationException("Invalid visit string (invalid VistA timestamp): " + s);
        }
        if (!"A".equals(pieces[2]) && !"H".equals(pieces[2]) && !"E".equals(pieces[2]))
        {
            throw new ValidationException("Invalid visit string (type must be 'A', 'H' or 'E'): " + s);
        }
    }
	public static boolean isWellFormedDUZ( String duz)
	{
		if ( duz==null ) return false;
		if ( duz.trim().equals("")) return false;
		if ( duz.trim().equals("0")) return false;
		if ( !StringUtil.isNumeric(duz)) return false;
		return true;
	}
	
	public static boolean isWellFormedIen(String ien)
    {
		if ( ien==null ) return false;
		if ( ien.trim().equals("")) return false;
		if ( ien.trim().equals("0")) return false;
		return  ( StringUtil.isNumeric(ien) || StringUtil.isDecimal(ien));
    }
	
	
}
