package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.AllOrdersParser;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

// TODO: Auto-generated Javadoc
/**
 * The Class AllOrdersDao.
 * 
 * @author DOMAINROMERJ
 */
public class AllOrdersDao extends BaseDao 
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#callRpc(java.lang.String, java.util.Map, gov.va.med.via.vistalink.ConnectionFactoryLocator)
	 */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		

		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		// Retrieve the order IDs for all orders associated to the patient		
		List<String> orderIds = getOrderIds(dfn, vistaLinkCommand);
		
		Map<String,String>paramsMap = new HashMap<String, String>();
		
		for (int i = 0; i < orderIds.size(); i++)
		{
			paramsMap.put((new Integer(i+1)).toString(), orderIds.get(i));
		}

		Calendar today = Calendar.getInstance();
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWORR_GET4LST, "2", Timestamp.fromDateTime(today), paramsMap);		
		
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);		
	}


	/**
	 * Gets the order ids by invoking RPC ORWORR_AGET.
	 *
	 * @param dfn the dfn
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return the order ids for all orders associated to a patient in a List<String>
	 */
	private List<String> getOrderIds(String dfn, VistaLinkCommand vistaLinkCommand)
	{
		List<String> orderIds = new ArrayList<String>();
		
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWORR_AGET, dfn, "2^0", "1", "-1", "0", "");
		
		response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        for (int i = 1; i < lines.length; i++)
        {
        	if (RPCStringUtils.isEmpty(lines[i])) continue;
        	
        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        	orderIds.add(fields[0]);        	
        }
        
		return orderIds;
	}
	
	/* 
	 * @return - the DAO name
	 */
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.BaseDao#getDaoName()
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.ALL_ORDERS_DAO;
	}


}