package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.AppointmentTextDataBeanParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC ORWCV DTLVST to get appointmenttext. 
 * It also helps parse the response received from RPC
 * @author DOMAINALLURS
 *
 */
public class AppointmentTextDao extends BaseDao 
{

    /**
     * This method is used to invoke the RPC ORWCV DTLVST to get appointmenttext and parse the response
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return List<DataBean> -a DataBean array is returned containing a list of Text found if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);

		String apptId = queryBean.getItemId();
		
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWCV_DTLVST, dfn,  "0", apptId);
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		AppointmentTextDataBeanParser appointmentTextDataBeanParser = (AppointmentTextDataBeanParser)getDataBeanParser(webMethodName);
		List<DataBean> textBeans = appointmentTextDataBeanParser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
		return textBeans;
	}


	/* 
	 * Returns the APPOINTMENT_TEXT_DAO 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.APPOINTMENT_TEXT_DAO;
	}
}
