package gov.va.med.via.dao;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.ChemHemReport;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.LabResult;
import gov.va.med.via.databeans.LabSpecimen;
import gov.va.med.via.databeans.LabTest;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;


public class ChemHemReportsDao extends BaseDao 
{	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#callRpc(java.lang.String, java.util.Map, gov.va.med.via.vistalink.ConnectionFactoryLocator)
	 */
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{			
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		//String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		return getChemHemReports(currentSiteId, dfn, queryBean, vistaLinkCommand);
		 					
	}
	
	
	
	  /*
     * MDWS - comment - 
     * In order to get the correct results with the correct reports using the DDR calls and RPCs 
     * available, we are going to use the user specified fromDate but use today as the toDate (like
     * CPRS does) and retrieve all the specimens with in that time period. Then, we will find the
     * reports, match them up with their specimens and lastly filter those results using our original 
     * date range
     */ 
	
	//String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWLRR_INTERIMG, dfn);
    protected List<DataBean> getChemHemReports(String siteId, String dfn, QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
    {
    	String fromDate = queryBean.getStartDate();
		String toDate= queryBean.getEndDate();
		int defaultPieceNum = 3;
		
        if (StringUtils.isNotEmpty(toDate) && toDate.indexOf(".") == -1)
        {
            toDate += ".235959";
        }

        StringBuilder fromDateBuilder = new StringBuilder(fromDate);
        StringBuilder toDateBuilder = new StringBuilder(toDate);
        
        ArrayList<LabSpecimen> specimens = getChemHemSpecimens(dfn, fromDateBuilder, toDateBuilder, defaultPieceNum, vistaLinkCommand);
        if (specimens == null || specimens.size()== 0)
        {
            return null;
        }

        ArrayList<ChemHemReport> reports = new ArrayList<ChemHemReport>();
        //Note: getChemHemSpecimens method updated toDate to the last date of specimens retrieved
        String nextDate = Timestamp.fromUtcString(toDateBuilder.toString());

        /* MDWS comment and logic
         * Due to comment above, we want to loop through "ORWLRR INTERIMG" RPC until we
         * get as many reports as the DDR call above (I think they mean the call to getSpecimens) told us we would 
         * TBD - this while statement screams infinite loop (i.e. what happens if we don't get all the 
         * reports the DDR call above said we would?) Should we put an arbitrarily large stop in here?
         * e.g. if count > 1,000,000 then throw an exception that we didn't get what VistA said we should?
         */
        int i = 0;
        while (reports.size() < specimens.size())
        {
            LabSpecimen specimen = specimens.get(i);
        	// ORWLRR INTERIMG rpc and function below updates nextDate for subsequent calls so LEAVE REF ALONE
            StringBuilder nextDateBuilder = new StringBuilder(nextDate);
            String nextDateBefore = nextDate;
            ChemHemReport rpt = getChemHemReport(siteId, dfn, nextDateBuilder, vistaLinkCommand);
            if (rpt != null)
            {
                LabSpecimen rptSpecimen = rpt.getSpecimen();
            	if ((rptSpecimen != null) && (rptSpecimen.getAccessionNumber().equals(specimen.getAccessionNumber())))
                {
                    i++;
                    rpt.setId(specimen.getId());
                    rpt.setFacility(specimen.getFacility());
                    rpt.setSpecimen(specimen);
                    rpt.setTimestamp(specimen.getReportDate());
                    reports.add(rpt);
                }
            }
            // MDWS comment: if the next date variable was not changed below, it means we are no longer looping through 
            // the reports so we should go ahead and break out of this loop - should take care of
            // infinite loop concerns
            nextDate = nextDateBuilder.toString();
            if (nextDate.equals(nextDateBefore))
            {
                break;
            }
        }
        
        //filter the reports based on their timestamps
        List<DataBean> filteredList = new ArrayList<DataBean>();
        long startDate = convertToLong(fromDate);
        long stopDate = convertToLong(toDate);
        for (ChemHemReport report : reports)
        {
            // if the report doesn't have a timestamp, we will just add it to our list
            if(StringUtils.isEmpty(report.getTimestamp()))
            {
                filteredList.add(report);
                continue;
            }
            long reportTimeStamp = convertToLong(report.getTimestamp());
            if (startDate <= reportTimeStamp && reportTimeStamp <= stopDate)
            {
                filteredList.add(report);
            }
        }
        return filteredList;
      
    }
    
    private long convertToLong(String dateString) {
    	
    	Calendar dateCal = DateUtils.IsoDateStringendDateTime(dateString);
    	return dateCal.getTimeInMillis();
	}



	public ChemHemReport getChemHemReport(String siteId, String dfn, StringBuilder nextDateBuilder, VistaLinkCommand vistaLinkCommand)
    {
    	
    	ChemHemReport report = null;
    	String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWLRR_INTERIMG, dfn, nextDateBuilder.toString(), "1", "1");
    	
    	response = RPCStringUtils.replaceEndofLineCharacters(response);
    	if (StringUtils.isNotEmpty(response))
		{
    		String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);	
     	
    		if (lines[0].equals("")) return report;
		
    		String[] fields = RPCStringUtils.split(lines[0], RPCStringUtils.CARET);
			nextDateBuilder.delete(0, nextDateBuilder.length());
			
			nextDateBuilder.append(fields[2]);
    		if (lines.length == 1 || !fields[1].equals("CH"))
    		{
    			return report;
    		}
    		
    		int ntests = new Integer(fields[0]);
    		report = new ChemHemReport();
    		report.setId(fields[2] + '^' + fields[5]);
    		report.setTimestamp(fields[2]);
    		
    		LabSpecimen specimen = new LabSpecimen();
    		specimen.setCollectionDate(Timestamp.toUtcString(fields[2]));
    		specimen.setName(fields[4]);
    		specimen.setAccessionNumber(fields[5]);
    		report.setSpecimen(specimen);
    		
    		Author author = new Author();
    		author.setName(fields[6]);
    		report.setAuthor(author);
    		int lineIdx = 1;
    		for (lineIdx = 1; lineIdx <= ntests; lineIdx++)
    		{
    			LabResult lr = new LabResult();
    			String[] flds = RPCStringUtils.split(lines[lineIdx], RPCStringUtils.CARET);
    			if (flds.length < 6)
    			{
    				continue;
    			}
    			LabTest lt = new LabTest();
    			lt.setId(flds[0]);
    			lt.setName(flds[1]);
    			lt.setUnits(flds[4].trim());
    			lt.setRefRange(flds[5]);
    			lr.setLabTest(lt);
    			
    			lr.setValue(flds[2].trim());
    			lr.setBoundaryStatus(flds[3]);
    			// MDWS comment: MHV patch - probably no one needs this
    			lr.setLabSiteId(siteId);

    			report.addLabResult(lr);
    		}
	
	    	StringBuilder reportComment = new StringBuilder(lines.length);
	    	while (lineIdx < lines.length)
	    	{
	    		reportComment.append(lines[lineIdx++].trim() + RPCStringUtils.CRLF );
	    	}
	    	
	    	report.setComment(reportComment.toString());
    	
        }

        return report;
    }

    protected ArrayList<LabSpecimen> getChemHemSpecimens(String dfn, StringBuilder fromDateBuilder, StringBuilder toDataBuilder, int pieceNum, VistaLinkCommand vistaLinkCommand)
    {
		/*
    	 * Retrieves Chem Specimens
    	 * FILE 63.04 - result position - field number and field definitions
    	 *  E flag note
    	 * .01 - DATE/TIME SPECIMEN TAKEN
    	 * .03 - DATE REPORT COMPLETED
    	 * .05 - SPECIMEN TYPE
    	 * .06 - ACCESSION
    	 * .08 - METHOD OR SITE
    	 * .112 - ACCESSIONING INSTITUTION
    	 * 
    	 */
		
		String lrdfn = getLrDfn(dfn, vistaLinkCommand);
		if(lrdfn == null){
			throw new RpcException(ErrorCodeEnum.DAO_EXCEPTION_UNFOUND_LRDFN, dfn);
		}
		
		//DateUtils.isValidDateRange(fromDate, toDate);
		
		List<Object> params =new ArrayList<Object>();		
 		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",63.04);
 		hashMap.put("IENS", ","+lrdfn+",");
 	    hashMap.put("FIELDS",".01;.03;.05E;.06;.08;.112E");
 		hashMap.put("FLAGS","IP");
 		String screen = RPCUtils.buildFromendDateScreenParam(fromDateBuilder.toString(), toDataBuilder.toString(), pieceNum);
 		hashMap.put("SCREEN", screen);
 		hashMap.put("XREF","#");
 		String id = "S X=$P(^(0),U,14) I X'= \"\" S Y=$P($G(^DIC(4,X,99)),U,1) D EN^DDIOL(Y)";
 		hashMap.put("ID", id);
 		 		 		
 		params.add(hashMap); 

 		String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
        return toChemHemSpecimens(response, fromDateBuilder, toDataBuilder);
 		
     }

  
     protected ArrayList<LabSpecimen> toChemHemSpecimens(String response, StringBuilder fromDateBuilder, StringBuilder toDateBuilder)
     {
        ArrayList<LabSpecimen> specimens = new ArrayList<LabSpecimen>();
    	response = RPCStringUtils.replaceEndofLineCharacters(response);
     	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
     	
     	for (int i= 0; i < lines.length; i++)
     	{
     		if (lines[i].contains("[Data]") || lines[i].equals("")) continue;
     		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
     		
     		String collectionDate = Timestamp.toUtcString(fields[1]);
     		String reportDate = Timestamp.toUtcString(fields[2]);
     		LabSpecimen specimen = new LabSpecimen(fields[0],collectionDate,reportDate,fields[3], fields[4]);
     		
     		//why is this being done?
            if (i == 0)
            {
            	fromDateBuilder.delete(0, fromDateBuilder.length());
            	fromDateBuilder.append(collectionDate);
            }
            if (i == response.length()-1)
            {
            	toDateBuilder.delete(0, toDateBuilder.length());
            	toDateBuilder.append(collectionDate);
            }
            
            if (fields.length > 6)
            {
                specimen.setSite(fields[5]);
            }
            if (fields.length > 7)
            {
                specimen.setFacility( new SiteId(fields[7], fields[6]));
            }
            
            specimens.add(specimen);
         }
         return specimens;
     }
	

	 protected String getLrDfn(String dfn, VistaLinkCommand vistaLinkCommand)
     {
		String lrDfn = null;
		
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("FILE",2);
		hashMap.put("FIELDS","63");
		hashMap.put("FLAGS","IP");
		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(dfn));
		hashMap.put("PART",RPCUtils.adjustForNumericSearch(dfn));
		hashMap.put("XREF","#");
		hashMap.put("MAX","1");
				
		params.add(hashMap); 
				
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	    if (response.equals("")) return lrDfn;
	    response = RPCStringUtils.replaceEndofLineCharacters(response);  
	    String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
	    String line4 = lines[3];
	    if (!StringUtils.isEmpty(line4)){
	    	String[] flds = RPCStringUtils.split(line4, RPCStringUtils.CARET);
        	lrDfn = flds[1];
	    }
	    
		return lrDfn;
     }

  	
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.CHEM_HEM_REPORTS_DAO;
	}

}
