package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.ClinicsDataBeanParser;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC to get clinics. 
 * It also helps parse the response received from RPC
 * 
 * @author DOMAINALLURS
 *
 */
public class ClinicsDao extends BaseDao 
{

    /**
     * This method is used to invoke the RPC ORWU CLINLOC to get clinics and parse its response
     * 
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return List<InpatientStay> -a InpatientStay array is returned containng a list of InpatientStay found if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		@SuppressWarnings("unchecked")
		HashMap<Object, Object> hashMap = (HashMap<Object, Object>)requestParameterMap.get(ViaRequestMapKeyConstants.HASHMAP_KEY);

		String target = queryBean.getTarget();
		
	    if (target == null || target.equals("")) 
	    	target = "";
	    else
	    	target = RPCUtils.adjustForNameSearch(target);
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWU_CLINICLOC, target, "1");

		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 				
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}


	/* 
	 * Returns the Clinics DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.CLINICS_DAO;
	}


}
