package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.StringDataBean;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class CloseNoteDao extends BaseDao 
{


	@Override
	public List<DataBean> callRpc(String currentSiteId,
			Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) 
	{

			QueryBean queryBean = (QueryBean) requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			String noteIEN = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_IEN);
			String consultIEN = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_CONSULT_IEN);

			/// create a single VistALink command to execute all the RPCs against
			VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
			unlockNote(noteIEN, vistaLinkCommand);
			
			String text = getNoteText(noteIEN, vistaLinkCommand);
			if (!StringUtil.isNullOrEmpty(consultIEN))
			{
				String orderIEN = getOrderNumberForConsult(consultIEN, vistaLinkCommand);
				if ( "".equals(orderIEN))
				{
					/// Do nothing - CPRS assumes it's an Inter-facility Consult
				}
				else
				{
					if ( !unlockOrder(consultIEN,vistaLinkCommand))
					{
						//TODO change exception type to business exception
						throw new RpcException(ErrorCodeEnum.NOTE_CANNOT_UNLOCK_CONSULT, consultIEN);
					}					
				}
			}
			
			String permission = isUserAuthorized(noteIEN, "VIEW", vistaLinkCommand);
			String result = null;
	        if (!"OK".equals(permission))     // Note is signed but unviewable (weird)
	        {
	        	result= "OK";
	        }
	        else
	        {
	        	result = text;
	        }
	        
	        Text textDataBean = new Text(result, currentSiteId);
	        List<DataBean> resultList = new ArrayList<DataBean>();
			resultList.add(textDataBean);
			return resultList;
	}
	
	private String unlockNote(String noteIEN, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_UNLOCK_RECORD, params);
		return RPCUtils.errMsgOrZero(response);
	}
	
	 public String getNoteText(String noteIEN, VistaLinkCommand vistaLinkCommand )
     {
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_GET_RECORD_TEXT, params);
		return response; 
     }

	 
	 public String getOrderNumberForConsult(String consultIEN, VistaLinkCommand vistaLinkCommand)
     {
		 List<Object> params =new ArrayList<Object>();
		 params.add(consultIEN);
		 String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORQQCN_GET_ORDER_NUMBER, params);
		 return response;
     }
	
	
	 private boolean unlockOrder(String ien, VistaLinkCommand vistaLinkCommand )
     {
		 List<Object> params =new ArrayList<Object>();
		 params.add(ien);
		 String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_UNLOCK_ORDER, params);
		 return "1".equals(response);
     }
	
	
	/**
	 * 
	 * @param noteIEN
	 * @param permission
	 * @param vistaLinkCommand
	 * @return "OK" if the TIU_AUTHORIZATION rpc call returns "1", otherwise returns error message
	 */
	private String isUserAuthorized(String noteIEN, String permission, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		params.add(permission);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_AUTHORIZATION, params);
		if ( response.equals("1")) return "OK";
		return StringUtil.piece(response,StringUtil.CARET,2);
	}




	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.CLOSE_NOTE_DAO;
	}

}
