package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;

import java.util.List;
import java.util.Map;

/**
 * Class used to extend the PatientSelectDao so that this Dao can extend the web method service name parameter (where only one is defined per webservice method
 * and add an extension to the webservice method name which in turn allows additional application logic to be applied and/or in this case
 * to map multiple databean parsers to the set of daos that have been assigned to the service method ... 
 *
 */
public class CprsLaunchPatientDao extends PatientSelectDao {
	
	
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		
		//This is a 'hacked' work-around to the limitation of being able to assign multiple databean parsers to one web service method name
		//1. get the original web method service name
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		//2. remove original web method service name from the parameters map
		requestParameterMap.remove(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		//3. now replace web service method name with an extended name - this will allow the super's callRpc to locate an instance of the PatientDataBeanParser mapped to this webservice method name
		requestParameterMap.put(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY, ViaWebMethodNameConstants.EMRSVC_CPRSLAUNCH_PATIENTSELECT);
		//4. call rpc
		List<DataBean> results =  super.callRpc(currentSiteId, requestParameterMap, connectionFactoryLocator);
		//5.replace original webmethod service name
		requestParameterMap.remove(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		requestParameterMap.put(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY, webMethodName);
		
		return results;
	}

	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.CPRS_LAUNCH_PATIENT_SELECT_DAO;
	}

}
