package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaQuery;
import gov.va.med.via.util.TestRpcParam;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.VistaLinkTemplate;
import gov.va.med.via.vistalink.command.VistaLinkAppProxyCommandImpl;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.via.vistalink.command.VistaLinkDuzCommandImpl;
import gov.va.med.vistalink.rpc.RpcReferenceType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class GenericRpcDao extends BaseDao {

	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) {

		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		Map<String,String> map = queryBean.getAdditionalQueryParams();
		String scListerKey = map.get("SC LISTER");
		boolean isScLister = false;
		if ( scListerKey!=null )
		{
			isScLister = true;
		}
		if ( isScLister)
		{
			return callScListerRpc(currentSiteId, requestParameterMap, connectionFactoryLocator );
		}


		boolean useProxy = (Boolean) requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_USE_PROXY);
		String duz = (String)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_DUZ);//the query bean (and we will have additional parameters for some methods - will create a key name at a later time for those
		String division = (String)requestParameterMap.get(ViaRequestMapKeyConstants.LOGIN_SITE_CODE_KEY);
		String user = (String) requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_USER);
		String rpcContext = (String)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_CONTEXT );
		String rpcName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_RPC_NAME);

		@SuppressWarnings("unchecked")
		List<TestRpcParam> testRpcParamsList = (List<TestRpcParam>)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_RPC_PARAMETERS_LIST_KEY);

		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);

		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand( connectionFactoryLocator, division, useProxy, rpcContext, duz, user);
		boolean addParams = testRpcParamsList!=null && testRpcParamsList.size()>0;
		try
		{
			if ( !addParams )
			{
				String result= vistaLinkCommand.executeRpc(rpcName);
				return parseResponseToDataBeans(result, queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
			}

			ArrayList<Object> params = new ArrayList<Object>();
			for ( TestRpcParam rpcParam: testRpcParamsList )
			{
				Object paramValue=rpcParam.getParamValue();
				if ( paramValue!=null )
				{
					if ( rpcParam.isEncryptValue() ) 
					{
						paramValue =  VistaQuery.encrypt(rpcParam.getParamValue());
					}
					/// handle RpcReferenceType
					String type = rpcParam.getParamType();
					if ( type!=null )  type = type.toLowerCase().trim();
					if ( type!=null && type.indexOf("ref")!=-1)
					{
						paramValue = new RpcReferenceType();
						((RpcReferenceType)paramValue).setValue(rpcParam.getParamValue());					
					}
					if ( type!=null && type.equalsIgnoreCase("list"))
					{
						HashMap<Object,Object> hm =new HashMap<Object,Object>();
						String[] items = rpcParam.getParamValue().split(";");
						for ( String item: items)
						{
							String[] keyValues = item.split("=");
							if ( keyValues.length!=2) continue;
							hm.put(keyValues[0], keyValues[1]);
						}
						paramValue= hm;
					}

					params.add(paramValue);
				}
			}
			String result=null;
			if ( addParams )
			{
				result =vistaLinkCommand.executeRpc(rpcName, params);
			}
			else
			{
				result= vistaLinkCommand.executeRpc(rpcName);
			}
			return parseResponseToDataBeans(result, queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
		} 
		catch (Exception e) 
		{
			return parseResponseToDataBeans(e.toString(), queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
		}

	}


	public List<DataBean> callScListerRpc(String currentSiteId, Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) {

		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		VistaLinkCommand vistaLinkCommand = null; 

		try
		{

			boolean useProxy = (Boolean) requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_USE_PROXY);
			String duz = (String)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_DUZ);//the query bean (and we will have additional parameters for some methods - will create a key name at a later time for those
			String division = (String)requestParameterMap.get(ViaRequestMapKeyConstants.LOGIN_SITE_CODE_KEY);
			String user = (String) requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_USER);
			String rpcContext = (String)requestParameterMap.get(ViaRequestMapKeyConstants.GENERIC_CONTEXT );
			vistaLinkCommand = createVistaLinkCommand( connectionFactoryLocator, division, useProxy, rpcContext, duz, user);

			Map<String,String> map = queryBean.getAdditionalQueryParams();
			String file = map.get("FILE");
			String fields = map.get("FIELDS");
			String flags = map.get("FLAGS");
			String iens = map.get("IENS");
			String part = map.get("PART");
			String xref = map.get("XREF");
			String id = map.get("ID");
			String from = map.get("FROM");
			String screen = map.get("SCREEN");
			String max = map.get("MAX");


			List<Object> params =new ArrayList<Object>(); 
			HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			if ( file!=null && !file.trim().equals(""))
			{
				try
				{
					Integer nFile = Integer.parseInt(file.trim());
					hashMap.put("FILE", nFile);						
				}
				catch ( NumberFormatException e)
				{
					try
					{
						Double dFile = Double.parseDouble(file.trim());
						hashMap.put("FILE", dFile);
					}
					catch ( NumberFormatException e2)
					{
						hashMap.put("FILE",file); /// hospital location is file 44
					}
				}
			}

			
			if ( fields!=null) hashMap.put("FIELDS",fields); /// E can limit what returns
			if ( flags!=null) hashMap.put("FLAGS",flags);
			if ( iens!=null) hashMap.put("IENS",iens);
			if ( part!=null) hashMap.put("PART",part);  /// part is like equal
			if ( xref!=null) hashMap.put("XREF",xref);  /// xref index  
			if ( id!=null) hashMap.put("ID",id);
			if ( from!=null) hashMap.put("FROM",from);
			if ( screen!=null) hashMap.put("SCREEN",screen);
			if ( max!=null) hashMap.put("MAX",max);

			params.add(hashMap); 
			String result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
			return parseResponseToDataBeans(result, queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
		} 
		catch (Exception e) 
		{
			return parseResponseToDataBeans(e.toString(), queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
		}

	}



	@Override
	public String getDaoName() {
		return DaoNameConstants.GENERIC_RPC_DAO;
	}

	/**
	 * Determines the type (DUZ of APP PROXY) of connectionSpec and creates a Command object for that type that will delegate calls of that type to the VistaLinkTemplate 
	 * 
	 * @param connectionFactoryLocator
	 * @param targetSiteCode
	 * @param provider
	 * @param rpcContext
	 * @return VistaLinkCommand object for the type of ConnectionSpec
	 */
	protected VistaLinkCommand createVistaLinkCommand(ConnectionFactoryLocator connectionFactoryLocator, String targetSiteCode, boolean useProxy, String rpcContext, String duz, String appProxyUserName) {
		//check to see if this should be a DUZ connection or App Proxy connection - OR something more involved - depending upon Business need - this might need
		//to be delegated to another component for authorization - BSE or some other authorization - this logic may be too simple
		int connectionSpecType = useProxy?VistaLinkTemplate.USER_TYPE_APPLICATION_PROXY:VistaLinkTemplate.USER_TYPE_DUZ;//connectionSpecManager.getConnectionSpecType(provider, targetSiteCode);

		VistaLinkTemplate vistaLinkTemplate = new VistaLinkTemplate(connectionFactoryLocator);

		VistaLinkCommand vistaLinkCommand = null;

		if(connectionSpecType == VistaLinkTemplate.USER_TYPE_DUZ){
			vistaLinkCommand = new VistaLinkDuzCommandImpl(targetSiteCode, rpcContext, duz,vistaLinkTemplate);
		}
		else if(connectionSpecType == VistaLinkTemplate.USER_TYPE_APPLICATION_PROXY)
		{
			vistaLinkCommand = new VistaLinkAppProxyCommandImpl(targetSiteCode, rpcContext, appProxyUserName, vistaLinkTemplate);
		}
		return vistaLinkCommand;
	}

}
