package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.HospitalLocationsParser;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.List;
import java.util.Map;

public class HospitalLocationsDao extends BaseDao {

	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId, provider, VistaConstants.CPRS_CONTEXT);

		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);

		String target = queryBean.getTarget();
		String direction = queryBean.getDirection();

		if (target == null || target.equals("")) 
			target = "";
		else
			target = RPCUtils.adjustForNameSearch(target);

		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWU1_NEWLOC, target, RPCUtils.setDirectionParam(direction));


		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		HospitalLocationsParser parser = (HospitalLocationsParser)getDataBeanParser(webMethodName);
		return parser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	/* 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.HOSPITAL_LOCATIONS_DAO;
	}


}