package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.IssueConfidentialityBulletinDataBeanParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC to get issue Issue Confidentiality Bulletin. 
 * It also helps parse the response received from RPC
 * @author DOMAINALLURS
 *
 */
public class IssueConfidentialityBulletinDao extends BaseDao {

    /**
     * This method is used to invoke the RPC DG SENSITIVE RECORD BULLETIN to get issue Issue Confidentiality Bulletin and parse its response
     * 
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return List<Text> -a Text array is returned containing a list of Text found if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.DG_SENSITIVE_RECORD_BULLETIN, dfn);
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		IssueConfidentialityBulletinDataBeanParser issueConfidentialityBulletinDataBeanParser = (IssueConfidentialityBulletinDataBeanParser)getDataBeanParser(webMethodName);
		return issueConfidentialityBulletinDataBeanParser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	/* 
	 * Returns the Confidentiality DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.ISSUE_CONFIDENTIALITY_BULLETIN_DAO;
	}


}
