package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.LocationsDataBeanParser;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC to get admissions. 
 * It also helps parse the response received from RPC
 * @author DOMAINALLURS
 *
 */
public class LocationsDao extends BaseDao {

    /**
     * This method is used to invoke the RPC ORWPT ADMITLST to get patient admissions and parse its response
     * 
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return List<InpatientStay> -a InpatientStay array is returned containng a list of InpatientStay found if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		String target = queryBean.getTarget();
		String direction = queryBean.getDirection();
		
         if (target == null || target.equals("")) 
        	 target = "";
         else
             target = RPCUtils.adjustForNameSearch(target);
		
		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		//dfn not required for this call
		String dfn = null;// getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWU1_NEWLOC, target, RPCUtils.setDirectionParam(direction));
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		LocationsDataBeanParser locationsDataBeanParser = (LocationsDataBeanParser)getDataBeanParser(webMethodName);
		return locationsDataBeanParser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	/* 
	 * Returns the Visits DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.LOCATIONS_DAO;
	}


}
