package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.response.MedsDataBeanParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC to get Meds. 
 * It also helps parse the response received from RPC
 * @author DOMAINALLURS
 *
 */
public class MedsDao extends BaseDao {

    /**
     * This method is used to invoke the RPC ORWPS ACTIVE and ORWRP REPORT TEXT(for NV Meds)to get Meds and parse the response
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return List<DataBean> -a DataBean array is returned containing a list of Medication found if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		String duz = provider.getUserId();
		
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPS_ACTIVE, dfn, duz, "0", "1");
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		MedsDataBeanParser medsDataBeanParser = (MedsDataBeanParser)getDataBeanParser(webMethodName);
		List<DataBean> meds = medsDataBeanParser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
		List<DataBean> allMeds = new ArrayList<DataBean>();
		allMeds.addAll(meds);
		
		// If request is getAllMeds or getOtherMeds,
		// get the NV(non VIA) meds report and add to the list
		if ( webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETOTHERMEDS)==0 ||
				webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETALLMEDS)==0
				){
			String nonvaMedsResult = vistaLinkCommand.executeRpc(RpcNameConstants.ORWRP_REPORT_TEXT, dfn, RpcNameConstants.ORRXN_NONVA_MEDS, "", "", "", "0", "0" );
			List<DataBean> nonvaMeds =  medsDataBeanParser.getOtherMedsRdv(nonvaMedsResult);
			allMeds.addAll(nonvaMeds);
		}
		return allMeds;
	}


	/* 
	 * Returns the Meds DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.MEDS_DAO;
	}
}
