package gov.va.med.via.dao;

import java.util.List;
import java.util.Map;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

/**
 * Class manages access, connection and parsing to the select patient functionality.
 * 
 * @author DOMAINROMERJ
 *
 */
public class NoteTypeQuestionsDao extends BaseDao 
{
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		String noteDefinitionIEN = queryBean.getItemId();		
		String result = "";
		if (ViaWebMethodNameConstants.TBISVC_ISCONSULTNOTE.equals(webMethodName) 
				|| ViaWebMethodNameConstants.EMRSVC_ISCONSULTNOTE.equals(webMethodName) 
				|| ViaWebMethodNameConstants.CALLSVC_ISCONSULTNOTE.equals(webMethodName))
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_IS_THIS_A_CONSULT, noteDefinitionIEN);
		}
		else if (ViaWebMethodNameConstants.TBISVC_ISONEVISITNOTE.equals(webMethodName) 
				 || ViaWebMethodNameConstants.EMRSVC_ISONEVISITNOTE.equals(webMethodName)
				 || ViaWebMethodNameConstants.CALLSVC_ISONEVISITNOTE.equals(webMethodName))
		{
			String visitString = queryBean.getCriteria();  
			result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_ONE_VISIT_NOTE, noteDefinitionIEN, dfn, visitString);
		}
		else if (ViaWebMethodNameConstants.TBISVC_ISSURGERYNOTE.equals(webMethodName)
				|| ViaWebMethodNameConstants.EMRSVC_ISSURGERYNOTE.equals(webMethodName) 
				|| ViaWebMethodNameConstants.CALLSVC_ISSURGERYNOTE.equals(webMethodName))
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_IS_THIS_A_SURGERY, noteDefinitionIEN);
		}
		else if (ViaWebMethodNameConstants.TBISVC_ISPRFNOTE.equals(webMethodName) 
				|| ViaWebMethodNameConstants.EMRSVC_ISPRFNOTE.equals(webMethodName) 
				|| ViaWebMethodNameConstants.CALLSVC_ISPRFNOTE.equals(webMethodName))
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_ISPRF, noteDefinitionIEN);
		}
		else if (ViaWebMethodNameConstants.TBISVC_ISCOSIGNERREQUIRED.equals(webMethodName)
				|| ViaWebMethodNameConstants.EMRSVC_ISCOSIGNERREQUIRED.equals(webMethodName)
				|| ViaWebMethodNameConstants.CALLSVC_ISCOSIGNERREQUIRED.equals(webMethodName))
		{
			// This RPC requires multiple verifications:
			// a) Verify if the user requires CoSignature, if not, verify if the user requires CoSignature for this note.
			// b) if the authorDuz is null, return false; otherwise verify if the author requires CoSignature, if not, verify if the author requires CoSignature for this note.
			
			String duz=queryBean.getCriteria();  /// the criteria provides the caller with the opportunity to see if another user requires cosignature, if not defined, then the value defaults to the logged in provider's DUZ
			if ( duz==null ) duz = queryBean.getProvider().getUserId();
			
			result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_REQUIRES_COSIGNATURE, "3", "0", duz); // Does User require CoSignature?
			if (!result.equals("1"))
			{
				result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_REQUIRES_COSIGNATURE, noteDefinitionIEN, "0", duz); // Does user require CoSignature for this note?
				String authorDuz = queryBean.getCriteria();	

				if (!result.equals("1"))
				{
					if (RPCStringUtils.isNullOrEmpty(authorDuz)) 
						result = "0"; // If authorDuz is null, return "0"
					else
					{
						result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_REQUIRES_COSIGNATURE, "3", "0", authorDuz); // Does author require CoSignature?
						if (!result.equals("1"))
							result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_REQUIRES_COSIGNATURE, noteDefinitionIEN, "0", authorDuz); // Does author require CoSignature for this note?
					}
				}
			}
		}
 				
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}

	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.NOTE_TYPE_QUESTIONS_DAO;
	}

}
