package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.NotesWithTextParser;
import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.util.DateTimeUtil;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

public class NotesWithTextDao extends BaseDao {

	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		@SuppressWarnings("unchecked")
		HashMap<Object, Object> hashMap = (HashMap<Object, Object>)requestParameterMap.get(ViaRequestMapKeyConstants.HASHMAP_KEY);

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId, provider, VistaConstants.CPRS_CONTEXT);
		
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		/// setup params
		List<Object> params = setupParams(queryBean, dfn);
		
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWRP_REPORT_TEXT, params);
		
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		NotesWithTextParser parser = (NotesWithTextParser)getDataBeanParser(webMethodName);
		return parser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	private List<Object> setupParams(QueryBean queryBean, String dfn) {
		List<Object> params = new ArrayList<Object>();
		params.add(dfn);
		
		String reportId = "OR_PN:PROGRESS NOTES~TIUPRG;ORDV04;15;";
		if ( queryBean.getMaxRecords()!=null && new Integer(queryBean.getMaxRecords()).intValue()>0)
		{
			reportId+=queryBean.getMaxRecords();
		}
		params.add(reportId);
		params.add("");
		params.add("");
		params.add("");
		
		String startDate = queryBean.getStartDate();
		if ( startDate==null || startDate.isEmpty() || startDate.trim().equals("0"))
		{
			startDate =DateTimeUtil.VISTA_MIN_DATE_STRING;
		}
		else 
		{
			startDate = VistaTimestamp.fromUtcString(startDate);			
		}
		
		String endDate = queryBean.getEndDate();
		if ( endDate==null || endDate.trim().equals(""))
		{
			endDate="0";
		}
		else if ( !endDate.trim().equals("0"))
		{
			endDate = VistaTimestamp.fromUtcString(endDate);
		}
		
		//TODO date range checks needed
//		/*
//		
//		
//		 if (  !endDate.equals("0") && !endDate.equals("-1"))
//         {
//             DateUtils.checkDateRange(startDate, endDate);
//         }
//         else
//         {
//             if (!String.IsNullOrEmpty(fromDate) && fromDate != "0")
//             {
//                 if (!DateUtils.isWellFormedUtcDateTime(fromDate))
//                 {
//                     throw new MdoException(MdoExceptionCode.ARGUMENT_DATE_FORMAT, "Invalid 'from' date: " + fromDate);
//                 }
//             }
//         }
//		*/
//		
		params.add(startDate);
		params.add(endDate);
		
		return params;
	}


	/* 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.NOTES_WITH_TEXT_DAO;
	}


}