package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcReferenceType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PatientAssociatesDao extends BaseDao 
{

	/**
	 * Retrieves the patient associates information from Vista for the patient received.
	 * The Patient Associates are (fields from Vista Patient file in parenthesis):
	 * 	- Primary NOK (.211 - Name, .212 - Relationship, .213.214.215 - Streets 1,2,3; .216 - City, .217 - State, .218 - Zip Code, .219 - Phone Number) 
	 *  - Secondary NOK (.2191 - Name, .2192 - Relationship, .2193.2194.2195 - Streets 1,2,3; .2196 - City, .2197 - State, .2198 - Zip Code, .2919 - Phone Number)
	 *  - Primary Emergency Contact (.331 - Name, .332 - Relationship, .333.334.335 - Streets 1,2,3; .336 - City, .337 - State, .338 - Zip Code, .339 - Phone Number) 
	 *  - Secondary Emergency Contact (.3311 - Name, .3312 - Relationship, .3313.3314.3315 - Streets 1,2,3; .3316 - City, .3317 - State, .3318 - Zip Code, .3319 - Phone Number)
	 *  - Designee (.341 - Name, .342 - Relationship, .343.344.345 - Streets 1,2,3; .346 - City, .347 - State, .348 - Zip Code, .349 - Phone Number)
	 *  - VA Guardian (.2912 - Name, .2913 - Relationship, .2914.2915 - Streets 1,2; .2916 - City, .2917 - State, .2918 - Zip Code, .2919 - Phone Number)
	 *  - Civil Guardian (.2922 - Name, .2923 - Relationship, .2924.2925 - Streets 1,2; .2926 - City, .2927 - State, .2928 - Zip Code, .2929 - Phone Number)
	 * 
	 */	
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);		

		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
	    List<Object> params =new ArrayList<Object>();                             
        hashMap = new HashMap<Object,Object>();
        hashMap.put("FILE",2);
        hashMap.put("FIELDS",".211;.212;.213;.214;.215;.216;.217;.218;.219;" +	// Primary NOK
        		".2191;.2192;.2193;.2194;.2195;.2196;.2197;.2198;.2199;" +	// Secondary NOK
        		".331;.332;.333;.334;.335;.336;.337;.338;.339;" + 			// Primary Emergency Contact
        		".3311;.3312;.3313;.3314;.3315;.3316;.3317;.3318;.3319;" +	// Secondary Emergency Contact
        		".341;.342;.343;.344;.345;.346;.347;.348;.349;" +			// Designee
        		".2912;.2913;.2914;.2915;.2916;.2917;.2918;.2919;" +		// VA Guardian
        		".2922;.2923;.2924;.2925;.2926;.2927;.2928;.2929");			// Civil Guardian 	 
        hashMap.put("FLAGS","IP");
        hashMap.put("FROM",RPCUtils.adjustForNumericSearch(dfn));
        hashMap.put("PART",dfn);
        hashMap.put("XREF","#");
        hashMap.put("MAX","1");
        params.add(hashMap);        
        String scResult = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);        

        return parseResponseToDataBeans(scResult, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}


	/* 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.PATIENT_ASSOCIATES_DAO;
	}


}