package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PatientCorrespondingIdsDao extends BaseDao {

	public List<DataBean> callRpc(String currentSiteId, Map<String, Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {

		QueryBean queryBean = (QueryBean) requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, provider, VistaConstants.CPRS_CONTEXT);

		String dfn = "";
		String type = queryBean.getType();
		if (type.equalsIgnoreCase("DFN")) {
			dfn = queryBean.getPatient().getLocalPid();
		} 
		else if (type.equalsIgnoreCase("ICN")) {
			String icn = queryBean.getPatient().getMpiPid();
			dfn = populateLocalPid(icn, vistaLinkCommand);
		}
		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
	    List<Object> params =new ArrayList<Object>();                             
        hashMap = new HashMap<Object,Object>();
        hashMap.put("FILE",391.91);
        hashMap.put("FIELDS",".01;.02;.03;.07;.08;.09;10;11;12");
        hashMap.put("FLAGS","IP");
        hashMap.put("FROM",RPCUtils.adjustForNumericSearch(dfn));
        hashMap.put("PART",dfn);
        hashMap.put("XREF","B");
        hashMap.put("ID","S ID=$G(^(1,1,0)) D EN^DDIOL(ID)");
      
        params.add(hashMap);        
        String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);      
		
		
		return parseCorrespondingIdsToText(response, dfn);

	}

	private List<DataBean> parseCorrespondingIdsToText(String response, String dfn) {

		List<DataBean> correspondingIds = new ArrayList<DataBean>();
		
		response = RPCStringUtils.replaceEndofLineCharacters(response);
     	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
     	
     	for (int i= 0; i < lines.length; i++)
     	{
     		
     		if (lines[i].contains("[Data]") || lines[i].equals("")) continue;
     		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
     		
     		//if our record is for our local dfn and it is active
     		if(fields[1].equals(dfn) && fields[9].equals("A"))
     		{
     			//if this index value is a siteId then the record is a corresponding Id
     			if(fields[2].length()==3)
     			{
     				//create a text data bean with the site's dfn as text (fields[8]) and the siteId as tag (fields[2])
     				Text text = new Text(fields[8],fields[2]);
     				correspondingIds.add(text);
     			}
     		}
     	}
	
		return correspondingIds;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.va.med.via.dao.BaseDao#getDaoName()
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.PATIENT_CORRESPONDINGIDS_DAO;
	}

}
