package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.List;
import java.util.Map;

public class PatientSelectByIdDao extends BaseDao 
{	
	/** 
	 * Invokes the correct RPC call to select the patient in Vista associated to the Id received (WardId, ClinicId, TeamId, etc). 
	 * 
	 * @return a list of databeans populated from the RPC call response.
	 */
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		
		String id = queryBean.getItemId(); // Id to select patients by
		String startDate = queryBean.getStartDate();
		String endDate = queryBean.getEndDate();
		String result = "";
		// Select Patients by Ward ID
		if ((ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYWARD.compareTo(webMethodName) == 0) ||
			(ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYWARD.compareTo(webMethodName) == 0))
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT_BYWARD, id); // id = wardId
		}
		// Select Patients by Team ID
		else if (ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYTEAM.compareTo(webMethodName) == 0)
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.ORQPT_TEAM_PATIENTS, id); // id = TeamId
		}
		// Select Patients by Provider ID
		else if (ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYPROVIDER.compareTo(webMethodName) == 0)
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.ORQPT_PROVIDER_PATIENTS, id); // id = ProviderId
		}
		// Select Patients by Specialty ID
		else if (ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYSPECIALTY.compareTo(webMethodName) == 0)
		{
			result = vistaLinkCommand.executeRpc(RpcNameConstants.ORQPT_SPECIALTY_PATIENTS, id); // id = SpecialtyId
		}				
		// Select Patients by Clinic ID
		else if ((ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYCLINIC.compareTo(webMethodName) == 0) ||
				 (ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYCLINIC.compareTo(webMethodName) == 0))
		{
			if (RPCStringUtils.isNullOrEmpty(startDate)) 
				startDate = "T"; // If startDate is null, send "T"
			if (RPCStringUtils.isNullOrEmpty(endDate)) 
				endDate = "T"; // If endDate is null, send "T"			
			result = vistaLinkCommand.executeRpc(RpcNameConstants.ORQPT_CLINIC_PATIENTS, id, startDate, endDate); // id = clinicID
		}
		
		return parseResponseToDataBeans(result, queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);
	}

	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.PATIENT_SELECT_BY_ID_DAO;
	}

}
