package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.List;
import java.util.Map;

/**
 * Class manages access, connection and parsing to the select patient functionality.
 * 
 * @author DOMAINROMERJ
 *
 */
public class PatientSelectDao extends BaseDao 
{
	
	/** 
	 * Invokes the RpcNameConstants.ORWPT_SELECT RPC call to select the patient in Vista associated to the dfn received.
	 * 
	 * @return a list of databeans populated from the RPC call response.
	 */
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
				
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT_SELECT, dfn);	
		
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}

	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.PATIENT_SELECT_DAO;
	}

}
