package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.PrfNoteActionsParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PrfNoteActionsDao extends BaseDao {

	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		@SuppressWarnings("unchecked")
		HashMap<Object, Object> hashMap = (HashMap<Object, Object>)requestParameterMap.get(ViaRequestMapKeyConstants.HASHMAP_KEY);

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId, provider, VistaConstants.CPRS_CONTEXT);
		
		// The IEN is the TIU DOCUMENT DEFINITION (TITLE) IEN
		String ien = queryBean.getItemId();
		
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		
		List<Object> params = new ArrayList<Object>();
		params.add(ien);
		params.add(dfn);
		
		//Execute RPC
		///This RPC gets the Patient Record Flag History Assignments/Actions for a Patient/Title Combination.
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_GET_PRF_ACTIONS, params);
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		PrfNoteActionsParser parser = (PrfNoteActionsParser)getDataBeanParser(webMethodName);
		return parser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	/* 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.PRF_NOTE_ACTIONS_DAO;
	}


}