package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The call to retrieve prosthetic claims for a patient filtered by a date list.
 * 
 * @author ZZZZZZAPOSTI
 *
 */
public class ProstheticClaimsDao extends BaseDao 
{
	
	private VistaLinkCommand vistaLinkCommand = null;
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#callRpc(java.lang.String, java.util.Map, gov.va.med.via.vistalink.ConnectionFactoryLocator)
	 */
	@Override
	public List<DataBean> callRpc(String currentSiteId,
			Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) 
	{
			String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);			
			QueryBean queryBean = (QueryBean) requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		
			vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, 
					currentSiteId, 
					queryBean.getProvider(), 
					VistaConstants.CPRS_CONTEXT);
			
			//Get dfn
			String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);			
			String result = getProstheticClaims(dfn);						

			return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}
	
	private String getProstheticClaims(String dfn) {
		
		String result = "";
		
		if (dfn != null) {
			
            /*DdrLister query = new DdrLister(cxn);
            query.File = "356";
            query.Fields = ".02;.02E;.06;.09;1.01;1.03";
            query.Flags = "IP";
            query.Xref = "C";
            query.From = VistaUtils.adjustForNumericSearch(dfn);
            query.Part = dfn;
            query.Screen = "I $P(^(0),U,9)'=\"\"";*/
			
			/*
			 * Add the “@” sign to the fields.
			 * S JC("FIELDS")="@;.01;.02;.02E;.06;.09"
			 * There is a Write Identifier in 356 and without the “@” it causes an error when it processes.
			 * @: This suppresses all the default values normally returned by the Lister, except for the IEN
			 * and any fields and values specified in the FIELDS parameter. It is recommended that 
			 * developers ALWAYS use the "@" specifier in their Lister calls. Use of the "@" specifier 
			 * allows the developer to control exactly what will be returned in the output. 
			 * See the default values below to see what is normally returned by the Lister.
			 */
			
			List<Object> params =new ArrayList<Object>();		
			HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			hashMap.put("FILE",356);
			hashMap.put("FIELDS","@;.02;.02E;.06;.09;1.01;1.03");
			hashMap.put("FLAGS","IP");
			hashMap.put("XREF","C");
			hashMap.put("FROM",RPCUtils.adjustForNumericSearch(dfn));
			hashMap.put("PART",dfn);
			params.add(hashMap);	
			
			result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
			
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.BaseDao#getDaoName()
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.PROSTHETIC_CLAIMS_DAO;
	}

}
