package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ReportRequestDao extends BaseDao 
{
    /**
     * This method is used to invoke the ORWRP REPORT TEXT to get the report based on the argument(s) received
     * 
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * 
     * @return List<DataBean> -a Text array is returned containing the report requested if successful.
     */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);		
		String argFromDate = queryBean.getStartDate();	
		String utcFromDate = "0";
		if ( argFromDate != null && argFromDate.length() > 0)
			utcFromDate = Timestamp.fromUtcString(argFromDate);
				
		String argToDate = queryBean.getEndDate();
		String utcToDate = "0";
		if ( argToDate != null && argToDate.length() > 0)
			utcToDate = Timestamp.fromUtcString(argToDate);
						
		String args = (String)requestParameterMap.get(ViaRequestMapKeyConstants.REPORT_REQUEST_ARG_KEY);
		String ien = "";
		String value = "";
						
		if (args.contains("LOCAL ONLY")) // This means the report should only retrieve the data from the local Vista Server
		{
			ien = queryBean.getItemId() != null? queryBean.getItemId() : "";	
		}
		else
		{
			// Using 10 as the default maximum number of records returned based on CPRS's default. -> chnages MDWS returned records 
			String maxRecords = RPCStringUtils.isNullOrEmpty(queryBean.getMaxRecords())? "" : queryBean.getMaxRecords();

			// Value is defaulted to 00000 when one, or both date parameters is missing. MaxRecords default changes to 101 per CPRS
			if ("0".equals(utcFromDate) || "0".equals(utcToDate))
			{
				value = "00000";
				maxRecords = "101";
			}
			
			if (!RPCStringUtils.isNullOrEmpty(maxRecords))
			{
				args += maxRecords;	
			}
		}
		

		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWRP_REPORT_TEXT, dfn, args, "", value, ien, utcFromDate, utcToDate );
		
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}


	/* 
	 * Returns the REPORT_REQUEST_DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.REPORT_REQUEST_DAO;
	}


}