package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.StringDataBean;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.rpc.VistaQuery;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SignNoteDao extends BaseDao 
{


	@Override
	public List<DataBean> callRpc(String currentSiteId,
			Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) 
			{
	
			QueryBean queryBean = (QueryBean) requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			String noteIEN = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_IEN);
			String esig = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_ESIG);
			/// the author should be the provider DUZ
			//String authorDuz = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_AUTHOR_DUZ);
			
			/// create a single VistALink command to execute all the RPCs against
			VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);

			if ( !checkText(noteIEN,vistaLinkCommand))
			{
				throw new RpcException(ErrorCodeEnum.NOTE_IEN_EXCEPTION);
			}
			String checkNoteSavedResult = this.checkNoteSaved(noteIEN, vistaLinkCommand);
			if ( !checkNoteSavedResult.equals("OK"))
			{
				throw new RpcException(ErrorCodeEnum.NOTE_NOT_SAVED_EXCEPTION, checkNoteSavedResult);
			}
			String permission = whichSignatureAction(noteIEN, vistaLinkCommand);
			if (!permission.equals("SIGNATURE"))
			{
				throw new RpcException(ErrorCodeEnum.NOTE_INVALID_SIGNATURE_PERMISSION_EXCEPTION, permission);
			}
			String userAuthorized = isUserAuthorized(noteIEN, permission, vistaLinkCommand);
			if (!"OK".equals(userAuthorized))
            {
				throw new RpcException(ErrorCodeEnum.NOTE_USER_NOT_AUTHORIZED_EXCEPTION, userAuthorized);
            }
			
			///TODO need to clarify with TBI team whether this is a logical error
//			if (!hasAuthorSignedNote(noteIEN, authorDuz, vistaLinkCommand))
//	        {
//				throw new Exception( "Author has not signed note");
//	        }
			
			if (!isValidEsig(esig,vistaLinkCommand))
            {
				throw new RpcException(ErrorCodeEnum.NOTE_INVALID_SIGNATURE_EXCEPTION);
            }
			String signNoteResult = signNote( noteIEN, esig, vistaLinkCommand);

			Text textDatabean = new Text(signNoteResult, currentSiteId);
			List<DataBean> resultList = new ArrayList<DataBean>();
			resultList.add(textDatabean);
			return resultList;
		
	}
	
	/** signs a note by calling the TIU_SIGN_RECORD rpc.
	 * 
	 * @param noteIEN the first parameter in the Rpc call
	 * @param esig the second parameter in the RPC call. gets encrypted before being added as param 
	 * @return "OK" if the rpc response is "0", otherwise returns the rpc error message
	 */
	private String signNote(String noteIEN, String esig,VistaLinkCommand vistaLinkCommand)
    {
		String encryptedEsig = VistaQuery.encrypt(esig);
		
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		params.add(encryptedEsig);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_SIGN_RECORD, params);
		if ( response.equals("0")) return "OK";
		return StringUtil.piece(response,StringUtil.CARET,2);
    }
	
	
	private boolean isValidEsig(String esig,VistaLinkCommand vistaLinkCommand)
    {
		String encryptedEsig = VistaQuery.encrypt(esig);
		List<Object> params =new ArrayList<Object>();
		params.add(encryptedEsig);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWU_VALIDSIG, params);
        return response.equals("1");
    }
	
	
	

	/** returns true if the author has signed this note, otherwise false.
	 * 
	 * @param noteIEN
	 * @param duz
	 * @param vistaLinkCommand
	 * @return
	 */
    @SuppressWarnings("unused")
	private boolean hasAuthorSignedNote(String noteIEN, String duz,VistaLinkCommand vistaLinkCommand)
    {
    	List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		params.add(duz);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_HAS_AUTHOR_SIGNED, params);
    	return response.equals("1");
    }
	
	/**
	 * 
	 * @param noteIEN
	 * @param permission
	 * @param vistaLinkCommand
	 * @return "OK" if the TIU_AUTHORIZATION rpc call returns "1", otherwise returns error message
	 */
	private String isUserAuthorized(String noteIEN, String permission, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		params.add(permission);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_AUTHORIZATION, params);
		if ( response.equals("1")) return "OK";
		return StringUtil.piece(response,StringUtil.CARET,2);
	}

	public String whichSignatureAction(String noteIEN, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		return vistaLinkCommand.executeRpc(RpcNameConstants.TIU_WHICH_SIGNATURE_ACTION, params);
	}


	private boolean checkText(String noteIEN, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWTIU_CHKTXT, params);
		return response.equals("1");
	}

	/** determines if note was saved.
	 * 
	 * @param noteIEN the IEN of the NOTE
	 * @param vistaLinkCommand
	 * @return "OK" if the TIU WAS THIS SAVED? rpc call returns 1,otherwise returns error message
	 */
	public String checkNoteSaved(String noteIEN, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_WAS_THIS_SAVED, params);
		if ( response.equals("1")) return "OK";
		return StringUtil.piece(response, StringUtil.CARET,2);
	}


	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.SIGN_NOTE_DAO;
	}

}
