package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.SignedNotesParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SignedNotesDao extends BaseDao {

	public static final String PROGRESS_NOTES_DOCUMENT_CLASS_PARAMETER="3";
	
	public static final String SIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER="1";
	public static final String UNSIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER="2";
	public static final String UNCOSIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER="3";
	public static final String SIGNED_DOCUMENTS_AUTHOR_CONTEXT_PARAMETER="4";
	public static final String SIGNED_DOCUMENTS_DATERANGE_CONTEXT_PARAMETER="5";
	
	public static final String ASCENDING_OLDEST_FIRST_PARAMETER="A";
	public static final String DESCENDING_NEWEST_FIRST_PARAMETER="D";
	
	public static final String SHOW_ADD_TRUE_PARAMETER="1";
	public static final String SHOW_ADD_FALSE_PARAMETER="0";

	
	
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId, provider, VistaConstants.CPRS_CONTEXT);
		
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		//TODO convert line below to validation
		String fromDate = queryBean.getStartDate();
		String toDate = queryBean.getEndDate();

		Integer maxNumberOfDocuments = queryBean.getMaxRecords() != null ? new Integer(queryBean.getMaxRecords()): null;
		
		
		String progressNotesDocumentClassParameter=PROGRESS_NOTES_DOCUMENT_CLASS_PARAMETER;
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		
		List<Object> params = constructParams(progressNotesDocumentClassParameter,fromDate, toDate, dfn, maxNumberOfDocuments, webMethodName);
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_DOCUMENTS_BY_CONTEXT, params);
		
		//Invoke parser and return databeans
		SignedNotesParser parser = (SignedNotesParser)getDataBeanParser(webMethodName);
		return parser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}

	
	/* 
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.SIGNED_NOTES_DAO;
	}

	
	private List<Object> constructParams(String documentClassParameter, String fromDate, String toDate, String dfn, Integer maxNumberOfDocuments, String webMethod )
	{
		List<Object> params = new ArrayList<Object>();
		
		/// This is a pointer to the Class of TIU DOCUMENTS which the user is
		/// interested in (e.g., 3 for PROGRESS NOTES, 244 for DISCHARGE SUMMARIES, etc.).
		params.add(documentClassParameter);

		
/*		This is an integer number which maps the context in which the user wishes
		 the notes to be returned.  The mapping is as follows:
		  
		 1 - signed documents (all)
		 2 - unsigned documents  
		 3 - uncosigned documents
		 4 - signed documents/author
		 5 - signed documents/date range
*/
		String contextParameterArgument=null;
		if ( fromDate==null || fromDate.trim().equals(""))
		{
			// no date parameters have been defined
			
            fromDate = "-1";
            toDate = "-1";
            if ( webMethod.equals("EmrSvc.getSignedNotes"))
            {
            	contextParameterArgument=SIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER;
            }
            else if ( webMethod.equals("EmrSvc.getCosignedNotes"))
            {
            	contextParameterArgument=UNSIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER;
            }
            else if ( webMethod.equals("EmrSvc.getUncosignedNotes"))
            {
            	contextParameterArgument=UNCOSIGNED_DOCUMENTS_ALL_CONTEXT_PARAMETER;
            }
            else
            {
            	throw new IllegalArgumentException("'webMethod' "+webMethod+" has an unexpected value. Value should be one of: EmrSvc.getSignedNotes,EmrSvc.getCosignedNotes,EmrSvc.getUncosignedNotes");
            }
            
		}
        else
        {
        		/// date parameters have been defined
                 if (toDate==null || toDate.trim().equals(""))
                 {
                     //Changed the call to be -1 and let the Vista Procedure change
                     //it, this way we can test calls with blank/default dates, as the
                     //call signature will be constant (b/w Mock XML Doc and the test 
                     //call)
                     //VistaToolsDao toolsDao = new VistaToolsDao(cxn);
                     //toDate = toolsDao.getTimestamp();
                     toDate = "-1";
                 }
                 
                 /// note that since no context parameter (option) for DATERANGE exists for unsigned and uncosigned documents,
                 /// the DATERANGE cannot be allowed for unsigned and uncosigned documents.  Thus, the original 
                 /// MDWS method signature is incorrect
                 
                 
                 contextParameterArgument=SIGNED_DOCUMENTS_DATERANGE_CONTEXT_PARAMETER;
         }
		params.add(contextParameterArgument);
		
		
		
/*		INPUT PARAMETER: DFN                    PARAMETER TYPE: LITERAL
		  REQUIRED: YES
		 DESCRIPTION:   
		 Pointer to Patient File (#2).
*/
		params.add(dfn);
		
		/// make sure to date has been correctly formatted
		if (toDate==null || toDate.trim().equals(""))
        {
       	 	toDate = "-1";
        };
        
/*        INPUT PARAMETER: EARLY                  PARAMETER TYPE: LITERAL
        REQUIRED: NO
       DESCRIPTION:   
       EARLY DATE/TIME in regular FileMan format for the inclusive lower bound on
       the search.
*/
        if ( fromDate.equals("-1"))
        {
        	params.add(fromDate);
        }
        else
        {
        	params.add(VistaTimestamp.fromUtcFromDate(fromDate));
        }

        
/*        INPUT PARAMETER: LATE                   PARAMETER TYPE: LITERAL
        REQUIRED: NO                          SEQUENCE NUMBER: 5
       DESCRIPTION:   
       LATE DATE/TIME in regular FileMan format for the inclusive upper bound on
       the search.
*/
        
        
        if ("-1".equals(toDate))
        {
       	 	params.add(toDate); 
        }
        else
        {
            params.add(VistaTimestamp.fromUtcToDate(toDate));
        }
        
        /*INPUT PARAMETER: PERSON                 PARAMETER TYPE: LITERAL
        REQUIRED: NO                          SEQUENCE NUMBER: 6
       DESCRIPTION:   
       This is a pointer to the NEW PERSON FILE (#200) for the author of the
       documents to be retrieved.*/
        String personPointer="0";
        params.add(personPointer);
        
//        INPUT PARAMETER: OCCLIM                 PARAMETER TYPE: LITERAL
//        REQUIRED: NO                          SEQUENCE NUMBER: 7
//       DESCRIPTION:   
//       This is the maximum number of documents to be retrieved in the current
//       query.
        if ( maxNumberOfDocuments==null || maxNumberOfDocuments < 1 )
        {
        	maxNumberOfDocuments = new Integer(10000);  /// arbitrarily high number chosen
        }
        params.add(maxNumberOfDocuments.toString());
        
/*        INPUT PARAMETER: SEQUENCE               PARAMETER TYPE: LITERAL
        REQUIRED: NO                          SEQUENCE NUMBER: 8
       DESCRIPTION:   
       This is the sequence by reference date/time (A=ascending (OLDEST FIRST),
       D=descending (NEWEST FIRST)) in which the user would like the list
       returned.
*/
        params.add(DESCENDING_NEWEST_FIRST_PARAMETER);
        
        
/*        INPUT PARAMETER: SHOWADD                PARAMETER TYPE: LITERAL
        REQUIRED: NO                          SEQUENCE NUMBER: 10
       DESCRIPTION:   
       This BOOLEAN parameter determines whether addenda will be included in the
       return array, when their parent documents are identified by the search
       criteria, and vice versa. Also if SHOWADD is TRUE (1), the "+" at the
       beginning of the return text will be omitted (since it offers only
       redundant information).
*/
        params.add(SHOW_ADD_TRUE_PARAMETER);
		
		
		
		
		return params;
	}

	
	

}