package gov.va.med.via.dao;

import gov.va.med.via.databeans.Adt;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.InpatientStay;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class StayMovementOrchestratorDao extends BaseDao {

	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) {
	
		List<DataBean> result = new ArrayList<DataBean>();
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		
		if (ViaWebMethodNameConstants.NUMISVC_GETSTAYMOVEMENTS.compareTo(webMethodName) == 0){
			
			InpatientStay inpatientStay = getStayMovementsByCheckinId(currentSiteId, requestParameterMap, connectionFactoryLocator, null);
			if(inpatientStay != null){
				result.add(inpatientStay);
			}
			
		}
		else if(ViaWebMethodNameConstants.NUMISVC_GETSTAYMOVEMENTSBYDATERANGE.compareTo(webMethodName) == 0){
			Dao dao = getDao(DaoNameConstants.INPATIENT_MOVE_DAO);
			List<DataBean> adtResult = dao.callRpc(currentSiteId, requestParameterMap, connectionFactoryLocator);
			
			for(DataBean adt: adtResult){
				String checkinId = ((Adt)adt).getCheckInId();
				InpatientStay inpatientStay = getStayMovementsByCheckinId(currentSiteId, requestParameterMap, connectionFactoryLocator, checkinId);
				if(inpatientStay != null){
					result.add(inpatientStay);
				}
			}
			
		}
		else if (ViaWebMethodNameConstants.NUMISVC_GETSTAYMOVEMENTSBYPATIENT.compareTo(webMethodName) == 0){
				
			requestParameterMap.remove(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
			Dao dao = getDao(DaoNameConstants.INPATIENT_MOVE_DAO);
			//replace web service method the name that will result in the InpatientMove dao being used first
			requestParameterMap.put(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY, ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVES);
			List<DataBean> adtResult = dao.callRpc(currentSiteId, requestParameterMap, connectionFactoryLocator);
			
			QueryBean qryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			String dfn = getLocalDfn(qryBean.getPatient(), currentSiteId, null);
			
			 Set<String> uniqueCheckinIds = new HashSet<String>();
			 
			 //get a list of uniqueCheckinIds
			 for(DataBean dataBean: adtResult){
				 
				 Adt adt = (Adt)dataBean;
				 String localPid = adt.getPatient().getLocalPid();
				 String checkinId = adt.getCheckInId();
				 if(!uniqueCheckinIds.contains(checkinId) && localPid.equals(dfn)){
					 uniqueCheckinIds.add(checkinId);
				 }
			 }
			 
			 //get the InpatientStays for each checkinId
			 for(String checkinId: uniqueCheckinIds){
				InpatientStay inpatientStay = getStayMovementsByCheckinId(currentSiteId, requestParameterMap, connectionFactoryLocator, checkinId);
				if(inpatientStay != null){
					result.add(inpatientStay);
				}
			}
		}
		
		
		return result;
	}
	
	protected InpatientStay getStayMovementsByCheckinId(String currentSiteId, Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator, String newCheckinId){
		
		//return might be a null object
		InpatientStay inpatientStay = null;
		
		//may need to swap out the itemId with a new one
		if(newCheckinId != null){
			QueryBean qryBean = (QueryBean)requestParameterMap.remove(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			qryBean.setItemId(newCheckinId);
			requestParameterMap.put(ViaRequestMapKeyConstants.QUERY_BEAN_KEY, qryBean);
		}
		
		requestParameterMap.remove(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
		Dao dao = getDao(DaoNameConstants.INPATIENT_MOVE_DAO);
		//replace web service method the name that will result in the InpatientMove dao being used first
		requestParameterMap.put(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY, ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVESBYCHECKINID);
		List<DataBean> adtResult = dao.callRpc(currentSiteId, requestParameterMap, connectionFactoryLocator);
		
		//any and all beans will be for the same patient - just need to first in the list to get a patient's dfn
		
		Adt adtBean = (Adt)adtResult.get(0);
		List<DataBean> patientSelected = null;
		if(adtBean != null){
			QueryBean queryBean = (QueryBean)requestParameterMap.remove(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			String checkinId = queryBean.getItemId();
			Patient patient = adtBean.getPatient();
			//adt does not capture the siteId
			patient.setLocalSiteId(currentSiteId);
			if(patient == null){
				throw new RpcException(ErrorCodeEnum.MISSING_ADT_PATIENT, queryBean.getItemId(), currentSiteId);
			}
			queryBean.setPatient(patient);
			requestParameterMap.put(ViaRequestMapKeyConstants.QUERY_BEAN_KEY, queryBean);
			//replace method name with one for patient select
			requestParameterMap.remove(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);
			requestParameterMap.put(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY, ViaWebMethodNameConstants.EMRSVC_CPRSLAUNCH_PATIENTSELECT);
			
			dao = getDao(DaoNameConstants.PATIENT_SELECT_DAO);
			patientSelected = dao.callRpc(currentSiteId, requestParameterMap, connectionFactoryLocator);
			Patient patientSelect = (Patient)patientSelected.get(0);
			
			//now we can construct our InpatientStay
			inpatientStay = new InpatientStay();
			inpatientStay.setMovementCheckinId(checkinId);
			inpatientStay.setAdts(adtResult);
			inpatientStay.setPatient(patientSelect);
				
		}
		
		return inpatientStay;
	}

	@Override
	public String getDaoName() {
		
		return DaoNameConstants.STAY_MOVEMENT_ORHESTRATOR_DAO;
	}

}
