package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class UserScListerDao extends BaseDao {

	/**
	 * Retrieves the User information from VistA
	 * FILE 200 - result position - filed number and field definitions
	 * POSITION								FIELD NUMBER
	 * 0:IEN								.01 (NOTE: field .01 seems to return IEN and NAME in two fields)
	 * 1:NAME								.01
	 * 2:OFFICE PHONE						.132
	 * 3:VOICE PAGER						.137
	 * 4:ROOM								.141
	 * 5:FILE MANAGER ACCESS CODE			3
	 * 6:SEX								4
	 * 7:DOB								5
	 * 8:DIUSER								7
	 * 9:TITLE								8E 
	 * 10:SSN								9
	 * 11:NICKNAME							13
	 * 12:SIGNATURE BLOCK PRINTED NAME		20.2
	 * 13:SIGNATURE BLOCK TITLE				20.3
	 * 14:SERVICE/SECTION					29
	 * 15:									29E
	 * 16:PROVIDER CLASS					53.5	POINTER
	 * 17:									53.5E	PROVIDER CLASS
	 * 18:PROVIDER TYPE						53.6
	 * 19:PRIMARY MENU OPTION				201 	POINTER
	 * 20:									201E 	PRIMARY MENU OPTION
	 * 
	 * Example of one result line:	"1497^MACEDONIO,ROSIE M^^^^^F^2541208^^STAFF NURSE^573055356^ROSIE^ROSIE M MACEDONIO^RN^57^NURSING SERVICE-AMB CARE^11^REGISTERED NURSE^^2791^NURS-RN"
	 */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) {
		
        QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
        String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);          
        
        VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
        
        String maxRex = queryBean.getMaxRecords();  
        String target = queryBean.getTarget();
        target = target.toUpperCase();
        
        List<Object> params =new ArrayList<Object>();          

        HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
        hashMap.put("FILE",200);
        hashMap.put("FIELDS","@;.01;.132;.137;.141;3;4;5;7;8E;9;13;20.2;20.3;29;29E;53.5;53.5E;53.6;201;201E");
        hashMap.put("FLAGS","IP");
        hashMap.put("MAX",maxRex);
        hashMap.put("FROM",RPCUtils.adjustForNameSearch(target));
        hashMap.put("PART",target);
        hashMap.put("XREF","B");
        
        params.add(hashMap);
        
        String result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
                    
        return parseResponseToDataBeans(result, queryBean, null, currentSiteId, webMethodName, vistaLinkCommand);

	}

	@Override
	public String getDaoName() {
		return DaoNameConstants.USER_SCLISTER_DAO;
	}
	
}
