package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.VisitsDataBeanParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This class is used to manage access and invoke the RPC to get patient visits. 
 * It also helps parse the response received from RPC
 * @author DOMAINALLURS
 *
 */
public class VisitsDao extends BaseDao {

    /**
     * This method is used to invoke the RPC ORWCV VST to get patient visits and parse its response
     * 
     * @param currentSiteId - the current site id
     * @param requestParameterMap - the hashmap that holds all input data needed
     * @param connectionFactoryLocator - the connection factory
     * @return PatientArray -a patient array is returned containng a list of Patients found in MPI if successful.
     */
	@Override
	@SuppressWarnings("unchecked")
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {		

		// Get data from Query Bean
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
		Provider provider = queryBean.getProvider();

		//Create VistaLink Command
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator,currentSiteId,  provider, VistaConstants.CPRS_CONTEXT);
			
		//Get dfn
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		String argFromDate = Timestamp.fromUtcToDate(queryBean.getStartDate());
		
		//endDate is optional
		String argToDate = queryBean.getEndDate();
		if ( argToDate== null || argToDate.isEmpty()){
			argToDate = "";
		}else{
			argToDate = Timestamp.fromUtcToDate(argToDate);
		}
		//Execute RPC
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWCV_VST, dfn, argFromDate, argToDate, "1");
		
		//Invoke parser and return databeans
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY); 		
		VisitsDataBeanParser dataBeanParser = (VisitsDataBeanParser)getDataBeanParser(webMethodName);
		return dataBeanParser.populateDataBeans(result, queryBean, dfn, currentSiteId, webMethodName,vistaLinkCommand);
	}


	/* 
	 * Returns the Visits DAO name
	 * @return - the DAO name
	 */
	@Override
	public String getDaoName() {
		return DaoNameConstants.VISITS_DAO;
	}


}
