package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCException;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.rpc.VistaQuery;
import gov.va.med.via.rpc.client.RPCClient;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.VistaLinkTemplate;
import gov.va.med.via.vistalink.command.VistaLinkAppProxyCommandImpl;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class VistaAccountDao extends BaseDao
{


	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) {

		String context = VistaConstants.CPRS_CONTEXT;
		String accessCode = (String)requestParameterMap.get(ViaRequestMapKeyConstants.LOGIN_ACCESS_CODE_KEY); 
		String verifyCode = (String)requestParameterMap.get(ViaRequestMapKeyConstants.LOGIN_VERIFY_CODE_KEY); 
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String siteCode = queryBean.getProvider().getLoginSiteCode();
		Provider provider = loginAndGetProviderWithRPCSockets(context, accessCode,verifyCode, siteCode);
		
		if (!provider.hasErrors() && !RPCStringUtils.isNullOrEmpty(provider.getProviderName()))
		{
			// Retrieve provider's SSN
			List<Object> params =new ArrayList<Object>();          
	        HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			hashMap.put("FILE",200);
			hashMap.put("FIELDS",".01;9");
			hashMap.put("FLAGS","IP");
			hashMap.put("MAX",1);
			hashMap.put("FROM",RPCUtils.adjustForNameSearch(provider.getProviderName()));
		    hashMap.put("PART",provider.getProviderName());
			hashMap.put("XREF","B");	        
	        params.add(hashMap);	  	
	
	        VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, provider, VistaConstants.CPRS_CONTEXT);
	        String result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	        
	        result = RPCStringUtils.replaceEndofLineCharacters(result);  
		    String[] sclines = RPCStringUtils.split(result, RPCStringUtils.CRLF);	        
	        for (int j = 0; j < sclines.length; j++)
	        {    
		    	if (sclines[j].contains("[Data]") || RPCStringUtils.isEmpty(sclines[j])) continue;
		    	
	        	String[] fields = RPCStringUtils.split(sclines[j], RPCStringUtils.CARET);
	        	//ensure the record contains the IEN (numeric value) and that the IEN is the same as our patient's PID (dfn)
	    	   if (fields != null && fields.length > 1 && provider.getUserId().equals(fields[0]))
	    	   {
	    		   provider.setSsn(fields[2]);
	    		   break;
	    	   }
	        } 
		}
        
		return Arrays.asList(new DataBean[]{provider}); 

	}

	@Override
	/**
	 * Determines the type (DUZ of APP PROXY) of connectionSpec and creates a Command object for that type that will delegate calls of that type to the VistaLinkTemplate 
	 * 
	 * @param connectionFactoryLocator
	 * @param targetSiteCode
	 * @param provider
	 * @param rpcContext
	 * @return VistaLinkCommand object for the type of ConnectionSpec
	 */
	protected VistaLinkCommand createVistaLinkCommand(ConnectionFactoryLocator connectionFactoryLocator, String targetSiteCode, Provider provider, String rpcContext) {
		//int connectionSpecType = VistaLinkTemplate.USER_TYPE_APPLICATION_PROXY;
		VistaLinkTemplate vistaLinkTemplate = new VistaLinkTemplate(connectionFactoryLocator);
		VistaLinkCommand vistaLinkCommand = new VistaLinkAppProxyCommandImpl(targetSiteCode, rpcContext, APP_PROXY_NAME, vistaLinkTemplate);
		return vistaLinkCommand;
	}




	/** logs in a provider using JMeadows/MDWS-style RPC Sockets.  This RPC Socket approach is necessary to overcome a security limitation of VistALink.  
	 * 
	 * @param context
	 * @param accessCode
	 * @param verifyCode
	 * @param siteCode
	 * @return Provider with DUZ, SSN, Name, login Site Id, login site name, etc.; LoginException (DataBean) if Exception occurs
	 *  returns LoginException if DUZ cannot be determined, other information cannot be determined, or 
	 *  RPC connections cannot be established
	 */
	protected Provider loginAndGetProviderWithRPCSockets(String context,
			String accessCode, String verifyCode, String siteCode) {
		/// The RPCClient constructor below creates a raw RPC connection to the specified site code, 
		/// authenticates the user, parses the DUZ, then disconnects.  This will minimize the number 
		/// of calls using raw RPC and also minimize the time spent using the underlying connection.
		Provider provider =null;
		RPCClient client = null;
		try
		{
			/// the RPC Client establishes a connection which can then be reused
			boolean immediatelyDisconnect = false;
			client = new RPCClient( accessCode, verifyCode, context, siteCode, immediatelyDisconnect);

			String duz= client.getDuz();
			if ( duz==null ) throw new RpcException(ErrorCodeEnum.LOGIN_UNABLE_TO_DETERMINE_PROVIDER_DUZ);
			String greeting = client.getGreeting();
			
			
			/// now that the DUZ has been determined, the following calls utilize VistALink  
			provider = new Provider();
			provider.setUserId(duz);
			provider.setPermissionString(context);
			provider.setLoginSiteCode(siteCode);
			provider.setGreeting(greeting);
			
			///ORWU USERINFO DESCRIPTION from Roll-And-Scroll:
			/// 1>Returns preferences for the current user.
			VistaQuery vq = new VistaQuery(RpcNameConstants.ORWU_USERINFO);
			String rtn = (String) client.getRpcConnection().query(vq.buildMessage());
			if (rtn == null)
			{
				throw new RpcException(ErrorCodeEnum.LOGIN_UNABLE_TO_GET_USER_INFO);
			}
			String[] parts = StringUtil.split(rtn, StringUtil.CARET);
			//provider.setUserId(parts[0]); <-- user DUZ is userID, set above

			//	        ArrayList<KeyValuePair> siteUids = new ArrayList<KeyValuePair>();
			//	        siteUids.add(new KeyValuePair((String) cxn.getSiteId().getKey(), parts[0]));
			//	        provider.setSiteUserIds(siteUids.toArray(new KeyValuePair[0]));
			provider.setProviderName(parts[1]);
			if ( parts.length>=13)
			{
				String loginSiteName = parts[12];
				if ( loginSiteName!=null && !loginSiteName.trim().equals(""))
				{
					provider.setLoginSiteName(loginSiteName);
				}
			}
			if (StringUtil.isEmpty(provider.getUserId()))
			{
				throw new RpcException(ErrorCodeEnum.LOGIN_NO_USER_ID_FOR_SITE);
			}
			
			/** CANNOT USE GET VARIABLE VALUE - KERNEL CALL THAT IS NOT ALLOWED ***
			String arg = "@\"^VA(200," + provider.getUserId() + ",1)\"";
			rtn = RPCUtils.getVariableValue(client.getRpcConnection(), arg);
			if (rtn.equals(""))
			{
				throw new RpcException(ErrorCodeEnum.LOGIN_UNABLE_TO_GET_USER_SSN);
			}
			parts = StringUtil.split(rtn, StringUtil.CARET);

			if (parts != null && parts.length > 7)
				provider.setSsn(parts[8]);
			**/
						
			client.getRpcConnection().setUid(provider.getUserId());
			return provider;
		}
		catch (RPCException e)  /// note that this Exception is not an RpcException (uppercase vs. lowercase)
		{
			/*
			when invalid access/verify codes are entered, then the vista response is similar to 
			"sign-in returned '0' value for DUZ; signin was not OK"
			*/
			if ( e.getMessage().toLowerCase().indexOf("signin was not ok")!=-1)
			{
				Provider p = new Provider();
				p.setFault(new Fault("Not a valid ACCESS CODE/VERIFY CODE pair."));
				return p;
			}			
			throw new RuntimeException(e);
		}
		finally
		{
			try 
			{
				if ( client!=null ) client.disconnect();
			} 
			catch (Exception ignore) 
			{
			}			
		}

	}

	@Override
	public String getDaoName() 
	{
		return getClass().getSimpleName();
	}

}
