package gov.va.med.via.dao;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Encounter;
import gov.va.med.via.databeans.Note;
import gov.va.med.via.databeans.NoteResult;
import gov.va.med.via.databeans.PatientRecordFlag;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.PatientRecordFlagParser;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.util.VistaUtils;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcRequest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class WriteNoteDao extends BaseDao 
{
	public static final int MAX_CHARS_PER_LINE = 80;
	public static final int MAX_LINES_PER_PAGE =300;


	@Override
	public List<DataBean> callRpc(String currentSiteId,
			Map<String, Object> requestParameterMap,
			ConnectionFactoryLocator connectionFactoryLocator) 
			{

			QueryBean queryBean = (QueryBean) requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
			String titleIEN = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_DEFINITION_IEN);
			Encounter encounter = (Encounter)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_ENCOUNTER);
			String text = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_TEXT);
			String authorDuz = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_AUTHOR_DUZ);
			String cosignerDUZ = (String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_COSIGNER_DUZ);
			String consultIEN=(String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_CONSULT_IEN );
			String prfIEN=(String)requestParameterMap.get(ViaRequestMapKeyConstants.NOTE_PRF_IEN);
			String dfn = queryBean.getPatient().getLocalPid();

			Note note = new Note();
			note.setDocumentDefinitionId(titleIEN);
			note.setAuthor(new Author(authorDuz,"",""));
			String timestamp = getTimestampFromVista(currentSiteId,	connectionFactoryLocator, queryBean);
			note.setTimestamp(timestamp);
			note.setConsultId(consultIEN);
			note.setAddendum(false);
			note.setPrfId(prfIEN);
			if ( cosignerDUZ!=null && !cosignerDUZ.trim().equals(""))
			{
				note.setCosigner(new Author(cosignerDUZ,"",""));
			}
			note.setText(text);

			/// create a single VistALink command to execute all the RPCs against
			VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);

			/// ok to create note
			validateOkToCreateNote(note, queryBean.getPatient().getLocalPid(), encounter, queryBean, vistaLinkCommand);

			if ( !StringUtil.isNullOrEmpty(consultIEN))
			{
				String orderIEN = getOrderNumberForConsult(consultIEN, vistaLinkCommand);
				if ( orderIEN.equals(""))
				{
					// Do nothing - CPRS assumes it's an Interfacility Consult
				}
				else
				{
					lockOrder(orderIEN,vistaLinkCommand);
				}
			}
			String noteIEN = createNote(queryBean.getPatient().getLocalPid(), encounter, note, vistaLinkCommand);
			if (noteIEN.startsWith("0^"))
			{
				String reason = StringUtil.piece(noteIEN,StringUtil.CARET,2);
				//TODO change exception type
				throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_CREATE_NOTE, reason);
			}

			String lockNoteResponse = lockNote( noteIEN, vistaLinkCommand);
			if ( !lockNoteResponse.equals("OK"))
			{
				//TODO change exception type
				throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_LOCK_NOTE, lockNoteResponse);
			}
			note.setId(noteIEN);

			String updateNoteResponse = updateNote(dfn,note,vistaLinkCommand);
			if ( !updateNoteResponse.equals(noteIEN))
			{
				//TODO change exception type
				throw new RpcException(ErrorCodeEnum.NOTE_ERROR_UPDATING_NOTE, noteIEN, updateNoteResponse);
			}
			NoteResult result = null;
			try
			{
				result = setNoteText(noteIEN,text,false,vistaLinkCommand);
			}
			catch ( java.lang.IndexOutOfBoundsException ie)
			{
				String msg = "setNoteText: " + ie.getMessage() + "\r\n" + ie.getStackTrace();
				throw new RpcException(ErrorCodeEnum.NOTE_INDEX_OUT_OF_BOUNDS,msg);
			}

			if (!StringUtil.isNullOrEmpty(prfIEN))
			{
				//Handle PRF
				PatientRecordFlag[] flags = getPrfNoteActions(noteIEN,dfn, vistaLinkCommand);
				PatientRecordFlag patientRecordFlag = getFlag(flags, prfIEN);
				if (patientRecordFlag == null)
				{
					throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_LINK_PRF_NOTE, noteIEN,prfIEN);
				}

				String s = linkNoteToFlag(dfn,noteIEN, patientRecordFlag, vistaLinkCommand);
				if (s != "OK")
				{
					throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_LINK_PRF_NOTE_REASON,noteIEN, prfIEN,s);
				}
			}
			
			/// unlock the note since it is now locked... this approach is not found in MDWS, however
			/// the stateless approach we are using would suggest that we should not leave records in locked states
			/// Leaving a note in a locked state makes it not possible to perform other activities on the note
			String unlockNoteResponse = unlockNote( noteIEN, vistaLinkCommand);
			if ( !unlockNoteResponse.equals("OK"))
			{
				throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_UNLOCK_NOTE, unlockNoteResponse);
			}
			
			List<DataBean> resultList = new ArrayList<DataBean>();
			resultList.add(result);
			return resultList;
	}

	private String linkNoteToFlag(String dfn, String noteIEN, PatientRecordFlag flag, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();

		params.add(noteIEN);
		params.add(flag.getId());
		params.add(flag.getActionId());
		params.add(dfn);

		///TODO identify the RPC name - perhaps the code have never been active because the PRF IEN is always null or empty
		String response = vistaLinkCommand.executeRpc("", params);
		return ("1".equals(response) ? "OK" : StringUtil.piece(response,StringUtil.CARET,2));

	}


	private PatientRecordFlag getFlag(PatientRecordFlag[] flags, String prfIEN)
	{
		for ( PatientRecordFlag flag: flags)
		{
			if ( flag.getId().equals(prfIEN))
			{
				return flag;
			}
		}
		return null;
	}

	private PatientRecordFlag[] getPrfNoteActions(String noteIEN,String dfn, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		params.add(dfn);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_GET_PRF_ACTIONS, params);
		/// the parser below is not injected, but directly instantiated since the note's IEN must be set dynamically
		/// At a minimum, the parsing logic is being kept in one place
		PatientRecordFlagParser parser = new PatientRecordFlagParser();
		parser.setNoteIEN(noteIEN);
		return parser.toPatientRecordFlags(response);
	}


	class Page 
	{
		int pageNumber;
		List<String> pageLines= new ArrayList<String>();
		
		public void addLine(String line)
		{
			pageLines.add(line);
		}
		
	}
	
	private NoteResult setNoteText(String noteIEN, String text, boolean fSuppressCommit, VistaLinkCommand vistaLinkCommand)
	{
		ArrayList<String> lineList = extractLinesFromNoteText(text);
		List<Page> pages = extractPages(lineList);
		int numPages = pages.size();
		for ( Page page : pages)
		{
			//String noteIEN, boolean suppress,Page page, int numPages, VistaLinkCommand vistaLinkCommand
			String writeTextResponse = writeText( noteIEN, fSuppressCommit,page,numPages, vistaLinkCommand);
			writeTextResponse = writeTextResultOK(writeTextResponse, noteIEN, page.pageNumber, numPages);
			if (writeTextResponse != "OK")
			{
				throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_SET_NOTE_TEXT,writeTextResponse);
			}
		}
		return new NoteResult(noteIEN, numPages, numPages);
	}

	private List<Page> extractPages(ArrayList<String> lineList) 
	{
		List<Page> pages = new ArrayList<Page>();
		int pageNumber=1;
		
		/// initialize the first page
		Page page=new Page();
		page.pageNumber=pageNumber;
		for ( String line: lineList)
		{
			if ( page.pageLines.size()== MAX_LINES_PER_PAGE)
			{
				/// add the existing page
				pages.add(page);
				// setup the next page
				pageNumber++;
				page=new Page();
				page.pageNumber=pageNumber;
			}
			page.addLine(line);
		}
		/// make sure the last page gets added if a line of text is present in it
		if ( page.pageLines.size()>0)
		{
			pages.add(page);
		}
		return pages;		
	}

	/** extracts the lines from Note Text.  This method first separates lines by the '|' (pipe) character.  Then, the lines
	 *  are extracted to ensure that each line is maximum 80 characters long.  
	 * 
	 * @param text the complete note text
	 * @return
	 */
	private ArrayList<String> extractLinesFromNoteText(String text) {
		ArrayList<String> lineList = new ArrayList<String>();
		/// the text argument gets split up into a number of lines separated by the "pipe" character
		String[] lines = StringUtil.split(text, "|");
		/// make sure that each line has maximum 80 characters
		for (String line : lines)
		{
			line = StringUtil.trimEnd(line); /// remove the blank space at the end of each line
			if ( line.equals(""))
			{
				lineList.add(line);
				continue;
			}
			while (line.length() > MAX_CHARS_PER_LINE)
			{
				int index = StringUtil.getFirstWhiteSpacePreceding(line, MAX_CHARS_PER_LINE);
				if (index == -1)
				{
					index = MAX_CHARS_PER_LINE;
				}
				lineList.add(line.substring(0, index));

				while (index < line.length() && StringUtil.isWhiteSpace(line.charAt(index)))
				{
					index++;
				}
				line = line.substring(index);
			}
			if (line.length() > 0)
			{
				lineList.add(line);
			}
		}
		return lineList;
	}

	private String writeTextResultOK(String s, String noteIEN, int lastPageSent, int totalPages)
	{
		try
		{
			String[] flds = StringUtil.split(s, StringUtil.CARET);

			if (flds[0].equals("0"))
			{
				return flds[3];
			}
			if (!flds[0].equals(noteIEN))
			{
				return "IEN mismatch";
			}
			if (Integer.parseInt(flds[1])!=(lastPageSent))
			{
				return "Last page received not last page sent";
			}
			if (Integer.parseInt(flds[2]) != totalPages)
			{
				return "Total pages mismatch";
			}
		}
		catch (Exception ex)
		{
			return ex.getMessage();
		}
		return "OK";
	}

	/**
	 * 
	 * @param rpcName should have value TIU SET DOCUMENT TEXT 
	 * @param noteIEN This is the IEN of the TIU Document in the TIU DOCUMENT file (#8925).
	 * @param argList This is the array in which the body of the TIU Document is being passed,<br/>as:<br/> 
	 *
	 *           TIUX("HDR")=<# of Current Page>^	<br/>
	 *           TIUX("TEXT",1,0)=					<br/>
	 *           TIUX("TEXT",2,0)=					<br/>
	 *           TIUX("TEXT",3,0)=					<br/>
	 *                   .                       .	<br/>
	 *                   .                       .	<br/>
	 *                   .                       .	<br/>
	 *           TIUX("TEXT",i,0)= 					<br/>
	 * 
	 * 
	 * @param fSuppressComm Boolean flag that indicates whether to commit the data (as in the Save w/o signature action in CPRS), or to simply save it in the EDIT BUFFER for the document (as in the auto-save function).
	 * @param vistaLinkCommand
	 * @return
	 */
	private String writeText( String noteIEN, boolean suppress,Page page, int numPages, VistaLinkCommand vistaLinkCommand)
	{
		
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		
		HashMap<String,String> hm = new HashMap<String,String>();
		hm.put("HDR",page.pageNumber +"^"+numPages);
		int lineNumber=1;  /// the line number starts with one
		for ( String line: page.pageLines)
		{
			String key = RpcRequest.buildMultipleMSubscriptKey("\"TEXT\","+lineNumber++ +",0");
			String value = StringUtil.filteredString(line);
			hm.put(key, value);//,"some framework-friendly experimental note text");
		}
		/// from vistalink 1.6 developer guide page 4-7
		params.add(hm);
		
		String suppressLiteralParam = suppress?"1":"0"; 
		params.add(suppressLiteralParam);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_SET_DOCUMENT_TEXT, params);
		return response;
	}
	
	private String updateNote(String dfn, Note note, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(note.getId());

		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put(".01", note.getDocumentDefinitionId());
		hashMap.put("1202", note.getAuthor().getId());
		if (note.getCosigner() != null)
		{
			hashMap.put("1208", note.getCosigner().getId());
		}
		if (!StringUtil.isEmpty(note.getConsultId()))
		{
			hashMap.put("1405", note.getConsultId()+ ";GMR(123,");
		}
		hashMap.put("1301", note.getTimestamp());

		String subject = (note.getSubject() == null) ? "" : note.getSubject();
		if (subject.length() > 80)
		{
			subject = subject.substring(0, 80);
		}
		hashMap.put("1701", subject);
		if (!StringUtil.isEmpty(note.getParentId()))
		{
			hashMap.put("2101", note.getParentId());
		}
		if (!StringUtil.isEmpty(note.getProcId()))
		{
			hashMap.put("70201", note.getProcId());
		}
		if (!StringUtil.isEmpty(note.getProcTimestamp()))
		{
			hashMap.put("70202", VistaTimestamp.fromUtcString(note.getProcTimestamp()));
		}
		params.add(hashMap);

		String updateNoteResponse = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_UPDATE_RECORD, params);
		return updateNoteResponse;
	}

	private String lockNote(String noteIEN, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_LOCK_RECORD, params);
		return RPCUtils.errMsgOrZero(response);
	}

	private String unlockNote(String noteIEN, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(noteIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_UNLOCK_RECORD, params);
		return RPCUtils.errMsgOrZero(response);
	}

	
	
	/**
	 * 
	 * @param dfn
	 * @param encounter
	 * @param note
	 * @param vistaLinkCommand
	 * @return the note's IEN
	 */
	private String createNote(String dfn, Encounter encounter, Note note,VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(dfn);
		params.add(note.getDocumentDefinitionId());

		//VistA documentation says it will accept an encounter timestamp here, but CPRS hard codes a blank
		params.add("");

		//VistA documentation says encounter location IEN next, but CPRS hard codes a blank
		params.add("");

		//VistA documentation says encounter IEN next, but CPRS hard codes a blank
		params.add("");   

		//String vistaTS = null;//VistaTimestamp.Now;
		HashMap<Object,Object> lst = new HashMap<Object,Object>();
		lst.put("1202", note.getAuthor().getId());
		lst.put("1301", note.getTimestamp());
		lst.put("1205", encounter.getLocationId());
		if (note.getCosigner() != null)
		{
			lst.put("1208", note.getCosigner().getId());
		}
		if (note.getConsultId()!=null && !"".equals(note.getConsultId()))
		{
			lst.put("1405", note.getConsultId() + ";GMR(123,");
		}
		String subject = (note.getSubject() == null) ? "" : note.getSubject();
		if (subject.length() > 80)
		{
			subject = subject.substring(0,80);
		}
		lst.put("1701", subject);
		if (!StringUtil.isEmpty(note.getParentId()))
		{
			lst.put("2101", note.getParentId());
		}
		params.add(lst);

		String timestamp = encounter.getTimestamp();
		timestamp = VistaTimestamp.trimZeroes(timestamp);
		String visitStr = encounter.getLocationId() + ';' + timestamp + ';' + encounter.getType();
		params.add(visitStr);

		//Vista documentation says it accepts 1 or 0, but 
		//CPRS hard codes a 1 to suppress commit logic
		params.add("1");

		//Vista documentation adds this param, but CPRS omits it
		//if (noAsave)
		//{
		//    vq.addParameter(vq.LITERAL, "1");
		//}

		String createNoteResponse = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_CREATE_RECORD, params);
		return createNoteResponse;
	}

	private void lockOrder(String orderIEN, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(orderIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_LOCK_ORDER, params);
		String[] flds = response.split("\\^");
		/// the first part of the response indicates success of failure
		/// if the order result ==1, then the order has successfully been locked
		/// otherwise, the order has not been locked
		if ( flds[0].equals("1"))
		{
			/// order successfully locked
			return;
		}
		/// the 2nd part of the caret separated return string will contain some type of error message
		throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_LOCK_ORDER, flds[1]);
	}

	private String getOrderNumberForConsult(String consultIEN, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(consultIEN);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORQQCN_GET_ORDER_NUMBER, params);
		return response;
	}



	private void validateOkToCreateNote(Note note, String localPid,Encounter encounter,QueryBean queryBean, VistaLinkCommand vistaLinkCommand) 
	{
		if ( isSurgeryNote(note.getDocumentDefinitionId(),vistaLinkCommand))
		{
			throw new RpcException( ErrorCodeEnum.NOTE_CANNOT_CREATE_SURGERY_NOTE);			
		}

		String visitString = VistaUtils.getVisitString(encounter);
		if ( isOneVisitNote(note.getDocumentDefinitionId(), localPid, visitString ,vistaLinkCommand))
		{
			throw new RpcException(ErrorCodeEnum.NOTE_ONE_PER_VISIT, note.getLocalTitle());
		}

		String consultId = note.getConsultId();
		if ( isConsultNote( note.getDocumentDefinitionId(),vistaLinkCommand) && (consultId==null || consultId.equals("")))
		{
			throw new RpcException( ErrorCodeEnum.NOTE_MISSING_CONSULT_IEN);
		}

		if ( !note.isAddendum())
		{
			if (  isPrfNote(note.getDocumentDefinitionId(), vistaLinkCommand)&& StringUtil.isNullOrEmpty(note.getPrfId()))
			{
				throw new RpcException( ErrorCodeEnum.NOTE_MISSING_PRF_IEN);				
			}

			boolean cosignerRequired = false;

			// if (note.Author.Id == cxn.Uid)  ///asking whether the signed in provider is the authoer
			String authorId = note.getAuthor().getId();
			String signedInProviderId = queryBean.getProvider().getUserId();
			if ( authorId.equals(signedInProviderId))
			{
				cosignerRequired = isCosignerRequired(signedInProviderId, note.getDocumentDefinitionId(), authorId, vistaLinkCommand);
			}
			else
			{
				cosignerRequired = isCosignerRequired(signedInProviderId, note.getDocumentDefinitionId(), authorId, vistaLinkCommand);
			}
			if (cosignerRequired && (note.getCosigner() == null || StringUtil.isNullOrEmpty(note.getCosigner().getId())))
			{
				throw new RpcException( ErrorCodeEnum.NOTE_MISSING_COSIGNER);
			}			
		}
		/// success - no exception thrown
	}

	/**  Checks to see if this user needs a cosigner for this note.
	 * 
	 * @param userDuz
	 * @param noteDefinitionIen
	 * @param authorDuz
	 * @return
	 */
	public boolean isCosignerRequired(String userDuz, String noteDefinitionIen, String authorDuz , VistaLinkCommand vistaLinkCommand)
	{
		if (userRequiresCosignature(userDuz,vistaLinkCommand))
		{
			return true;
		}
		if (userRequiresCosignatureForNote(userDuz, noteDefinitionIen,vistaLinkCommand))
		{
			return true;
		}
		if (StringUtil.isNullOrEmpty(authorDuz))
		{
			return false;
		}
		if (userRequiresCosignature(authorDuz,vistaLinkCommand))
		{
			return true;
		}
		if (userRequiresCosignatureForNote(authorDuz, noteDefinitionIen,vistaLinkCommand))
		{
			return true;
		}

		return false;
	}



	private boolean userRequiresCosignature(String duz, VistaLinkCommand vistaLinkCommand)
	{
		/// the "3" value is hardcoded in the MDWS 2.5 code (VistaNoteDao:userRequiresCosignature-Line 992)
		/// one can assume that the "3" is a value that covers any note ???
		return userRequiresCosignatureForNote(duz, "3", vistaLinkCommand); 
	}


	private boolean userRequiresCosignatureForNote(String duz, String noteIEN,	VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add( noteIEN );
		params.add( "0" );
		params.add( duz );  /// check for unusual behavior with this call (using duz with call and also adding duz as parameter)

		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_REQUIRES_COSIGNATURE,params);	
		if ( response!=null ) response=response.trim();
		return "1".equals(response);
	}

	private boolean isPrfNote(String ien,	VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(ien);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_ISPRF,params);	
		if ( response!=null ) response=response.trim();
		return "1".equals(response);
	}

	private boolean isConsultNote(String ien, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(ien);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_IS_THIS_A_CONSULT,params);	
		if ( response!=null ) response=response.trim();
		return "1".equals(response);
	}

	private boolean isOneVisitNote(String documentDefinitionId, String dfn, String visitStr,VistaLinkCommand vistaLinkCommand) 
	{
		// validation already performed, no need for redundant checks 
		// VistaUtils.CheckRpcParams(docDefId); <-- validation already performed, no need for redundant check
		// VistaUtils.CheckRpcParams(dfn);
		// VistaUtils.checkVisitString(visitStr);

		List<Object> params =new ArrayList<Object>();
		params.add(documentDefinitionId);
		params.add(dfn);
		params.add(visitStr);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_ONE_VISIT_NOTE, params);
		if ( response!=null ) response=response.trim();
		return "1".equals(response);  /// if response is null, then no null pointer exception thrown
	}

	private boolean isSurgeryNote(String ien, VistaLinkCommand vistaLinkCommand) 
	{
		List<Object> params =new ArrayList<Object>();
		params.add(ien);
		String isSurgeryNoteResponse = vistaLinkCommand.executeRpc(RpcNameConstants.TIU_IS_THIS_A_SURGERY,params);	
		return (isSurgeryNoteResponse=="1");
	}

	private String getTimestampFromVista(String currentSiteId,	ConnectionFactoryLocator connectionFactoryLocator,		QueryBean queryBean) 
	{
		/// get timestamp via RPC call to vista system
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		List<Object> params =new ArrayList<Object>();
		params.add("NOW");
		String vistaTimestampResponse = vistaLinkCommand.executeRpc(RpcNameConstants.ORWU_DT,params);
		return vistaTimestampResponse;
//		String timestamp = VistaTimestamp.fromUtcString(vistaTimestampResponse);
//		return timestamp;
	}

	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.WRITE_NOTE_DAO;
	}

}
