package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.OrderDialogName;
import gov.va.med.via.databeans.OrderResponse;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.OrderDialog;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcRequest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The Class WriteSimpleOrderByPolicyDao.
 * 
 * @author DOMAINROMERJ
 */
public class WriteSimpleOrderByPolicyDao extends BaseDao 
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#callRpc(java.lang.String, java.util.Map, gov.va.med.via.vistalink.ConnectionFactoryLocator)
	 */
	@Override
	public List<DataBean> callRpc(String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);		
				
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		OrderDialog dialog = getOrderDialog(queryBean, vistaLinkCommand); 
		dialog.setResponses(getOrderResponses(dialog.getResponseId(), vistaLinkCommand));		
		OrderDialogName dialogName = getOrderDialogName(queryBean.getItemId(), vistaLinkCommand); //orderIEN = queryBean.getItemId()
		
		List<Object> params =new ArrayList<Object>();
		params.add(dfn);									// Patient ID
		params.add(queryBean.getProvider().getUserId());	// Provider DUZ
		params.add(queryBean.getCriteria());				// Location IEN
		params.add(dialogName.getBaseName());
		params.add(dialog.getDisplayGrp());
		params.add(queryBean.getItemId());					// Order Dialog IEN
		params.add("");

		HashMap<String,String> hashMap = new HashMap<String,String>();

		OrderResponse[] orderResponses = dialog.getResponses();
		if (orderResponses != null)
		{
			for (int i =0; i < orderResponses.length; i++)
			{
				String key = RpcRequest.buildMultipleMSubscriptKey(orderResponses[i].getPromptId() + ',' + orderResponses[i].getInstance());
				if ("6".equals(orderResponses[i].getPromptId()))
				{
					hashMap.put(key, Timestamp.fromUtcFromDate(queryBean.getStartDate()));
					
				}
				else
				{
					hashMap.put(key, orderResponses[i].getIvalue());				
				}
			}		
		}
		
		hashMap.put("\"ORCHECK\"", "0");				
		hashMap.put("\"ORTS\"", "0"); 
		params.add(hashMap);		
		params.add("");
		params.add("");
		params.add("");
		params.add("0");
		String response = vistaLinkCommand.executeRpc("ORWDX SAVE", params); 

		return parseResponseToDataBeans(response, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}


	/**
	 * Gets the order dialog.
	 *
	 * @param queryBean the query bean
	 * @param vistaLinkCommand the vista link command
	 * @return the order dialog
	 */
	private OrderDialog getOrderDialog(QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
	{
		// Call RPC with: orderIen and information string
		String information =  queryBean.getPatient().getLocalPid() + '^' + queryBean.getCriteria() + '^' + 
				queryBean.getProvider().getUserId() + '^' +
                (queryBean.getPatient().isInpatient() ? "1" : "0") + '^' + queryBean.getPatient().getGender() + '^' + 
                queryBean.getPatient().getAge() + '^' + "0;C;0;0" + '^' + queryBean.getPatient().getScPercent() + "^^^" + ""; 

		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDXM1_BLDQRSP, queryBean.getItemId(), information, "0", "0"); // orderIEN = queryBean.getItemId() 
		
		if (RPCStringUtils.isNullOrEmpty(response))
        {
            return null; 
        }
		
		/* if (response.Substring(0, 3) == "8^0")
         {
             string[] lines = StringUtils.split(response, StringUtils.CRLF);
             throw new Exception(lines[1]);
         }*/
		
		String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
		OrderDialog orderDialog = new OrderDialog();
		orderDialog.setQuickLevel(RPCStringUtils.isNumeric(fields[0])? new Integer(fields[0]): null);
		orderDialog.setResponseId(fields[1]);
		orderDialog.setDialogId(fields[2]);
		orderDialog.setType(fields[3]);
		orderDialog.setFormId(fields[4]);
		orderDialog.setDisplayGrp(fields[5]);
		
		return orderDialog;			
	}
	
	/**
	 * Gets the order responses for the order dialog ID using teh response ID received .
	 *
	 * @param responseId the response id
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return the order responses populated with the information received from Vista.
	 */
	private OrderResponse[] getOrderResponses(String responseId, VistaLinkCommand vistaLinkCommand)
	{
		List<OrderResponse> responseArray = new ArrayList<OrderResponse>();
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_LOADRSP, responseId);
		
		if (RPCStringUtils.isNullOrEmpty(response))
        {
            return null; 
        }
		
		OrderResponse orderResponse = null;
		response = RPCStringUtils.replaceEndofLineCharacters(response);    
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        for (int i = 0; i < lines.length; i++)
        {
             if (RPCStringUtils.isNullOrEmpty(lines[i]))
             {
                 continue;
             }
                           
             switch (lines[i].charAt(0))
             {
                 case '~':
                     if (orderResponse != null)
                     {
                    	 responseArray.add(orderResponse);
                     }
                     orderResponse = new OrderResponse();
                     String[] flds = RPCStringUtils.split(lines[i].substring(1), RPCStringUtils.CARET);
                     orderResponse.setPromptId(flds[0]);
                     orderResponse.setInstance(new Integer(flds[1]));
                     orderResponse.setName(flds[2]);
                     orderResponse.setIvalue("");
                     orderResponse.setEvalue("");
                     orderResponse.setTvalue("");
                     break;
                 case 'i':
                     orderResponse.setIvalue(orderResponse.getIvalue() + lines[i].substring(1));
                     break;
                 case 'e':
                	 orderResponse.setEvalue(orderResponse.getEvalue() + lines[i].substring(1));
                     break;
                 case 't':
                	 orderResponse.setTvalue(orderResponse.getTvalue() + lines[i].substring(1));
                     break;
             }
        }
         
        if (orderResponse != null)
        {
        	 responseArray.add(orderResponse);
        }
         
        return responseArray.toArray(new OrderResponse[0]);
	}
	
	/**
	 * Retrieves the order dialog name for the order IEN received.
	 *
	 * @param orderIen the order ien
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return the order dialog name instance populated with the information received from Vista for the order IEN received.
	 */
	private OrderDialogName getOrderDialogName(String orderIen, VistaLinkCommand vistaLinkCommand)
	{
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDXM_DLGNAME, orderIen);
		if (RPCStringUtils.isNullOrEmpty(response))
        {
            return null; 
        }
		
		OrderDialogName orderDialogName = new OrderDialogName();
		String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
		orderDialogName.setId(fields[0]);
		orderDialogName.setDisplayName(fields[1]);
		orderDialogName.setBaseId(fields[2]);
		orderDialogName.setBaseName(fields[3]);
		
		return orderDialogName;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.BaseDao#getDaoName()
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.WRITE_SIMPLE_ORDER_BY_POLICY_DAO;
	}
}
