

package gov.va.med.via.persistence;


import java.util.ArrayList;
import java.util.Collection;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Required;


public class JNDIBindingsLocatorImpl
    implements
        JNDIBindingsLocator
{

    private Log logger = LogFactory.getLog( JNDIBindingsLocatorImpl.class );

    private Context initialContext;
    private String namedContext = "";// The initial context


    @Override
    public Collection<Binding> getVistaLinkBindings( )
    {
        Collection<Binding> result = new ArrayList<Binding>();
        try
        {

            // Get the JNDI bindings
            NamingEnumeration<Binding> enumeration = initialContext.listBindings( this.namedContext );
        	
            try
            {
                while ( enumeration.hasMore() )
                {
                    Binding binding = enumeration.next();
                    Object obj = binding.getObject();
                    if ( obj instanceof DataSource )
                    {
                        result.add( binding );
                    }
                }
            }
            catch ( NamingException ex )
            {
                throw new RuntimeException( getClass().getName() + " could not retrieve all vista link bindings", ex );
            }
            finally
            {
                enumeration.close();
            }
        }
        catch ( NamingException nnfe )
        {
            throw new RuntimeException( getClass().getName() + " could not list vista link bindings", nnfe );
        }
        if ( logger.isInfoEnabled() )
            
        {
            String msg = "Located " + result.size() + " vista link site bindings in context " + this.namedContext ;

            logger.info( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessageInfo( msg ), null );

        }
        return result;
    }


    /**
     * Sets the namedContext that is passed into the 
     * JNDI InitialContext.listBinding() method. If not 
     * specified, the initial context is used.
     * @param namedContext - The name of the JNDI context to search.
     */
    public void setNamedContext( String namedContext )
    {
        this.namedContext = namedContext;
    }


    /**
     * Sets the JNDI initial context.
     * @param initialContext - the initial context instance.
     */

    @Required
    public void setInitialContext( Context initialContext )
    {
        this.initialContext = initialContext;
    }
}
