package gov.va.med.via.request;

import gov.va.med.via.business.ViaResponse;
import gov.va.med.via.transaction.RpcTransactionManager;

import java.util.Map;

/**
 * Interface for core pojo processing - generic processor interface used by all services, extends main Business Interface VistaService
 * 
 * @author ZZZZZZTALBOM
 *
 */
public interface ViaReadProcessor {
	
	
	/**
	 * Setter method of the transaction manager the processor delegates processing to
	 * 
	 * @param transactionManager
	 */
	public void setTransactionManager(RpcTransactionManager transactionManager);
	
	
	/**
	 * Logs Provider User into the Via Application - authenticates with login site
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse loginProvider(Map<String,Object>requestParameterMap);
	
	/**
	 * Mathes a patient from the search strings provided
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse matchPatient(Map<String,Object>requestParameterMap);
	
	
	/**
	 * Selects a patient from criteria provided by calling application
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse selectPatient(Map<String,Object>requestParameterMap);
	
	/**
	 * 
	 * Generic processing method of service/business core processing
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse processVistaRead(Map<String,Object>requestParameterMap);

	
	

}
