package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.InpatientStay;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class parses the response from the get admissions RPC ORWPT ADMITLST call into DataBeans
 * @author DOMAINALLURS
 *
 */
public class AdmissionsDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){		
		List<DataBean> list = Arrays.asList((DataBean[])toAdmissions(response, dfn));
		return list;
	}
	
	/**
	 * Parses the get admissions RPC response string and populates an array of DataBean objects
	 * @param response - The response from RPC ORWPT ADMITLST
	 * @param dfn - the dfn
	 * @return an array of InpatientStay DataBeans populated from the content stored in the RPC response string
	 */
	protected InpatientStay[] toAdmissions(String response, String dfn)
    {
        
		if (response.equals("")) {
            return new InpatientStay[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);       
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
       
        // Mock test data 
        // String[] lines = {"3101026.0836^310^CHY FLU AM^A", "3101026.0836^310^CHY FLU AM^A"};
        
        
        ArrayList<InpatientStay> lst = new ArrayList<InpatientStay>();
                
        for (int i = 0; i < lines.length; i++){
            if (lines[i].isEmpty()){
                continue;
            }
            InpatientStay inpatientStay = new InpatientStay();
            Patient patient = new Patient();
            patient.setLocalPid(dfn);
            String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);                       
            inpatientStay.setAdmitTimestamp(Timestamp.toUtcString(fields[0]));
            HospitalLocation hospitalLocation = new HospitalLocation(fields[1], fields[2]);
            inpatientStay.setLocation(hospitalLocation);
            inpatientStay.setType(fields[3]);
            inpatientStay.setPatient(patient);
            lst.add(inpatientStay);
        }
        InpatientStay[] array =  (InpatientStay[])lst.toArray(new InpatientStay[lst.size()]);
        return array;
    }	
}
