package gov.va.med.via.response;

import gov.va.med.via.databeans.Appointment;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;

public class AppointmentsDataBeanParser extends BaseDataBeanParser {

	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) {
		
		return Arrays.asList((DataBean[])toAppointments(response, queryBean, dfn, siteId, vistaLinkCommand));
	}
	
	protected Appointment[] toAppointments(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		if (RPCStringUtils.isNullOrEmpty(response))
        {
            return new Appointment[0];
        }
		 
		response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        lines = RPCStringUtils.trimArray(lines);
        
        List<Appointment> result = new ArrayList<Appointment>();
        ReferenceDataDelegate referenceDataDelegate = new ReferenceDataDelegate(vistaLinkCommand);
      
        for (int i = 0; i < lines.length; i++)
        {
        	if (lines[i].contains("[Data]")) continue;
        	if (lines[i].contains("[Errors]")) continue;
        	
        	String[] recordParts = RPCStringUtils.split(lines[i],RPCStringUtils.TILDE);
        	
        	String[] flds = RPCStringUtils.split(recordParts[0], RPCStringUtils.CARET);
        	Appointment appt = new Appointment();
        	
        	//Part I
        	appt.setId("A;" + flds[0] + ";" + flds[1]);
        	String siteName = SiteTable.getInstance().getSite(siteId).getName();  
        	appt.setFacility(new SiteId(siteId, siteName));
            appt.setTimestamp(Timestamp.toUtcString(flds[0]));
            HospitalLocation hospitalLocation = new HospitalLocation(flds[1], "");
            
            appt.setStatus(decodeApptStatus(flds[2]));
            if (!flds[3].equals(""))
            {
                appt.setLabDateTime(Timestamp.toUtcString(flds[3]));
            }
            if (!flds[4].equals(""))
            {
                appt.setXrayDateTime(Timestamp.toUtcString(flds[4]));
            }
            if (!flds[5].equals(""))
            {
                appt.setEkgDateTime(Timestamp.toUtcString(flds[5]));
            }
            appt.setPurpose(decodeApptPurpose(flds[6]));
            if (!flds[7].equals(""))
            {
            	String apptType = referenceDataDelegate.getCodeDescription(VistaConstants.APPT_TYPES,flds[7]);
            	appt.setType(apptType);
            }
            if (flds.length == 9)
            {
                appt.setCurrentStatus(flds[8]);
            }
             
            //Part II
            if(recordParts.length > 1)
            {
            	String parts2 = recordParts[1].replace(RPCStringUtils.RECORD_SECTION_FIELD_SEPARATOR, RPCStringUtils.CARET);
            	flds = RPCStringUtils.split(parts2, RPCStringUtils.CARET);
            	hospitalLocation.setName(flds[0]);
            	
            	if(flds.length > 1 && StringUtils.isNotEmpty(flds[1]))
            	{
            		SiteId clinicSite = new SiteId();
            		siteName = referenceDataDelegate.getCodeDescription(VistaConstants.MEDICAL_CENTER_DIVISION,flds[1]);
            		if(siteName != null){
            			clinicSite.setName(siteName);
            		}
            		hospitalLocation.setFacility(clinicSite);	
            	}
            	
            }
            
            //Part III
            if(recordParts.length > 2){
            	String parts3 = recordParts[2].replace(RPCStringUtils.RECORD_SECTION_FIELD_SEPARATOR, RPCStringUtils.CARET);
            	flds = RPCStringUtils.split(parts3, RPCStringUtils.CARET);
            	hospitalLocation.setPhone(flds[0]);
            }
            
            appt.setClinic(hospitalLocation);
            result.add(appt);
             
        }
		
        return result.toArray(new Appointment[0]);
		
    }
	

    private String decodeApptStatus(String status)
    {
    	if(status == null) return status;
    	
        if (status.equals("N"))
        {
            return "NO-SHOW";
        }
        if (status.equals( "C"))
        {
            return "CANCELLED BY CLINIC";
        }
        if (status.equals( "NA"))
        {
            return "NO-SHOW & AUTO RE-BOOK";
        }
        if (status.equals( "CA"))
        {
            return "CANCELLED BY CLINIC & AUTO RE-BOOK";
        }
        if (status.equals( "I"))
        {
            return "INPATIENT APPOINTMENT";
        }
        if (status.equals( "PC"))
        {
            return "CANCELLED BY PATIENT";
        }
        if (status.equals( "PCA"))
        {
            return "CANCELLED BY PATIENT & AUTO-REBOOK";
        }
        if (status.equals( "NT"))
        {
            return "NO ACTION TAKEN";
        }
        return status;
    }
    
    private String decodeApptPurpose(String code)
    {
        if(code == null) return code;
        
    	if (code.equals( "1"))
        {
            code = "C&P";
        }
        else if (code.equals( "2"))
        {
        	code = "10-10";
        }
        else if(code.equals( "3"))
        {
        	code = "SCHEDULED VISIT";
        }
        else if (code.equals("4"))
        {
        	code = "UNSCHED. VISIT";
        }
        return code;
    }

}
