package gov.va.med.via.response;

import gov.va.med.via.databeans.CytologyReport;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.ChemHemReport;
import gov.va.med.via.databeans.LabResult;
import gov.va.med.via.databeans.LabTest;

import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class ChemHemReportsSimpleDataBeanParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toChemHemReports(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * Convert the response into an array of Order containing the data received from Vista.
	 * Populates the order with the correct order type and translate the number received for 
	 * order status and signature status to a valid description.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return Array of Order objects. 
	 */
	protected ChemHemReport[] toChemHemReports(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<ChemHemReport> responseArray = new ArrayList<ChemHemReport>();
		HashMap<String, ChemHemReport> reportsByTime = new HashMap<String, ChemHemReport>();
		ChemHemReport report = new ChemHemReport();
        LabResult result = null;
		
        if (response == null|| response.equals("")) 
        {
        	report.setFault(new Fault("Cannot retrieve orders using the information provided."));
        	responseArray.add(report);
        }
        else
        {        	
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	
            String timestamp = "";
            String facilityTag = "";
            String facilityName = "";
            
            for (int i = 0; i < lines.length; i++)
            {
                String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);             
                int fieldNumber = 0;
                try
                {
               	 	fieldNumber = new Integer(fields[0]).intValue();         
                } catch (NumberFormatException e) 
                {
               	 continue;
                }
                switch (fieldNumber)
                {
                	case 1:
                		if (fields.length == 2)
                		{
                			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
                			if (parts.length == 2)
                			{
                				facilityTag = parts[1];
                				facilityName = parts[0];
                			}
                			else if (!fields[1].isEmpty())
                			{
                				facilityTag = siteId;
                				facilityName = fields[1];
                			}
                			else
                			{
                				try
                				{
                    				facilityTag = siteId;
                    				facilityName = SiteTable.getInstance().getSiteName(siteId);
                				}
                		       	catch(SiteException e)
                		       	{
                		       		// Ignore it, site will not have name. Should never happen if valid site.
                		       	}
                			}
                		}
                		break;
                	case 2:
                		if (report != null && result != null && result.getLabTest() != null)
                		{
                			if (RPCStringUtils.isEmpty(result.getLabTest().getRefRange()))
                			{
               					if (!RPCStringUtils.isEmpty(result.getLabTest().getLowRef()) &&                				
               						!RPCStringUtils.isEmpty(result.getLabTest().getHiRef()))
               					{
               						result.getLabTest().setRefRange(result.getLabTest().getLowRef() + "-" + result.getLabTest().getHiRef());
               					}
               					report.addLabResult(result);
                			}
                		}
                		result = new LabResult();
                		result.setLabTest(new LabTest());
                		timestamp = Timestamp.toUtcFromRdv(fields[1]);

                		// Look for a Report with the same Timestamp, if none can be found, create a new ChemHem Report
                		//if (reportsByTime.get(timestamp) == null);
                		//if (reportsByTime.containsKey(timestamp));                		
                		boolean found = false;
                		Set<String> keys = reportsByTime.keySet();
                		for (String key: keys)
                		{
                			if (key.equals(timestamp))
                			{
                				found = true;
                				break;
                			}                			
                		}
                		if (!found)
                		{
                			report = new ChemHemReport();
                			report.setFacility(new SiteId(facilityTag, facilityName));
                			report.setTimestamp(timestamp);
                			reportsByTime.put(timestamp, report);
                		}                		
                		break;
                	case 3:
                		if (result != null && result.getLabTest() != null)
                		{
                			result.getLabTest().setName(fields.length == 2? fields[1] + RPCStringUtils.CRLF: null);
                		}
                		break;             		
                	case 4:
                		if (fields.length == 2 && result != null)
                		{
               				result.setSpecimenType( fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 5:
                		if (result != null)
                		{
                			result.setValue(fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 6:
                		if (result != null)
                		{
                			result.setBoundaryStatus(fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 7:
                		if (result != null && result.getLabTest() != null)
                		{
                			result.getLabTest().setUnits(fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 8:
                		if (result != null && result.getLabTest() != null)
                		{
                			result.getLabTest().setLowRef(fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 9:
                		if (result != null && result.getLabTest() != null)
                		{
                			result.getLabTest().setHiRef(fields.length == 2? fields[1] : null);
                		}
                		break;
                	case 10:
                		if (result != null)
                		{
                			result.setComment(fields.length == 2? fields[1] : null);
                		}
                		break;                		
                }
            }             
        }        
        
		if (report != null)
		{
			if (RPCStringUtils.isEmpty(result.getLabTest().getRefRange()))
			{
				if ( result != null & result.getLabTest() != null)
				{
					if (!RPCStringUtils.isEmpty(result.getLabTest().getLowRef()) &&                				
						!RPCStringUtils.isEmpty(result.getLabTest().getHiRef()))
					{
						result.getLabTest().setRefRange(result.getLabTest().getLowRef() + "-" + result.getLabTest().getHiRef());
					}
					report.addLabResult(result);
				}
			}
		}
		
		Set<String> keys = reportsByTime.keySet();
		for (String key: keys)
		{
			responseArray.add(reportsByTime.get(key));
		}

        return responseArray.toArray(new ChemHemReport[0]);
	}

}
