package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;

import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.via.rpc.RpcNameConstants;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * This class parses the response from the get Clinics RPC ORWU CLINLOC call into DataBeans
 * @author DOMAINALLURS
 *
 */
public class ClinicsDataBeanParser extends BaseDataBeanParser 
{	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){		
		List<DataBean> list = Arrays.asList((DataBean[])toHospitalLocations(response, dfn, vistaLinkCommand));
		return list;
	}
	
	/**
	 * Parses the get clinics RPC response string and populates an array of DataBean objects
	 * @param response - The response from RPC ORWU CLINLOCS
	 * @param dfn - the dfn
	 * @return an array of HospitalLocation DataBeans populated from the content stored in the RPC response string
	 */
	protected HospitalLocation[] toHospitalLocations(String response, String dfn, VistaLinkCommand vistaLinkCommand)
    {        
		if (response.equals("")) 
		{
            return new HospitalLocation[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);       
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
                
        ArrayList<HospitalLocation> lst = new ArrayList<HospitalLocation>();
                        
        for (int i = 0; i < lines.length; i++)
        {
            if (lines[i].isEmpty())
            {
                continue;
            }
            
            String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            HospitalLocation hospitalLocation = new HospitalLocation(fields[0], fields[1]);
            
	        List<Object> params =new ArrayList<Object>();          
	        HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			hashMap.put("FILE",44);
			hashMap.put("FIELDS","10");
			hashMap.put("FLAGS","IP");
			hashMap.put("FROM",RPCStringUtils.adjustForNameSearch(fields[1]));
			hashMap.put("XREF","B");	        
	        params.add(hashMap);	        
	        String result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	        
	        result = RPCStringUtils.replaceEndofLineCharacters(result);  
		    String[] sclines = RPCStringUtils.split(result, RPCStringUtils.CRLF);
		    for (int j = 0; j < sclines.length; j++)
	        {    
		    	if (sclines[j].contains("[Data]") || RPCStringUtils.isEmpty(sclines[j])) continue;

		    	String[] flds = RPCStringUtils.split(sclines[j], RPCStringUtils.CARET);
	        	//ensure the record contains the IEN (numeric value) and that the IEN is the same as our patient's PID (dfn)
	           if (RPCStringUtils.isNumeric(flds[0]) && fields[0].equals(flds[0]))
	           {
	        	   String physicalLocation = flds[1];	           
	        	   if (physicalLocation!= null && !physicalLocation.isEmpty())
	        	   {
		        		hospitalLocation.setPhysicalLocation(physicalLocation);
		        		break;
	        	   }
	            }
	        }
	           
            lst.add(hospitalLocation);
        }
		    
        HospitalLocation[] array =  (HospitalLocation[])lst.toArray(new HospitalLocation[lst.size()]);
        return array;
    }	
}
