package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class parses the response from the get Confidentiality RPC DG SENSITIVE RECORD ACCESS call into DataBeans
 * 
 * @author DOMAINROMERJ
 *
 */
public class ConfidentialityDataBeanParser extends BaseDataBeanParser 
{	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{		
		List<DataBean> list = Arrays.asList((DataBean[])toText(response, dfn));
		return list;
	}
	
	/**
	 * Parses the get Confidentiality RPC and populates an array of DataBean objects
	 * @param response - The response from RPC  DG SENSITIVE RECORD ACCESS
	 * @param dfn - the dfn
	 * @return an array of Text DataBeans populated from the content stored in the RPC response string
	 */
	protected Text[] toText(String response, String dfn)
    {
        
		if (response.equals("")) {
            return new Text[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);       
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        if ( lines[0].indexOf("-1") >= 0 ){
        	throw new RpcException(ErrorCodeEnum.CONFIDENTIALITY_SENSITIVE_RECORD_ACCESS_EXCEPTION);
        }
        
        ArrayList<Text> lst = new ArrayList<Text>();
        String message = "";
        int level = (new Integer(lines[0])).intValue();
               
        if ( level != 0 ){
        	int p = response.indexOf("\r\n");
        	message = response.substring(p + 2);
        }
        Text text = new Text();
        text.setTag(new Integer(level).toString());
        text.setText(message);
        lst.add(text);
        
        Text[] array =  (Text[])lst.toArray(new Text[lst.size()]);
        return array;
    }	
}
