package gov.va.med.via.response;

import gov.va.med.via.databeans.Consult;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.OrderStatus;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.Arrays;
import java.util.List;

public class ConsultDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toConsults(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of Consult data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of Consult databeans populated from the content stored in the RPC response string
	 */
	protected Consult[] toConsults(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response) || response.startsWith("< PATIENT DOES NOT HAVE ANY CONSULTS/REQUESTS"))
         {
             return new Consult[0]; // TODO - we probably want to return an Fault saying no consults were retrieved!
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         Consult[] result = new Consult[lines.length];
         for (int i = 0; i < lines.length; i++)
         {
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
             Consult consult = new Consult();
             consult.setId(fields[0]);
             consult.setTimestamp(Timestamp.toDateTime(fields[1]));             
             consult.setStatus(OrderStatus.getNameByCode(fields[2]));
             consult.setTitle(fields[6]);
             consult.setSiteId(siteId);
             
             ConsultDelegate consultDelegate = new ConsultDelegate(queryBean, siteId, vistaLinkCommand);
             consult.setText(consultDelegate.getConsultNote(consult));
             result[i] = consult;
         }
         return result;
    }

}
