package gov.va.med.via.response;

import gov.va.med.via.databeans.Consult;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

public class ConsultDelegate
{
	/** The query bean. */
	private QueryBean queryBean;
	
	/** The site id. */
	private String siteId;
	
	/** The vista link command. */
	private VistaLinkCommand vistaLinkCommand;
	
	/**
	 * Instantiates a new consult delegate.
	 *
	 * @param queryBean the query bean
	 * @param siteId the site id
	 * @param vistaLinkCommand the vista link command
	 */
	public ConsultDelegate(QueryBean queryBean, String siteId, VistaLinkCommand vistaLinkCommand)
	{
		this.queryBean = queryBean;
		this.siteId = siteId;
		this.vistaLinkCommand = vistaLinkCommand;
	}
	
	/**
	 * Retrieves the consult note for the consult received as parameter from Vista.
	 * 
	 * @param the consult
	 * 
	 * @return String with the consult note retrieved from Vista if successful.
	 */
	public String getConsultNote(Consult consult)
	{		
		if (consult.getId() == null) return null;
		
    	String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORQQCN_DETAIL, consult.getId());

    	response = RPCStringUtils.replaceEndofLineCharacters(response);
    	response = response.replace(RPCStringUtils.CRLF, " ");
    	
    	return RPCStringUtils.stripInvalidXmlCharacters(response);
	    
	}
}
