package gov.va.med.via.response;

import gov.va.med.via.databeans.CytologyReport;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.LabSpecimen;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CytologyReportDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toCytologyReport(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of CytologyReport data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of CytologyReport databeans populated from the content stored in the RPC response string
	 */
	protected CytologyReport[] toCytologyReport(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new CytologyReport[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<CytologyReport> responseArray = new ArrayList<CytologyReport>();
         CytologyReport report = null;
         String collectionDate = "";
         String specimenDesc = "";
         String exam = "";
         String accessionNumber = "";
         for (int i = 0; i < lines.length; i++)
         {
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);             
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		// A new cytology report is received when the field number is 1, add it to list and create a new CytologyReport instance
             		if (report != null)  
             		{
             			if (!RPCStringUtils.isNullOrEmpty(specimenDesc))
             			{
             				LabSpecimen specimen = new LabSpecimen("", RPCStringUtils.CSubstring(specimenDesc, 0, specimenDesc.length() - 1), collectionDate, "");
             				specimen.setAccessionNumber(accessionNumber);
             				report.setSpecimen(specimen);             				
             			}
             			report.setExam(!RPCStringUtils.isNullOrEmpty(exam) ? exam: null);
             			responseArray.add(report);
             		}
             		report = new CytologyReport();
             		report.setTitle("Cytopathology Report");
             		specimenDesc = "";
             		exam = "";
             		if (fields.length == 2)
             		{
             			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             			if (parts.length == 2)
             			{
             				report.setFacility(new SiteId(parts[1], parts[0], "",""));
             			}
             			else if (!fields[1].isEmpty())
             			{
             				report.setFacility(new SiteId(siteId, fields[1], "",""));
             			}
             			else
             			{
             				try
             				{
             					report.setFacility(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             				}
             		       	catch(SiteException e)
             		       	{
             		       		// Ignore it, site will not have name. Should never happen if valid site.
             		       	}
             			}
             		}
             		break;
             	case 2:
             		collectionDate = fields.length == 2? Timestamp.toUtcFromRdv(fields[1]): "";
             		break;
             	case 3:
             		if (fields.length == 2)
             		{
             			specimenDesc += fields[1] + '\n';
             		}
             		break;             		
             	case 4:
             		accessionNumber = fields.length == 2? fields[1] : "";
             		break;
             	case 5:
             		if (fields.length == 2)
             		{
             			exam += fields[1] + '\n';
             		}
             		break;
             }
         } 
           
         if (report != null)
         {             
        	 if (!RPCStringUtils.isNullOrEmpty(specimenDesc))
  			{
  				LabSpecimen specimen = new LabSpecimen("", RPCStringUtils.CSubstring(specimenDesc, 0, specimenDesc.length() - 1), collectionDate, "");
  				specimen.setAccessionNumber(accessionNumber);
  				report.setSpecimen(specimen);             				
  			}
  			report.setExam(!RPCStringUtils.isNullOrEmpty(exam) ? exam: null);
  			responseArray.add(report);
         }         

         return responseArray.toArray(new CytologyReport[0]);
    }
}


