package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.DictionaryTextArray;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

public class DictionaryTextArrayParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toDictionaryTextArray(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return the text[]
	 */
	protected DictionaryTextArray[] toDictionaryTextArray(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<DictionaryTextArray> responseArray = new ArrayList<DictionaryTextArray>();
		DictionaryTextArray dictionaryTextArray = new DictionaryTextArray();
		
        if (response == null|| response.equals("")) 
        {
        	dictionaryTextArray.setFault(new Fault("Cannot retrieve text using the information provided."));
        }
        else
        {
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	
        	LinkedHashMap<String, List<String>> result = new LinkedHashMap<String, List<String>>(lines.length);
        	for (int i= 0; i < lines.length; i++)
        	{
        		if (lines[i].equals("")) continue;
        		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        		if (result.containsKey(fields[0]))
        		{
        			List<String> list = result.get(fields[0]);
        			list.add(fields[1]);
        		}
        		else
        		{
        			List<String> list = new ArrayList<String>();
        			list.add(fields[1]);
        			result.put(fields[0], list);
        		}
        	}
        	dictionaryTextArray.setDictionary(result);
        	dictionaryTextArray.setSiteId(siteId);
        }        
        responseArray.add(dictionaryTextArray);
        
        return responseArray.toArray(new DictionaryTextArray[0]);
	}
}
