package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.Note;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

// TODO: Auto-generated Javadoc
/**
 * The Class DischargeSummariesDataBeanParser.
 * 
 * @author DOMAINROMERJ
 */
public class DischargeSummariesDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toNote(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of Notes containing the Discharge Summaries data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkCommand the vista link command
	 * @return an array of MicrobiologyReport databeans populated from the content stored in the RPC response string
	 */
	protected Note[] toNote(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new Note[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<Note> responseArray = new ArrayList<Note>();
         Note note = null;
         for (int i = 0; i < lines.length; i++)
         {
        	 if ("".equals(lines[i])) continue;
        	 
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
             
             if ("[+]".equals(fields[1]))
             {
            	 responseArray.add(note);
            	 
             }
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		note = new Note();
             		note.setStandardTitle("DISCHARGE SUMMARY");
             		note.setLocalTitle("Discharge Summary");
             		note.setText(RPCUtils.removeCtlChars(note.getText()));
             		if (fields.length == 2)
             		{
             			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             			if (parts.length == 2)
             			{
             				note.setSiteId(new SiteId(parts[1], parts[0], "",""));
             			}
             			else if (!fields[1].isEmpty())
             			{
             				note.setSiteId(new SiteId(siteId, fields[1], "",""));
             			}
             			else
             			{
             				try
             				{
             					note.setSiteId(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             				}
             		       	catch(SiteException e)
             		       	{
             		       		// Ignore it, site will not have name. Should never happen if valid site.
             		       	}
             			}
             		}
             		break;
             	case 2:
             		note.setAdmitTimestamp( fields.length == 2? Timestamp.toUtcFromRdv(fields[1]): null);
             		break;
             	case 3:
             		note.setDischargeTimestamp(fields.length == 2? fields[1]: null);
             		break;             		
             	case 4:
             		if (fields.length == 2)
             		{
             			note.setAuthor( new Author("",fields[1],""));
             		}
             		break;
             	case 5:
             		if (fields.length == 2)
             		{
             			note.setApprovedBy( new Author("",fields[1],""));
             		}
             		break;
             	case 6:
           			note.setStatus(fields.length == 2? fields[1]: null);
             		break;
             	case 7:
             		if (fields.length == 2)
             		{
             			note.setText(note.getText() != null? 
             					note.getText().concat(fields[1] + "\r\n"): fields[1] + "\r\n");
             		}
             }
         } 

         return responseArray.toArray(new Note[0]);
    }
}
