package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Drg;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.TaggedDrgArrayDataBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;

public class GetDRGRecordsDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		List<DataBean> list = new ArrayList<DataBean>();
		TaggedDrgArrayDataBean bean = toTaggedDrgArrayDataBean(response, siteId);
		list.add(bean);
		return list;
	}
	
	protected TaggedDrgArrayDataBean toTaggedDrgArrayDataBean(String response,String siteId )
	{
		List<Drg> drgList = toDrgList(response);
		String tag = siteId;
		TaggedDrgArrayDataBean drgArray = new TaggedDrgArrayDataBean(tag, drgList);
		return drgArray;
	}
	
	protected List<Drg> toDrgList( String response )
	{
		if (response.equals(""))
        {
            return new ArrayList<Drg>();
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
		
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Drg> lst = new ArrayList<Drg>();
        for (int i = 0; i < lines.length; i++)
        {    
        	if (lines[i].contains("[Data]")) continue;

        	String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        	//ensure the record contains the IEN
            if (RPCStringUtils.isNumeric(flds[0]))
            {
            	Drg drg = new Drg();
            	drg.setId(flds[0]);
            	drg.setDescription(flds[1]);
                lst.add(drg);
            }   
        }
        return lst;
		
	}
	
	
	
	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * MDWS result appears as follows:  

				<TaggedDrgArrays xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="http://domaindomaindomain/NumiService">
				  <count>1</count>
				  <arrays>
				    <TaggedDrgArray>
				      <count>871</count>
				      <tag>442</tag>
				      <items>
				        <DrgTO>
				          <id>1</id>
				          <description>CRANIOTOMY AGE &gt;17 W CC</description>
				        </DrgTO>
				        ...
				        <DrgTO>
				          <id>116</id>
				          <description>NO LONGER VALID</description>
				        </DrgTO>
				      </items>
				    </TaggedDrgArray>
				  </arrays>
				</TaggedDrgArrays>
*/			

	
}
