package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.ImagingExam;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.RadiologyReport;
import gov.va.med.via.rpc.DateFormat;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ImagingReportParser extends BaseDataBeanParser
{

	public ImagingReportParser() {
		super();
	}


	public static final int ID_INDEX=0;

	public static final String CASE_NUMBER_FIELD="4";
	public static final int CASE_NUMBER_INDEX=1;

	public static final String REPORT_STATUS_FIELD="5";
	public static final int REPORT_STATUS_INDEX=2;

	public static final String EXAM_STATUS_FIELD="103";
	public static final int EXAM_STATUS_INDEX=3;

	public static final String TITLE_FIELD="102"; ///procedure title
	public static final int TITLE_INDEX=4;

	public static final String TIMESTAMP_FIELD="3";
	public static final int TIMESTAMP_INDEX=5;
	public static final SimpleDateFormat sdf=new SimpleDateFormat("MMM dd, yyyy@HH:mm");


	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{	
		List<DataBean> reports = new ArrayList<DataBean>();

		/// in this case, we match the SSN before getting the full report
		PatientMatchDataBeanParser parser = new PatientMatchDataBeanParser();
		Patient[] patients = parser.toPatients(response);		
		if ( patients.length>1)
		{
			RadiologyReport radiologyReport = new RadiologyReport();
			Fault fault = new Fault();
			fault.setMessage("More than one patient has that SSN in this site ("+siteId+")");
			radiologyReport.setFault(fault);
			reports.add(radiologyReport);
			return reports;
		}
		if ( patients.length==0)
		{
			RadiologyReport radiologyReport = new RadiologyReport();
			Fault fault = new Fault();
			fault.setMessage("No patient has that SSN in this site ("+siteId+")");
			radiologyReport.setFault(fault);
			reports.add(radiologyReport);
			return reports;
		}


		String accessionNumber= queryBean.getCriteria();

		String patientName = patients[0].getName();

		/// get ExamList
		Map<String, RadiologyReport> accessionNumberRadiologyReportMap = getScListerExamList(patientName, vistaLinkCommand);
		RadiologyReport radiologyReport = accessionNumberRadiologyReportMap.get(accessionNumber);
		if ( accessionNumberRadiologyReportMap==null || radiologyReport==null )
		{
			radiologyReport = new RadiologyReport();
			Fault fault = new Fault();
			fault.setMessage("No radiology report could be found for accessionNumber '"+accessionNumber+"'.");
			reports.add(radiologyReport);
			return reports;
		}

		/// get report text
		List<Object> params = buildGetReportTextParams( radiologyReport.getId());//buildGetReportRequestParams(dfn, radiologyReport.getId(), radiologyReport.getCaseNumber());
		response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_GETS_ENTRY_DATA, params);
		String reportText = toReportText(response);
		radiologyReport.setText(reportText);

		List<DataBean> result = new ArrayList<DataBean>();
		result.add(radiologyReport);
		return result;
	}

	private String toReportText(String response) 
	{
		if ( response==null || response.trim().isEmpty())
		{ 
			return null;
		}
		
/*		The response will appear like:
		 
		 	[Data]
			74^146015^200^[WORD PROCESSING]
			There is shortening of the 5th metacarpal with a deformity suggesting an 
			old healed boxers fracture. No evidence of joint space narrowing, 
			spurring or sclerosis of the joints. There is calcification of the radial 
			artery extending into the thenar eminence. 
			$$END$$
*/		

		String wordProcessingMarker = "[WORD PROCESSING]";
		String endMarker = "$$END$$";
		String result = response;
		
		int wordProcessingIndex = result.indexOf(wordProcessingMarker);
		if ( wordProcessingIndex!=-1)
		{
			result = result.substring(wordProcessingIndex+ wordProcessingMarker.length());			
		}
		int endMarkerIndex = result.indexOf(endMarker);
		if ( endMarkerIndex!=-1)
		{
			result = result.substring(0,endMarkerIndex);
		}
		if ( result.indexOf("^")!=-1)
		{
			String[] parts = result.split("\\^");
			result = parts[parts.length-1];  // return the last part if the WORD PROCESSING and $$END$$ elements not present
		}
		return result;
	}

	/**
	 * gets a Map of RadiologyReport objects with the patient's name
	 *  
	 * @param the patient's name 	 * @param vistaLinkCommand
	 * @return Map of RadiologyReport objects with accession number as the key
	 */
	protected Map<String, RadiologyReport> getScListerExamList(String patientName,VistaLinkCommand vistaLinkCommand)
	{


		String fields = CASE_NUMBER_FIELD+";"
				+ REPORT_STATUS_FIELD+";"
				+ EXAM_STATUS_FIELD+";"
				+ TITLE_FIELD+";"
				+ TIMESTAMP_FIELD;

		HashMap<Object,Object> hm = new HashMap<Object,Object>();
		hm.put("FILE", 74);
		hm.put("FIELDS", fields);
		hm.put("XREF","C");/// patient name
		hm.put("PART", patientName);
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, hm);
		return toScListerImagingExameIds(response);
	}


	protected Map<String, RadiologyReport> toScListerImagingExameIds(String response)
	{
		if (StringUtil.isNullOrEmpty(response))
		{
			return new HashMap<String, RadiologyReport>();
		}
		Map<String, RadiologyReport> result = new HashMap<String, RadiologyReport>();
		String[] lines = response.split("\\n");
		for ( String line: lines)
		{
			line = line.trim();
			if ( line.toLowerCase().startsWith("[data]")) continue;

			RadiologyReport newReport = new RadiologyReport();
			String[] flds = line.split("\\^");

			newReport.setId(flds[0]);  ///sc lister returns the IEN (ID) first
			newReport.setCaseNumber(flds[CASE_NUMBER_INDEX]);
			newReport.setStatus(flds[REPORT_STATUS_INDEX]);
			newReport.setExam( new ImagingExam());
			newReport.getExam().setStatus( flds[EXAM_STATUS_INDEX]);
			newReport.setTitle(flds[TITLE_INDEX]);
			try
			{
				String dateString = flds[TIMESTAMP_INDEX];
				Date date = sdf.parse(dateString);
				String vistaTimestamp = VistaTimestamp.convertDateToVistAFormat(date);
				newReport.setTimestamp(vistaTimestamp);
			}
			catch ( Exception e)
			{
				throw new RuntimeException(e);
			}
			newReport.setAccessionNumber(newReport.getAccessionNumber(newReport.getTimestamp(), newReport.getCaseNumber(), DateFormat.VISTA));

			// maybe map some more fields? lot's of stuff being returned by RPC. For example:
			//ANN ARBOR VAMC;506
			//^6899085.8548-1
			//^3100914.1451
			//^TIBIA & FIBULA 2 VIEWS
			//^794
			//^Verified
			//^
			//^878633
			//^9~COMPLETE
			//^AA RADIOLOGY,AA
			//^RAD~GENERAL RADIOLOGY
			//^
			//^73590
			//^20323714
			//^N
			//^
			//^
			result.put(newReport.getAccessionNumber(), newReport); 
		}
		return result;
	}

	List<Object> buildGetReportTextParams(String reportIen)
	{

		List<Object> params =new ArrayList<Object>();                               
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("FILE",74);
		hashMap.put("FIELDS","200");
		hashMap.put("IENS", reportIen+",");  /// I can't believe some SOB required a comma for this to work!
		hashMap.put("FLAGS","IP");
		params.add(hashMap);
		return params;

	}



	/**
	 * matches the MDWS approach, but the RPC returns no data
	 * 
	 * @param dfn
	 * @param radiologyReportId
	 * @param caseNumber
	 * @return
	 */
	@Deprecated
	protected List<Object> buildGetReportRequestParams(String dfn, String radiologyReportId, String caseNumber)
	{

		List<Object> list = new ArrayList<Object>();

		/*      INPUT PARAMETER: DFN                    PARAMETER TYPE: LITERAL
        MAXIMUM DATA LENGTH: 20               REQUIRED: YES
        DESCRIPTION:   
        Internal entry number of entry in the Patient file.
		 */
		list.add( dfn);

		/*      INPUT PARAMETER: REPORT ID              PARAMETER TYPE: LITERAL
        MAXIMUM DATA LENGTH: 20               REQUIRED: YES
        DESCRIPTION:   
        Identification number of the desired report.
		 */
		list.add( "18:IMAGING (LOCAL ONLY)~");

		/*      INPUT PARAMETER: HEALTH SUMMARY TYPE    PARAMETER TYPE: LITERAL
        MAXIMUM DATA LENGTH: 20               REQUIRED: NO
        DESCRIPTION:   
        Internal entry number of entry in the HEALTH SUMMARY TYPE file.
		 */        
		list.add( "");

		/*      INPUT PARAMETER: DATE RANGE             PARAMETER TYPE: LITERAL
        MAXIMUM DATA LENGTH: 5                REQUIRED: NO
        DESCRIPTION:   
        This parameter indicates the number of days from TODAY
        that the report should search for information.
		 */        
		list.add( "");


		/*      This parameter is required for vital reports.
        INPUT PARAMETER: REPORT SECTION         PARAMETER TYPE: LITERAL
        MAXIMUM DATA LENGTH: 2                REQUIRED: YES
        DESCRIPTION:   
        This parameter specifics which portion of the report should be
        retrieved.
		 */   
		//list.add("0");
		list.add( radiologyReportId + '#' + caseNumber);

		list.add( "0");
		list.add( "0");
		return list;
	}





	/**
	 * gets a Map of RadiologyReport objects with the accession number as the key. 
	 *  
	 * @param dfn the patient's local ID
	 * @param vistaLinkCommand
	 * @return Map of RadiologyReport objects with accession number as the key
	 */
	@Deprecated
	protected Map<String, RadiologyReport> getExamList(String dfn,VistaLinkCommand vistaLinkCommand)
	{
		throw new UnsupportedOperationException();
		//		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWRA_IMAGING_EXAMS1, dfn);
		//		return toImagingExameIds(response);
	}

	@Deprecated
	protected Map<String, RadiologyReport> toImagingExameIds(String response)
	{
		if (StringUtil.isNullOrEmpty(response))
		{
			return new HashMap<String, RadiologyReport>();
		}
		Map<String, RadiologyReport> result = new HashMap<String, RadiologyReport>();
		String[] lines = response.split("\\n");
		for ( String line: lines)
		{
			line = line.trim();

			RadiologyReport newReport = new RadiologyReport();
			String[] flds = line.split("\\^");

			newReport.setAccessionNumber(newReport.getAccessionNumber(flds[2], flds[4], DateFormat.VISTA));
			newReport.setId(flds[1]);
			newReport.setTimestamp(flds[2]);
			newReport.setTitle(flds[3]);
			newReport.setCaseNumber(flds[4]);
			newReport.setStatus( flds[5]);
			newReport.setExam( new ImagingExam());
			//newReport.getExam().setStatus( (flds[8].Split(new char[] { '~' }))[1] ); // e.g. 9~COMPLETE
			// set status 
			String field8 = flds[8];
			String[] sa = field8.split("\\~");
			String imagingExamStatus = sa[1];
			newReport.getExam().setStatus(imagingExamStatus);



			// maybe map some more fields? lot's of stuff being returned by RPC. For example:
			//ANN ARBOR VAMC;506
			//^6899085.8548-1
			//^3100914.1451
			//^TIBIA & FIBULA 2 VIEWS
			//^794
			//^Verified
			//^
			//^878633
			//^9~COMPLETE
			//^AA RADIOLOGY,AA
			//^RAD~GENERAL RADIOLOGY
			//^
			//^73590
			//^20323714
			//^N
			//^
			//^
			result.put(newReport.getAccessionNumber(), newReport); 
		}
		return result;
	}


	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return 
	 */
	protected List<DataBean> toRadiologyReportTO(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<DataBean> reports = new ArrayList<DataBean>();
		RadiologyReport radiologyReport = new RadiologyReport();

		if (response == null|| response.equals("")) 
		{
			return reports;
		}

		reports.add(radiologyReport);
		return reports;
	}
}
