package gov.va.med.via.response;

import gov.va.med.via.databeans.Adt;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.util.DateTimeIteratorUtil;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * The Class InpatientMoveDataBeanParser.
 * 
 * @author DOMAINROMERJ
 *  
 */
public class InpatientMoveAdtCheckinIdDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toAdts(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of Consult data beans using the data in the response received.
	 *   
	 * Patient Movement Fields in Vista (405)
     *  0:  IEN
     *  1:  DATE/TIME                       .01     Timestamp
     *  2:  ADMISSION/CHECK-IN MOVEMENT     .14     CheckInId
     *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of Consult databeans populated from the content stored in the RPC response string
	 */
	protected Adt[] toAdts(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new Adt[0]; // TODO - is this correct?
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         
         List<Adt> result = new ArrayList<Adt>();
         Set<String> uniqueCheckinIds = new HashSet<String>();
         for (int i = 0; i < lines.length; i++)
         {
             // ADT data starts after the [Data] tag in the response received. Skip processing the [Data] tag in the response.        	 
        	 if (lines[i].contains("[Data]"))
        		 continue;
        	 
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
             if (fields != null && fields.length >1)
             {
            	 //only return the unique checkinId records
            	 String checkinId = fields[2];
            	 if(!uniqueCheckinIds.contains(checkinId)){
            		 uniqueCheckinIds.add(checkinId);
            		 Adt adt = new Adt();
            		 adt.setTimestamp(!RPCStringUtils.isNullOrEmpty(fields[1])? Timestamp.toDateTime(fields[1]): null);
                	 adt.setCheckInId(fields[2]);
                	        	 
                	 result.add(adt);
            	 }            	
             }
         }
       
         
         return result.toArray(new Adt[0]);
    }
	
}
