package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.ObservationType;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.VitalSign;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class LatestVitalSignsParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toVitalSigns(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return 
	 */
	protected VitalSign[] toVitalSigns(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		
        if (response == null|| response.equals("")) 
        {
        	VitalSign vs = new VitalSign();
        	vs.setFault(new Fault("Cannot retrieve text using the information provided."));
        	return new VitalSign[]{vs};
        }
        else
        {
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	String category = "Vital Signs";

        	ArrayList<VitalSign> vitalSignList = new ArrayList<VitalSign>();
        	for (int i= 0; i < lines.length; i++)
        	{
        		if (lines[i].trim().equals("")) continue;
        		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        		
        		ObservationType observationType = getObservationType(category, fields);
        		if (observationType == null)
                {
                    continue;
                }
        		VitalSign observation = new VitalSign();
        		observation.setType(observationType);
        		observation.setValue1(fields[4]);
        		if (fields.length == 6)
                {
                    observation.setValue2(fields[6]);
                }
        		observation.setTimestamp(VistaTimestamp.toUtcString(fields[3]));
        		vitalSignList.add(observation);
        		
        	}
        	return (VitalSign[])vitalSignList.toArray(new VitalSign[vitalSignList.size()]);
        }

	}
	
	private ObservationType getObservationType( String category, String[] fields)
	{
		ObservationType observationType = null;
		String id=fields[0];
		
        if ("T".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Temperature");
        }
        else if ("P".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Pulse");
        }
        else if ("R".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Respiration");
        }
        else if ("BP".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Blood Pressure");
        }
        else if ("HT".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Height");
        }
        else if ("WT".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Weight");
        }
        else if ("PN".equals(fields[1]))
        {
            observationType = new ObservationType(id, category, "Pain");
        }
		return observationType;
	}
	
}
