package gov.va.med.via.response;

import gov.va.med.via.databeans.Address;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PhoneNum;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;
//import gov.va.med.via.dto.common.*;

public class MatchByNameCityStateParser extends BaseDataBeanParser
{



	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return toPatientArray(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand);
	}

	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return 
	 */
	protected List<DataBean> toPatientArray(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<DataBean> list = new ArrayList<DataBean>();
		if ( response==null || response.trim().isEmpty() || response.trim().equals("[Data]"))
		{
			/// no results, return empty list
			return list;
		}
		String[] lines = response.split("\\n",-1);
		for ( String line: lines )
		{
			if ( line.trim().toLowerCase().equals("[data]")) continue;
			Patient p = toPatient(line,siteId);
			if ( p!=null ) list.add(p);
		}
		return list;
	}

	private Patient toPatient( String line, String siteId )
	{
		if ( line==null || line.trim().isEmpty()) return null;
		
		/// fields retrieved from SC LISTER from FILE 2
		//hashMap.put("FIELDS",".01;.02;.03;.09;.111;.112;.113;.114;.115E;.116;.117;.131;991.01;991.02");

		/// returns value like
		//[Data]
		//7177262^AARSVOLD,FAITH K^F^2420716^101012294^712 S OAK^^^KIMBALL^NEBRASKA^69145^65^(444) 444-4444^1009609054^897943^C

		int DFN=0;//IEN
		int NAME=1;/// .01
		int SEX=2;/// .02
		int DATE_OF_BIRTH=3; //.03
		int SSN=4;// .09
		int STREET1=5;//.111
		int STREET2=6;//.112
		int STREET3=7;//.113
		int CITY=8;//.114
		int STATE=9;//.115E
		int ZIP=10;//.116
		int COUNTY_CODE=11;//.117
		int PHONE=12;//.131  the telephone number [4-20 characters] to this applicant's place of residence.
		int ICN=13;//991.01
		int ICN_CHECKSUM=14;//991.02
		int CELL_PHONE=15;//.134  the telephone number [4-20 characters] to the applicant's mobile phone.
		int AGE=16;///.033 A computed field which, based on today's date (or the patient's date of death, if applicable) and the patient's date of birth, will arrive at his/her age.
		
		String[] p = line.split("\\^",-1);
		
		Patient patient = new Patient();
		patient.setSiteId(siteId);
		patient.setLocalPid(p[DFN]);
		patient.setName(p[NAME]);
		patient.setGender(p[SEX]);
		patient.setDob(Timestamp.toUtcString(p[DATE_OF_BIRTH]));
		patient.setSsn(p[SSN]);
		
		Address a = new Address();
		a.setStreetAddress1(p[STREET1]);
		a.setStreetAddress2(p[STREET2]);
		a.setStreetAddress3(p[STREET3]);
		a.setCity(p[CITY]);
		a.setState(p[STATE]);
		a.setCounty(p[COUNTY_CODE]);
		a.setZipcode(p[ZIP]);
		
		patient.setHomeAddress(a);
		
		PhoneNum homePhone = new PhoneNum();
		homePhone.setNumber(p[PHONE]);
		patient.setHomePhone(homePhone);
		
		PhoneNum cellPhone = new PhoneNum();
		cellPhone.setNumber(p[CELL_PHONE]);
		patient.setCellPhone(cellPhone);
		
		patient.setMpiPid(p[ICN]);
		patient.setMpiChecksum(p[ICN_CHECKSUM]);
		try{	patient.setAge(Integer.parseInt(p[AGE]));	}catch ( Exception ignore)	{}
		
		return patient;
		
		

	}


}
