package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.MentalHealthInstrumentAdministration;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * The Class MentalHealthInstrumentsForPatientParser.
 * 
 * @author DOMAINROMERJ
 */
public class MentalHealthInstrumentsForPatientParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toMentalHealthInstrumentAdministration(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return the mental health instrument administration[]
	 */
	protected MentalHealthInstrumentAdministration[] toMentalHealthInstrumentAdministration(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<MentalHealthInstrumentAdministration> responseArray = new ArrayList<MentalHealthInstrumentAdministration>();
		MentalHealthInstrumentAdministration mentalHealthInstrumentAdministration = new MentalHealthInstrumentAdministration();
		
        if (response == null|| response.equals("")) 
        {
        	mentalHealthInstrumentAdministration.setFault(new Fault("Cannot retrieve text using the information provided."));
        	responseArray.add(mentalHealthInstrumentAdministration);
        }
        else
        {
        	ReferenceDataDelegate referenceDataDelegate = new ReferenceDataDelegate(vistaLinkCommand);
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

        	for (int i= 0; i < lines.length; i++)
        	{
        		if (lines[i] == null || lines[i].equals("") || "[Data]".equals(lines[i])) continue;
        		
        		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        		String instrumentName = referenceDataDelegate.getCodeDescription(VistaConstants.MH_TESTS_AND_SURVEYS, fields[3]);
        		
        		mentalHealthInstrumentAdministration = new MentalHealthInstrumentAdministration();
        		mentalHealthInstrumentAdministration.setId(fields[0]);
        		mentalHealthInstrumentAdministration.setPatient(new KeyValuePair<String,String>(fields[1], fields[2]));
        		mentalHealthInstrumentAdministration.setInstrument(new KeyValuePair<String,String>(fields[3], instrumentName));
        		mentalHealthInstrumentAdministration.setDateAdministered(Timestamp.toUtcString(fields[4]));
        		mentalHealthInstrumentAdministration.setDateSaved(Timestamp.toUtcString(fields[5]));
        		mentalHealthInstrumentAdministration.setOrderedBy(new KeyValuePair<String,String>(fields[6], fields[7]));
        		mentalHealthInstrumentAdministration.setAdministeredBy(new KeyValuePair<String,String>(fields[8], fields[9]));
        		mentalHealthInstrumentAdministration.setSigned("Y".equals(fields[10]));
        		mentalHealthInstrumentAdministration.setComplete("Y".equals(fields[11]));
        		mentalHealthInstrumentAdministration.setNumberOfQuestionsAnswered(fields[12]);
        		mentalHealthInstrumentAdministration.setTransmitStatus(decodeMentalHealthInstrumentTransimissionStatus(fields[13]));
        		mentalHealthInstrumentAdministration.setTransmitTime(Timestamp.toUtcString(fields[14]));
        		mentalHealthInstrumentAdministration.setHospitalLocation(new KeyValuePair<String, String>(fields[15], fields[16]));
                responseArray.add(mentalHealthInstrumentAdministration);
        	}
        }        
        
        return responseArray.toArray(new MentalHealthInstrumentAdministration[0]);
	}
	
	/**
	 * Decode mental health instrument transimission status.
	 *
	 * @param code the code
	 * @return the string
	 */
	private String decodeMentalHealthInstrumentTransimissionStatus(String code)
	{
		String status = "Invalid code:" + code;
		
		if ("S".equals(code))
			status = "Successfully added to db";
		else if ("T".equals(code))
			status = "Transmitted, not yet added";
		else if ("E".equals(code))
			status = "Error";
		
		return status;			
	}
}
