package gov.va.med.via.response;

import gov.va.med.via.databeans.LabSpecimen;
import gov.va.med.via.databeans.MicrobiologyReport;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class MicrobiologyReportDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toMicrobiologyReport(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of MicrobiologyReport data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of MicrobiologyReport databeans populated from the content stored in the RPC response string
	 */
	protected MicrobiologyReport[] toMicrobiologyReport(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new MicrobiologyReport[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<MicrobiologyReport> responseArray = new ArrayList<MicrobiologyReport>();
         MicrobiologyReport report = null;
         String sample = "";
         String text = "";
         for (int i = 0; i < lines.length; i++)
         {
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);             
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		// A new microbiology report is received when the field number is 1, add it to list and create a new MicrobiologyReport instance
             		if (report != null)  
             		{
             			report.setSample(!RPCStringUtils.isNullOrEmpty(sample) ? sample: null);
             			report.setText(!RPCStringUtils.isNullOrEmpty(text) ? text: null);
             			responseArray.add(report);
             		}
             		report = new MicrobiologyReport();
             		report.setTitle("Microbiology Report");
             		sample = "";
             		text = "";
             		if (fields.length == 2)
             		{
             			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             			if (parts.length == 2)
             			{
             				report.setFacility(new SiteId(parts[1], parts[0], "",""));
             			}
             			else if (!fields[1].isEmpty())
             			{
             				report.setFacility(new SiteId(siteId, fields[1], "",""));
             			}
             			else
             			{
             				try
             				{
             					report.setFacility(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             				}
             		       	catch(SiteException e)
             		       	{
             		       		// Ignore it, site will not have name. Should never happen if valid site.
             		       	}
             			}
             		}
             		break;
             	case 2:
             		report.setTimestamp( fields.length == 2? Timestamp.toUtcFromRdv(fields[1]): null);
             		break;
             	case 3:
             		report.setTitle(fields.length == 2? fields[1]: null);
             		break;             		
             	case 4:
             		if (fields.length == 2)
             		{
             			sample += fields[1] + '\n';
             		}
             		break;
             	case 5:
             		report.setSpecimen(fields.length == 2? new LabSpecimen("",fields[1],"","") : null);
             		break;
             	case 6:
             		if (fields.length == 2)
             		{
             			report.getSpecimen().setAccessionNumber(fields[1]);
             		}
             		break;
             	case 7:            	 
             		if (fields.length == 2)
             		{
             			text += fields[1] + '\n';
             		}
             }
         } 
           
         if (report != null)
         {             
        	 report.setSample(!RPCStringUtils.isNullOrEmpty(sample) ? sample: null);
        	 report.setText(!RPCStringUtils.isNullOrEmpty(text) ? text: null);
          	 responseArray.add(report);
         }         

         return responseArray.toArray(new MicrobiologyReport[0]);
    }
}
