package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Note;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class NotesWithTextParser extends BaseDataBeanParser
{
	
	public static final SimpleDateFormat sdfTimestampOut = new SimpleDateFormat("yyyyMMdd.HHmmss");
	public static final SimpleDateFormat sdfFromHL7 = new SimpleDateFormat("MM/dd/yyyy HH:mm");//08/01/2008 14:45
	
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{	
		return toNotes(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand);
	}


	private void parseLine( String line, Note note, String scannedDoc)
	{
		if ( note==null ) return;

		if ( line.startsWith("1^"))
		{
			note.setSiteId(parseSiteId(line));
		}
		else if ( line.startsWith("2^"))
		{
			note.setId(parseId(line));
		}
		else if ( line.startsWith("3^"))
		{
			note.setTimestamp(parseTimestamp(line));
		}
		else if ( line.startsWith("4^"))
		{
			note.setLocalTitle(parseLocalTitle(line));
		}
		else if ( line.startsWith("5^"))
		{
			note.setAuthor(parseAuthor(line));
		}
		else if ( line.startsWith("6^"))
		{
			parseLine6( line,note,scannedDoc );
		}
		

		/*
		a typical note record

		1^CHEYENNE VAMC;442
		2^3111375
		3^08/01/2008 14:45
		4^OUTSIDE DOCUMENTS - MISC.
		5^GONZALO E SMREKAR
		6^ LOCAL TITLE: OUTSIDE DOCUMENTS - MISC.                          
		6^STANDARD TITLE: SCANNED REPORT                                  
		6^DATE OF NOTE: AUG 01, 2008@14:45     ENTRY DATE: AUG 01, 2008@14:45:26      
		6^      AUTHOR: SMREKAR,GONZALO E    EXP COSIGNER:                           
		6^     URGENCY:                            STATUS: COMPLETED                     
		6^
		6^See scanned document. GMC DEPT. OF RHEUMATOLOGY 05-17-02
		6^
		6^ 
		6^/es/ WILLIAM E OLSON
		6^File Clerk
		6^Signed: 08/01/2008 14:46
		7^[+]
		8^1

		 */


	}

	private void parseLine6(String line, Note note, String scannedDoc) 
	{
		if ( line ==null ) return;
		
		if (!StringUtil.isNullOrEmpty(scannedDoc))
        {
			String noteText = note.getText();
			if ( noteText==null ) noteText="";
			noteText+=scannedDoc + "\r\n";
			note.setText(noteText.trim());
        }		
		
		try
		{
			line=line.split("\\^")[1].trim();
		
			if ( note.getText()==null ) note.setText("");
			note.setText(note.getText().trim()+" " + line.trim());			
			
			String localTitleMarker = "LOCAL TITLE:";
			if ( line.startsWith(localTitleMarker) && note.getLocalTitle()==null )
			{
				note.setLocalTitle(line.substring(localTitleMarker.length()).trim());	
				return;
			}
			String standardTitleMarker = "STANDARD TITLE:";
			if ( line.startsWith(standardTitleMarker) )
			{
				note.setStandardTitle(line.substring(standardTitleMarker.length()).trim());	
				return;
			}
			String cosignerMarker = "COSIGNER:";
			int cosignerIndex = line.indexOf(cosignerMarker);
			if ( cosignerIndex!=-1 )
			{
				String cosignerText = line.substring(cosignerIndex+cosignerMarker.length());
				if ( !cosignerText.trim().isEmpty())
				{
					Author cosigner = new Author();
					cosigner.setName(cosignerText);
					note.setCosigner(cosigner);
				}
			}
			String statusMarker = "STATUS:";
			int statusIndex = line.indexOf(statusMarker);
			if ( statusIndex!=-1 )
			{
				String statusText = line.substring(statusIndex+statusMarker.length());
				if ( !statusText.trim().isEmpty())
				{
					note.setStatus(statusText.trim());
					return;
				}
			}
			String esigMarker = "/es/";
			if ( line.startsWith(esigMarker))
			{
				String esig = line.substring(esigMarker.length()).trim();
				Author approverSigner = new Author();
				approverSigner.setSignature(esig);
				note.setApprovedBy(approverSigner);
				return;
			}
			String signatureTimestampMarker ="Signed:"; 
			if ( line.startsWith(signatureTimestampMarker))
			{
				String ts = line.substring(signatureTimestampMarker.length()).trim();
				note.setSignatureTimestamp(parseTimestamp(ts));
				return;
			}			
			
		}
		catch ( Exception e)
		{
			return;
		}
		
		
/*		note.sets
		fInTextPortion = true;
        if (!String.IsNullOrEmpty(scannedDoc))
        {
            note.Text += scannedDoc + "\r\n";
            scannedDoc = "";
        }
        note.Text += flds[1] + "\r\n";
        if (flds[1].StartsWith("STANDARD TITLE"))
        {
            string[] parts = StringUtils.split(flds[1], StringUtils.COLON);
            note.StandardTitle = parts[1].Trim();
        }
*/		
	}


	private Author parseAuthor(String line) {
		try
		{
			String[] parts = line.split("\\^");
			Author author = new Author();
			author.setName(parts[1]);
			return author; 
		}
		catch ( Exception e)
		{
			return null;
		}
	}


	private String parseLocalTitle(String line) {
		try
		{
			String[] parts = line.split("\\^");
			return parts[1]; 
		}
		catch ( Exception e)
		{
			return null;
		}
	}


	private String parseId(String line) 
	{
		try
		{
			String[] parts = line.split("\\^");
			return parts[1]; 
		}
		catch ( Exception e)
		{
			return null;
		}	
	
	}


	/**
	 * parses timestamp from line like: 08/01/2008 14:45
	 * 
	 * @param line
	 * @return a string like 20080801.144500
	 */
	private String parseTimestamp(String line) 
	{
		try
		{
			String[] parts = line.split("\\^");
			String vistaTimestamp = parts[1]; 
			Date date = sdfFromHL7.parse(vistaTimestamp);
			return sdfTimestampOut.format(date);		
		}
		catch ( Exception e)
		{
			return null;
		}	
	}


	/** parses site id from line like:  1^CHEYENNE VAMC;442
	 * 
	 * @param line
	 * @return
	 */
	private SiteId parseSiteId(String line) 
	{
		try
		{
			String[] parts = line.split("\\^");
			String[] infoParts = parts[1].split(";");
			SiteId siteId = new SiteId();
			siteId.setId(infoParts[1]);
			siteId.setName(infoParts[0]);
			return siteId;
		}
		catch ( Exception e)
		{
			return null;
		}	
	}

	
	public static String removeCtrlChars(String s)
    {
		if ( s==null ) return null;
		if ( s.isEmpty()) return s;
		StringBuilder result = new StringBuilder();
        int i = 0;
        int lastChar =-1;
        while (i < s.length())
        {
        	int c = s.charAt(i);
            if (c == 9 || c == 10 || c == 13 || (c > 31 && c < 127))
            {
            	if ( lastChar==32 && c==32 )
            	{
            		/// don't append - allow only single blank space
            	}
            	else
            	{
            		result.append(s.charAt(i));
            	}
            	lastChar=c;
            }
            i++;
        }
        /// remove all the double space characters from the note
        return result.toString().replaceAll("  ", " ");
    }

	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return 
	 */
	protected List<DataBean> toNotes(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<DataBean> noteList= new ArrayList<DataBean>();
		if (response==null || response.trim().equals(""))
		{
			return noteList;
		}
		String[] lines = response.split("\\n",-1);//RPCStringUtils.split(response, RPCStringUtils.CRLF);

		/*
		  MDWS code
		  
		  string[] flds = StringUtils.split(lines[i], StringUtils.CARET);
          if (flds.Length == 1 && fInTextPortion)
          {
              scannedDoc += lines[i] + "\r\n";
              continue;
          }
		 */
		
		String scannedDoc="";
		Note note =null;
		for (String line: lines)
		{
			String[] flds = StringUtil.split(line, StringUtil.CARET);
			if ( flds.length==1)
			{
				/// this line has no ^ separator
				scannedDoc+= line + "\r\n";
				continue;
			}
			
			
			line = line.trim();
			if (line.equals("") || line.startsWith("^"))
			{
				continue;
			}
			if ( line.startsWith("1^"))
			{
				if ( note!=null )
				{
					/// clear out any unusual characters from the existing note
					note.setText( removeCtrlChars(note.getText()));  /// clean out any unusual characters
				}
				note = new Note();  /// create a new note to add later
				noteList.add(note);  /// add the new note and populate it   
			}
			parseLine(line,note,scannedDoc);
			
			
			//TODO need to add additional MDWS logic to parser
			//String[] fields = RPCStringUtils.split(line, RPCStringUtils.CARET);

		}
		return noteList;
	}
}
