package gov.va.med.via.response;

import gov.va.med.via.databeans.Address;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.PatientAssociate;
import gov.va.med.via.databeans.PhoneNum;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * The Class PatientAssociatesParser.
 * 
 *  @author DOMAINROMERJ
 */
public class PatientAssociatesParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toPatientAssociates(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * Returns a list of Patient Associates associated to the patient selected with the information received from Vista.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return a list of patient associates
	 */
	protected PatientAssociate[] toPatientAssociates(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<PatientAssociate> responseArray = new ArrayList<PatientAssociate>();
		PatientAssociate patientAssociate = new PatientAssociate();
		String[] associationList = {"Primary NOK","Secondary NOK","Primary Emergency Contact","Secondary Emergency Contact","Designee","VA Guardian","Civil Guardian"};
		
        if (response == null|| response.equals("")) 
        {
        	patientAssociate.setFault(new Fault("Cannot retrieve the patient associates using the information provided."));
        }
        else
        {
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	
        	if (lines.length > 3)
        	{
        		// Retrieve Site Name
        		String siteName = "";
        		try
 				{
 					siteName = SiteTable.getInstance().getSiteName(siteId);
 				}
 		       	catch(SiteException e)
 		       	{
 		       		// Ignore it, site will not have name. Should never happen if valid site.
 		       	}
        		
        		// Only process the third line received, the rest contain TAGS returned by the SC Lister RPC ([Misc], [Data], etc.)
       			String[] fields = RPCStringUtils.split(lines[3], RPCStringUtils.CARET);
       			int count = 1;
       			int i = 1;
       			// Processing 7 Associates: Primary NOK, Secondary NOK, Primary Emergency Contact, Secondary Emergency Contact, Designee, VA Guardian, Civil Guardian
       			while (count < 8)
       			{
       				if (!RPCStringUtils.isNullOrEmpty(fields[i]))
       				{
	       				patientAssociate = new PatientAssociate();	       				
	       				patientAssociate.setAssociate(associationList[count-1]);
	       				patientAssociate.setFacilityName(siteName);
	       				patientAssociate.setName(fields[i++]);
	       				patientAssociate.setRelationshipToPatient(fields[i++]);
	       				Address address = new Address();
	       				address.setStreetAddress1(fields[i++]);
	       				address.setStreetAddress2(fields[i++]);
	       				// The last 2 associates do not have a street address 3 field (VA Guardian/Civil Guardian)
	       				if (count < 6) 
	       				{
	       					address.setStreetAddress3(fields[i++]);
	       				}
	       				address.setCity(fields[i++]);
	       				address.setState(fields[i++]);
	       				address.setZipcode(fields[i++]);
	       				patientAssociate.setHomeAddress(address);
	       				
	       				// Process the String representing the Phone number. It must be a valid number, it must contain 10 digits.
	       				if (!RPCStringUtils.isNullOrEmpty(fields[i]))
	       				{
	       					String phoneNumberRetrieved = fields[i++];
	       					StringBuffer validPhone = new StringBuffer();
	       					for (int index=0; index < phoneNumberRetrieved.length(); index++)
	       					{
	       						if (Character.isDigit(phoneNumberRetrieved.charAt(index)))
	       						{
	       							validPhone.append(phoneNumberRetrieved.charAt(index));
	       						}
	       					}
	       					
	       					if (validPhone.length() == 10)
	       					{
	       						PhoneNum phoneNumber = new PhoneNum();	       					
	       						phoneNumber.setAreaCode(validPhone.substring(0, 3));
	       						phoneNumber.setExchange(validPhone.substring(3, 6));
	       						phoneNumber.setNumber(validPhone.substring(6, 10));
	       						patientAssociate.setHomePhone(phoneNumber);
	       					}	       						
	       				}
	       				count++;
	       				responseArray.add(patientAssociate);
       				}
       				else  // increment counter to next associate
       				{
       					if (count < 6)
       						i = i+9;
       					else
       						i = i+8;
       					count++;
       				}
       				patientAssociate = null;
       			}
        	}
        	else
        		patientAssociate.setFault(new Fault("Cannot retrieve the patient associates using the information provided."));
        }        
        	
        if (patientAssociate != null)
        {
        	responseArray.add(patientAssociate);
        }
        
        return responseArray.toArray(new PatientAssociate[0]);
	}
}
