package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

// if the same for match and select -> separate rpc calls... maybe send in the method name to populate extra rpcs
/**
 * The Class PatientDataBeanParser.
 * 
 */
public class PatientDataBeanParser extends BaseDataBeanParser 
{    
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toPatients(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	protected Patient[] toPatients(String response, QueryBean queryBean, String dfn, String siteId,  VistaLinkCommand vistaLinkCommand)
    {
        if (response.equals(""))
        {
            return new Patient[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Patient> lst = new ArrayList<Patient>(lines.length);
        for (int i = 0; i < lines.length; i++)
        {
            if (lines[i].equals(""))
            {
                continue;
            }
            Patient p = toPatient(lines[i], queryBean, dfn, siteId, vistaLinkCommand);
            if (p != null)
            {
                lst.add(p);
            }
        }

        return lst.toArray(new Patient[0]);
    }
	
	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkTemplate
	 * @return an array of Allergy DataBeans populated from the content stored in the RPC response string
	 */
	 protected Patient toPatient(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {		
        if (response.equals(""))
        {
            return new Patient();
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
       /* if (fields[0].equals("-1")) // TODO is this an error? what would we return?
        {
            throw new RPCException(fields[5]);
        }*/
        Patient patient = new Patient();
        patient.setName(fields[0]);
        patient.setGender(fields[1]);
        patient.setDob(DateUtils.trimTime(Timestamp.toUtcString(fields[2])));
        patient.setSsn(fields[3]);
        patient.setLocalPid(dfn); 
         if (fields[4] != "" && fields[5] != "")
        {
            HospitalLocation hl = new HospitalLocation();
            hl.setId(fields[4]);
            hl.setName(fields[5]);
            hl.setRoom(fields[6]);
            if (fields[15] != "")
            {
                hl.setSpecialty(new KeyValuePair<String, String>(fields[15], ""));
            }
            patient.setLocation(hl);
            patient.setInpatient(true);
        }
        patient.setCwad(fields[7]);
        if (fields[8].equals("1"))
        {
        	patient.setRestricted(true);
        } 
        if (fields[9] != "")
        {
            //patient.AdmitTimestamp = VistaTimestamp.toUtcString(fields[9]);
        }
        if (fields[11].equals("1"))
        {
        	patient.setServiceConnected(true);
        } 
        if (fields[12] != "")
        {
            //patient.ScPercent = Convert.ToInt16(fields[12]);
        }
        patient.setMpiPid(fields[13]);
        if (!RPCStringUtils.isEmpty(fields[14]))
        {
            //patient.setAge(fields[14]);
        }        
        
        patient.setVistaLocations(getSiteIDs(patient.getLocalPid(), siteId, queryBean, vistaLinkCommand));      

        return patient;
    }	 
	 
	    private String[] getSiteIDs(String dfn, String siteId, QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
	    {
	    	ArrayList<String> result = new ArrayList<String>();
	    	result.add(siteId);
	    	
	      /*  String request = buildGetRemoteSitesRequest(dfn);
	        String response = (String) cxn.query(request);
	        if (response.startsWith("-1^"))
	        {
	            ArrayList<String> sites = new ArrayList<String>();
	            sites.add((String) cxn.getSiteId().getKey());
	            return sites.toArray(new String[0]);
	        }
	        String[] lines = StringUtils.split(response, StringUtils.CRLF);
	        lines = StringUtils.trimArray(lines);
	        ArrayList<String> result = new ArrayList<String>();
	        result.add((String)cxn.getSiteId().getKey());
	        for (int i = 0, j = 1; i < lines.length; i++, j++)
	        {
	            String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
	            //SB: do not include station 200 in patient site id list
	            if (flds[0].equals("200"))
	            {
	                continue;
	            }
	            result.add(flds[0]);
	        }*/
	        return result.toArray(new String[0]);
	    }

	    /*protected String buildGetRemoteSitesRequest(String dfn)
	    {
	        VistaQuery vq = new VistaQuery("ORWCIRN FACLIST");
	        vq.addParameter(vq.LITERAL, dfn);
	        return vq.buildMessage();
	    }*/

}