package gov.va.med.via.response;

import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.databeans.Team;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcReferenceType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.lang.StringUtils;

// TODO: Auto-generated Javadoc
/**
 * The Class PatientDelegate.
 * 
 * @author DOMAINROMERJ
 */
public class PatientDelegate
{	
	/** The query bean. */
	private QueryBean queryBean;
	
	/** The site id. */
	private String siteId;
	
	/** The vista link command. */
	private VistaLinkCommand vistaLinkCommand;
	
	/**
	 * Instantiates a new patient delegate.
	 *
	 * @param patient the patient
	 * @param queryBean the query bean
	 * @param siteId the sitwhe id
	 * @param vistaLinkCommand the vista link command
	 */
	public PatientDelegate(QueryBean queryBean, String siteId, VistaLinkCommand vistaLinkCommand)
	{
		this.queryBean = queryBean;
		this.siteId = siteId;
		this.vistaLinkCommand = vistaLinkCommand;
	}		
          
	 /**
 	 * Sets the value of the attribute "needsMeansTest" on the patient received.
 	 * 
 	 * @param Patient - the patient to be updated with the needsMeansTest value
 	 * 
 	 * @returns patient's needsMeansTest attribute updated if successful.
 	 */
 	public void addNeedsMeansTest(Patient patient)
     {
		 if (patient.getLocalPid() == null) return;
		 
	 	List<Object> params =new ArrayList<Object>();		
	 	HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",2);
 	    hashMap.put("FIELDS",".14;.6");		//current test status; test patient indicator
 		hashMap.put("FLAGS","IP");
 		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(patient.getLocalPid()));
 		hashMap.put("PART",patient.getLocalPid());
 		hashMap.put("MAX",1);
 		hashMap.put("XREF","#");
 		params.add(hashMap);
 		String 	scresponse = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);

 		scresponse = RPCStringUtils.replaceEndofLineCharacters(scresponse);  
 		String[] lines = RPCStringUtils.split(scresponse, RPCStringUtils.CRLF);	 
		for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]") || lines[i].contains("More") || RPCStringUtils.isEmpty(lines[i])) continue;
	    	
        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (fields != null && patient.getLocalPid().equals(fields[0]))
            {
            	patient.setNeedsMeansTest(fields.length > 1 && fields[1] != null? fields[1].equals("1") : false);
    	        patient.setTestPatient(fields.length > 2 && fields[2] != null? fields[2].equals("1") : false);
    	        break;
            }
        }
    }
	
	/**
	 * Adds the patient id info (patient type, room, bed) from Vista to the patient instance.
	 * 
	 * @param the patient to be updated with the room and bed data
	 * 
	 * @return patient's type, room and bed attributes populated if successful.
	 */
	public void addPatientIdInfo(Patient patient)
	{		
		if (patient.getLocalPid() == null) return;
		
    	String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT16_ID_INFO, patient.getLocalPid());
    	
	    if (!response.equals(""))
	     {
	        String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
	        patient.setPatientType(fields.length >=6 && fields[5] != null ? fields[5] : null);
	        if (fields.length >=7 && fields[7] != null && !fields[7].equals(""))
	        {
	        	String[] roomBed = RPCStringUtils.split(fields[7], "-");
	        	HospitalLocation location = patient.getLocation();
	        	if (location != null && roomBed != null && roomBed.length >= 2)
	        	{
	        		location.setRoom(roomBed[0]);
	        		location.setBed(roomBed[1]);
	        	}
	        }
	     }
	}
	
    /**
     * Adds the active insurance information from Vista to the patient instance.
     * 
     * @param the patient to be updated with the active insurance information.
     * 
     * @returns patient's activeInsurance attribute populated if successful
     */
    public  void addActiveInsurance(Patient patient)
    {
        if (patient.getLocalPid() == null) return;

        String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORVAA_ACTIVE_INSURANCE, patient.getLocalPid());  
        result = RPCStringUtils.replaceEndofLineCharacters(result);                
        patient.setActiveInsurance(result !=  null ? result.trim() : null);	    
    }
    
    
    /**
     * Adds the deceased date if applicable to the patient instance.
     * 
     * @param the patient to be updated with the deceased date.
     * 
     * @returns patient's deceased date populated if successful
     */
    public  void addDeceasedDate(Patient patient)
    {
        if (patient.getLocalPid() == null) return;

        String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT_DECEASED_DATE, patient.getLocalPid());
        result = RPCStringUtils.replaceEndofLineCharacters(result); 
        String[] fields = RPCStringUtils.split(result, RPCStringUtils.CRLF);
        patient.setDeceasedDate(fields[0] ==  null || fields[0].equals("0") ? null : Timestamp.toUtcString(fields[0]));
    }      
    
    /**
     * Adds checksum and locally assigned MPI PID to the patient and the CmorSiteId
     * 
     * @param patient - the patient to be updated with the checksum and locally assigned mpi pid flag and cmor site Id.
     * 
     * @param siteId - the current site being processed
     * 
     * @returns patient's "mpiChecksum" and "locallyAssignedMpiPid" and "cmorSiteId"  attributes populated if successful.
     */
    public void addSCListerNodeMPI(Patient patient, String siteId)
    {
    	
    	/*
    	 * Retrieves Patient information
    	 * FILE 2 - result position - field number and field definitions
    	 * POSITION								FIELD NUMBER
    	 * 0:IEN								.01 (NOTE: field .01 seems to return IEN and NAME in two fields)
    	 * 1:NAME
    	 * 2:ICN								991.01
    	 * 3:CHECKSUM							991.02
    	 * 4:CMOR SITE ID						991.03
    	 * 5:Locally Assigned ICN				991.04
    	 * 
    	 */
    	
    	if (patient.getLocalPid() == null) return;
    	
    	List<Object> params =new ArrayList<Object>();		
 		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",2);
 	    hashMap.put("FIELDS",".01;991.01;991.02;991.03;991.04");
 		hashMap.put("FLAGS","IP");
 		hashMap.put("FROM",RPCUtils.adjustForNameSearch(patient.getName()));
 		hashMap.put("PART",patient.getName());
 		hashMap.put("XREF","B");
 		 		 		
 		params.add(hashMap); 

 		String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	    if (response.equals("")) return;
	    response = RPCStringUtils.replaceEndofLineCharacters(response);  
	    String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
	    for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]")) continue;
        	if (StringUtils.isEmpty(lines[i])) continue;

        	String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        	//ensure the record contains the IEN (numeric value) and that the IEN is the same as our patient's PID (dfn)
           if (RPCStringUtils.isNumeric(flds[0]) && patient.getLocalPid().equals(flds[0]))
           {
            	patient.setMpiPid(flds[2]);
            	patient.setMpiChecksum(flds[3]);
            	patient.setCmorSiteId(flds[4]);
            	boolean assignedValue = determineAssignedValue(flds[5], siteId, flds[2]);
            	patient.setLocallyAssignedMpiPid(assignedValue);
            }
        }
    }
    
    /**
     * Method needs to determine if an mpiPid (ICN) has been locallyAssigned - the value returned from the RPC may or may not have been populated
     * so some logic needs to be applied.  All locally assigned mpiPids will begin with the local siteId 3 character code - or the value in the locallyAssigned 
     * field will be set to 0 or 1 or N or Y
     *  
     * @param assigned
     * @param siteId
     * @param mpiPid
     * @return - String representing a yes or no decision
     */
    private boolean determineAssignedValue(String assigned, String siteId, String mpiPid) {
		
    	boolean assignedValue = false;
		
    	//the first condition should be enough to assign true
    	if(mpiPid.startsWith(siteId) || assigned.equals("1") || assigned.equals("Y") ){
    		assignedValue = true;
    	}
		return assignedValue;
	}

	/**
     * Adds the confidentiality to the patient received
     * 
     * @param the patient to be updated with the confidentiality data.
     * 
     * @returns patient's confidentiality attributes populated if successful
     */
    public void addConfidentiality(Patient patient)
    {
       	if (patient.getLocalPid() == null) return;
		 
        String response = vistaLinkCommand.executeRpc(RpcNameConstants.DG_SENSITIVE_RECORD_ACCESS, patient.getLocalPid());
	    response = RPCStringUtils.replaceEndofLineCharacters(response); 
    	String[] fields = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        if (fields[0].equals("") || fields[0].equals("0")) return; // TODO can response start with -1 error?

        try
        {
        	int level = (new Integer(response)).intValue();
            int p = response.indexOf("\r\n");
            String message = response.substring(p + 2);
            KeyValuePair<Integer, String> confidentialityPair = new KeyValuePair<Integer, String>();
            confidentialityPair.setKey(new Integer(level));
            confidentialityPair.setValue(message);
            patient.setConfidentiality(confidentialityPair);        	
        }
        catch (Exception e)
        {
        	return; // TODO  bad response
        }  	    
    }    
    
    /**
     * Adds the patient flags to the patient received.
     * 
     * @param the patient to be updated with the patient flags data.
     * 
     * @returns patient's flags attributes populated if successful
     */
    public void addPatientFlags(Patient patient)
    {
       	if (patient.getLocalPid() == null) return;
		 
        String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORPRF_HASFLG, patient.getLocalPid());
	    response = RPCStringUtils.replaceEndofLineCharacters(response); 
    	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        if (lines[0].equals("") || lines[0].equals("0")) return; // TODO can response start with -1 error?

        HashMap<String,String> flags = new HashMap<String,String>();  
        for (int i = 0; i < lines.length; i++)
        {
            if (lines[i].equals("")) continue;

            String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            flags.put(fields[0], fields[1]);
        }
        
        patient.setPatientFlags(flags);
    }
    
    /**
     * Adds the team data to the patient received.
     * 
     * @param the patient to be updated with the team data.
     * 
     * @returns patient's Team attributes populated if successful.
     */
    public void addTeam(Patient patient)
    {
       	if (patient.getLocalPid() == null) return;
		 
        String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT1_PRCARE, patient.getLocalPid());
	    response = RPCStringUtils.replaceEndofLineCharacters(response);
	    if (response.equals("^^^")) return;
	    
	    String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
	    Team team = new Team();
	    if (fields.length >= 1) team.setId(fields[0]);
	    if (fields.length >= 2) team.setName(fields[1]);
	    if (fields.length >= 3) team.setPcpName(fields[2]);
	    if (fields.length >= 4) team.setAttendingName(fields[3]);		    
	    patient.setTeam(team);
    }
    
    
    /**
     * Adds the site ids.
     *
     * @param patient the patient
     * @param siteId the site id
     */
    public void addSiteIds(Patient patient, String siteId)
    {
       	if (patient.getLocalPid() == null) return;
     
       	String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWCIRN_FACLIST, patient.getLocalPid());
       	SiteId site = new SiteId(siteId); 
       	try
       	{
       		site.setName(SiteTable.getInstance().getSiteName(siteId));
       	}
       	catch(SiteException e)
       	{
       		// Ignore it, site will not have name. Should never happen if valid site.
       	}
       	
   		HashMap<String, SiteId> siteMap = new HashMap<String, SiteId>();
   		siteMap.put(siteId, site); // Add connected site to site hash map to avoid duplicating if it is returned back.

   		if (!response.startsWith("-1^"))
       	{       		
       		response = RPCStringUtils.replaceEndofLineCharacters(response); 
       		String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
       		for (int i = 0; i < lines.length; i++)
            {
       			String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
       			if (fields.length > 3)
       			{
       				SiteId newSite = new SiteId(fields[0],fields[1],fields[2],fields[3]);
       				if (siteMap.containsKey(newSite.getId())) 
       				{
       					continue;
       				}
       				siteMap.put(newSite.getId(), newSite);
       			}
            }
       	}
       	
       	SiteId[] sites = new SiteId[siteMap.size()];       	
       	siteMap.values().toArray(sites);
       	
       	patient.setSiteIDs(Arrays.asList((SiteId[])sites));
    }
}
