package gov.va.med.via.response;

import gov.va.med.via.databeans.Address;
import gov.va.med.via.databeans.DemographicSet;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PhoneNum;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

// TODO: Auto-generated Javadoc
/**
 * The Class PatientDemographicsDelegate.
 * 
 * @author DOMAINROMERJ
 */
public class PatientDemographicsDelegate
{
	
	/** The query bean. */
	private QueryBean queryBean;
	
	/** The site id. */
	private String siteId;
	
	/** The vista link command. */
	private VistaLinkCommand vistaLinkCommand;
	
	/**
	 * Instantiates a new patient demographics delegate.
	 *
	 * @param queryBean the query bean
	 * @param siteId the site id
	 * @param vistaLinkCommand the vista link command
	 */
	public PatientDemographicsDelegate(QueryBean queryBean, String siteId, VistaLinkCommand vistaLinkCommand)
	{
		this.queryBean = queryBean;
		this.siteId = siteId;
		this.vistaLinkCommand = vistaLinkCommand;
	}
		
	/**
	 * Adds the demographics information for the patient.
	 *
	 * @param patient the patient
	 */
	public void addDemographics(Patient patient)
	{		 
		if (patient.getLocalPid() == null) return;
		
		HashMap<String, DemographicSet> demographicsMap = getDemographics(patient, false); // false - return County Number		
		patient.setDemographics(demographicsMap);
	}

	/**
	 * Adds Home Data to the demographics information for the patient.
	 *
	 * @param patient the patient
	 */
	public void addHomeData(Patient patient)
	{		 
		if (patient.getLocalPid() == null) return;
		
		HashMap<String, DemographicSet> demographicsMap = getDemographics(patient, true); // true - return County Name
		
		for (Iterator<String> i = demographicsMap.keySet().iterator(); i.hasNext(); )
		{
			String tag = (String)i.next();
 			DemographicSet demographicsBean = demographicsMap.get(tag);
 			// Set Home Address for Patient
 			Address homeAddress = demographicsBean.getAddresses().get(0);
 			patient.setHomeAddress(homeAddress);
 			// Set Home Phones for Patient
 			List<PhoneNum> phoneList = demographicsBean.getPhones();
 			for (PhoneNum phone: phoneList)
 			{
 				if ("Home Phone".equals(phone.getDescription()))
 				{
 					patient.setHomePhone(phone);
 				}
 			}
		}		
	}


	/**
	 * Gets the demographics for the patient. If the translateCountyName is true, retrieve the name of the county;
	 * otherwise return the County number.
	 *
	 * @param patient the patient that need demographic information
	 * @param useCountyName the use county name
	 * @return the demographics - patient demographics for the Patient DFN received in the patient object.
	 */
	private HashMap<String, DemographicSet> getDemographics(Patient patient, boolean useCountyName)
	{
		if (patient.getLocalPid() == null) return null;
		
		HashMap<String, DemographicSet> demographicsMap = new HashMap<String, DemographicSet>();        
		List<Object> params =new ArrayList<Object>();		
 		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",2);
 		hashMap.put("FIELDS",".01;.06;.05;.111;.112;.114;.115;.116;.117;.131;.132"); // Race, Marital Status, Street Address, Street address 2, city, state, zip code, county; home phone; work phone
 		hashMap.put("FLAGS","IP");
 		hashMap.put("FROM",RPCUtils.adjustForNameSearch(patient.getName()));
 		hashMap.put("PART",patient.getName());
 	    hashMap.put("XREF","B"); 	    	 		 	
 		params.add(hashMap); 
 		String scResponse = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params); 		
 		
 		scResponse = RPCStringUtils.replaceEndofLineCharacters(scResponse);  
	    String[] sclines = RPCStringUtils.split(scResponse, RPCStringUtils.CRLF);	        
        for (int j = 0; j < sclines.length; j++)
        {    
	    	if (sclines[j].contains("[Data]") || RPCStringUtils.isEmpty(sclines[j])) continue;
	    	        	
        	String[] fields = RPCStringUtils.split(sclines[j], RPCStringUtils.CARET);
        	if (fields != null && fields.length > 11 && patient.getName().equals(fields[1]))
        	{
        		DemographicSet demographics = new DemographicSet();        		
        		patient.setEthnicity(getRace(fields[2]));
        		patient.setMaritalStatus(getMaritalStatus(fields[3]));        		
        		
        		List<Address> addresList = new ArrayList<Address>(); 
    			Address address = new Address();
    			address.setStreetAddress1(RPCStringUtils.isEmpty(fields[4]) ? null : fields[4]);
    			address.setStreetAddress2(RPCStringUtils.isEmpty(fields[5]) ? null : fields[5]);
    			address.setCity(RPCStringUtils.isEmpty(fields[6]) ? null : fields[6]);			
    			address.setState(getStateName(fields[7]));
   				address.setZipcode(fields[8]);
   				address.setCounty(useCountyName? getCountyName(fields[7], fields[9]) : fields[9]);
    			addresList.add(address);
    			demographics.setAddresses(addresList);
    			
    			List<PhoneNum> phoneList = new ArrayList<PhoneNum>();
    			PhoneNum homePhone = new PhoneNum();
    			homePhone.setDescription("Home Phone");
    			if (!RPCStringUtils.isNullOrEmpty(fields[10]))
    			{
    			   	String[] homePhoneFields = RPCStringUtils.split(fields[10], RPCStringUtils.DASH);
    			   	if (homePhoneFields != null && homePhoneFields.length >= 2)
    			   	{
    			   		// homePhoneFields[0]  must have a format (XXX) XXX and homePhoneFields[1] must contain the last 4 digits of the number 
    			   		String areaExchange[] = RPCStringUtils.split(homePhoneFields[0], RPCStringUtils.SPACE);
    			   		homePhone.setAreaCode(areaExchange != null && areaExchange.length >= 1 ? areaExchange[0]: null);
    			   		homePhone.setExchange(areaExchange != null && areaExchange.length >= 2 ? areaExchange[1]: null);
    			   		homePhone.setNumber(homePhoneFields != null && homePhoneFields.length >= 2 ? homePhoneFields[1]: null);
    			   	}
    			}
    			phoneList.add(homePhone);
    			
    			PhoneNum workPhone = new PhoneNum();
    			workPhone.setDescription("Work Phone");
    			if (!RPCStringUtils.isEmpty(fields[11]))
    			{
			    	String[] workPhoneFields = RPCStringUtils.split(fields[11], RPCStringUtils.DASH);
			    	if (workPhoneFields != null && workPhoneFields.length >= 2)
			    	{
			    		// workPhoneFields[0]  must have a format (XXX) XXX and workPhoneFields[1] must contain the last 4 digits of the number  
			    		String areaExchange[] = RPCStringUtils.split(workPhoneFields[0], RPCStringUtils.SPACE);
			    		workPhone.setAreaCode(areaExchange != null && areaExchange.length >= 1 ? areaExchange[0]: null);
			    		workPhone.setExchange(areaExchange != null && areaExchange.length >= 2 ? areaExchange[1]: null);
			    		workPhone.setNumber(workPhoneFields != null && workPhoneFields.length >= 3 ? workPhoneFields[2]: null);
			    	}
    			 }            
    			phoneList.add(workPhone);
    			demographics.setPhones(phoneList);
    			demographicsMap.put(siteId, demographics);
    			break;
        	} 
        }
		
        // Retrieve isVeteran indicator
 		String responseVeteran = vistaLinkCommand.executeRpc(RpcNameConstants.ORWPT_ID_INFO, patient.getLocalPid());
 		if (!RPCStringUtils.isNullOrEmpty(responseVeteran))
		{
			String[] fields = RPCStringUtils.split(responseVeteran, RPCStringUtils.CARET);
			if (!RPCStringUtils.isNullOrEmpty(fields[3]))			// Field #3 contains value for isVeteran
			{
				patient.setVeteran(fields[3].equals("Y") ? new Boolean(true): new Boolean(false));
			}
		}
		 
		return demographicsMap;
	}

	/**
	 * Gets the race.
	 *
	 * @param raceIEN the race ien
	 * @return the race
	 */
	private String getRace(String raceIEN)
	{
		if (RPCStringUtils.isNullOrEmpty(raceIEN))
			return null;
		
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("FILE",10);
		hashMap.put("FIELDS",".01"); 
		hashMap.put("FLAGS","IP");
		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(raceIEN));
	    hashMap.put("XREF","#"); 	    	 		 	
		params.add(hashMap); 
		String raceStatus = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
		raceStatus = RPCStringUtils.replaceEndofLineCharacters(raceStatus);  
		String[] lines = RPCStringUtils.split(raceStatus, RPCStringUtils.CRLF);	 
		for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]") || RPCStringUtils.isEmpty(lines[i])) continue;
	    	
        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (fields != null && fields.length > 1 && raceIEN.equals(fields[0]))
            {
       		   return fields[1];
           }
        }

		return null;		
	}

	/**
	 * Gets the marital status.
	 *
	 * @param maritalStatusIEN the marital status ien
	 * @return the marital status
	 */
	private String getMaritalStatus(String maritalStatusIEN)
	{
		if (RPCStringUtils.isNullOrEmpty(maritalStatusIEN))
			return null;
				
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("FILE",11);
		hashMap.put("FIELDS",".01;1"); 
		hashMap.put("FLAGS","IP");
		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(maritalStatusIEN));
	    hashMap.put("XREF","#"); 	    	 		 	
		params.add(hashMap); 
		String maritalStatus = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
		maritalStatus = RPCStringUtils.replaceEndofLineCharacters(maritalStatus);  
		String[] lines = RPCStringUtils.split(maritalStatus, RPCStringUtils.CRLF);	 
		for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]") || RPCStringUtils.isEmpty(lines[i])) continue;
	    	
        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (fields != null && fields.length > 1 && maritalStatusIEN.equals(fields[0]))
            {
       		   return fields[1];
           }
        }
		
		return null;
	}
	
	/**
	 * Gets the county name.
	 *
	 * @param stateIEN the state ien
	 * @param countyIEN the county ien
	 * @return the county name
	 */
	private String getCountyName(String stateIEN, String countyIEN)
	{
		if (RPCStringUtils.isNullOrEmpty(countyIEN) || RPCStringUtils.isNullOrEmpty(stateIEN))
			return null;
		
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",5.01);
 		hashMap.put("FIELDS",".01"); 
 		hashMap.put("FLAGS","IP");
 		hashMap.put("IENS","," + stateIEN + ",");
 		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(countyIEN));
 	    hashMap.put("XREF","#"); 	    	 		 	
 		params.add(hashMap); 
 		String countyResponse = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
 		countyResponse = RPCStringUtils.replaceEndofLineCharacters(countyResponse);  
 		String[] lines = RPCStringUtils.split(countyResponse, RPCStringUtils.CRLF);	 
		for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]") || RPCStringUtils.isEmpty(lines[i])) continue;

        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (fields != null && fields.length > 1 && countyIEN.equals(fields[0]))
            {
       		   return fields[1];
           }
        }
			
		return null;
	}
			
	/**
	 * Gets the state name.
	 *
	 * @param stateIEN the state ien
	 * @return the state name
	 */
	private String getStateName(String stateIEN)
	{
		if (RPCStringUtils.isNullOrEmpty(stateIEN))
			return null;
	
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
 		hashMap.put("FILE",5);
 		hashMap.put("FIELDS",".01"); 
 		hashMap.put("FLAGS","IP");
 		hashMap.put("FROM",RPCUtils.adjustForNumericSearch(stateIEN));
 	    hashMap.put("XREF","#"); 	    	 		 	
 		params.add(hashMap); 
 		String stateResponse = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
 		
 		stateResponse = RPCStringUtils.replaceEndofLineCharacters(stateResponse);  
 		String[] lines = RPCStringUtils.split(stateResponse, RPCStringUtils.CRLF);	 
		for (int i = 0; i < lines.length; i++)
        {    
	    	if (lines[i].contains("[Data]") || RPCStringUtils.isEmpty(lines[i])) continue;
	    	
        	String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (fields != null && fields.length > 1 && stateIEN.equals(fields[0]))
            {
       		   return fields[1];
           }
        }
		
		return null;
	}
}
