package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.PatientRecordFlag;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.Arrays;
import java.util.List;

public class PatientRecordFlagParser extends BaseDataBeanParser 
{
	private String noteIEN;
	
	
	public PatientRecordFlagParser() 
	{
	}

	/** This implementation just needs the response argument.  Before this method is invoked, the noteIEN attribute must be set.
	 * 
	 * @return a list of PatientRecordFlag[]
	 */
	@Override
	public List<DataBean> populateDataBeans(String response,
			QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) 
	{
        PatientRecordFlag[] patientRecordFlags = toPatientRecordFlags(response); 
        return Arrays.asList((DataBean[])patientRecordFlags);
	}
	
	public PatientRecordFlag[] toPatientRecordFlags( String response)
	{
		if ( noteIEN==null )
		{
			throw new IllegalStateException("The noteIEN attribute of the PatientRecordFlagParser must be set prior to invoking this method");
		}

		if (StringUtil.isEmpty(response))
        {
            return null;
        }
		
        String[] lines = StringUtil.split(response, StringUtil.CRLF);
        lines = StringUtil.trimArray(lines);
        PatientRecordFlag[] patientRecordFlags = new PatientRecordFlag[lines.length];
        for (int i = 0; i < lines.length; i++)
        {
            patientRecordFlags[i] = toPatientRecordFlag(lines[i]);
            patientRecordFlags[i].setNoteId( noteIEN );
        }
        return patientRecordFlags;
		
	}
	
	public PatientRecordFlag toPatientRecordFlag(String line)
    {
        String[] flds = StringUtil.split(line, StringUtil.CARET);
        if (flds.length < 5)
        {
            return null;
        }
        PatientRecordFlag result = new PatientRecordFlag();
        result.setId( flds[1]);
        result.setName(flds[0]);
        result.setActionId(flds[3]);
        result.setActionName(flds[2]);
        result.setActionTimestamp( VistaTimestamp.toUtcString(flds[4]));
        return result;
    }

	public String getNoteIEN() {
		return noteIEN;
	}

	public void setNoteIEN(String noteIEN) {
		this.noteIEN = noteIEN;
	}

}
