package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PatientSelectByIdDataBeanParser extends BaseDataBeanParser
{	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toPatients(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * To note type response.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return the text[]
	 */
	protected Patient[] toPatients(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<Patient> responseArray = new ArrayList<Patient>();
		Patient patient = new Patient();
		
        if (response == null|| response.equals("") || response.contains("No appointments") || response.contains("No patients found")) 
        {
        	patient.setFault(new Fault("Cannot retrieve patients using the Id provided"));
        	responseArray.add(patient);
        }
        else
        {
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	lines = RPCStringUtils.trimArray(lines);
        	for (int i = 0; i < lines.length; i++)
        	{
        		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        		patient = new Patient();
        		patient.setLocalPid(fields[0]);
        		patient.setName(fields[1]);
        		//  Only GetPatientsByWard and GetPatientsByClinic contain Hospital Location information
        		if (ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYWARD.equals(webMethodName) ||
        			ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYWARD.equals(webMethodName) ||
        			ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYCLINIC.equals(webMethodName) ||
        			ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYCLINIC.equals(webMethodName))
        		{
	        		if (fields.length > 2 && !fields[2].equals(""))
	        		{
	        			String[] parts = RPCStringUtils.split(fields[2], "-");
	        			HospitalLocation location = new HospitalLocation();
	        			if (ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYWARD.equals(webMethodName) ||
	        				ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYWARD.equals(webMethodName))
	        			{
	        				location.setRoom(parts[0]);
	        				location.setBed(parts[1]);
	        				patient.setInpatient(new Boolean(true));
	        			}
	        			if (ViaWebMethodNameConstants.TBISVC_GETPATIENTSBYCLINIC.equals(webMethodName) ||
	        				ViaWebMethodNameConstants.EMRSVC_GETPATIENTSBYCLINIC.equals(webMethodName))
	        			{
	        				location.setId(fields[2]);
	        				location.setName(fields.length > 8 && !fields[8].equals("")? fields[8] : null);
	        				location.setAppointmentTimestamp(fields.length > 3 && !fields[3].equals("")? Timestamp.toUtcString(fields[3]) : null);
	        				patient.setInpatient(new Boolean(false));
	        			}
	        			patient.setLocation(location);	        			
	        		}
        		}
        	    responseArray.add(patient);
        	}
        }
        
        return responseArray.toArray(new Patient[0]);
	}
}

