package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

public class PcpForPatientDataBeanParser extends BaseDataBeanParser {


	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) {
		
		return Arrays.asList((DataBean[])toTextTO(response, vistaLinkCommand));
	}
	
	/**
	 * Parses the RPC ORWPT PCDETAIL response and populates an array of DataBean objects - one object
	 * method must make a second call back to VistA using the SC LISTER to get the second piece of data - the IEN of the PCP
	 * 
	 * @param response - The response from RPC ORWPT PCDETAIL
	 * @return an array of Text DataBeans populated from the content stored in the RPC response string and the callback response string
	 */
	protected Text[] toTextTO(String response, VistaLinkCommand vistaLinkCommand)
    {        
		if (RPCStringUtils.isNullOrEmpty(response)){
            return new Text[0];
        }
		
		//check end of line characters - standardize to linux format
		response = RPCStringUtils.replaceEndofLineCharacters(response); 
	    String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
		Text text = new Text();
		if( lines.length == 1){
			
			 //our data is in the 4th line - 2nd field 
			 String[] flds = RPCStringUtils.split(lines[0], RPCStringUtils.CARET);
			 String pcpName = flds[1];
			
			 //now call SC Lister
			 List<Object> params =new ArrayList<Object>();          
			
			 HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			 hashMap.put("FILE",200);
			 hashMap.put("FIELDS",".01;53.5;53.6");
			 hashMap.put("FLAGS","IP");
			 hashMap.put("FROM",RPCUtils.adjustForNameSearch(pcpName));
			 hashMap.put("PART",pcpName);
			 hashMap.put("XREF","B");
					    
			 params.add(hashMap);  
			 String scListerResult = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
			 scListerResult = RPCStringUtils.replaceEndofLineCharacters(scListerResult); 
			 String[] scLines = RPCStringUtils.split(scListerResult, RPCStringUtils.CRLF);
			 
			 if(scLines.length >1){
				 String[] scFlds = RPCStringUtils.split(scLines[1], RPCStringUtils.CARET);
				 text.setTag(scFlds[0]);
				 text.setText(scFlds[1]);
			 }
			
		}
		
		ArrayList<Text> lst = new ArrayList<Text>();
		lst.add(text);
		return (Text[])lst.toArray(new Text[lst.size()]);
       
    }	

}
