package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.PatientRecordFlag;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.List;

public class PrfNoteActionsParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{	
		
/*		parses an HL7 string like:
		HIGH RISK FOR SUICIDE^1319^NEW ASSIGNMENT^2134^3081107.135616^NOV 07, 2008@13:56:16^3733434
		HIGH RISK FOR SUICIDE^1319^INACTIVATE^2780^3090921.110708^SEP 21, 2009@11:07:08^
*/		
		
		String noteDefinitionIEN = queryBean.getItemId();
		return toPrfs( response, noteDefinitionIEN);
	}
	
	private List<DataBean> toPrfs(String response, String noteDefinitionIEN)
    {
		List<DataBean> prfList = new ArrayList<DataBean>();
        if (StringUtil.isEmpty(response))
        {
            return prfList;
        }
        String[] lines =  response.split("\\n");
        lines = StringUtil.trimArray(lines);
        
        for (String line : lines)
        {
        	line = line.trim();
        	PatientRecordFlag prf = toPrf(line);
        	if (prf==null ) continue;
        	prf.setNoteId(noteDefinitionIEN);
        	prfList.add(prf);
        }
        return prfList;
    }

	
	
	/** parses an HL7 string into a PatientRecordFlag
	 * 
	 * @param s a string like: HIGH RISK FOR SUICIDE^1319^INACTIVATE^2780^3090921.110708^SEP 21, 2009@11:07:08^
	 * @return PatientRecordFlag
	 */
	private PatientRecordFlag toPrf(String s)
    {
        String[] flds = s.split("\\^");
        if (flds.length < 5)
        {
            return null;
        }
        PatientRecordFlag result = new PatientRecordFlag();
        result.setName(flds[0]);
        result.setId(flds[1]);        
        result.setActionName(flds[2]);
        result.setActionId(flds[3]);        
        result.setActionTimestamp( VistaTimestamp.toUtcString(flds[4]));
        return result;
    }
	
	
	
	
	
}
