package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.ProstheticClaim;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * Class that parses the prosthetic claims response.
 * 
 * @author ZZZZZZAPOSTI
 * 
 */
public class ProstheticClaimsParser extends BaseDataBeanParser {
	
	private VistaLinkCommand vistaLinkCommand = null;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang
	 * .String, gov.va.med.via.databeans.QueryBean, java.lang.String,
	 * java.lang.String, java.lang.String,
	 * gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response,
			QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) {
		
		this.vistaLinkCommand = vistaLinkCommand;
		
		return Arrays.asList((DataBean[]) toProstheticClaims(response,
				queryBean));
	}


	private List<ProstheticClaim> getProstheticClaims(String response) {
		
		List<ProstheticClaim> prostheticClaims = new ArrayList<ProstheticClaim>();
		
		ProstheticClaim prostheticClaim = null;
		
		boolean hasProstheticId = false;
		
		if (RPCStringUtils.isNullOrEmpty(response)) {
			return prostheticClaims;
		}
		
		response = RPCStringUtils.replaceEndofLineCharacters(response);
						
		String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

		for (int i = 0; i < lines.length; i++) {
			
			String currentLine = lines[i];
			
			if (StringUtil.isNullOrEmpty(currentLine)
					|| currentLine.contains("[Data]")
					|| currentLine.contains("[Misc]")
					|| currentLine.contains("MORE")) {
				continue;
			}

   			String[] flds = RPCStringUtils.split(currentLine, RPCStringUtils.CARET);

			if (flds.length == 0) {
				continue;
			}

			if (flds != null && flds.length > 0) {
		
				prostheticClaim = new ProstheticClaim();	

				prostheticClaim.setId(flds[0]);
				
				if (flds.length > 1) {

					prostheticClaim.setPatientId(RPCStringUtils.isEmpty(flds[1]) ? null
							: flds[1]);
				}

				if (flds.length > 2) {
					
					prostheticClaim.setPatientName(RPCStringUtils.isEmpty(flds[2]) ? null
									: flds[2]);

				}

				if (flds.length > 3) {
					
					String episodeDate = null;
					
					if (!StringUtil.isNullOrEmpty(flds[3])) {
						
						episodeDate = flds[3];
						prostheticClaim.setEpisodeDate(VistaTimestamp.toUtcString(episodeDate));
						
					} else {
						
						prostheticClaim.setEpisodeDate(episodeDate);
					}
			
				}

				if (flds.length > 4) {
					
					hasProstheticId = !RPCStringUtils.isEmpty(flds[4]);
					
					if (hasProstheticId) {
						prostheticClaim.setProstheticId(flds[4]);
					} 				
					
				}

				if (flds.length > 5) {
					
					String timeStamp = null;
					
					if (!StringUtil.isNullOrEmpty(flds[5])) {
						
						timeStamp = flds[5];
						prostheticClaim.setTimestamp(VistaTimestamp
								.toUtcString(timeStamp));
						
					} else {
						
						prostheticClaim.setTimestamp(timeStamp);
					}
										
				} 

				if (flds.length > 6) {
					
					String lastEditTimeStamp = null;
					
					if (!StringUtil.isNullOrEmpty(flds[6])) {
						
						lastEditTimeStamp = flds[6];
						prostheticClaim.setLastEditTimestamp(VistaTimestamp.toUtcString(lastEditTimeStamp));
						
					} else {
						
						prostheticClaim.setLastEditTimestamp(lastEditTimeStamp);
					}

				}
	
				if (flds.length > 1 && hasProstheticId) {
					prostheticClaims.add(prostheticClaim);
				}
			}
		}		
		
		
		return prostheticClaims;
	}
	
	/**
	 * Returns the populated toProstheticClaims array given the response.
	 * 
	 * @param response the response
	 * @param queryBean the query bean
	 * @return RatedDisability[] the array of RatedDisability
	 */
	protected ProstheticClaim[] toProstheticClaims(String response,
			QueryBean queryBean) {
		
		List<ProstheticClaim> allProstheticClaims = getProstheticClaims(response);
		
		List<ProstheticClaim> filteredProstheticClaims = new ArrayList<ProstheticClaim>(); 
		
		String[] dateArray = queryBean.getSupplementalParameters();
		
		List<String> dateList = Arrays.asList(dateArray);
		
		for (ProstheticClaim prostheticClaim: allProstheticClaims) {
			
			if (prostheticClaim.getEpisodeDate() != null) {
				
				String key = convertDate(prostheticClaim.getEpisodeDate());
				
				if (dateList.contains(key)) {

					addMoreProstheticClaimData(prostheticClaim);
					filteredProstheticClaims.add(prostheticClaim);
				}
			}		
		}
		
		return filteredProstheticClaims.toArray(new ProstheticClaim[0]);
	}

	/*
	 * Adds additional data to the claim.  In particular, the prosthetic cost, prosthetic name 
	 * and consult id. 
	 * @param claim The claim to augment.
	 */
	private void addMoreProstheticClaimData(ProstheticClaim claim) {

		String response = "";

		boolean hasHcpcs = false;

		if (claim != null && claim.getProstheticId() != null) {

			String prostheticId = claim.getProstheticId();

			/*
			 * http://livevista.caregraf.info/schema#!660 4.5 psas hcpcs 1;4
			 * POINTER 661.1 H Standardized Prosthetics HCPCS that points to
			 * file 661.1. 14 total cost(+) 0;16 NUMERIC This field contains the
			 * total cost of the transaction. 8.9 consult 10;9 POINTER 123 This
			 * is a pointer to Consult file #123.
			 * 
			 * 
			 * String arg = "$P($G(^RMPR(660," + prostheticId + ",1)),U,4)" +
			 * "_U_" + // 1:4 "$P($G(^RMPR(660," + prostheticId + ",0)),U,16)" +
			 * "_U_" + // 0:16 "$P($G(^RMPR(660," + prostheticId + ",10)),U,9)";
			 * // 10:9
			 */

			List<Object> params = new ArrayList<Object>();
			HashMap<Object, Object> hashMap = new HashMap<Object, Object>();
			hashMap.put("FILE", 660);
			hashMap.put("FIELDS", "@;4.5;14;8.9");
			hashMap.put("FLAGS", "IP");
			hashMap.put("XREF", "#");
			hashMap.put("MAX", "1");
			hashMap.put("FROM", RPCUtils.adjustForNumericSearch(prostheticId));
			hashMap.put("PART", prostheticId);
			params.add(hashMap);

			response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);

			response = RPCStringUtils.replaceEndofLineCharacters(response);

			String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

			for (int i = 0; i < lines.length; i++) {

				String currentLine = lines[i];

				if (StringUtil.isNullOrEmpty(currentLine) || currentLine.contains("[Data]")
						|| currentLine.contains("[Misc]") || currentLine.contains("MORE")) {
					continue;
				}

				String[] flds = RPCStringUtils.split(currentLine, RPCStringUtils.CARET);

				claim.setCost(flds[2]);
				claim.setConsultId(flds[3]);

				/*
				 * http://livevista.caregraf.info/schema#661_1 .02 short name
				 * 0;2 FREE TEXT C A short description for the Prosthetics HCPCS
				 * code as in CPT file #81.
				 * 
				 * arg = "$P($G(^RMPR(661.1," + flds[0] + ",0)),U,2)";
				 */
				hasHcpcs = !RPCStringUtils.isEmpty(flds[1]);

				if (hasHcpcs) {

					String hcpcs = flds[1];

					params = new ArrayList<Object>();
					hashMap = new HashMap<Object, Object>();
					hashMap.put("FILE", 661.1);
					hashMap.put("FIELDS", ".02");
					hashMap.put("FLAGS", "IP");
					hashMap.put("XREF", "#");
					hashMap.put("MAX", "1");
					hashMap.put("FROM", RPCUtils.adjustForNumericSearch(hcpcs));
					hashMap.put("PART", hcpcs);
					params.add(hashMap);

					response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);

					response = RPCStringUtils.replaceEndofLineCharacters(response);

					lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

					for (int y = 0; y < lines.length; y++) {

						currentLine = lines[y];

						if (StringUtil.isNullOrEmpty(currentLine) || currentLine.contains("[Data]")
								|| currentLine.contains("[Misc]") || currentLine.contains("MORE")) {
							continue;
						}

						flds = RPCStringUtils.split(currentLine, RPCStringUtils.CARET);

						claim.setProstheticName(RPCStringUtils.isEmpty(flds[1]) ? null
								: flds[1]);

					}
				}
			}
		}
	}
	
	/*
	 * Converts a vista date into Month/Day/Year format.
	 * @param episodeDate The date to convert.
	 * @return
	 */
	private String convertDate(String episodeDate) {

		String convertedDate = "";

		if (episodeDate != null) {

			int p = episodeDate.indexOf(RPCStringUtils.PERIOD);

			if (p == -1) {
				return convertedDate;
			}

			String strippedDate = episodeDate.substring(0, 8);

			if (strippedDate != null) {

				String year = strippedDate.substring(0, 4); // mdoDate.Substring(0,
															// 4).Substring(2);

				if (year != null) {

					year = year.substring(2);

				}

				String month = strippedDate.substring(4, 6); // mdoDate.Substring(4,
																// 2);

				if (month != null) {

					if (month.charAt(0) == '0') {

						month = month.substring(1);

					}
				}

				String day = strippedDate.substring(6);

				convertedDate = month + day + year;

			}
		}

		return convertedDate;
	}
}
