package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcReferenceType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Arrays;
import java.util.List;

public class ProviderUserInfoDataBeanParser extends BaseDataBeanParser 
{	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		
		return Arrays.asList((DataBean[])toProvider(response, dfn, siteId, queryBean, vistaLinkCommand));
	}
	

	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkCommand -VistaLinkCommand used to executer RPC calls thru VistaLink for additional data
	 * @return an array of Provider DataBeans populated from the content stored in the RPC response string
	 */
	protected Provider[] toProvider(String response, String dfn, String siteId, QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
    {
        if (response.equals(""))
        {
            return new Provider[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Provider> lst = new ArrayList<Provider>();
        Provider rec = null;
        //TODO - check that we have one - redundant but throw error and remove this for loop and remove the null check below - instantiat provider on line above line 52
        for (int i = 0; i < lines.length; i++)
        {    
            rec = new Provider();
        	String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            //TODO check that this value equals what we used to connect - redundant but safe
            String duzReturned = flds[0];//now check that it is equal to providr userId or throw an error - Duz error
            rec.setUserId(flds[0]);
            rec.setProviderName(flds[1]);
            rec.setLoginSiteName(flds[12]);
            rec.setLoginSiteCode(flds[23]);

            // Obtain SSN for provider
	        List<Object> params =new ArrayList<Object>();          
	        HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			hashMap.put("FILE",200);
			hashMap.put("FIELDS",".01;9");
			hashMap.put("FLAGS","IP");
			hashMap.put("MAX",1);
			hashMap.put("FROM",RPCUtils.adjustForNameSearch(rec.getProviderName()));
		    hashMap.put("PART",rec.getProviderName());
			hashMap.put("XREF","B");	        
	        params.add(hashMap);	  	        
	        String result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	        
	        result = RPCStringUtils.replaceEndofLineCharacters(result);  
		    String[] sclines = RPCStringUtils.split(result, RPCStringUtils.CRLF);	        
	        for (int j = 0; j < sclines.length; j++)
	        {    
		    	if (sclines[j].contains("[Data]") || RPCStringUtils.isEmpty(sclines[j])) continue;
		    	
	        	String[] fields = RPCStringUtils.split(sclines[j], RPCStringUtils.CARET);
	        	//ensure the record contains the IEN (numeric value) and that the IEN is the same as our patient's PID (dfn)
        	   if (fields != null && fields.length > 1 && rec.getUserId().equals(fields[0]))
        	   {
        		   rec.setSsn(fields[2]);
        		   break;
        	   }
	        }            
        }
        
        if (rec != null)
        {
            lst.add(rec);
        }
        
        return lst.toArray(new Provider[0]);
    }

}
