package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.vistalink.rpc.RpcReferenceType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ProviderUserLookupDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		
		return Arrays.asList((DataBean[])toProvider(response, dfn, siteId, queryBean, vistaLinkCommand));
	}
	
	
	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkCommand -VistaLinkCommand used to executer RPC calls thru VistaLink for additional data
	 * @return an array of Provider DataBeans populated from the content stored in the RPC response string
	 */
	protected Provider[] toProvider(String response, String dfn, String siteId, QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
    {
        if (response.equals(""))
        {
            return new Provider[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Provider> lst = new ArrayList<Provider>();
        for (int i = 0; i < lines.length; i++)
        {    
        	String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if (RPCStringUtils.isNumeric(flds[0]))
            {
                Provider user = new Provider();
                user.setUserId(flds[0]);
                user.setProviderName(flds[1]);
                if (flds.length > 2 && !RPCStringUtils.isNullOrEmpty(flds[2]))
                {
                   user.setTitle(flds[2]);
                }
                lst.add(user);
            }   
        }
        
        return lst.toArray(new Provider[0]);
    }

}
