package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.RadiologyReport;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.DateFormat;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class RadiologyReportDataBeanParser extends BaseDataBeanParser
{
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toRadiologyReport(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * Parses a response string and creates a list of Radiology Reports data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of RadiologyReport databeans populated from the content stored in the RPC response string
	 */
	protected RadiologyReport[] toRadiologyReport(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new RadiologyReport[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<RadiologyReport> responseArray = new ArrayList<RadiologyReport>();
         RadiologyReport report = null;
         String text = "";
         for (int i = 0; i < lines.length; i++)
         {
        	 if ("".equals(lines[i]))
        		 continue;
        	 
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
             if ("[+]".equals(fields[1]))
             {
            	 report.setText(text);
            	 responseArray.add(report);
            	 report = null;
            	 continue;
             }
             
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		// This line starts a new radiology report
             		report = new RadiologyReport();
             		text = "";
             		String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             		if (parts.length == 2)
             		{
             				report.setFacility(new SiteId(parts[1], parts[0], "",""));
             		}
             		else if (!fields[1].isEmpty())
             		{
             				report.setFacility(new SiteId(siteId, fields[1], "",""));
             		}
             		else
             		{
             			try
             			{
             				report.setFacility(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             			}
             	       	catch(SiteException e)
             	       	{
             		       	// Ignore it, site will not have name. Should never happen if valid site.
             		    }
             		}
             		break;
             	case 2:
             		report.setTimestamp( fields.length == 2? Timestamp.toUtcFromRdv(fields[1]) : null);
             		break;
             	case 3:
           			report.setTitle( fields.length == 2? fields[1] : null);
             		break;             		
             	case 4:
           			report.setStatus(fields.length == 2? fields[1] : null);
             		break;
             	case 5:
             		if (fields.length == 2)
             		{
             			report.setCaseNumber(fields[1]);
             			report.setAccessionNumber(report.getAccessionNumber(report.getTimestamp(), report.getCaseNumber(), DateFormat.ISO));
             		}
             		break;
             	case 6:
           			 text += fields.length == 2? fields[1] + "\n" : null;
             		break;
             }
         } 
           
         if (report != null)
         {             
          	 responseArray.add(report);
         }         

         return responseArray.toArray(new RadiologyReport[0]);
    }
}

