package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.RatedDisability;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Class that parses the rated disabilities response.
 * 
 * @author ZZZZZZAPOSTI
 * 
 */
public class RatedDisabilitiesForPatientParser extends BaseDataBeanParser {
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang
	 * .String, gov.va.med.via.databeans.QueryBean, java.lang.String,
	 * java.lang.String, java.lang.String,
	 * gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response,
			QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) {
		return Arrays.asList((DataBean[]) toRatedDisabilities(response,
				queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}

	/**
	 * Returns the populated RatedDisability array given the response.
	 * 
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return RatedDisability[] the array of RatedDisability
	 */
	protected RatedDisability[] toRatedDisabilities(String response,
			QueryBean queryBean, String dfn, String siteId,
			String webMethodName, VistaLinkCommand vistaLinkCommand) {
		List<RatedDisability> responseArray = new ArrayList<RatedDisability>();
		RatedDisability ratedDisability = null;

		if (RPCStringUtils.isNullOrEmpty(response)) {
			return new RatedDisability[0];
		}

		response = RPCStringUtils.replaceEndofLineCharacters(response);
		String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

		for (int i = 0; i < lines.length; i++) {
			
			if (lines[i].contains("[Data]")) {
				continue;
			}

			String[] flds = RPCStringUtils
					.split(lines[i], RPCStringUtils.CARET);

			if (flds.length == 0) {
				continue;
			}

			if (flds != null && flds.length > 0) {

				ratedDisability = new RatedDisability();	

				if (flds.length > 1) {

					ratedDisability
							.setId(RPCStringUtils.isEmpty(flds[1]) ? null
									: flds[1]);
				}

				if (flds.length > 2) {
					ratedDisability
							.setName(RPCStringUtils.isEmpty(flds[2]) ? null
									: flds[2]);
				}

				if (flds.length > 3) {
					ratedDisability
							.setPercent(RPCStringUtils.isEmpty(flds[3]) ? null
									: flds[3]);
				}

				if (flds.length > 4) {
					if (RPCStringUtils.isEmpty(flds[4])) {
						ratedDisability.setServiceConnected(null);
					} else {
						ratedDisability.setServiceConnected(flds[4].equals("1") ? "Y"
								: "N");
					}
				}

				if (flds.length > 5) {
					
					ratedDisability.setExtremityAffected(RPCStringUtils
							.isEmpty(flds[5]) ? "" : flds[5]);
				} else {
					
					ratedDisability.setExtremityAffected("");
					
				}

				if (flds.length > 6) {
					String originalEffectiveDate = null;
					if (!StringUtil.isNullOrEmpty(flds[6])) {
						originalEffectiveDate = flds[6];
					}
					ratedDisability.setOriginalEffectiveDate(VistaTimestamp
							.toUtcString(originalEffectiveDate));
				}

				if (flds.length > 7) {
					String currentEffectiveDate = null;
					if (!StringUtil.isNullOrEmpty(flds[7])) {
						currentEffectiveDate = flds[7];
					}
					ratedDisability.setCurrentEffectiveDate(VistaTimestamp
							.toUtcString(currentEffectiveDate));
				}
				
				if (flds.length > 1) {
					responseArray.add(ratedDisability);
				}
			}
		}

		return responseArray.toArray(new RatedDisability[0]);
	}
}
