package gov.va.med.via.response;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import gov.va.med.via.dao.lookup.ReferenceDataCache;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

public class ReferenceDataDelegate
{
	/** The vista link command. */
	private VistaLinkCommand vistaLinkCommand;

	public ReferenceDataDelegate(VistaLinkCommand vistaLinkCommand)
	{
		this.vistaLinkCommand = vistaLinkCommand;
	}

	/**
	 * Adds the patient id info (patient type, room, bed) from Vista to the patient instance.
	 * 
	 * @param the patient to be updated with the room and bed data
	 * 
	 * @return patient's type, room and bed attributes populated if successful.
	 */
	public String getCodeDescription(String fileNumber, String code)
	{				
		String result = "";
		if (RPCStringUtils.isNullOrEmpty(fileNumber) ||
				RPCStringUtils.isNullOrEmpty(code)) 
		{
			return result;
		}

		String cachedResult = ReferenceDataCache.getInstance().get(fileNumber, code);
		if ( cachedResult!=null)
		{
			result =cachedResult;
		}
		else
		{
			
			List<Object> params =new ArrayList<Object>();		
			HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
			hashMap.put("FILE",fileNumber);
			hashMap.put("FIELDS",".01");
			hashMap.put("FLAGS","IP");		
			hashMap.put("FROM", RPCUtils.adjustForNumericSearch(code));
			hashMap.put("XREF","#");
			params.add(hashMap);
			String response = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);

			response = RPCStringUtils.replaceEndofLineCharacters(response);        
			String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);

			for (int i=0; i < lines.length; i++)
			{
				// Skip processing if empty or [Data] tag in the response.        	 
				if (lines[i].equals("") || lines[i].contains("[Data]"))
					continue;

				String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
				if (fields != null && fields.length > 0)
				{
					if (code.equals(fields[0]))
					{
						result = fields[1];	
						break;
					};
				}
			}
			  	
		}
		/// always put the cached value in the ReferenceDataCache, doing so marks the entry as "fresh" in the LRU cache
		ReferenceDataCache.getInstance().put(fileNumber, code, result);  
		
		return result;

	}
}
