package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Note;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class SignedNotesParser extends BaseDataBeanParser
{
	public static final int ID_FIELD_INDEX=0;
	public static final int LOCAL_TITLE_FIELD_INDEX=1;
	public static final int VISTA_TIMESTAMP_FIELD_INDEX=2;
	public static final int AUTHOR_FIELD_INDEX=4;
	public static final int NOTE_LOCATION_FIELD_INDEX=5;
	public static final int HAS_IMAGES_FIELD_INDEX=10;
	public static final int HAS_ADDENDUM_FIELD_INDEX=12;
	public static final int ADDENDUM_FIELD_INDEX=13;
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return toNotes(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand);
	}
	
	/**
	 * To text.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return 
	 */
	protected List<DataBean> toNotes(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<DataBean> noteList= new ArrayList<DataBean>();
		if (response==null || response.trim().equals(""))
        {
            return noteList;
        }
		String[] lines = response.split("\\n",-1);//RPCStringUtils.split(response, RPCStringUtils.CRLF);
		
        
        
        for (String line: lines)
        {
        	line = line.trim();
            if (line.equals("") || line.startsWith("^"))
            {
                continue;
            }
            String[] fields = RPCStringUtils.split(line, RPCStringUtils.CARET);
            
            Note note = new Note();
            note.setId(fields[ID_FIELD_INDEX]);
            note.setLocalTitle( fields[LOCAL_TITLE_FIELD_INDEX]);
            note.setTimestamp(VistaTimestamp.toUtcString(fields[VISTA_TIMESTAMP_FIELD_INDEX]));
            String authorField = fields[AUTHOR_FIELD_INDEX];
            String[] authorFields = authorField.split(";",-1);//StringUtil.split(, StringUtil.SEMICOLON);
            String authorDUZ = authorFields[0];
            if (!authorDUZ.equals("0"))
            {
            	String authorId = authorFields[0];
            	String authorSignature = authorFields[1];
            	String authorName = authorFields[2];            	
            	note.setAuthor(new Author( authorId, authorName, authorSignature));
            }
            if (fields.length > NOTE_LOCATION_FIELD_INDEX)
            {
                //[DP] 4/6/2011 Fixed the Location ID and Name fields on the 
                //Location Object on the Note Object.
                //The location name was being put into the location id field.  
                //The RPC returns the Name of the Location within the "Site" 
                //(Hospital), this was putting it into the id field, it now 
                //puts it into the name field, and gets the id with a get
                //variable value call to the Vista database.
            	String noteLocationName = fields[NOTE_LOCATION_FIELD_INDEX];
                String locationId = "";
                if ((noteLocationName != null) && (!"".equals(noteLocationName)))
                {
                    //Get the ID from the Location Name
                    /*String arg = "$O(^SC(\"B\",\"" + noteLocationName + "\",0))";
                    RpcReferenceType variableArg = new RpcReferenceType(arg);
                    locationId = vistaLinkCommand.executeRpc(RpcNameConstants.XWB_GET_VARIABLE_VALUE, variableArg);
                    */
                	
                    List<Object> params =new ArrayList<Object>(); 
                    HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
                    hashMap.put("FILE",44); /// hospital location is file 44
                    hashMap.put("FIELDS",".01E"); /// E can limit what returns
                    hashMap.put("FLAGS","IP");
                    hashMap.put("PART",noteLocationName);  /// part is like equal
                    hashMap.put("XREF","B");  /// xref index  
                    /// something like:  index B matches to the value of part
                    
                    
                    params.add(hashMap); 
                    String locationResult = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
                    locationId = parseHospitalId(locationResult);
                    
                    /// MDWS code from VistaNoteDao:toNotes
                    //MdoQuery request = VistaUtils.buildGetVariableValueRequest(arg);
                    //locationId = (string)cxn.query(request);
                    
                    /* example from ClinicsDataBeanParser 

 						String arg = "@\"^SC(" + fields[0] + ",0)\"";
            			RpcReferenceType variableArg = new RpcReferenceType(arg);
	       				String variableValue = vistaLinkCommand.executeRpc(RpcNameConstants.XWB_GET_VARIABLE_VALUE, variableArg);
                     
                     */
                    
                }
                note.setLocation( new HospitalLocation(locationId, noteLocationName));
            }
            if (fields.length > HAS_IMAGES_FIELD_INDEX)
            {
            	String hasImagesValue = fields[HAS_IMAGES_FIELD_INDEX];
                note.setHasImages(  "0".equals(hasImagesValue)?false:true );
            }
            if (fields.length > HAS_ADDENDUM_FIELD_INDEX)
            {
            	String hasAddendumValue = fields[HAS_ADDENDUM_FIELD_INDEX];
                note.setHasAddendum( "+".equals(hasAddendumValue)?true:false);
            }
            if (fields.length > ADDENDUM_FIELD_INDEX)
            {
            	String addendumValue = fields[ADDENDUM_FIELD_INDEX];
                if (addendumValue!=null && addendumValue.length() > 1)
                {
                    note.setOriginalNoteID(fields[ADDENDUM_FIELD_INDEX]);
                    note.setAddendum(true);
                }
                else
                {
                	note.setAddendum(false);
                }
            }
            note.setSiteId(new SiteId(siteId));
            noteList.add((DataBean)note);
        }
        return noteList;
	}
	
	/** 
	 * 
	 * @param result
	 * @return
	 */
	public String parseHospitalId( String result)
	{
		/* get a string like below and returns the value 1184  
		[Data]
				1184^ZZCHY DIAB TRI READ OPTO^        ATTEBURG,CEDRIC B
		*/ 
		try
		{
			if ( result.trim().startsWith("[Data]"))
			{
				result = result.trim().substring("[Data]".length()).trim();				
			}
			String[] parts = result.split("\\^");
			Integer.parseInt(parts[0]);  // throw an exception if the parts[0] is not parseable to an Integer
			return parts[0];			
		}
		catch ( Exception e)
		{
			return null;
		}
		
		
	}
}
