package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.databeans.SurgeryReport;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SurgeryReportDataBeanParser extends BaseDataBeanParser
{
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toSurgeryReport(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * To note type response.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * @return the text[]
	 */
	protected SurgeryReport[] toSurgeryReport(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<SurgeryReport> responseArray = new ArrayList<SurgeryReport>();
		
        if (response == null|| response.equals("")) 
        {
        	return new SurgeryReport[0];
        }

        response = RPCStringUtils.replaceEndofLineCharacters(response);        
    	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
    	SurgeryReport report;
    	for (int i=0; i < lines.length; i++)
    	{
    		if (lines[i].equals(""))
    			continue;
    		
    		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
    		report = new SurgeryReport();
    		String[] parts = RPCStringUtils.split(fields[0], RPCStringUtils.SEMICOLON);
    		if (parts != null && parts.length >1)
    		{
    			report.setFacility(new SiteId(parts[0], parts[1],"",""));
    		}
    		report.setId(fields[1]);
    		report.setTimestamp(fields[2]);
    		report.setTitle(fields[3]);
    		report.setAuthor(new Author("",fields[4],""));
    		if (ViaWebMethodNameConstants.EMRSVC_GETSURGERYREPORTSWITHTEXT.compareTo(webMethodName) == 0)
    		{
    			ReportDelegate reportDelegate = new ReportDelegate(vistaLinkCommand);
    			report.setText(reportDelegate.getReportText(dfn, report.getId(), "28:SURGERY (LOCAL ONLY)~"));
    		}
    		responseArray.add(report);
    	}
        
        return responseArray.toArray(new SurgeryReport[0]);
	}

}
