package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.LabSpecimen;
import gov.va.med.via.databeans.SurgicalPathologyReport;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SurgicalPathologyReportDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toSurgicalPathologyReport(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of MicrobiologyReport data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of MicrobiologyReport databeans populated from the content stored in the RPC response string
	 */
	protected SurgicalPathologyReport[] toSurgicalPathologyReport(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new SurgicalPathologyReport[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<SurgicalPathologyReport> responseArray = new ArrayList<SurgicalPathologyReport>();
         SurgicalPathologyReport report = null;
         String specimenName = "";
         String exam = "";
         String accessionNum = "";
         for (int i = 0; i < lines.length; i++)
         {
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);             
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		// A new surgical pathology report is received when the field number is 1, add it to list and create a new MicrobiologyReport instance
             		if (report != null)  
             		{
             			if (!RPCStringUtils.isNullOrEmpty(specimenName))
             			{
             				specimenName = RPCStringUtils.CSubstring(specimenName, 0, specimenName.length() - 1);
             				report.setSpecimen(new LabSpecimen("",specimenName,"",accessionNum));
             			}
             			
             			if (!RPCStringUtils.isNullOrEmpty(exam))
             			{
             				exam = RPCStringUtils.CSubstring(exam, 0, exam.length() - 1);
             				report.setExam(exam);
             			}
             			responseArray.add(report);
             		}
             		report = new SurgicalPathologyReport();
             		report.setTitle("Surgical Pathology Report");
             		exam = "";
             		specimenName = "";
             		if (fields.length == 2)
             		{
             			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             			if (parts.length == 2)
             			{
             				report.setFacility(new SiteId(parts[0], parts[1], "",""));
             			}
             			else if (!fields[1].isEmpty())
             			{
             				report.setFacility(new SiteId(siteId, fields[1], "",""));
             			}
             			else
             			{
             				try
             				{
             					report.setFacility(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             				}
             		       	catch(SiteException e)
             		       	{
             		       		// Ignore it, site will not have name. Should never happen if valid site.
             		       	}
             			}
             		}
             		break;
             	case 2:
             		report.setTimestamp( fields.length == 2? Timestamp.toUtcFromRdv(fields[1]): null);
             		break;
             	case 3:
             		if (fields.length == 2)
             		{
             			specimenName += fields[1] + '\n';
             		}
             		break;             		
             	case 4:
             		if (fields.length == 2)
             		{
             			accessionNum = fields[1];
             		}             		
             		break;
             	case 5:
             		if (fields.length == 2 && !RPCStringUtils.isNullOrEmpty(fields[1].trim()))
             		{
             			exam += fields[1] + '\n';
             		}
             		break;
             }
         } 
           
         if (report != null)
         {             
          	 responseArray.add(report);
         }         

         return responseArray.toArray(new SurgicalPathologyReport[0]);
    }
}
