package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class UserScListerLookupDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		
		return Arrays.asList((DataBean[])toProviders(response, dfn, siteId, queryBean, vistaLinkCommand));
	}
	
	
	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkCommand -VistaLinkCommand used to executer RPC calls thru VistaLink for additional data
	 * @return an array of Provider DataBeans populated from the content stored in the RPC response string
	 */
	protected Provider[] toProviders(String response, String dfn, String siteId, QueryBean queryBean, VistaLinkCommand vistaLinkCommand)
    {
		/*
		 * Retrieves the User information from VistA
		 * FILE 200 - result position - filed number and field definitions
		 * POSITION								FIELD NUMBER
		 * 0:IEN								.01 (NOTE: field .01 seems to return IEN and NAME in two fields)
		 * 1:NAME								.01
		 * 2:OFFICE PHONE						.132
		 * 3:VOICE PAGER						.137
		 * 4:ROOM								.141
		 * 5:FILE MANAGER ACCESS CODE			3
		 * 6:SEX								4
		 * 7:DOB								5
		 * 8:DIUSER								7
		 * 9:TITLE								8E 
		 * 10:SSN								9
		 * 11:NICKNAME							13
		 * 12:SIGNATURE BLOCK PRINTED NAME		20.2
		 * 13:SIGNATURE BLOCK TITLE				20.3
		 * 14:SERVICE/SECTION					29
		 * 15:									29E
		 * 16:PROVIDER CLASS					53.5	POINTER
		 * 17:									53.5E	PROVIDER CLASS
		 * 18:PROVIDER TYPE						53.6
		 * 19:PRIMARY MENU OPTION				201 	POINTER
		 * 20:									201E 	PRIMARY MENU OPTION
		 * 
		 * Example of one result line:	"1497^MACEDONIO,ROSIE M^^^^^F^2541208^^STAFF NURSE^573055356^ROSIE^ROSIE M MACEDONIO^RN^57^NURSING SERVICE-AMB CARE^11^REGISTERED NURSE^^2791^NURS-RN"
		 */
        if (response.equals(""))
        {
            return new Provider[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Provider> lst = new ArrayList<Provider>();
        for (int i = 0; i < lines.length; i++)
        {    
        	if (lines[i].contains("[Data]")) continue;

        	String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
        	//ensure the record contains the IEN
            if (RPCStringUtils.isNumeric(flds[0]))
            {
                Provider user = new Provider();
                user.setUserId(flds[0]);
                user.setProviderName(flds[1]);
                user.setName(flds[1]);
                user.setPhone(flds[2]);
                //flds[3] is voice pager
                user.setOffice(flds[4]);
                //flds[5] is FileManager access
                user.setGender(flds[6]);
                user.setDob(flds[7]);
                //flds[8] is disuser
                user.setTitle(flds[9]);
                user.setSsn(flds[10]);
                //flds[11] is nickname
                user.setSigText(flds[12] + "\r\n" + flds[13]);//this is the same structure the legacy code attempts to capture this value in
                //flds[14] is service IEN
                user.setService(flds[15]);
                //flds[16] is Provider Class IEN
                user.setUserClass(flds[17]);
                //NOTE - at this time the permission is NOT transferred to client apps - so not capturing or creating objects to store it in unless a need arises
                //user.setPrimaryPermission(new KeyValuePair(flds[19], flds[20]));
                //result[i].PrimaryPermission = new MenuOption(flds[19], flds[20]);
  
                lst.add(user);
            }   
        }
        
        return lst.toArray(new Provider[0]);
    }
	
}
