package gov.va.med.via.response;

import gov.va.med.via.business.ViaResponse;
import gov.va.med.via.databeans.DataBean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ViaResponseImpl implements ViaResponse {
	
	protected Map<String,Map<String,List<DataBean>>> responseMap;

	
	public ViaResponseImpl(Map<String,Map<String,List<DataBean>>> responseMap){
		this.responseMap = responseMap;
	}
	
	public Map<String,Map<String,List<DataBean>>> getResponseMap(){
		
		if(this.responseMap != null){
			return this.responseMap;
		}
		else{
			return new HashMap<String,Map<String,List<DataBean>>>();
		}
	}
	
	public List<DataBean> getDomainDataBeansForSite(String daoName, String siteCode){
		
		List<DataBean> emptyDefaultDataBeanList = new ArrayList<DataBean>();
		
		Map<String,List<DataBean>> siteDataBeanMap = responseMap.get(daoName);
		if(siteDataBeanMap !=null){
			List<DataBean> dataBeans =  siteDataBeanMap.get(siteCode);
			if(dataBeans != null){
				return dataBeans;
			}
		}
		else{
			return emptyDefaultDataBeanList;
		}
		
		return emptyDefaultDataBeanList;
		
	}
	
	public Map<String,List<DataBean>> getSiteDataBeanMapForDao(String daoName){
		
		Map<String,List<DataBean>> siteDataBeanMap = responseMap.get(daoName);
		if(siteDataBeanMap != null){
			return siteDataBeanMap;
		}
		else{
			return new HashMap<String,List<DataBean>>();
		}
	}
	
	@Override
	public DataBean getSingleResponseDataBeanFromSingleDaoAndSite()
	{
		/// daos
		int daoCount = responseMap.keySet().size();
		if ( daoCount!=1 )
		{
			throw new UnsupportedOperationException("This method only works when a ViaResponse contains just one Dao.  Your ViaReponse contains "+daoCount+" DAOs - so you cannot use this method.");
		}
		
		/// sites
		Map<String,List<DataBean>> siteDataBeanMap = (Map<String,List<DataBean>>)responseMap.values().iterator().next();
		int siteCount = siteDataBeanMap.keySet().size();
		if ( siteCount!=1 )
		{
			throw new UnsupportedOperationException("This method only works when a ViaResponse contains just one Dao and one Site.  Your ViaReponse contains "+siteCount+" Sites - so you cannot use this method.");
		}
		
		/// list of objects
		int itemCount = siteDataBeanMap.values().size();
		if ( itemCount!=1 )
		{
			throw new UnsupportedOperationException("This method only works when a ViaResponse contains just one Dao, one Site, and one object in the list.  Your ViaReponse contains "+itemCount+" items in the list - so you cannot use this method.");
		}
		List<DataBean> databeans = (List<DataBean>)siteDataBeanMap.values().iterator().next();
		return databeans.get(0);
			
	}
	
	@Override
	public List<DataBean> getResponseListFromSingleDaoAndSite()
	{
		/// daos
		int daoCount = responseMap.keySet().size();
		if ( daoCount!=1 )
		{
			throw new UnsupportedOperationException("This method only works when a ViaResponse contains just one Dao.  Your ViaReponse contains "+daoCount+" DAOs - so you cannot use this method.");
		}
		
		/// sites
		Map<String,List<DataBean>> siteDataBeanMap = (Map<String,List<DataBean>>)responseMap.values().iterator().next();
		int siteCount = siteDataBeanMap.keySet().size();
		if ( siteCount!=1 )
		{
			throw new UnsupportedOperationException("This method only works when a ViaResponse contains just one Dao and one Site.  Your ViaReponse contains "+siteCount+" Sites - so you cannot use this method.");
		}
		
		/// list of objects
		List<DataBean> databeans = (List<DataBean>)siteDataBeanMap.values().iterator().next();
		return databeans;
			
	}
	
	

}
