package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.Visit;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class parses the response from the Visits RPC call into DataBeans
 * @author DOMAINALLURS
 *
 */
public class VisitsDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){		
		List<DataBean> list = Arrays.asList((DataBean[])toVisits(response, dfn, siteId, vistaLinkCommand));
		return list;
	}
	
	/**
	 * Parses the Visits RPC response string and populates an array of DataBean objects
	 * @param response - The response from RPC ORWCV VST
	 * @param dfn - the dfn
	 * @param siteId - the siteId
	 * @param vistaLivistaLinkCommandnkTemplate -VistaLinkCommand used to execute RPC calls thru VistaLink
	 * @return an array of Visit DataBeans populated from the content stored in the RPC response string
	 */
	protected Visit[] toVisits(String response, String dfn, String siteId, VistaLinkCommand vistaLivistaLinkCommandnkTemplate)
    {
        if (response.equals("")) {
            return new Visit[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);       
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Visit> lst = new ArrayList<Visit>();

        for (int i = 0; i < lines.length; i++){
            if (lines[i].isEmpty()){
                continue;
            }
            Visit visit = new Visit();
            String[] caretSplitFields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);           
            String[] subFields = caretSplitFields[0].split(";");
            
            visit.setType(subFields[0]);
            visit.setTimestamp(Timestamp.toUtcString(subFields[1]));
            HospitalLocation hospitalLocation = new HospitalLocation(subFields[2], caretSplitFields[2]);
            visit.setHospitalLocation(hospitalLocation);
            if ( caretSplitFields.length > 3){
            	visit.setStatus(caretSplitFields[3]);
            }          
            lst.add(visit);
        }
        Visit[] array =  (Visit[])lst.toArray(new Visit[lst.size()]);
        return array;
    }

	
}
