package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.ObservationType;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.databeans.VitalSign;
import gov.va.med.via.databeans.VitalSignSet;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Vitals DataBean parser type class - parses RPC responses and populates the content into Allergy DataBeans
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class VitalsDataBeanParser extends BaseDataBeanParser {
	
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		
		return Arrays.asList((DataBean[])toVitals(response, dfn, siteId, vistaLinkCommand));
	}
	

	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkCommand -VistaLinkCommand used to executer RPC calls thru VistaLink for additional data
	 * @return an array of Vitals DataBeans populated from the content stored in the RPC response string
	 */
	protected VitalSignSet[] toVitals(String response, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
        if (RPCStringUtils.isNullOrEmpty(response))
        {
            return new VitalSignSet[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        List<VitalSignSet> vitalsSetList = new ArrayList<VitalSignSet>();
        VitalSignSet rec = null;
        VitalSign s = null;
        for (int i = 0; i < lines.length; i++)
        {
                      
        	if (lines[i].equals(""))
            {
                continue;
            }
            String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            short fldnum = Short.parseShort(flds[0]);
            switch (fldnum)
            {
                case 1:
                    if (rec != null)
                    {
                    	vitalsSetList.add(rec);
                    }
                    rec = new VitalSignSet();
                    String[] subflds = RPCStringUtils.split(flds[1], RPCStringUtils.SEMICOLON);
                    if (subflds.length == 1)
                    {
                        rec.setFacility(new SiteId("200", subflds[0]));
                    }
                    else
                    {
                        rec.setFacility(new SiteId(subflds[1], subflds[0]));
                    }
                    break;
                case 2:
                    if (flds.length == 2)
                    {
                        rec.setTimestamp(VistaTimestamp.toUtcFromRdv(flds[1]));
                    }
                    break;
                case 3:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.TEMPERATURE));
                    if (flds.length == 2)
                    {
                        s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.TEMPERATURE, s);
                    break;
                case 4:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.PULSE));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.PULSE, s);
                    break;
                case 5:
                    s = new VitalSign();
                    s.setType(new ObservationType(null,VitalSign.VITAL_SIGN, VitalSign.RESPIRATION));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.RESPIRATION, s);
                    break;
                case 6:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.BLOOD_PRESSURE));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.BLOOD_PRESSURE, s);
                    break;
                case 7:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.HEIGHT));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.HEIGHT, s);
                    break;
                case 8:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.WEIGHT));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.WEIGHT, s);
                    break;
                case 9:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.PAIN));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.PAIN, s);
                    break;
                case 10:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.PULSE_OXYMETRY));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.PULSE_OXYMETRY, s);
                    break;
                case 11:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.CENTRAL_VENOUS_PRESSURE));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.CENTRAL_VENOUS_PRESSURE, s);
                    break;
                case 12:
                    s = new VitalSign();
                    s.setType(new ObservationType(null,VitalSign.VITAL_SIGN, VitalSign.CIRCUMFERENCE_GIRTH));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.CIRCUMFERENCE_GIRTH, s);
                    break;
                case 15:
                	 if (flds.length == 2)
                    {
                        setVitalSignQualifierStrings(rec, flds[1], "Qualifiers");
                        rec.setQualifiers(flds[1]);
                    }
                    break;
                case 16:
                    s = new VitalSign();
                    s.setType(new ObservationType(null, VitalSign.VITAL_SIGN, VitalSign.BODY_MASS_INDEX));
                    if (flds.length == 2)
                    {
                    	 s.setValue1(flds[1]);
                    }
                    rec.addVitalSign(VitalSign.BODY_MASS_INDEX, s);
                    break;
                case 17:
                	if (flds.length == 2)
                    {
                        setVitalSignQualifierStrings(rec, flds[1], "Units");
                        rec.setUnits(flds[1]);
                    }
                    break;
                
                default:
                    break;
            }
        }
                
        if (rec != null)
        {
        	vitalsSetList.add(rec);
        }
        
        return vitalsSetList.toArray(new VitalSignSet[0]);
    }
	

     private void setVitalSignQualifierStrings(VitalSignSet set, String s, String qualifier)
     {
         String[] keys = new String[]
         {
             "TEMP","PULSE","BP","RESP","WT","HT","PAIN","O2","POx","CG","CVP","BMI"
         };
         
         for (int i = 0; i < keys.length; i++)
         {
             String value = getVitalSignQualifierItem(s, keys[i]);
             if (!RPCStringUtils.isNullOrEmpty(value))
             {
                 VitalSign theSign = getSignFromSet(set, keys[i]);
                 if (theSign == null)
                 {
                     continue;
                 }
                 if (qualifier.equalsIgnoreCase("Units"))
                 {
                     theSign.setUnits(value);
                     if (keys[i] == "BP")
                     {
                         theSign = set.getVitalSign(VitalSign.SYSTOLIC_BP);
                         if (theSign != null)
                         {
                             theSign.setUnits(value);
                         }
                         theSign = set.getVitalSign(VitalSign.DIASTOLIC_BP);
                         if (theSign != null)
                         {
                             theSign.setUnits(value);
                         }
                     }
                 }
                 else if (qualifier.equalsIgnoreCase("Qualifiers"))
                 {
                     theSign.setQualifiers(value);
                     if (keys[i] == "BP")
                     {
                         theSign = set.getVitalSign(VitalSign.SYSTOLIC_BP);
                         if (theSign != null)
                         {
                        	 theSign.setQualifiers(value);
                         }
                         theSign = set.getVitalSign(VitalSign.DIASTOLIC_BP);
                         if (theSign != null)
                         {
                        	 theSign.setQualifiers(value);
                         }
                     }
                 }
             }
         }
     }
     
     private String getVitalSignQualifierItem(String qualifiers, String key)
     {
         int p1 = qualifiers.indexOf(key + ':');
         if (p1 == -1)
         {
             return "";
         }
         p1 += key.length() + 1;
         int p2 = p1;
         while (p2 < qualifiers.length() && qualifiers.charAt(p2) != ':')
         {
             p2++;
         }
         if (p2 < qualifiers.length())
         {
             while (qualifiers.charAt(p2) != ',')
             {
                 p2--;
             }
         }
         //return qualifiers.substring(p1, p2 - p1).trim();
         return qualifiers.substring(p1, p1 +(p2 - p1)).trim();
     }
     
     private VitalSign getSignFromSet(VitalSignSet set, String key)
     {
         if (key == "TEMP")
         {
             return set.getVitalSign(VitalSign.TEMPERATURE);
         }
         if (key == "PULSE")
         {
             return set.getVitalSign(VitalSign.PULSE);
         }
         if (key == "RESP")
         {
             return set.getVitalSign(VitalSign.RESPIRATION);
         }
         if (key == "BP")
         {
             return set.getVitalSign(VitalSign.BLOOD_PRESSURE);
         }
         if (key == "WT")
         {
             return set.getVitalSign(VitalSign.WEIGHT);
         }
         if (key == "HT")
         {
             return set.getVitalSign(VitalSign.HEIGHT);
         }
         if (key == "PAIN")
         {
             return set.getVitalSign(VitalSign.PAIN);
         }
         if (key == "O2")
         {
             return set.getVitalSign(VitalSign.PULSE_OXYMETRY);
         }
         if (key == "POx")
         {
             return set.getVitalSign(VitalSign.PULSE_OXYMETRY);
         }
         if (key == "CG")
         {
             return set.getVitalSign(VitalSign.CIRCUMFERENCE_GIRTH);
         }
         if (key == "CVP")
         {
             return set.getVitalSign(VitalSign.CENTRAL_VENOUS_PRESSURE);
         }
         if (key == "BMI")
         {
             return set.getVitalSign(VitalSign.BODY_MASS_INDEX);
         }
         return null;
     }
 }

