package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;

import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class parses the response from the get wards RPC ORQPT WARDS call into DataBeans
 * @author DOMAINALLURS
 *
 */
public class WardsDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){		
		List<DataBean> list = Arrays.asList((DataBean[])toHospitalLocations(response, dfn));
		return list;
	}
	
	/**
	 * Parses the get wards RPC response string and populates an array of DataBean objects
	 * @param response - The response from RPC ORQPT WARDS
	 * @param dfn - the dfn
	 * @return an array of HospitalLocation DataBeans populated from the content stored in the RPC response string
	 */
	protected HospitalLocation[] toHospitalLocations(String response, String dfn)
    {
        
		if (response.equals("")) {
            return new HospitalLocation[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);       
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
                
        ArrayList<HospitalLocation> lst = new ArrayList<HospitalLocation>();
                
        
        for (int i = 0; i < lines.length; i++){
            if (lines[i].isEmpty()){
                continue;
            }
            String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            HospitalLocation hospitalLocation = new HospitalLocation(fields[0], fields[1]);
            lst.add(hospitalLocation);
        }
        HospitalLocation[] array =  (HospitalLocation[])lst.toArray(new HospitalLocation[lst.size()]);
        return array;
    }	
}
