package gov.va.med.via.transaction;

import gov.va.med.via.dao.Dao;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.exception.MissingDuzException;
import gov.va.med.via.vistalink.exception.VistaLinkDataAccessException;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Class intended to act as a facade to the Rpc layer - and to be used in transaction processing that does not require threaded processing of the Work management framework
 * 
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class NonThreadedWorkImpl implements RpcNonThreadedWork{
	
    protected Dao dao;
    protected String currentSiteId;
    protected Map<String,Object> requestParameterMap;
    protected ConnectionFactoryLocator connectionFactoryLocator;
    private Map<String,List<DataBean>> mappedResult = null;
          
    /**
     * Constructor
     * 
     * @param queryBean
     * @param dao
     * @param currentSiteId
     * @param webMethodName
     * @param connectionFactoryLocator
     */

    public NonThreadedWorkImpl(Dao dao, String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator )
    {
        this.dao = dao;
        this.currentSiteId = currentSiteId;
        this.requestParameterMap = requestParameterMap;
        this.connectionFactoryLocator = connectionFactoryLocator;
    }


    public Map<String,List<DataBean>> runRpc( )
    {
        try
        {      	
        	List<DataBean> result = dao.callRpc(this.currentSiteId, this.requestParameterMap, this.connectionFactoryLocator);
        	mappedResult = new HashMap<String, List<DataBean>>();
        	mappedResult.put(currentSiteId, result);
        	return mappedResult;
        }
        catch ( RpcException re )
        {           
            throw re;
        }
        catch ( MissingDuzException de )
        {           
            //Web service validation should ensure we never throw/catch this exception ...
            throw new RpcException (ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, de, currentSiteId);
        }
        catch ( VistaLinkDataAccessException ve )
        {           
            throw new RpcException (ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, ve, currentSiteId);
        } 
        catch (Exception e){
        	throw new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, e, currentSiteId);
        }
        
    }

}
