

package gov.va.med.via.transaction;

import gov.va.med.via.dao.Dao;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.exception.MissingDuzException;
import gov.va.med.via.vistalink.exception.VistaLinkDataAccessException;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import commonj.work.Work;


/**
 * Runnable implementing class used for threaded work management - implements the commonj Work interface so this can be used by the WorkManagement framework in Weblogic
 * This class runs the RPC vista processing within the VIA system.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class ReadRpcWork
    implements
        Work
{
   
    private boolean completeFailure = false;
    private Map<String,List<DataBean>> mappedResult = null;
    protected Dao dao;
    protected String currentSiteId;
    protected Map<String,Object> requestParameterMap;
    protected ConnectionFactoryLocator connectionFactoryLocator;
    protected RpcException rpcException;
          
    /**
     * Constructor
     * 
     * @param queryBean
     * @param dao
     * @param currentSiteId
     * @param webMethodName
     * @param connectionFactoryLocator
     */

    public ReadRpcWork(Dao dao, String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator )
    {
        this.dao = dao;
        this.currentSiteId = currentSiteId;
        this.requestParameterMap = requestParameterMap;
        this.connectionFactoryLocator = connectionFactoryLocator;
    }


    /*
     * (non-Javadoc)
     * @see commonj.work.Work#release()
     */
    public void release( )
    {
        // no-op for now.
    }


    /*
     * (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run( )
    {
        try
        {      	
        	List<DataBean> result = dao.callRpc(this.currentSiteId, this.requestParameterMap, this.connectionFactoryLocator);
        	mappedResult = new HashMap<String, List<DataBean>>();
        	mappedResult.put(currentSiteId, result);	
        }
        catch ( RpcException re )
        {           
            completeFailure = true;
            this.rpcException = re;
            //throw re;
        }
        catch ( MissingDuzException de )
        {           
            //Web service validation should ensure we never throw/catch this exception ...
        	completeFailure = true;
            //throw new RpcException (ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, de, currentSiteId);
        	this.rpcException = new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES_FOR_DUZEXCEPTION, de, currentSiteId);
        }
        catch ( VistaLinkDataAccessException ve )
        {           
            completeFailure = true;
            //throw new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, ve, currentSiteId);
            this.rpcException = new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES_FOR_EXCEPTION, ve, currentSiteId, ve.getErrorMessage());
        } 
        catch (Exception e){
        	completeFailure = true;
        	//throw new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES, e, currentSiteId);
        	this.rpcException = new RpcException(ErrorCodeEnum.READ_WORK_EXCEPTION_WITH_CLIENT_MESSAGES_FOR_EXCEPTION, e, currentSiteId, e.getMessage());
        }
        
    }


    /**
     * Indicates if the read work failed.
     * @return True if the read operation failed in totality; otherwise false.
     */
    public boolean isCompleteFailure( )
    {
        return completeFailure;
    }


    /**
     * Gets the results.
     * @return The Map of the Databeans of results.
     */
    public Map<String,List<DataBean>> getResult( )
    {
        return mappedResult;
    }


    /**
     * Sets the results.
     */
    protected void setResult(Map<String,List<DataBean>> result )
    {
        this.mappedResult = result;
    }
    
    /**
     * Protected method for protected package transaction processing...
     * 
     * returns the dao
     */
    protected Dao getDao(){
    	return this.dao;
    }


    /*
     * (non-Javadoc)
     * @see commonj.work.Work#isDaemon()
     */
    public boolean isDaemon( )
    {
        return false;
    }
    
    public String getCurrentSiteId(){
    	return this.currentSiteId;
    }
    
    public RpcException getRpcException(){
    	return this.rpcException;
    }

}
