package gov.va.med.via.transaction;

import gov.va.med.via.business.ViaResponse;
import gov.va.med.via.dao.Dao;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;

import java.util.Map;

/**
 * Transaction management interface - manages classes/components used in defined transactions - in this case RPC calls made to a Dao instance.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public interface RpcTransactionManager
{
 
	/**
	* Method exposed to main generic processing core pojo
	* 
	* @param requestParameterMap
	* @return ViaResponse
	*/
	public ViaResponse performRpcRead(Map<String,Object>requestParameterMap);
 
	/**
	 * Setter injection method
	 * @param workManager
	 */
	public void setReadRpcWorkManager( ReadRpcWorkManager workManager );

	/**
	* Setter injection method
	* @param connectionFactoryLocator
 	*/
	public void setConnectionFactoryLocator( ConnectionFactoryLocator connectionFactoryLocator );
 
	/**
	* Setter injection method
	* @param daoMap
	*/
	public void setDaoMap( Map<String,Dao> daoMap );
	
	/**
	 * sets the entry point for login processing functionality.
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse loginProvider(Map<String, Object> requestParameterMap);
	
	/**
	 * Matches a patient from the search strings provided
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse matchPatient( Map<String, Object> requestParameterMap);

	/**
	 * Selects a patient from criteria provided by client application
	 * 
	 * @param requestParameterMap
	 * @return ViaResponse
	 */
	public ViaResponse selectPatient(Map<String, Object> requestParameterMap);
  
}
