
package gov.va.med.via.transaction;

import gov.va.med.via.business.ViaResponse;
import gov.va.med.via.dao.Dao;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.exceptionframework.ExceptionHandler;
import gov.va.med.via.exceptionframework.LoggingSeverity;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.response.ViaResponseImpl;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import commonj.work.WorkException;
import commonj.work.WorkItem;


/**
 * RpcTransactionManager implementing class - manages components used in RPC transaction threaded calls
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class RpcTransactionManagerImpl implements  RpcTransactionManager
{
	protected ReadRpcWorkManager readRpcWorkManager = null;
    protected ConnectionFactoryLocator connectionFactoryLocator = null;
    protected Map<String,Dao> daoMap;
    
    private Log log = LogFactory.getLog( RpcTransactionManagerImpl.class );
    
    //TODO - Need to work in the DataSources that are NON VistA - so far this focuses on VistA - need to use the JndiBindings to get at CDW/VADIR/NPT/CDS to
    //access those datasources for data - matchPatient / and the some reads - will need to add a new dao type or datasource type - new work type etc ... probaly will do it after
    //the vista calls - aggregate data, etc. will use same parsers, etc.
    
    /* (non-Javadoc)
     * @see gov.va.med.via.transaction.RpcTransactionManager#performRpcRead(java.util.Map)
     */
    public ViaResponse performRpcRead( Map<String,Object>requestParameterMap)
    {
    	
    	@SuppressWarnings("unchecked")
		Collection<Dao> daos = getDaos((List<String>)requestParameterMap.get(ViaRequestMapKeyConstants.DAO_NAME_LIST_KEY));
    	return this.performReadOnVistaData(daos, requestParameterMap );
    }

    /**
     * return a collection of Dao objects corresponding the the List of keys
     * 
     * @param daoNames
     * @return
     */
	private Collection<Dao> getDaos(List<String> daoNames) {
		
		Collection<Dao> daos = new ArrayList<Dao>();
		for(String daoKey : daoNames){
			daos.add(daoMap.get(daoKey));
		}
		return daos;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.transaction.RpcTransactionManager#matchPatient(java.util.Map)
	 */
	@Override
	public ViaResponse matchPatient(Map<String, Object> requestParameterMap) {
		
		Map<String,Map<String,List<DataBean>>> readResults = new HashMap<String,Map<String,List<DataBean>>>();
		Map<String,List<DataBean>> faultBeans = new HashMap<String,List<DataBean>>();
		Map<String,Map<String,List<DataBean>>> faults = new HashMap<String,Map<String,List<DataBean>>>();

		@SuppressWarnings("unchecked")
		Collection<Dao> daos = getDaos((List<String>)requestParameterMap.get(ViaRequestMapKeyConstants.DAO_NAME_LIST_KEY));

		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String siteCode = queryBean.getLoginSiteCode();
		
		for ( Dao dao: daos)
		{
			
			try
			{
				RpcNonThreadedWork rpcWork = createRpcNonThreadedWork(dao, siteCode, requestParameterMap, connectionFactoryLocator);		
				readResults.put(dao.getDaoName(),  rpcWork.runRpc());
			}
			catch ( RpcException re )
			{
				ExceptionHandler.logException(re, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), re.getMessage());
				Fault faultBean = new Fault(re);
				List<DataBean> faultList = new ArrayList<DataBean>();
				faultList.add(faultBean);
				faultBeans.put(siteCode, faultList );
			}
		
			if(faultBeans.size() > 0){
				faults.put(dao.getDaoName(), faultBeans);
			}
		}
		
		addFaultsToSiteMappedResults(readResults, faults, new String[]{siteCode});
		
		return createViaResponse(readResults);

	}
	
	
	@Override
	public ViaResponse selectPatient(Map<String, Object> requestParameterMap) {
		Map<String,Map<String,List<DataBean>>> readResults = new HashMap<String,Map<String,List<DataBean>>>();
		Map<String,List<DataBean>> faultBeans = new HashMap<String,List<DataBean>>();
		Map<String,Map<String,List<DataBean>>> faults = new HashMap<String,Map<String,List<DataBean>>>();

		@SuppressWarnings("unchecked")
		Collection<Dao> daos = getDaos((List<String>)requestParameterMap.get(ViaRequestMapKeyConstants.DAO_NAME_LIST_KEY));
		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String siteCode = queryBean.getLoginSiteCode();

		for ( Dao dao: daos)
		{
			
			try
			{
				RpcNonThreadedWork rpcWork = createRpcNonThreadedWork(dao, siteCode, requestParameterMap, connectionFactoryLocator);		
				readResults.put(dao.getDaoName(),  rpcWork.runRpc());
			}
			catch ( RpcException re )
			{
				ExceptionHandler.logException(re, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), re.getMessage());
				Fault faultBean = new Fault(re);
				List<DataBean> faultList = new ArrayList<DataBean>();
				faultList.add(faultBean);
				faultBeans.put(siteCode, faultList );
			}
		
			if(faultBeans.size() > 0){
				faults.put(dao.getDaoName(), faultBeans);
			}
		}
		
		addFaultsToSiteMappedResults(readResults, faults, new String[]{siteCode});
		
		return createViaResponse(readResults);

	}


	/* (non-Javadoc)
	 * @see gov.va.med.via.transaction.RpcTransactionManager#loginProvider(java.util.Map)
	 */
	@Override
	public ViaResponse loginProvider( Map<String, Object> requestParameterMap) 
	{
		Map<String,Map<String,List<DataBean>>> readResults = new HashMap<String,Map<String,List<DataBean>>>();
		Map<String,List<DataBean>> faultBeans = new HashMap<String,List<DataBean>>();
		Map<String,Map<String,List<DataBean>>> faults = new HashMap<String,Map<String,List<DataBean>>>();

		@SuppressWarnings("unchecked")
		Collection<Dao> daos = getDaos((List<String>)requestParameterMap.get(ViaRequestMapKeyConstants.DAO_NAME_LIST_KEY));
		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String siteCode = queryBean.getLoginSiteCode();

		for ( Dao dao: daos)
		{
			
			try
			{
				RpcNonThreadedWork rpcWork = createRpcNonThreadedWork(dao, siteCode, requestParameterMap, connectionFactoryLocator);		
				readResults.put(dao.getDaoName(),  rpcWork.runRpc());
			}
			catch ( RpcException re )
			{
				ExceptionHandler.logException(re, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), re.getMessage());
				Fault faultBean = new Fault(re);
				List<DataBean> faultList = new ArrayList<DataBean>();
				faultList.add(faultBean);
				faultBeans.put(siteCode, faultList );
			}
		
			if(faultBeans.size() > 0){
				faults.put(dao.getDaoName(), faultBeans);
			}
		}
		
		addFaultsToSiteMappedResults(readResults, faults, new String[]{siteCode});
		
		return createViaResponse(readResults);

	}

	
	/**
	 * Main transaction processing method.
	 * 
	 * @param daos
	 * @param queryBean
	 * @param webMethodName
	 * @return a Map of a List of DataBeans populated and parsed from RPC calls
	 * @throws RpcReadException
	 */
	protected ViaResponse performReadOnVistaData( Collection<Dao> daos ,  Map<String,Object>requestParameterMap )
        throws RpcException
    {

		Map<String,Map<String, List<DataBean>>> readResults = new HashMap<String,Map<String, List<DataBean>>>();
        List<WorkItem> workQueue = new ArrayList<WorkItem>();
        Map<String, List<DataBean>> faultBeans = new HashMap<String, List<DataBean>>();
		Map<String,Map<String,List<DataBean>>> faults = new HashMap<String,Map<String,List<DataBean>>>();
		
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY);
        String[] siteCodes = queryBean.getSiteCodes();
        
        for ( Dao dao : daos )
        {
        	
            for ( String siteId:siteCodes)
            {
            	
            	try
                {
            		workQueue.add( readRpcWorkManager.schedule( new ReadRpcWork( dao, siteId, requestParameterMap, connectionFactoryLocator  ) ) );
                }
                catch ( WorkException we )
                {
                	RpcException re = new RpcException(ErrorCodeEnum.WORK_EXCEPTION_WITH_CLIENT_MESSAGES, we, siteId);
                	ExceptionHandler.logException(re, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), "An error has occurred processing request for side code: " + siteId + " - exception message: " + we.getMessage());
    				Fault faultBean = new Fault(we);
    				List<DataBean> faultList = new ArrayList<DataBean>();
    				faultList.add(faultBean);
    				faultBeans.put(siteId, faultList );
                }
            }

            // wait for all of the scheduled work to complete before moving on.
            try
            {
            	readRpcWorkManager.waitForAll( workQueue );
            }
            catch ( WorkException we )
            {
            	ExceptionHandler.logException(we, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), we.getMessage());
				Fault faultBean = new Fault(we);
				List<DataBean> faultList = new ArrayList<DataBean>();
				faultList.add(faultBean);
				//we aren't sure which site errored so go for the login site code
				faultBeans.put(queryBean.getLoginSiteCode(), faultList );
            }

            ReadRpcWork doneWork = null;
            int failedCount = 0;

            for ( WorkItem workItem : workQueue )
            {
                try
                {
                    doneWork = ( ( ReadRpcWork )workItem.getResult() );

                    if ( doneWork != null )
                    {
                        if(doneWork.isCompleteFailure()){
                        	throw doneWork.getRpcException();
                        }
                        if(readResults.containsKey(doneWork.getDao().getDaoName())){
                        	String resultsSiteKey = doneWork.getCurrentSiteId();
                        	Map<String,List<DataBean>> siteResults = readResults.get(doneWork.getDao().getDaoName());
                        	siteResults.put(resultsSiteKey, doneWork.getResult().get(resultsSiteKey));	  
                        }
                        else{
                        	 readResults.put(doneWork.getDao().getDaoName(),doneWork.getResult());
                        }
                           
                    }

                }
                catch ( RpcException re )
                {
                	ExceptionHandler.logException(re, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), re.getMessage());
                	Fault faultBean = new Fault(re);
    				List<DataBean> faultList = new ArrayList<DataBean>();
    				faultList.add(faultBean);
    				faultBeans.put(doneWork.getCurrentSiteId(), faultList );
    				++failedCount;//To date we aren't doing anything with this, but we may want to know and capture this
                	//this is a bit of a hack, but we want a stack trace rather than just handling the exception caught and stored in the RpcReadWork - so throw this and catch it below ...
                }
                catch ( WorkException we )
                {
                	ExceptionHandler.logException(we, LoggingSeverity.ERROR, log, (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY), (String)requestParameterMap.get(ViaRequestMapKeyConstants.REQUESTING_APP_KEY), we.getMessage());
                	Fault faultBean = new Fault(we);
    				List<DataBean> faultList = new ArrayList<DataBean>();
    				faultList.add(faultBean);
                    failedCount++ ;
                }
            }
            
            if(faultBeans.size() > 0){
				faults.put(dao.getDaoName(), faultBeans);
				//TODO define a better configured BizLogic rule that can determine if processing should abort instead of this one off solution for cprsLaunch
				if(abortProcessing((String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY))){
					break;
				}
			}
		}
		
		readResults = addFaultsToSiteMappedResults(readResults, faults, siteCodes);
		
		return createViaResponse(readResults);
    }
	
	/**
	 * Combines readResults and faults into one DAO mapped collection
	 * 
	 * @param readResults
	 * @param faults
	 */
	private Map<String, List<DataBean>> addFaultsToResults(Map<String, List<DataBean>> readResults, Map<String, List<DataBean>> faults) {
		if(faults.size()>0){
			
			if(readResults.size()>0){
				Set<String> resultsDaoKeys = readResults.keySet();
				for(String daoKey : resultsDaoKeys){
					if(faults.containsKey(daoKey)){
						/*
						 * the dao data bean parsers return a [List] implementation that does not support the addAll method
						 * that list is constructed from the Arrays class and it acts like an immutable List - will not support adding beans
						 * so we need to convert the collection into a simple ArrayList in order 
						 * to combine our readResults and faults into one Dao mapped collection returned to the web layer
						 */
						//1.get and remove the immutable List
						List<DataBean> resultsImmutable = readResults.remove(daoKey);
						List<DataBean> results = new ArrayList<DataBean>();
						//2.add all the result beans into the new List
						results.addAll(resultsImmutable);
						//3. add/combine the faults with the result beans in the same list
						results.addAll(faults.get(daoKey));
						//4. re-add the mapped DAO collection
						readResults.put(daoKey, results);
					}
				}
			}
			//just return the faults since there are not read results - point readResults handle to faults
			readResults = faults;
		}
		return readResults;	
	}
	
	/**
	 * Combines readResults and faults into one DAO mapped collection of SITE mapped databeans
	 * 
	 * @param readResults
	 * @param faults
	 */
	private Map<String, Map<String, List<DataBean>>> addFaultsToSiteMappedResults(Map<String, Map<String, List<DataBean>>> readResults, Map<String, Map<String, List<DataBean>>> faults, String[] siteCodeKeys) {
		if(faults.size()>0){	
			if(readResults.size()>0){
				Set<String> resultsDaoKeys = readResults.keySet();
				for(String daoKey : resultsDaoKeys){
					if(faults.containsKey(daoKey)){
						
						//now get the list of databeans from the siteMap
						for(String siteKey:siteCodeKeys){
							
							/*
							 * the dao data bean parsers return a [List] implementation that does not support the addAll method
							 * that list is constructed from the Arrays class and it acts like an immutable List - will not support adding beans
							 * so we need to convert the collection into a simple ArrayList in order 
							 * to combine our readResults and faults into one Dao mapped collection returned to the web layer
							 */
							//1.get and remove the immutable List of databeans
							Map<String,List<DataBean>> dataBeanResultsMap = readResults.get(daoKey);
							List<DataBean> resultsImmutable = dataBeanResultsMap.remove(siteKey);
							//2.add all the result beans into the new List
							List<DataBean> results = new ArrayList<DataBean>();
							if(resultsImmutable != null){
								//there may or may not be data results (may only be faults) for this sitekey - hence the check before attempting to add them into the new mutable results list - null pointer potential on following line can't addAll with a null
								results.addAll(resultsImmutable);
							}
							//3. add/combine the faults with the result beans in the same list
							Map<String,List<DataBean>> faultResultsMap = faults.get(daoKey);
							if(faultResultsMap.get(siteKey)!=null){
								//there may or may not be faults for this sitekey (may only be data results) - hence the check before attempting to combine them into the results - null pointer potential on following line can't addAll with a null
								results.addAll(faultResultsMap.get(siteKey));
							}
							//4. re-add the mapped DAO collection
							dataBeanResultsMap.put(siteKey, results);
						}
					}
				}
			}
			else{
				//just return the faults since there are not read results - point readResults handle to faults
				readResults = faults;
			}
		}
		return readResults;	
	}


    private boolean abortProcessing(String webMethodName) {
    	
    	if(webMethodName.contains("cprsLaunch")){
    		return true;
    	}
		return false;
	}

	/* (non-Javadoc)
     * @see gov.va.med.via.transaction.RpcTransactionManager#setReadRpcWorkManager(gov.va.med.via.transaction.ReadRpcWorkManager)
     */
    public void setReadRpcWorkManager( ReadRpcWorkManager workManager )
    {
        this.readRpcWorkManager = workManager;
    }

	/* (non-Javadoc)
	 * @see gov.va.med.via.transaction.RpcTransactionManager#setConnectionFactoryLocator(gov.va.med.via.vistalink.ConnectionFactoryLocator)
	 */
	public void setConnectionFactoryLocator( ConnectionFactoryLocator connectionFactoryLocator )
    {
        this.connectionFactoryLocator = connectionFactoryLocator;
    }
    
	/* (non-Javadoc)
	 * @see gov.va.med.via.transaction.RpcTransactionManager#setDaoMap(java.util.Map)
	 */
	@Override
	public void setDaoMap(Map<String, Dao> daoMap) {
		this.daoMap = daoMap;
	}
	
	/**
	 * Factory method to create an rpc work object that runs outside of a thread management framework
	 * 
	 * @param dao
	 * @param currentSiteId
	 * @param requestParameterMap
	 * @param connectionFactoryLocator
	 * @return RpcNonThreadedWork
	 */
	protected RpcNonThreadedWork createRpcNonThreadedWork(Dao dao, String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator){
		
		return new NonThreadedWorkImpl(dao, currentSiteId, requestParameterMap, connectionFactoryLocator);
	}

	/**
	 * Factory method to create a response object interface containing the Map (by doaName) of maps(by sideCode) of DataBean lists (data from each site)
	 * 
	 * @param responseMap
	 * @return
	 */
	protected ViaResponse createViaResponse(Map<String,Map<String,List<DataBean>>> responseMap){
		
		return new ViaResponseImpl(responseMap);
	}
	
	

}
