package gov.va.med.via.vistalink;

import gov.va.med.vistalink.adapter.cci.VistaLinkConnectionFactory;
import gov.va.med.vistalink.institution.InstitutionMapNotInitializedException;
import gov.va.med.vistalink.institution.InstitutionMappingDelegate;
import gov.va.med.vistalink.institution.InstitutionMappingNotFoundException;
import org.springframework.dao.DataAccessResourceFailureException;
import java.text.MessageFormat;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.cci.ConnectionFactory;


/**
 * ConnectionFactoryLocator implementing class that locates connection factories based upon the Institution
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class InstitutionMappingConnectionFactoryLocator implements ConnectionFactoryLocator{


	private static final String NO_CONNECTION_FACTORY = "Unable to obtain a connection factory for station ''{0}''";
    private static final String NO_CONNECTION_FACTORY_JNDI = "Unable to obtain connection factory for station ''{0}'' with JNDI name ''{1}''";

	/* (non-Javadoc)
	 * @see gov.va.med.via.vistalink.ConnectionFactoryLocator#getConnectionFactory(java.lang.String)
	 */
	@Override
	public ConnectionFactory getConnectionFactory(String stationNumber) throws DataAccessResourceFailureException {

		try {
            String jndiConnectorName = InstitutionMappingDelegate.getJndiConnectorNameForInstitution(stationNumber);
            try {

                Context ic = new InitialContext();
                VistaLinkConnectionFactory vistaLinkConnectionFactory = (VistaLinkConnectionFactory) ic.lookup(jndiConnectorName);

                if (vistaLinkConnectionFactory == null){
                	throw new DataAccessResourceFailureException(MessageFormat.format(NO_CONNECTION_FACTORY_JNDI, stationNumber, jndiConnectorName));
                }

                return vistaLinkConnectionFactory;

            }catch(NamingException e){
            	throw new DataAccessResourceFailureException(MessageFormat.format(NO_CONNECTION_FACTORY_JNDI, stationNumber, jndiConnectorName), e);
            }
		} catch (InstitutionMappingNotFoundException e) {
			throw new DataAccessResourceFailureException(MessageFormat.format(NO_CONNECTION_FACTORY, stationNumber), e);
		} catch (InstitutionMapNotInitializedException e) {
			throw new DataAccessResourceFailureException(MessageFormat.format(NO_CONNECTION_FACTORY, stationNumber), e);
        }

	}

}
