package gov.va.med.via.vistalink;

import java.util.List;

import org.springframework.dao.DataAccessException;

/**
 * Interface defines App Proxy Connection methods - used to isolate connection reuse over several components
 * 
 * @author ZZZZZZTALBOM
 *
 */
public interface VistaRpcAppProxyTemplate {
	
	/**
	 * Method takes no rpc request parameters
	 * 
	 * @param division
	 * @param applicationProxyName
	 * @param rpcContext
	 * @param rpcName
	 * @return String rpc response
	 * @throws DataAccessException
	 */
	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext, final String rpcName) throws DataAccessException;
	
	/**
	 * Overloaded method allows optional rpc request parameters
	 * 
	 * @param division
	 * @param applicationProxyName
	 * @param rpcContext
	 * @param rpcName
	 * @param params
	 * @return String rpc response
	 * @throws DataAccessException
	 */
	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext, final String rpcName, Object... params) throws DataAccessException;
	
	/**
	 * Overloaded method accepts a List of rpc request parameters
	 * 
	 * @param division
	 * @param applicationProxyName
	 * @param rpcContext
	 * @param rpcName
	 * @param params
	 * @return String rpc response
	 * @throws DataAccessException
	 */
	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext, final String rpcName, final List<Object> params) throws DataAccessException;

}
