package gov.va.med.via.vistalink.command;

import java.util.List;

/**
 * Interface defines the command objects used to access VistaLinkTemplate - allows instances to be passed to client objects that need to interact
 * with a VistaLinkTemplate without needing to know or define the connection/connectionSpec of the VistaLinkTemplate.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public interface VistaLinkCommand {
	

	/**
	 * Sets the site code of the connection
	 * 
	 * @param targetSiteCode
	 */
	public void setTargetSiteCode(String targetSiteCode);
	/**
	 * Sets the context/permission of the Rpc
	 * 
	 * @param context
	 */
	public void setPermissionContext(String context);
	
	
	/**
	 * executes and RPC call on a VistaLinkTemplate with no parameters
	 * 
	 * @param rpcName - String name of the Rpc being called
	 * @return
	 */
	public String executeRpc(String rpcName);
	
	/**
	 * executes and RPC call on a VistaLinkTemplate with an comma separated set of parameters
	 * 
	 * @param rpcName - String name of the Rpc being called
	 * @param params - optional number of Object parameters used by the rpc
	 * @return String of results - fields - returned by the rpc
	 */
	public String executeRpc(String rpcName, Object... params);
	
	/**
	 * executes and RPC call on a VistaLinkTemplate withe an Object parameter List
	 * 
	 * @param rpcName - String name of the Rpc being called
	 * @param params - List of parameters used by the Rpc call
	 * @return - String of results - fields - returned by the rpc
	 */
	public String executeRpc(String rpcName, List<Object> params);

}
