package gov.va.med.via.vistalink.exception;

import gov.va.med.vistalink.adapter.record.VistaLinkFaultException;
import org.springframework.dao.DataRetrievalFailureException;

/**
 * 
 * General Exception class for VistALink access exceptions - wraps a fault exception of the VistaLink framework which
 * are generated when connection and rpc access fault exceptions are encountered.
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class VistaLinkDataAccessException extends DataRetrievalFailureException {
	
	private static final long serialVersionUID = 1L;
	private VistaLinkFaultException fault;
	private boolean logged = false;

    /**
     * Constructor
     * @param fault
     */
    public VistaLinkDataAccessException (VistaLinkFaultException fault) {
       
    	super(fault.getErrorMessage(), fault);
        this.fault = fault;
    }

    /**
     * Constructor
     * @param unableToExecuteRpc
     * @param fault
     */
    public VistaLinkDataAccessException(String unableToExecuteRpc, VistaLinkFaultException fault) {
		super(unableToExecuteRpc + " : " + fault.getMessage());
		this.fault = fault;
	}

	/**
	 * @return fault's error code
	 */
	public String getErrorCode() {

        return fault.getErrorCode();
    }

    /**
     * @return fault's error message
     */
    public String getErrorMessage() {

      return fault.getErrorMessage();

    }

    /**
     * @return fault's errorType
     */
    public String getErrorType() {
     return fault.getErrorType();
    }


    /**
     * @return fault's actor
     */
    public String getFaultActor() {
        return fault.getFaultActor();

    }

    /**
     * @return fault's code
     */
    public String getFaultCode() {
       return fault.getFaultCode();
    }


    /**
     * @return fault's faultString
     */
    public String getFaultString() {
    	return fault.getFaultString();

    }

	public boolean isLogged() {
		return logged;
	}

	public void setLogged(boolean logged) {
		this.logged = logged;
	}


}
