

package gov.va.med.via.ejb;

import gov.va.med.via.business.ViaResponse;
import gov.va.med.via.business.VistaService;
import gov.va.med.via.request.ViaReadProcessor;

import java.util.Map;

import javax.annotation.Resource;
import javax.ejb.CreateException;
import javax.ejb.Local;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.access.ContextSingletonBeanFactoryLocator;
import org.springframework.ejb.support.AbstractStatelessSessionBean;


@Local(VistaService.class)
@Stateless(name="ViaGenericServiceSession", mappedName="ejb/VistaService")
public class ViaGenericServiceSession extends AbstractStatelessSessionBean implements VistaService
{

	private static final long serialVersionUID = 1L;
	private Log logger = LogFactory.getLog( ViaGenericServiceSession.class );
	
    String VIA_READ_PROCESSOR = "viaReadProcessor";
    
    //@Resource
    private ViaReadProcessor viaReadProcessor;
	
    @Resource
    private SessionContext sessionContext;

	public ViaGenericServiceSession(){
		
	}

	@Override
    protected void onEjbCreate( )
        throws CreateException
    {	
		initProcessor();
    }
	
	@Override
    public void setSessionContext( SessionContext sessionContext )
    {
        this.sessionContext = sessionContext;
		super.setSessionContext( sessionContext );
        setBeanFactoryLocator( ContextSingletonBeanFactoryLocator.getInstance( "viaBeanRefContext.xml" ) );
        setBeanFactoryLocatorKey( ServiceConstantsInterface.PRIMARY_CONTEXT_ID );
    }
	
	
	protected void initProcessor(){
		viaReadProcessor = ( ViaReadProcessor )getBeanFactory().getBean(VIA_READ_PROCESSOR );
	}

    @Override
    public boolean isAlive( )
    {
        boolean alive = viaReadProcessor == null ? false:true;
        if(!alive){
        	initProcessor();
        	alive = viaReadProcessor == null ? false:true;
        }
        
    	if ( logger.isDebugEnabled() )
            logger.debug( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( null, null, "isAlive called - returning " + alive + " from: " + this.getClass().getName(), null ) );
    	
        return alive;
    }

	@Override
	 @TransactionAttribute(value=TransactionAttributeType.SUPPORTS)
	public ViaResponse loginProvider(Map<String, Object> requestParameterMap) {
		return viaReadProcessor.loginProvider(requestParameterMap);
	}
    
    @Override
    @TransactionAttribute(value=TransactionAttributeType.SUPPORTS)
	public ViaResponse processVistaRead(Map<String,Object>requestParameterMap) {
    	return this.viaReadProcessor.processVistaRead(requestParameterMap);
	}
    
    @Override
    @TransactionAttribute(value=TransactionAttributeType.SUPPORTS)
	public ViaResponse matchPatient(Map<String, Object> requestParameterMap) {
    	return this.viaReadProcessor.matchPatient(requestParameterMap);
	}
    
    @Override
    @TransactionAttribute(value=TransactionAttributeType.SUPPORTS)
   	public ViaResponse selectPatient(Map<String, Object> requestParameterMap) {
    	return this.viaReadProcessor.selectPatient(requestParameterMap);
   	}

}
