package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.Allergy;
import gov.va.med.via.databeans.Symptom;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class AllergyTO.
 */
@XmlRootElement(name="AllergyTO")
@XmlAccessorType(XmlAccessType.NONE)
public class AllergyTO
    extends AbstractTO
{

    /** The allergen id. */
	@XmlElement(name="allergenId")
    protected String allergenId;
    
    /** The allergen name. */
	@XmlElement(name="allergenName")
    protected String allergenName;
    
    /** The allergen type. */
	@XmlElement(name="allergenType")
    protected String allergenType;
    
    /** The reaction. */
	@XmlElement(name="reaction")
    protected String reaction;
    
    /** The severity. */
	@XmlElement(name="severity")
    protected String severity;
    
    /** The comment. */
	@XmlElement(name="comment")
    protected String comment;
    
    /** The timestamp. */
	@XmlElement(name="timestamp")
    protected String timestamp;
    
    /** The facility. */
	@XmlElement(name="facility")
    protected TaggedText facility;
    
    /** The location. */
	@XmlElement(name="location")
    protected HospitalLocationTO location;
    
    /** The type. */
	@XmlElement(name="type")
    protected ObservationTypeTO type;
    
    /** The observer. */
	@XmlElement(name="observer")
    protected AuthorTO observer;
    
    /** The recorder. */
	@XmlElement(name="recorder")
    protected AuthorTO recorder;
    
    /** The reactions. */
	@XmlElementWrapper(name="reactions")
	@XmlElement(name="symptomTO")	
    protected List<SymptomTO> reactions;
    
    /** The drug ingredients. */
	@XmlElementWrapper(name="drugIngredients")
	@XmlElement(name="taggedText")
    protected List<TaggedText> drugIngredients;
    
    /** The drug classes. */
	@XmlElementWrapper(name="drugClasses")
	@XmlElement(name="taggedText")
    protected List<TaggedText> drugClasses;
	
	public AllergyTO()
    {
    }
	
    public AllergyTO(Allergy allergy)
    {
    	this.allergenId = allergy.getAllergyId();
    	this.allergenName = allergy.getAllergyName();
    	this.allergenType = allergy.getAllergyType();
    	this.severity = allergy.getSeverity();
    	this.comment = allergy.getComment();
    	this.timestamp = allergy.getTimestamp();
    	this.facility = new TaggedText(allergy.getFacility()); // TaggedText
    	if(allergy.getObserver() !=null) this.observer = new AuthorTO(allergy.getObserver()); // AuthorTO
    	if(allergy.getRecorder()!=null)this.recorder = new AuthorTO(allergy.getRecorder()); // AuthorTO
    	if(allergy.getLocation()!=null)this.location = new HospitalLocationTO(allergy.getLocation());// HospitalLocationTO
    	if(allergy.getType()!=null) this.type = new ObservationTypeTO(allergy.getType()); // ObservationTypeTO
    	for(Symptom reaction: allergy.getReactions()){
    		this.addReactions(new SymptomTO(reaction));
    	}
    	//allergy.getDrugClasses(); // TaggedText array
    	//allergy.getDrugIngredients(); // TaggedText array
    }
    
	/**
	 * Gets the allergen id.
	 *
	 * @return the allergen id
	 */
	public String getAllergenId()
	{
		return allergenId;
	}
	
	/**
	 * Sets the allergen id.
	 *
	 * @param allergenId the new allergen id
	 */
	public void setAllergenId(String allergenId)
	{
		this.allergenId = allergenId;
	}
	
	/**
	 * Gets the allergen name.
	 *
	 * @return the allergen name
	 */
	public String getAllergenName()
	{
		return allergenName;
	}
	
	/**
	 * Sets the allergen name.
	 *
	 * @param allergenName the new allergen name
	 */
	public void setAllergenName(String allergenName)
	{
		this.allergenName = allergenName;
	}
	
	/**
	 * Gets the allergen type.
	 *
	 * @return the allergen type
	 */
	public String getAllergenType()
	{
		return allergenType;
	}
	
	/**
	 * Sets the allergen type.
	 *
	 * @param allergenType the new allergen type
	 */
	public void setAllergenType(String allergenType)
	{
		this.allergenType = allergenType;
	}
	
	/**
	 * Gets the reaction.
	 *
	 * @return the reaction
	 */
	public String getReaction()
	{
		return reaction;
	}
	
	/**
	 * Sets the reaction.
	 *
	 * @param reaction the new reaction
	 */
	public void setReaction(String reaction)
	{
		this.reaction = reaction;
	}
	
	/**
	 * Gets the severity.
	 *
	 * @return the severity
	 */
	public String getSeverity()
	{
		return severity;
	}
	
	/**
	 * Sets the severity.
	 *
	 * @param severity the new severity
	 */
	public void setSeverity(String severity)
	{
		this.severity = severity;
	}
	
	/**
	 * Gets the comment.
	 *
	 * @return the comment
	 */
	public String getComment()
	{
		return comment;
	}
	
	/**
	 * Sets the comment.
	 *
	 * @param comment the new comment
	 */
	public void setComment(String comment)
	{
		this.comment = comment;
	}
	
	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public String getTimestamp()
	{
		return timestamp;
	}
	
	/**
	 * Sets the timestamp.
	 *
	 * @param timestamp the new timestamp
	 */
	public void setTimestamp(String timestamp)
	{
		this.timestamp = timestamp;
	}
	
	/**
	 * Gets the facility.
	 *
	 * @return the facility
	 */
	public TaggedText getFacility()
	{
		return facility;
	}
	
	/**
	 * Sets the facility.
	 *
	 * @param facility the new facility
	 */
	public void setFacility(TaggedText facility)
	{
		this.facility = facility;
	}
	
	/**
	 * Gets the location.
	 *
	 * @return the location
	 */
	public HospitalLocationTO getLocation()
	{
		return location;
	}
	
	/**
	 * Sets the location.
	 *
	 * @param location the new location
	 */
	public void setLocation(HospitalLocationTO location)
	{
		this.location = location;
	}
	
	/**
	 * Gets the type.
	 *
	 * @return the type
	 */
	public ObservationTypeTO getType()
	{
		return type;
	}
	
	/**
	 * Sets the type.
	 *
	 * @param type the new type
	 */
	public void setType(ObservationTypeTO type)
	{
		this.type = type;
	}
	
	/**
	 * Gets the observer.
	 *
	 * @return the observer
	 */
	public AuthorTO getObserver()
	{
		return observer;
	}
	
	/**
	 * Sets the observer.
	 *
	 * @param observer the new observer
	 */
	public void setObserver(AuthorTO observer)
	{
		this.observer = observer;
	}
	
	/**
	 * Gets the recorder.
	 *
	 * @return the recorder
	 */
	public AuthorTO getRecorder()
	{
		return recorder;
	}
	
	/**
	 * Sets the recorder.
	 *
	 * @param recorder the new recorder
	 */
	public void setRecorder(AuthorTO recorder)
	{
		this.recorder = recorder;
	}
	
	/**
	 * Gets the reactions.
	 *
	 * @return the reactions
	 */
	public List<SymptomTO> getReactions()
	{
		return reactions;
	}
	
	/**
	 * Sets the reactions.
	 *
	 * @param reactions the new reactions
	 */
	public void setReactions(List<SymptomTO> reactions)
	{
		this.reactions = reactions;
	}
	
	public void addReactions(SymptomTO reaction)
	{
		if(this.reactions == null){
			this.reactions = new ArrayList<SymptomTO>();
		}
		this.reactions.add(reaction);
	}
	
	/**
	 * Gets the drug ingredients.
	 *
	 * @return the drug ingredients
	 */
	public List<TaggedText> getDrugIngredients()
	{
		return drugIngredients;
	}
	
	/**
	 * Sets the drug ingredients.
	 *
	 * @param drugIngredients the new drug ingredients
	 */
	public void setDrugIngredients(List<TaggedText> drugIngredients)
	{
		this.drugIngredients = drugIngredients;
	}
	
	/**
	 * Gets the drug classes.
	 *
	 * @return the drug classes
	 */
	public List<TaggedText> getDrugClasses()
	{
		return drugClasses;
	}
	
	/**
	 * Sets the drug classes.
	 *
	 * @param drugClasses the new drug classes
	 */
	public void setDrugClasses(List<TaggedText> drugClasses)
	{
		this.drugClasses = drugClasses;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "AllergyTO [allergenId=" + allergenId + ", allergenName="
				+ allergenName + ", allergenType=" + allergenType
				+ ", reaction=" + reaction + ", severity=" + severity
				+ ", comment=" + comment + ", timestamp=" + timestamp
				+ ", facility=" + facility + ", location=" + location
				+ ", type=" + type + ", observer=" + observer + ", recorder="
				+ recorder + ", reactions=" + reactions + ", drugIngredients="
				+ drugIngredients + ", drugClasses=" + drugClasses
				+ ", toString()=" + super.toString() + "]";
	}
    
    
    
}
