package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.ChemHemReport;
import gov.va.med.via.databeans.LabResult;
import gov.va.med.via.databeans.SiteId;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class ChemHemRpt.
 */
@XmlRootElement(name="ChemHemRpt")
@XmlAccessorType(XmlAccessType.NONE)
public class ChemHemRpt extends AbstractTO
{
	
	/** The id. */
	@XmlElement(name="id")
	protected String id;
	
	/** The title. */
	@XmlElement(name="title")
	protected String title;
	
	/** The timestamp. */
	@XmlElement(name="timestamp")
	protected String timestamp;
	
	/** The author. */
	@XmlElement(name="author")
	protected AuthorTO author;
	
	/** The facility. */
	@XmlElement(name="facility")
	protected TaggedText facility;
	
	/** The specimen. */
	@XmlElement(name="specimen")
	protected LabSpecimenTO specimen;
	
	/** The comment. */
	@XmlElement(name="comment")
	protected String comment;
	
	/** The results. */
	@XmlElementWrapper(name="results")
	@XmlElement(name="labResultTO")
	protected List<LabResultTO> results;
	
	/** The lab sites. */
	@XmlElementWrapper(name="labSites")
	@XmlElement(name="siteTO")
	protected List<SiteTO> labSites;
	
	public ChemHemRpt() {
		
	}
	
	public ChemHemRpt(ChemHemReport report) {
		this.id = report.getId();
		this.title = report.getTitle();
		this.timestamp = report.getTimestamp();
		this.author = new AuthorTO(report.getAuthor());
		this.facility = new TaggedText(report.getFacility());		
		this.comment = report.getComment();
		
		if (report.getSpecimen() != null)
		{
			this.specimen = new LabSpecimenTO(report.getSpecimen());
		}
		
		if(report.getResults()!=null)
		{
			for(LabResult result: report.getResults()){
	    		this.addResult(new LabResultTO(result));
	    	}
		}
		if(report.getLabSites() !=null)
		{
			for(SiteId site: report.getLabSites()){
	    		this.addLabSite(new SiteTO(site));
	    	}
		}
		
		
	}

	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public String getId()
	{
		return id;
	}
	
	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setId(String id)
	{
		this.id = id;
	}
	
	/**
	 * Gets the title.
	 *
	 * @return the title
	 */
	public String getTitle()
	{
		return title;
	}
	
	/**
	 * Sets the title.
	 *
	 * @param title the new title
	 */
	public void setTitle(String title)
	{
		this.title = title;
	}
	
	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public String getTimestamp()
	{
		return timestamp;
	}
	
	/**
	 * Sets the timestamp.
	 *
	 * @param timestamp the new timestamp
	 */
	public void setTimestamp(String timestamp)
	{
		this.timestamp = timestamp;
	}
	
	/**
	 * Gets the author.
	 *
	 * @return the author
	 */
	public AuthorTO getAuthor()
	{
		return author;
	}
	
	/**
	 * Sets the author.
	 *
	 * @param author the new author
	 */
	public void setAuthor(AuthorTO author)
	{
		this.author = author;
	}
	
	/**
	 * Gets the facility.
	 *
	 * @return the facility
	 */
	public TaggedText getFacility()
	{
		return facility;
	}
	
	/**
	 * Sets the facility.
	 *
	 * @param facility the new facility
	 */
	public void setFacility(TaggedText facility)
	{
		this.facility = facility;
	}
	
	/**
	 * Gets the specimen.
	 *
	 * @return the specimen
	 */
	public LabSpecimenTO getSpecimen()
	{
		return specimen;
	}
	
	/**
	 * Sets the specimen.
	 *
	 * @param specimen the new specimen
	 */
	public void setSpecimen(LabSpecimenTO specimen)
	{
		this.specimen = specimen;
	}
	
	/**
	 * Gets the comment.
	 *
	 * @return the comment
	 */
	public String getComment()
	{
		return comment;
	}
	
	/**
	 * Sets the comment.
	 *
	 * @param comment the new comment
	 */
	public void setComment(String comment)
	{
		this.comment = comment;
	}
	
	/**
	 * Gets the results.
	 *
	 * @return the results
	 */
	public List<LabResultTO> getResults()
	{
		return results;
	}
	
	/**
	 * Sets the results.
	 *
	 * @param results the new results
	 */
	public void setResults(List<LabResultTO> results)
	{
		this.results = results;
	}
	
	public void addResult(LabResultTO result)
	{
		if(this.results == null){
			this.results = new ArrayList<LabResultTO>();
		}
		this.results.add(result);
	}
	
	/**
	 * Gets the lab sites.
	 *
	 * @return the lab sites
	 */
	public List<SiteTO> getLabSites()
	{
		return labSites;
	}
	
	public void addLabSite(SiteTO site)
	{
		if(this.labSites == null){
			this.labSites = new ArrayList<SiteTO>();
		}
		this.labSites.add(site);
	}
	
	/**
	 * Sets the lab sites.
	 *
	 * @param labSites the new lab sites
	 */
	public void setLabSites(List<SiteTO> labSites)
	{
		this.labSites = labSites;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "ChemHemRpt [id=" + id + ", title=" + title + ", timestamp="
				+ timestamp + ", author=" + author + ", facility=" + facility
				+ ", specimen=" + specimen + ", comment=" + comment
				+ ", results=" + results + ", labSites=" + labSites
				+ ", toString()=" + super.toString() + "]";
	}
}