package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.Adt;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.InpatientStay;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class InpatientStayTO.
 */
@XmlRootElement(name="InpatientStayTO")
@XmlAccessorType(XmlAccessType.NONE)
public class InpatientStayTO
    extends AbstractTO
{
    
    /** The patient. */
	@XmlElement(name="patient")
    protected PatientTO patient;
    
    /** The location. */
    @XmlElement(name="location")
    protected HospitalLocationTO location;
    
    /** The admit timestamp. */
    @XmlElement(name="admitTimestamp")
    protected String admitTimestamp;
    
    /** The discharge timestamp. */
    @XmlElement(name="dischargeTimestamp")
    protected String dischargeTimestamp;
    
    /** The discharge diagnoses. */
    @XmlElement(name="dischargeDiagnoses")
    protected DischargeDiagnosesTO dischargeDiagnoses;
    
    /** The type. */
    @XmlElement(name="type")
    protected String type;
    
    /** The adts. */
    @XmlElementWrapper(name="adts")
	@XmlElement(name="adtTO")	
    protected List<AdtTO> adts;
    
    /** The movement checkin id. */
    @XmlElement(name="movementCheckinId")
    protected String movementCheckinId;
    
    public InpatientStayTO(){
    	
    }
    		
    public InpatientStayTO(InpatientStay inpatientStay)
    {
    	initContent(inpatientStay);
    }
    
    public InpatientStayTO(DataBean inpatientStay){
    	if(inpatientStay instanceof Fault){
			this.setFault(new FaultTO((Fault)inpatientStay));
		}
		else{
			initContent((InpatientStay)inpatientStay);
		}    	
    }
        
    private void initContent(InpatientStay inpatientStay){
    	boolean foundFault = false;
		if(inpatientStay.getFault() != null){
			this.setFault(new FaultTO(inpatientStay.getFault()));
			foundFault = true;
		}
		if(!foundFault)
		{
			this.type = inpatientStay.getType();
			this.admitTimestamp = inpatientStay.getAdmitTimestamp();
			this.dischargeTimestamp = inpatientStay.getDischargeTimestamp();
			this.movementCheckinId = inpatientStay.getMovementCheckinId();

			if ( inpatientStay.getLocation()!= null){
				this.location = new HospitalLocationTO(inpatientStay.getLocation());
			}
	
			if ( inpatientStay.getPatient()!= null){
				this.patient = new PatientTO(inpatientStay.getPatient());
			}
		    		
			this.adts = new ArrayList<AdtTO>();
			for(DataBean adt:inpatientStay.getAdts()){
				adts.add(new AdtTO((Adt)adt));
			}
		}
    }
    
    
    
	/**
	 * Gets the patient.
	 *
	 * @return the patient
	 */
	public PatientTO getPatient()
	{
		return patient;
	}
	
	/**
	 * Sets the patient.
	 *
	 * @param patient the new patient
	 */
	public void setPatient(PatientTO patient)
	{
		this.patient = patient;
	}
	
	/**
	 * Gets the location.
	 *
	 * @return the location
	 */
	public HospitalLocationTO getLocation()
	{
		return location;
	}
	
	/**
	 * Sets the location.
	 *
	 * @param location the new location
	 */
	public void setLocation(HospitalLocationTO location)
	{
		this.location = location;
	}
	
	/**
	 * Gets the admit timestamp.
	 *
	 * @return the admit timestamp
	 */
	public String getAdmitTimestamp()
	{
		return admitTimestamp;
	}
	
	/**
	 * Sets the admit timestamp.
	 *
	 * @param admitTimestamp the new admit timestamp
	 */
	public void setAdmitTimestamp(String admitTimestamp)
	{
		this.admitTimestamp = admitTimestamp;
	}
	
	/**
	 * Gets the discharge timestamp.
	 *
	 * @return the discharge timestamp
	 */
	public String getDischargeTimestamp()
	{
		return dischargeTimestamp;
	}
	
	/**
	 * Sets the discharge timestamp.
	 *
	 * @param dischargeTimestamp the new discharge timestamp
	 */
	public void setDischargeTimestamp(String dischargeTimestamp)
	{
		this.dischargeTimestamp = dischargeTimestamp;
	}
	
	/**
	 * Gets the discharge diagnoses.
	 *
	 * @return the discharge diagnoses
	 */
	public DischargeDiagnosesTO getDischargeDiagnoses()
	{
		return dischargeDiagnoses;
	}
	
	/**
	 * Sets the discharge diagnoses.
	 *
	 * @param dischargeDiagnoses the new discharge diagnoses
	 */
	public void setDischargeDiagnoses(DischargeDiagnosesTO dischargeDiagnoses)
	{
		this.dischargeDiagnoses = dischargeDiagnoses;
	}
	
	/**
	 * Gets the type.
	 *
	 * @return the type
	 */
	public String getType()
	{
		return type;
	}
	
	/**
	 * Sets the type.
	 *
	 * @param type the new type
	 */
	public void setType(String type)
	{
		this.type = type;
	}
	
	/**
	 * Gets the adts.
	 *
	 * @return the adts
	 */
	public List<AdtTO> getAdts()
	{
		return adts;
	}
	
	/**
	 * Sets the adts.
	 *
	 * @param adts the new adts
	 */
	public void setAdts(List<AdtTO> adts)
	{
		this.adts = adts;
	}
	
	/**
	 * Gets the movement checkin id.
	 *
	 * @return the movement checkin id
	 */
	public String getMovementCheckinId()
	{
		return movementCheckinId;
	}
	
	/**
	 * Sets the movement checkin id.
	 *
	 * @param movementCheckinId the new movement checkin id
	 */
	public void setMovementCheckinId(String movementCheckinId)
	{
		this.movementCheckinId = movementCheckinId;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "InpatientStayTO [patient=" + patient + ", location=" + location
				+ ", admitTimestamp=" + admitTimestamp
				+ ", dischargeTimestamp=" + dischargeTimestamp
				+ ", dischargeDiagnoses=" + dischargeDiagnoses + ", type="
				+ type + ", adts=" + adts + ", movementCheckinId="
				+ movementCheckinId + ", toString()=" + super.toString() + "]";
	}
}
