package gov.va.med.via.dto.common;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Order;
import gov.va.med.via.util.DateTimeUtil;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class OrderTO.
 * 
 * @author DOMAINROMERJ
 */
@XmlRootElement(name="OrderTO")
@XmlAccessorType(XmlAccessType.NONE)
public class OrderTO
    extends AbstractTO
{    
    /** The id. */
	@XmlElement(name="id")
    protected String id;
    
    /** The timestamp. */
	@XmlElement(name="timestamp")
    protected String timestamp;
    
    /** The ordering service name. */
	@XmlElement(name="orderingServiceName")
    protected String orderingServiceName;
    
    /** The treating specialty. */
	@XmlElement(name="treatingSpecialty")
    protected String treatingSpecialty;
    
    /** The start date. */
	@XmlElement(name="startDate")
    protected String startDate;
    
    /** The stop date. */
	@XmlElement(name="stopDate")
    protected String stopDate;
    
    /** The status. */
	@XmlElement(name="status")
    protected String status;
    
    /** The sig status. */
	@XmlElement(name="sigStatus")
    protected String sigStatus;
    
    /** The date signed. */
	@XmlElement(name="dateSigned")
    protected String dateSigned;
    
    /** The verifying nurse. */
	@XmlElement(name="verifyingNurse")
    protected String verifyingNurse;
    
    /** The date verified. */
	@XmlElement(name="treatingSpecialty")
    protected String dateVerified;
    
    /** The verifying clerk. */
	@XmlElement(name="verifyingClerk")
    protected String verifyingClerk;
    
    /** The chart reviewer. */
	@XmlElement(name="chartReviewer")
    protected String chartReviewer;
    	
    /** The date reviewed. */
	@XmlElement(name="dateReviewed")
    protected String dateReviewed;
    
    /** The provider. */
    @XmlElement(name="provider")
    protected UserTO provider;
    
    /** The text. */
    @XmlElement(name="text")
    protected String text;
    
    /** The detail. */
    @XmlElement(name="detail")
    protected String detail;
    
    /** The err msg. */
    @XmlElement(name="errMsg")
    protected String errMsg;
    
    /** The flag. */
    @XmlElement(name="flag")
    protected Boolean flag;
    
    /** The type. */
    @XmlElement(name="type")
    protected OrderTypeTO type;
	
    /**
     * Instantiates a new order to.
     */
    public OrderTO()
    {
    	
    }
    
    /**
     * Instantiates a new order to using the information in the databeans received
     *
     * @param daoSiteDataBeans the dao site data beans populatd with the Vista data
     */
    public OrderTO(Map<String, List<DataBean>> daoSiteDataBeans) 
    {		
		Set<String> databeanSiteKeys = daoSiteDataBeans.keySet();
		for(String siteKey : databeanSiteKeys)
		{
			List<DataBean> daoDataBeans = daoSiteDataBeans.get(siteKey);
			if (daoDataBeans.size() == 1)
			{
				mapOrderAttributes((Order)daoDataBeans.get(0));
			}
		}
	}
    /**
     * Instantiates a new order to.
     *
     * @param order the order
     */
    public OrderTO(Order order) 
	{
    	mapOrderAttributes(order);
	}

    
    /**
     * Copies the values from the Order instance received as parameter to this instance of OrderTO
     *
     * @param order the order instance containing the information to be used to populate this OrderTO instance. 
     * 
     * @return void 
     */
    private void mapOrderAttributes(Order order)
    {
		if ( order.getFault()!=null )
		{
			this.setFault(new FaultTO(order.getFault()));
		}
		else
		{
			this.id = order.getId();
	        this.orderingServiceName = order.getOrderingServiceName();
	        this.treatingSpecialty = order.getTreatingSpecialty();            
	        this.status = order.getStatus();
	        this.sigStatus = order.getSigStatus();            
	        this.verifyingNurse = order.getVerifyingNurse();
	        this.verifyingClerk = order.getVerifyingClerk();
	        this.chartReviewer = order.getChartReviewer();
	        this.text = order.getText();
	        this.detail = order.getDetail();
	        this.errMsg = order.getErrMsg();
	        this.flag = order.getFlag();
	        
	        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(DateTimeUtil.FORMAT_DATE_TIME);            
	       	this.timestamp = order.getTimestamp() != null ? simpleDateFormat.format(order.getTimestamp().getTime()) : null;
	        this.startDate = order.getStartDate() != null ? simpleDateFormat.format(order.getStartDate().getTime()) : null;
	        this.stopDate = order.getStopDate() != null ? simpleDateFormat.format(order.getStopDate().getTime()) : null;
	        this.dateSigned = order.getDateSigned() != null ? simpleDateFormat.format(order.getDateSigned().getTime()) : null;
	        this.dateVerified = order.getDateVerified() != null ? simpleDateFormat.format(order.getDateVerified().getTime()) : null;
	        this.dateReviewed = order.getDateReviewed() != null ? simpleDateFormat.format(order.getDateReviewed().getTime()) : null;
	
	       	this.provider = order.getProvider() != null? new UserTO(order.getProvider()): null;
	        this.type = order.getType() != null? new OrderTypeTO(order.getType()) : null;
		}
	}
    
	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public String getId()
	{
		return id;
	}
	
	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setId(String id)
	{
		this.id = id;
	}
	
	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public String getTimestamp()
	{
		return timestamp;
	}
	
	/**
	 * Sets the timestamp.
	 *
	 * @param timestamp the new timestamp
	 */
	public void setTimestamp(String timestamp)
	{
		this.timestamp = timestamp;
	}
	
	/**
	 * Gets the ordering service name.
	 *
	 * @return the ordering service name
	 */
	public String getOrderingServiceName()
	{
		return orderingServiceName;
	}
	
	/**
	 * Sets the ordering service name.
	 *
	 * @param orderingServiceName the new ordering service name
	 */
	public void setOrderingServiceName(String orderingServiceName)
	{
		this.orderingServiceName = orderingServiceName;
	}
	
	/**
	 * Gets the treating specialty.
	 *
	 * @return the treating specialty
	 */
	public String getTreatingSpecialty()
	{
		return treatingSpecialty;
	}
	
	/**
	 * Sets the treating specialty.
	 *
	 * @param treatingSpecialty the new treating specialty
	 */
	public void setTreatingSpecialty(String treatingSpecialty)
	{
		this.treatingSpecialty = treatingSpecialty;
	}
	
	/**
	 * Gets the start date.
	 *
	 * @return the start date
	 */
	public String getStartDate()
	{
		return startDate;
	}
	
	/**
	 * Sets the start date.
	 *
	 * @param startDate the new start date
	 */
	public void setStartDate(String startDate)
	{
		this.startDate = startDate;
	}
	
	/**
	 * Gets the stop date.
	 *
	 * @return the stop date
	 */
	public String getStopDate()
	{
		return stopDate;
	}
	
	/**
	 * Sets the stop date.
	 *
	 * @param stopDate the new stop date
	 */
	public void setStopDate(String stopDate)
	{
		this.stopDate = stopDate;
	}
	
	/**
	 * Gets the status.
	 *
	 * @return the status
	 */
	public String getStatus()
	{
		return status;
	}
	
	/**
	 * Sets the status.
	 *
	 * @param status the new status
	 */
	public void setStatus(String status)
	{
		this.status = status;
	}
	
	/**
	 * Gets the sig status.
	 *
	 * @return the sig status
	 */
	public String getSigStatus()
	{
		return sigStatus;
	}
	
	/**
	 * Sets the sig status.
	 *
	 * @param sigStatus the new sig status
	 */
	public void setSigStatus(String sigStatus)
	{
		this.sigStatus = sigStatus;
	}
	
	/**
	 * Gets the date signed.
	 *
	 * @return the date signed
	 */
	public String getDateSigned()
	{
		return dateSigned;
	}
	
	/**
	 * Sets the date signed.
	 *
	 * @param dateSigned the new date signed
	 */
	public void setDateSigned(String dateSigned)
	{
		this.dateSigned = dateSigned;
	}
	
	/**
	 * Gets the verifying nurse.
	 *
	 * @return the verifying nurse
	 */
	public String getVerifyingNurse()
	{
		return verifyingNurse;
	}
	
	/**
	 * Sets the verifying nurse.
	 *
	 * @param verifyingNurse the new verifying nurse
	 */
	public void setVerifyingNurse(String verifyingNurse)
	{
		this.verifyingNurse = verifyingNurse;
	}
	
	/**
	 * Gets the date verified.
	 *
	 * @return the date verified
	 */
	public String getDateVerified()
	{
		return dateVerified;
	}
	
	/**
	 * Sets the date verified.
	 *
	 * @param dateVerified the new date verified
	 */
	public void setDateVerified(String dateVerified)
	{
		this.dateVerified = dateVerified;
	}
	
	/**
	 * Gets the verifying clerk.
	 *
	 * @return the verifying clerk
	 */
	public String getVerifyingClerk()
	{
		return verifyingClerk;
	}
	
	/**
	 * Sets the verifying clerk.
	 *
	 * @param verifyingClerk the new verifying clerk
	 */
	public void setVerifyingClerk(String verifyingClerk)
	{
		this.verifyingClerk = verifyingClerk;
	}
	
	/**
	 * Gets the chart reviewer.
	 *
	 * @return the chart reviewer
	 */
	public String getChartReviewer()
	{
		return chartReviewer;
	}
	
	/**
	 * Sets the chart reviewer.
	 *
	 * @param chartReviewer the new chart reviewer
	 */
	public void setChartReviewer(String chartReviewer)
	{
		this.chartReviewer = chartReviewer;
	}
	
	/**
	 * Gets the date reviewed.
	 *
	 * @return the date reviewed
	 */
	public String getDateReviewed()
	{
		return dateReviewed;
	}
	
	/**
	 * Sets the date reviewed.
	 *
	 * @param dateReviewed the new date reviewed
	 */
	public void setDateReviewed(String dateReviewed)
	{
		this.dateReviewed = dateReviewed;
	}
	
	/**
	 * Gets the provider.
	 *
	 * @return the provider
	 */
	public UserTO getProvider()
	{
		return provider;
	}
	
	/**
	 * Sets the provider.
	 *
	 * @param provider the new provider
	 */
	public void setProvider(UserTO provider)
	{
		this.provider = provider;
	}
	
	/**
	 * Gets the text.
	 *
	 * @return the text
	 */
	public String getText()
	{
		return text;
	}
	
	/**
	 * Sets the text.
	 *
	 * @param text the new text
	 */
	public void setText(String text)
	{
		this.text = text;
	}
	
	/**
	 * Gets the detail.
	 *
	 * @return the detail
	 */
	public String getDetail()
	{
		return detail;
	}
	
	/**
	 * Sets the detail.
	 *
	 * @param detail the new detail
	 */
	public void setDetail(String detail)
	{
		this.detail = detail;
	}
	
	/**
	 * Gets the err msg.
	 *
	 * @return the err msg
	 */
	public String getErrMsg()
	{
		return errMsg;
	}
	
	/**
	 * Sets the err msg.
	 *
	 * @param errMsg the new err msg
	 */
	public void setErrMsg(String errMsg)
	{
		this.errMsg = errMsg;
	}
	
	/**
	 * Sets the flag.
	 *
	 * @param flag the new flag
	 */
	public void setFlag(Boolean flag)
	{
		this.flag = flag;
	}
	
	/**
	 * Gets the type.
	 *
	 * @return the type
	 */
	public OrderTypeTO getType()
	{
		return type;
	}
	
	/**
	 * Sets the type.
	 *
	 * @param type the new type
	 */
	public void setType(OrderTypeTO type)
	{
		this.type = type;
	}

}