package gov.va.med.via.dto.common;

import java.util.Calendar;


// TODO: Auto-generated Javadoc
/**
 * This class represents a Patient Admission.  This class has properties to represent procedures and diagnosis for an admission.  These attributes are only populated when requesting details for an admission and will not be populated when requesting a list/array of admissions.
 */
public class PatientAdmission extends DataBean
{
  /**
   * Registration Number for this admission.
   */
  protected String registerNumber;
  /**
   * Medical center division associated with this admission.
   */
  protected String division;
  /**
   * Ward where the patient was last assigned for this admission.
   */
  protected String ward;

  /** Ward Id (IEN) of the patient location. */
  protected String wardId;
  /**
   * Date of  the admission.
   */
  protected Calendar admissionDate;
  /**
   * Date the patient was discharged.
   */
  protected Calendar dispositionDate;
  /**
   * Admitting provider associated with this admission.
   */
  protected String admittingProvider;
  /**
   * Primary DRG code associated with this admission.
   */
  protected String drg;
  /**
   * Primary DRG description associated with this admission.
   */
  protected String drgDescription;
  /**
   * Internal Id of the admission. This property will be used to retrieve details
   * of an admission.
   */
  protected String admissionId;
  /**
   * Internal visit id associated with this admission.  Used to retrieve visit diagnosis
   * and/or procedures.
   */
  protected String visitId;
  /**
   * Calendar coding was complete for this admission.
   */
  protected Calendar codingCompleteDate;
  /**
   * Status of admission.
   */
  protected String recordStatus;
  
  /** Location of patient within the ward. */
  protected String roomBed;

  /** The provider id. */
  protected String providerId;
  
  /** The provider. */
  protected String provider;
  
  /** The admitting provider id. */
  protected String admittingProviderId;
	
	/**
	 * Gets the register number.
	 *
	 * @return the register number
	 */
	public String getRegisterNumber()
	{
		return registerNumber;
	}
	
	/**
	 * Sets the register number.
	 *
	 * @param registerNumber the new register number
	 */
	public void setRegisterNumber(String registerNumber)
	{
		this.registerNumber = registerNumber;
	}
	
	/**
	 * Gets the division.
	 *
	 * @return the division
	 */
	public String getDivision()
	{
		return division;
	}
	
	/**
	 * Sets the division.
	 *
	 * @param division the new division
	 */
	public void setDivision(String division)
	{
		this.division = division;
	}
	
	/**
	 * Gets the ward.
	 *
	 * @return the ward
	 */
	public String getWard()
	{
		return ward;
	}
	
	/**
	 * Sets the ward.
	 *
	 * @param ward the new ward
	 */
	public void setWard(String ward)
	{
		this.ward = ward;
	}
	
	/**
	 * Gets the ward id.
	 *
	 * @return the ward id
	 */
	public String getWardId()
	{
		return wardId;
	}
	
	/**
	 * Sets the ward id.
	 *
	 * @param wardId the new ward id
	 */
	public void setWardId(String wardId)
	{
		this.wardId = wardId;
	}
	
	/**
	 * Gets the admission date.
	 *
	 * @return the admission date
	 */
	public Calendar getAdmissionDate()
	{
		return admissionDate;
	}
	
	/**
	 * Sets the admission date.
	 *
	 * @param admissionDate the new admission date
	 */
	public void setAdmissionDate(Calendar admissionDate)
	{
		this.admissionDate = admissionDate;
	}
	
	/**
	 * Gets the disposition date.
	 *
	 * @return the disposition date
	 */
	public Calendar getDispositionDate()
	{
		return dispositionDate;
	}
	
	/**
	 * Sets the disposition date.
	 *
	 * @param dispositionDate the new disposition date
	 */
	public void setDispositionDate(Calendar dispositionDate)
	{
		this.dispositionDate = dispositionDate;
	}
	
	/**
	 * Gets the admitting provider.
	 *
	 * @return the admitting provider
	 */
	public String getAdmittingProvider()
	{
		return admittingProvider;
	}
	
	/**
	 * Sets the admitting provider.
	 *
	 * @param admittingProvider the new admitting provider
	 */
	public void setAdmittingProvider(String admittingProvider)
	{
		this.admittingProvider = admittingProvider;
	}
	
	/**
	 * Gets the drg.
	 *
	 * @return the drg
	 */
	public String getDrg()
	{
		return drg;
	}
	
	/**
	 * Sets the drg.
	 *
	 * @param drg the new drg
	 */
	public void setDrg(String drg)
	{
		this.drg = drg;
	}
	
	/**
	 * Gets the drg description.
	 *
	 * @return the drg description
	 */
	public String getDrgDescription()
	{
		return drgDescription;
	}
	
	/**
	 * Sets the drg description.
	 *
	 * @param drgDescription the new drg description
	 */
	public void setDrgDescription(String drgDescription)
	{
		this.drgDescription = drgDescription;
	}
	
	/**
	 * Gets the admission id.
	 *
	 * @return the admission id
	 */
	public String getAdmissionId()
	{
		return admissionId;
	}
	
	/**
	 * Sets the admission id.
	 *
	 * @param admissionId the new admission id
	 */
	public void setAdmissionId(String admissionId)
	{
		this.admissionId = admissionId;
	}
	
	/**
	 * Gets the visit id.
	 *
	 * @return the visit id
	 */
	public String getVisitId()
	{
		return visitId;
	}
	
	/**
	 * Sets the visit id.
	 *
	 * @param visitId the new visit id
	 */
	public void setVisitId(String visitId)
	{
		this.visitId = visitId;
	}
	
	/**
	 * Gets the coding complete date.
	 *
	 * @return the coding complete date
	 */
	public Calendar getCodingCompleteDate()
	{
		return codingCompleteDate;
	}
	
	/**
	 * Sets the coding complete date.
	 *
	 * @param codingCompleteDate the new coding complete date
	 */
	public void setCodingCompleteDate(Calendar codingCompleteDate)
	{
		this.codingCompleteDate = codingCompleteDate;
	}
	
	/**
	 * Gets the record status.
	 *
	 * @return the record status
	 */
	public String getRecordStatus()
	{
		return recordStatus;
	}
	
	/**
	 * Sets the record status.
	 *
	 * @param recordStatus the new record status
	 */
	public void setRecordStatus(String recordStatus)
	{
		this.recordStatus = recordStatus;
	}
	
	/**
	 * Gets the room bed.
	 *
	 * @return the room bed
	 */
	public String getRoomBed()
	{
		return roomBed;
	}
	
	/**
	 * Sets the room bed.
	 *
	 * @param roomBed the new room bed
	 */
	public void setRoomBed(String roomBed)
	{
		this.roomBed = roomBed;
	}
	
	/**
	 * Gets the provider id.
	 *
	 * @return the provider id
	 */
	public String getProviderId()
	{
		return providerId;
	}
	
	/**
	 * Sets the provider id.
	 *
	 * @param providerId the new provider id
	 */
	public void setProviderId(String providerId)
	{
		this.providerId = providerId;
	}
	
	/**
	 * Gets the provider.
	 *
	 * @return the provider
	 */
	public String getProvider()
	{
		return provider;
	}
	
	/**
	 * Sets the provider.
	 *
	 * @param provider the new provider
	 */
	public void setProvider(String provider)
	{
		this.provider = provider;
	}
	
	/**
	 * Gets the admitting provider id.
	 *
	 * @return the admitting provider id
	 */
	public String getAdmittingProviderId()
	{
		return admittingProviderId;
	}
	
	/**
	 * Sets the admitting provider id.
	 *
	 * @param admittingProviderId the new admitting provider id
	 */
	public void setAdmittingProviderId(String admittingProviderId)
	{
		this.admittingProviderId = admittingProviderId;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dto.common.DataBean#toString()
	 */
	@Override
	public String toString()
	{
		return "PatientAdmission [registerNumber=" + registerNumber + ", division="
				+ division + ", ward=" + ward + ", wardId=" + wardId
				+ ", admissionDate=" + admissionDate + ", dispositionDate="
				+ dispositionDate + ", admittingProvider=" + admittingProvider
				+ ", drg=" + drg + ", drgDescription=" + drgDescription
				+ ", admissionId=" + admissionId + ", visitId=" + visitId
				+ ", codingCompleteDate=" + codingCompleteDate + ", recordStatus="
				+ recordStatus + ", roomBed=" + roomBed + ", providerId="
				+ providerId + ", provider=" + provider + ", admittingProviderId="
				+ admittingProviderId + ", toString()=" + super.toString() + "]";
	}
	 
	  
}
