package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PatientDemographics;
import gov.va.med.via.databeans.SiteId;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class PatientTO.
 */
@XmlRootElement(name="PatientTO")
@XmlAccessorType(XmlAccessType.NONE)
public class PatientTO
	extends PersonTO
{
	// Following attributes found in MDWS 2.7
	/** The occupation. */
	@XmlElement(name="occupation")
	protected String occupation;
	
	/** The employment status. */
	@XmlElement(name="employmentStatus")
	protected String employmentStatus;
	
	/** The employer name. */
	@XmlElement(name="employerName")
	protected String employerName;
	
	/** The relationships. */
	@XmlElement(name="relationships")
	protected PersonArray relationships;

	// Following attributes found in MDWS 2.5
	/** The patient name. */
	@XmlElement(name="patientName")
	protected String patientName;
	
	/** The mpi pid. */
	@XmlElement(name="mpiPid")
	protected String mpiPid;
	
	/** The mpi checksum. */
	@XmlElement(name="mpiChecksum")
	protected String mpiChecksum;
	
	/** The local pid. */
	@XmlElement(name="localPid")
	protected String localPid;
	
	/** The site pids. */
	@XmlElement(name="sitePids")
	protected TaggedTextArray sitePids;	
	
	/** The vendor pid. */
	@XmlElement(name="vendorPid")
	protected String vendorPid;	
	
	/** The location. */
	@XmlElement(name="location")	
	protected HospitalLocationTO location;
	
	/** The cwad. */
	@XmlElement(name="cwad")
	protected String cwad;
	
	/** The restricted. */
	@XmlElement(name="restricted")
	protected Boolean restricted;
	
	/** The admit timestamp. */
	@XmlElement(name="admitTimestamp")
	protected String admitTimestamp;
	
	/** The service connected. */
	@XmlElement(name="serviceConnected")
	protected Boolean serviceConnected;
	
	/** The sc percent. */
	@XmlElement(name="scPercent")
	protected Integer scPercent;
	
	/** The inpatient. */
	@XmlElement(name="inpatient")
	protected Boolean inpatient;
	
	/** The deceased date. */
	@XmlElement(name="deceasedDate")
	protected String deceasedDate;
	
	/** The confidentiality. */
	@XmlElement(name="confidentiality")
	protected TaggedText confidentiality;
	
	/** The needs means test. */
	@XmlElement(name="needsMeansTest")
	protected Boolean needsMeansTest;
	
	/** The patient flags. */
	@XmlElement(name="patientFlags")
	protected TaggedTextArray patientFlags;
	
	/** The cmor site id. */
	@XmlElement(name="cmorSiteId")
	protected String cmorSiteId;
	
	/** The active insurance. */
	@XmlElement(name="activeInsurance")
	protected String activeInsurance;
	
	/** The is test patient. */
	@XmlElement(name="isTestPatient")
	protected Boolean isTestPatient;
	
	/** The current means status. */
	@XmlElement(name="currentMeansStatus")
	protected String currentMeansStatus;
	
	/** The has insurance. */
	@XmlElement(name="hasInsurance")
	protected Boolean hasInsurance;
	
	/** The preferred facility. */
	@XmlElement(name="preferredFacility")
	protected TaggedText preferredFacility;
	
	/** The patient type. */
	@XmlElement(name="patientType")
	protected String patientType;
	
	/** The is veteran. */
	@XmlElement(name="isVeteran")
	protected Boolean isVeteran;
	
	/** The is locally assigned mpi pid. */
	@XmlElement(name="isLocallyAssignedMpiPid")
	protected Boolean isLocallyAssignedMpiPid;
	
	/** The sites. */
	@XmlElement(name="sites")
	protected SiteArray sites;
	
	/** The team. */
	@XmlElement(name="team")
	protected TeamTO team;	  
	
	public PatientTO()
	{
		
	}
	
	
	public PatientTO(DataBean patient)
	{		
		super(patient);
		
		if ((patient == null)  ) {
			return;
		}
		if(( patient instanceof Fault)){
			this.setFault(new FaultTO((Fault)patient));
		}
		else{
			populateFields((Patient)patient);
		}

	}
	
	protected void populateFields(Patient patient){
		
		this.patientName = patient.getName();
		this.mpiPid = patient.getMpiPid();
		this.mpiChecksum = patient.getMpiChecksum();
		this.localPid = patient.getLocalPid();
		this.vendorPid = patient.getVendorPid();
		this.cwad = patient.getCwad();
		this.restricted = patient.isRestricted();
		this.admitTimestamp = patient.getAdmitTimestamp();
		this.serviceConnected = patient.isServiceConnected();
		this.scPercent = patient.getScPercent();
		this.inpatient = patient.isInpatient();
		this.deceasedDate = patient.getDeceasedDate();
		this.needsMeansTest = patient.isNeedsMeansTest();
		this.cmorSiteId = patient.getCmorSiteId();
		this.activeInsurance = patient.getActiveInsurance();
		this.isTestPatient = patient.isTestPatient();
		this.currentMeansStatus = patient.getCurrentMeansStatus();
		this.hasInsurance = patient.isHasInsurance();
		this.patientType = patient.getPatientType();
		this.isVeteran = patient.isVeteran();
		this.confidentiality = new TaggedText(new Integer(0), null); 

		this.isLocallyAssignedMpiPid = patient.isLocallyAssignedMpiPid();
		
		this.patientFlags = new TaggedTextArray(patient.getPatientFlags());
		
		if (patient.getLocation() != null)
		{
			this.location = new HospitalLocationTO(patient.getLocation());
		}
		if (patient.getConfidentiality() != null)
		{
			this.confidentiality = new TaggedText(patient.getConfidentiality().getKey(), patient.getConfidentiality().getValue() ); //CANNOT DEFINED CTR WITH KEYVALUEPAIR<INTEGER, STRING>!
		}			
		if (patient.getPreferredFacility() != null) // TODO - MDWS is not retrieving this value from Vista!
		{
			this.preferredFacility = new TaggedText(patient.getPreferredFacility());
		}
		if (patient.getSiteIDs() != null)
		{
			this.sites = new SiteArray(patient.getSiteIDs().toArray(new SiteId[0]));
		}
	}
	
	public PatientTO(PatientDemographics demographics)
	{
    	super(demographics);
	}	
	
	/**
	 * Gets the occupation.
	 *
	 * @return the occupation
	 */
	public String getOccupation()
	{
		return occupation;
	}
	
	/**
	 * Sets the occupation.
	 *
	 * @param occupation the new occupation
	 */
	public void setOccupation(String occupation)
	{
		this.occupation = occupation;
	}
	
	/**
	 * Gets the employment status.
	 *
	 * @return the employment status
	 */
	public String getEmploymentStatus()
	{
		return employmentStatus;
	}
	
	/**
	 * Sets the employment status.
	 *
	 * @param employmentStatus the new employment status
	 */
	public void setEmploymentStatus(String employmentStatus)
	{
		this.employmentStatus = employmentStatus;
	}
	
	/**
	 * Gets the employer name.
	 *
	 * @return the employer name
	 */
	public String getEmployerName()
	{
		return employerName;
	}
	
	/**
	 * Sets the employer name.
	 *
	 * @param employerName the new employer name
	 */
	public void setEmployerName(String employerName)
	{
		this.employerName = employerName;
	}
	
	/**
	 * Gets the relationships.
	 *
	 * @return the relationships
	 */
	public PersonArray getRelationships()
	{
		return relationships;
	}
	
	/**
	 * Sets the relationships.
	 *
	 * @param relationships the new relationships
	 */
	public void setRelationships(PersonArray relationships)
	{
		this.relationships = relationships;
	}
	
	/**
	 * Gets the patient name.
	 *
	 * @return the patient name
	 */
	public String getPatientName()
	{
		return patientName;
	}
	
	/**
	 * Sets the patient name.
	 *
	 * @param patientName the new patient name
	 */
	public void setPatientName(String patientName)
	{
		this.patientName = patientName;
	}
	
	/**
	 * Gets the mpi pid.
	 *
	 * @return the mpi pid
	 */
	public String getMpiPid()
	{
		return mpiPid;
	}
	
	/**
	 * Sets the mpi pid.
	 *
	 * @param mpiPid the new mpi pid
	 */
	public void setMpiPid(String mpiPid)
	{
		this.mpiPid = mpiPid;
	}
	
	/**
	 * Gets the mpi checksum.
	 *
	 * @return the mpi checksum
	 */
	public String getMpiChecksum()
	{
		return mpiChecksum;
	}
	
	/**
	 * Sets the mpi checksum.
	 *
	 * @param mpiChecksum the new mpi checksum
	 */
	public void setMpiChecksum(String mpiChecksum)
	{
		this.mpiChecksum = mpiChecksum;
	}
	
	/**
	 * Gets the local pid.
	 *
	 * @return the local pid
	 */
	public String getLocalPid()
	{
		return localPid;
	}
	
	/**
	 * Sets the local pid.
	 *
	 * @param localPid the new local pid
	 */
	public void setLocalPid(String localPid)
	{
		this.localPid = localPid;
	}
	
	/**
	 * Gets the site pids.
	 *
	 * @return the site pids
	 */
	public TaggedTextArray getSitePids()
	{
		return sitePids;
	}
	
	/**
	 * Sets the site pids.
	 *
	 * @param sitePids the new site pids
	 */
	public void setSitePids(TaggedTextArray sitePids)
	{
		this.sitePids = sitePids;
	}
	
	/**
	 * Gets the vendor pid.
	 *
	 * @return the vendor pid
	 */
	public String getVendorPid()
	{
		return vendorPid;
	}
	
	/**
	 * Sets the vendor pid.
	 *
	 * @param vendorPid the new vendor pid
	 */
	public void setVendorPid(String vendorPid)
	{
		this.vendorPid = vendorPid;
	}
	
	/**
	 * Gets the location.
	 *
	 * @return the location
	 */
	public HospitalLocationTO getLocation()
	{
		return location;
	}
	
	/**
	 * Sets the location.
	 *
	 * @param location the new location
	 */
	public void setLocation(HospitalLocationTO location)
	{
		this.location = location;
	}
	
	/**
	 * Gets the cwad.
	 *
	 * @return the cwad
	 */
	public String getCwad()
	{
		return cwad;
	}
	
	/**
	 * Sets the cwad.
	 *
	 * @param cwad the new cwad
	 */
	public void setCwad(String cwad)
	{
		this.cwad = cwad;
	}
	
	/**
	 * Checks if is restricted.
	 *
	 * @return true, if is restricted
	 */
	public Boolean isRestricted()
	{
		return restricted;
	}
	
	/**
	 * Sets the restricted.
	 *
	 * @param restricted the new restricted
	 */
	public void setRestricted(Boolean restricted)
	{
		this.restricted = restricted;
	}
	
	/**
	 * Gets the admit timestamp.
	 *
	 * @return the admit timestamp
	 */
	public String getAdmitTimestamp()
	{
		return admitTimestamp;
	}
	
	/**
	 * Sets the admit timestamp.
	 *
	 * @param admitTimestamp the new admit timestamp
	 */
	public void setAdmitTimestamp(String admitTimestamp)
	{
		this.admitTimestamp = admitTimestamp;
	}
	
	/**
	 * Checks if is service connected.
	 *
	 * @return true, if is service connected
	 */
	public Boolean isServiceConnected()
	{
		return serviceConnected;
	}
	
	/**
	 * Sets the service connected.
	 *
	 * @param serviceConnected the new service connected
	 */
	public void setServiceConnected(Boolean serviceConnected)
	{
		this.serviceConnected = serviceConnected;
	}
	
	/**
	 * Gets the sc percent.
	 *
	 * @return the sc percent
	 */
	public Integer getScPercent()
	{
		return scPercent;
	}
	
	/**
	 * Sets the sc percent.
	 *
	 * @param scPercent the new sc percent
	 */
	public void setScPercent(Integer scPercent)
	{
		this.scPercent = scPercent;
	}
	
	/**
	 * Checks if is inpatient.
	 *
	 * @return true, if is inpatient
	 */
	public Boolean isInpatient()
	{
		return inpatient;
	}
	
	/**
	 * Sets the inpatient.
	 *
	 * @param inpatient the new inpatient
	 */
	public void setInpatient(Boolean inpatient)
	{
		this.inpatient = inpatient;
	}
	
	/**
	 * Gets the deceased date.
	 *
	 * @return the deceased date
	 */
	public String getDeceasedDate()
	{
		return deceasedDate;
	}
	
	/**
	 * Sets the deceased date.
	 *
	 * @param deceasedDate the new deceased date
	 */
	public void setDeceasedDate(String deceasedDate)
	{
		this.deceasedDate = deceasedDate;
	}
	
	/**
	 * Gets the confidentiality.
	 *
	 * @return the confidentiality
	 */
	public TaggedText getConfidentiality()
	{
		return confidentiality;
	}
	
	/**
	 * Sets the confidentiality.
	 *
	 * @param confidentiality the new confidentiality
	 */
	public void setConfidentiality(TaggedText confidentiality)
	{
		this.confidentiality = confidentiality;
	}
	
	/**
	 * Checks if is needs means test.
	 *
	 * @return true, if is needs means test
	 */
	public Boolean isNeedsMeansTest()
	{
		return needsMeansTest;
	}
	
	/**
	 * Sets the needs means test.
	 *
	 * @param needsMeansTest the new needs means test
	 */
	public void setNeedsMeansTest(Boolean needsMeansTest)
	{
		this.needsMeansTest = needsMeansTest;
	}
	
	/**
	 * Gets the patient flags.
	 *
	 * @return the patient flags
	 */
	public TaggedTextArray getPatientFlags()
	{
		return patientFlags;
	}
	
	/**
	 * Sets the patient flags.
	 *
	 * @param patientFlags the new patient flags
	 */
	public void setPatientFlags(TaggedTextArray patientFlags)
	{
		this.patientFlags = patientFlags;
	}
	
	/**
	 * Gets the cmor site id.
	 *
	 * @return the cmor site id
	 */
	public String getCmorSiteId()
	{
		return cmorSiteId;
	}
	
	/**
	 * Sets the cmor site id.
	 *
	 * @param cmorSiteId the new cmor site id
	 */
	public void setCmorSiteId(String cmorSiteId)
	{
		this.cmorSiteId = cmorSiteId;
	}
	
	/**
	 * Gets the active insurance.
	 *
	 * @return the active insurance
	 */
	public String getActiveInsurance()
	{
		return activeInsurance;
	}
	
	/**
	 * Sets the active insurance.
	 *
	 * @param activeInsurance the new active insurance
	 */
	public void setActiveInsurance(String activeInsurance)
	{
		this.activeInsurance = activeInsurance;
	}
	
	/**
	 * Checks if is test patient.
	 *
	 * @return true, if is test patient
	 */
	public Boolean isTestPatient()
	{
		return isTestPatient;
	}
	
	/**
	 * Sets the test patient.
	 *
	 * @param isTestPatient the new test patient
	 */
	public void setTestPatient(Boolean isTestPatient)
	{
		this.isTestPatient = isTestPatient;
	}
	
	/**
	 * Gets the current means status.
	 *
	 * @return the current means status
	 */
	public String getCurrentMeansStatus()
	{
		return currentMeansStatus;
	}
	
	/**
	 * Sets the current means status.
	 *
	 * @param currentMeansStatus the new current means status
	 */
	public void setCurrentMeansStatus(String currentMeansStatus)
	{
		this.currentMeansStatus = currentMeansStatus;
	}
	
	/**
	 * Checks if is checks for insurance.
	 *
	 * @return true, if is checks for insurance
	 */
	public Boolean isHasInsurance()
	{
		return hasInsurance;
	}
	
	/**
	 * Sets the checks for insurance.
	 *
	 * @param hasInsurance the new checks for insurance
	 */
	public void setHasInsurance(Boolean hasInsurance)
	{
		this.hasInsurance = hasInsurance;
	}
	
	/**
	 * Gets the preferred facility.
	 *
	 * @return the preferred facility
	 */
	public TaggedText getPreferredFacility()
	{
		return preferredFacility;
	}
	
	/**
	 * Sets the preferred facility.
	 *
	 * @param preferredFacility the new preferred facility
	 */
	public void setPreferredFacility(TaggedText preferredFacility)
	{
		this.preferredFacility = preferredFacility;
	}
	
	/**
	 * Gets the patient type.
	 *
	 * @return the patient type
	 */
	public String getPatientType()
	{
		return patientType;
	}
	
	/**
	 * Sets the patient type.
	 *
	 * @param patientType the new patient type
	 */
	public void setPatientType(String patientType)
	{
		this.patientType = patientType;
	}
	
	/**
	 * Checks if is veteran.
	 *
	 * @return true, if is veteran
	 */
	public Boolean isVeteran()
	{
		return isVeteran;
	}
	
	/**
	 * Sets the veteran.
	 *
	 * @param isVeteran the new veteran
	 */
	public void setVeteran(Boolean isVeteran)
	{
		this.isVeteran = isVeteran;
	}
	
	/**
	 * Checks if is locally assigned mpi pid.
	 *
	 * @return true, if is locally assigned mpi pid
	 */
	public Boolean isLocallyAssignedMpiPid()
	{
		return isLocallyAssignedMpiPid;
	}
	
	/**
	 * Sets the locally assigned mpi pid.
	 *
	 * @param isLocallyAssignedMpiPid the new locally assigned mpi pid
	 */
	public void setLocallyAssignedMpiPid(Boolean isLocallyAssignedMpiPid)
	{
		this.isLocallyAssignedMpiPid = isLocallyAssignedMpiPid;
	}
	
	/**
	 * Gets the sites.
	 *
	 * @return the sites
	 */
	public SiteArray getSites()
	{
		return sites;
	}
	
	/**
	 * Sets the sites.
	 *
	 * @param sites the new sites
	 */
	public void setSites(SiteArray sites)
	{
		this.sites = sites;
	}
	
	/**
	 * Gets the team.
	 *
	 * @return the team
	 */
	public TeamTO getTeam()
	{
		return team;
	}
	
	/**
	 * Sets the team.
	 *
	 * @param team the new team
	 */
	public void setTeam(TeamTO team)
	{
		this.team = team;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dto.common.PersonTO#toString()
	 */
	@Override
	public String toString()
	{
		return "PatientTO [occupation=" + occupation + ", employmentStatus="
				+ employmentStatus + ", employerName=" + employerName
				+ ", relationships=" + relationships + ", patientName="
				+ patientName + ", mpiPid=" + mpiPid + ", mpiChecksum="
				+ mpiChecksum + ", localPid=" + localPid + ", sitePids="
				+ sitePids + ", vendorPid=" + vendorPid + ", location="
				+ location + ", cwad=" + cwad + ", restricted=" + restricted
				+ ", admitTimestamp=" + admitTimestamp + ", serviceConnected="
				+ serviceConnected + ", scPercent=" + scPercent
				+ ", inpatient=" + inpatient + ", deceasedDate=" + deceasedDate
				+ ", confidentiality=" + confidentiality + ", needsMeansTest="
				+ needsMeansTest + ", patientFlags=" + patientFlags
				+ ", cmorSiteId=" + cmorSiteId + ", activeInsurance="
				+ activeInsurance + ", isTestPatient=" + isTestPatient
				+ ", currentMeansStatus=" + currentMeansStatus
				+ ", hasInsurance=" + hasInsurance + ", preferredFacility="
				+ preferredFacility + ", patientType=" + patientType
				+ ", isVeteran=" + isVeteran + ", isLocallyAssignedMpiPid="
				+ isLocallyAssignedMpiPid + ", sites=" + sites + ", team="
				+ team + ", toString()=" + super.toString() + "]";
	}
}