package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DemographicSet;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PatientAssociate;
import gov.va.med.via.databeans.PatientDemographics;
import gov.va.med.via.databeans.Person;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;


// TODO: Auto-generated Javadoc
/**
 * The Class PersonTO.
 */
@XmlRootElement(name="PersonTO")
@XmlAccessorType(XmlAccessType.NONE)
public class PersonTO
    extends AbstractTO
{    
    /** The type. */
	@XmlElement(name="type")
    protected String type;
    
    /** The birth place. */
	@XmlElement(name="birthPlace")
    protected String birthPlace;
    
    /** The religion. */
	@XmlElement(name="religion")
    protected String religion;
    
    /** The name. */
	@XmlElement(name="name")
    protected String name;
    
    /** The ssn. */
	@XmlElement(name="ssn")
    protected String ssn;
    
    /** The gender. */
	@XmlElement(name="gender")
    protected String gender;
    
    /** The dob. */
	@XmlElement(name="dob")
    protected String dob;
    
    /** The ethnicity. */
	@XmlElement(name="ethnicity")
    protected String ethnicity;
    
    /** The age. */
	@XmlElement(name="age")
    protected Integer age;
    
    /** The marital status. */
	@XmlElement(name="maritalStatus")
    protected String maritalStatus;
    
    /** The home address. */
	@XmlElement(name="homeAddress")    
    protected AddressTO homeAddress;
    
    /** The home phone. */
	@XmlElement(name="homePhone")    
    protected PhoneNumTO homePhone;
    
    /** The cell phone. */
	@XmlElement(name="cellPhone")
    protected PhoneNumTO cellPhone;
    
    /** The demographics. */
	@XmlElementWrapper(name="demographics")
	@XmlElement(name="demographicSetTO")
    protected List<DemographicSetTO> demographics;
    
    /**
     * Instantiates a new person to.
     */
    public PersonTO()
    {
    	
    }
    
    public PersonTO(gov.va.med.via.databeans.DataBean patient)
    {
    	if(( patient instanceof Fault)){
			this.setFault(new FaultTO((Fault)patient));
		}
    	else if ( ((Patient)patient).getFault()!=null )
   		{
    		this.setFault(new FaultTO(((Patient)patient).getFault()));
    	}
    	else{
    		populatePersonFields((Person)patient);
    	}
    }
    
    public PersonTO(PatientAssociate associate)
    {
    	if (associate.getFault() != null)
    	{
    		this.setFault(new FaultTO(associate.getFault()));
    	}
    	else
    	{
    		populatePersonFields((Person)associate);
    	}
    }
    /**
     * Instantiates a new PersonTO using the information in the patient received.
     *
     * @param patient the patient
     */
    public PersonTO(Patient patient)
    {
   		if ( patient.getFault()!=null )
   		{
    		this.setFault(new FaultTO(patient.getFault()));
    	}
    	else
    	{
    		populatePersonFields((Person)patient);
    	}
    }
    
    protected void populatePersonFields(Person person)
    {    	
    	this.dob = person.getDob();
    	this.gender = person.getGender();
    	this.name = person.getName();
    	this.ssn = person.getSsn();
    	this.ethnicity =person.getEthnicity();
    	this.maritalStatus = person.getMaritalStatus();
    	
    	if( person.getAge() != null)
    	{
    		this.age = new Integer(person.getAge()).intValue(); 
    	}

    	if (person.getHomeAddress() != null)
    	{
    		this.homeAddress = new AddressTO(person.getHomeAddress());
    	}
    	
    	if (person.getHomePhone() != null)
    	{
    		this.homePhone = new PhoneNumTO(person.getHomePhone());
    	}
    	
    	if (person.getHomePhone() != null)
    	{
    		this.homePhone = new PhoneNumTO(person.getHomePhone());
    	}
    	
    	if (person.getDemographics() != null && person.getDemographics().size() > 0)
    	{
    		demographics = new ArrayList<DemographicSetTO>();
    		
    		for (Iterator<String> i = person.getDemographics().keySet().iterator(); i.hasNext(); )
    		{
    			String tag = (String)i.next();
     			DemographicSet demographicsBean = person.getDemographics().get(tag);
    			DemographicSetTO demographic = new DemographicSetTO(tag, demographicsBean);
    			demographics.add(demographic);
    		}
    	}
    }
    
    public PersonTO(PatientDemographics demographics)
    {
    	this((Patient) demographics);
    }
    
	/**
	 * Gets the type.
	 *
	 * @return the type
	 */
	public String getType()
	{
		return type;
	}
	
	/**
	 * Sets the type.
	 *
	 * @param type the new type
	 */
	public void setType(String type)
	{
		this.type = type;
	}
	
	/**
	 * Gets the birth place.
	 *
	 * @return the birth place
	 */
	public String getBirthPlace()
	{
		return birthPlace;
	}
	
	/**
	 * Sets the birth place.
	 *
	 * @param birthPlace the new birth place
	 */
	public void setBirthPlace(String birthPlace)
	{
		this.birthPlace = birthPlace;
	}
	
	/**
	 * Gets the religion.
	 *
	 * @return the religion
	 */
	public String getReligion()
	{
		return religion;
	}
	
	/**
	 * Sets the religion.
	 *
	 * @param religion the new religion
	 */
	public void setReligion(String religion)
	{
		this.religion = religion;
	}
	
	/**
	 * Gets the name.
	 *
	 * @return the name
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Sets the name.
	 *
	 * @param name the new name
	 */
	public void setName(String name)
	{
		this.name = name;
	}
	
	/**
	 * Gets the ssn.
	 *
	 * @return the ssn
	 */
	public String getSsn()
	{
		return ssn;
	}
	
	/**
	 * Sets the ssn.
	 *
	 * @param ssn the new ssn
	 */
	public void setSsn(String ssn)
	{
		this.ssn = ssn;
	}
	
	/**
	 * Gets the gender.
	 *
	 * @return the gender
	 */
	public String getGender()
	{
		return gender;
	}
	
	/**
	 * Sets the gender.
	 *
	 * @param gender the new gender
	 */
	public void setGender(String gender)
	{
		this.gender = gender;
	}
	
	/**
	 * Gets the dob.
	 *
	 * @return the dob
	 */
	public String getDob()
	{
		return dob;
	}
	
	/**
	 * Sets the dob.
	 *
	 * @param dob the new dob
	 */
	public void setDob(String dob)
	{
		this.dob = dob;
	}
	
	/**
	 * Gets the ethnicity.
	 *
	 * @return the ethnicity
	 */
	public String getEthnicity()
	{
		return ethnicity;
	}
	
	/**
	 * Sets the ethnicity.
	 *
	 * @param ethnicity the new ethnicity
	 */
	public void setEthnicity(String ethnicity)
	{
		this.ethnicity = ethnicity;
	}
	
	/**
	 * Gets the age.
	 *
	 * @return the age
	 */
	public Integer getAge()
	{
		return age;
	}
	
	/**
	 * Sets the age.
	 *
	 * @param age the new age
	 */
	public void setAge(Integer age)
	{
		this.age = age;
	}
	
	/**
	 * Gets the marital status.
	 *
	 * @return the marital status
	 */
	public String getMaritalStatus()
	{
		return maritalStatus;
	}
	
	/**
	 * Sets the marital status.
	 *
	 * @param maritalStatus the new marital status
	 */
	public void setMaritalStatus(String maritalStatus)
	{
		this.maritalStatus = maritalStatus;
	}
	
	/**
	 * Gets the home address.
	 *
	 * @return the home address
	 */
	public AddressTO getHomeAddress()
	{
		return homeAddress;
	}
	
	/**
	 * Sets the home address.
	 *
	 * @param homeAddress the new home address
	 */
	public void setHomeAddress(AddressTO homeAddress)
	{
		this.homeAddress = homeAddress;
	}
	
	/**
	 * Gets the home phone.
	 *
	 * @return the home phone
	 */
	public PhoneNumTO getHomePhone()
	{
		return homePhone;
	}
	
	/**
	 * Sets the home phone.
	 *
	 * @param homePhone the new home phone
	 */
	public void setHomePhone(PhoneNumTO homePhone)
	{
		this.homePhone = homePhone;
	}
	
	/**
	 * Gets the cell phone.
	 *
	 * @return the cell phone
	 */
	public PhoneNumTO getCellPhone()
	{
		return cellPhone;
	}
	
	/**
	 * Sets the cell phone.
	 *
	 * @param cellPhone the new cell phone
	 */
	public void setCellPhone(PhoneNumTO cellPhone)
	{
		this.cellPhone = cellPhone;
	}
	
	/**
	 * Gets the demographics.
	 *
	 * @return the demographics
	 */
	public List<DemographicSetTO> getDemographics()
	{
		return demographics;
	}
	
	/**
	 * Sets the demographics.
	 *
	 * @param demographics the new demographics
	 */
	public void setDemographics(List<DemographicSetTO> demographics)
	{
		this.demographics = demographics;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "PersonTO [type=" + type + ", birthPlace=" + birthPlace
				+ ", religion=" + religion + ", name=" + name + ", ssn=" + ssn
				+ ", gender=" + gender + ", dob=" + dob + ", ethnicity="
				+ ethnicity + ", age=" + age + ", maritalStatus="
				+ maritalStatus + ", homeAddress=" + homeAddress
				+ ", homePhone=" + homePhone + ", cellPhone=" + cellPhone
				+ ", demographics=" + demographics + ", toString()="
				+ super.toString() + "]";
	}
    
}
