package gov.va.med.via.dto.common;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlList;
import javax.xml.bind.annotation.XmlRootElement;

/**
 *
 */
@XmlRootElement(name="QueryBean")
@XmlAccessorType(XmlAccessType.NONE)
public class QueryBean
{	
	@XmlElement(name="provider")	
    protected Provider provider;
	
	@XmlElement(name="patient")
	protected Patient patient;

	@XmlElement(name="recordSiteCode")
	protected String recordSiteCode;
	
	@XmlElement(name="requestingApp")
	protected String requestingApp;
    
    /**
    * Set to 1 if requesting only active data (appointments, prescriptions, clinical
    * reminders)
    */
	@XmlElement(name="active")
	protected String active;
    /**
    * End date for the query.
    */
    @XmlElement(name="endDate")
    protected String endDate;
    /**
    * Start date for the query.
    */
    @XmlElement(name="startDate")
    protected String startDate;
    /**
    * Maximum number of items to return.  This is usually a required attribute when
    * the return is an array of items.
    */
    @XmlElement(name="maxRecords")
    protected String maxRecords;

    /**
    * Status of data being requested.  This could be Inpatient vs outpatient (procedures,
    * prescriptions/medications)
    */
    @XmlElement(name="status")
    protected String status;
    
    /** Supplemental parameters. */
  	@XmlElement(name="supplementalParameters")
  	@XmlList
    protected String[] supplementalParameters; 

    /**
    * This is the internal id of the data item when detailed informations is being
    * requested.
    */
    @XmlElement(name="itemId")
    private String itemId;
    
    /**
     * This is a query value used by several RPC queries - it has a generic use, however many client apps of the
     * legacy service methods VIAS replaces will recognize this query parameter by name
     * replacing legacy services wil
     * requested.
     */
     @XmlElement(name="target")
     private String target;
     
     /**
      * This is a query value, used by several RPC queries - it has a generic use across methods
      * requested.
      */
      @XmlElement(name="criteria")
      private String criteria;
      
      /**
       * This is a specific named query parameter
       * requested.
       */
       @XmlElement(name="type")
       private String type;
       
       /**
        * This is a specific named query parameter
        * requested.
        */
        @XmlElement(name="direction")
        private String direction;
        
        /**
         * This is a specific named query parameter
         * requested.
         */
         @XmlElement(name="name")
         private String name;
         
         @XmlElement()
    

    private boolean multiSiteQuery = true;//default true, but can be switched to limit sites processed.
    
    public Patient getPatient() {
        return patient;
    }

    public void setPatient(Patient patient) {
        this.patient = patient;
    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public String getActive() {
        return active;
    }

    public void setActive(String active) {
        this.active = active;
    }

    public String getEndDate() {
        return endDate;
    }

    public void setEndDate(String endDate) {
        this.endDate = endDate;
    }

    public String getItemId() {
        return itemId;
    }

    public void setItemId(String itemId) {
        this.itemId = itemId;
    }

    public String getMaxRecords() {
        return maxRecords;
    }

    public void setMaxRecords(String max) {
        this.maxRecords = max;
    }

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getRecordSiteCode() {
        return recordSiteCode;
    }

    public void setRecordSiteCode(String recordSiteCode) {
        this.recordSiteCode = recordSiteCode;
    }

    public String getRequestingApp() {
        return requestingApp;
    }

    public void setRequestingApp(String requestingApp) {
        this.requestingApp = requestingApp;
    }
        
    public String getLoginSiteCode() {
    	
    	String siteCode = null;
    	Provider provider = this.getProvider();
    	if (provider != null){
    		siteCode = (provider.getLoginSiteCode() != null)?provider.getLoginSiteCode():this.getRecordSiteCode();
    	}
    	else{
    		//QueryBean not yet populated with a provider 
    		siteCode = this.getRecordSiteCode();    		
    	}
    	
    	return siteCode;
	}

	public boolean getMultiSiteQuery()
	{
		return multiSiteQuery;
	}

	public void setMultiSiteQuery(boolean multiSiteQuery)
	{
		this.multiSiteQuery = multiSiteQuery;
	}

	public String getTarget() {
		return target;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	public String getCriteria() {
		return criteria;
	}

	public void setCriteria(String criteria) {
		this.criteria = criteria;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getDirection() {
		return direction;
	}

	public void setDirection(String direction) {
		this.direction = direction;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	
	public void setSupplementalParameters(String[] supplementalParameters) {
		this.supplementalParameters = supplementalParameters;
	}

	public String[] getSupplementalParameters() {
		return supplementalParameters;
	}
    
    

}
