package gov.va.med.via.dto.common;

import java.util.List;
import java.util.Map;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.RadiologyReport;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;


// TODO: Auto-generated Javadoc
/**
 * The Class RadiologyReportTO.
 */
@XmlRootElement(name="RadiologyReportTO")
@XmlAccessorType(XmlAccessType.NONE)
public class RadiologyReportTO
    extends AbstractTO
{    
    /** The accession number. */
	@XmlElement(name="accessionNumber")
    protected String accessionNumber;
    
    /** The case number. */
	@XmlElement(name="caseNumber")
    protected String caseNumber;
    
    /** The id. */
	@XmlElement(name="id")
    protected String id;
    
    /** The title. */
	@XmlElement(name="title")
    protected String title;
    
    /** The timestamp. */
	@XmlElement(name="timestamp")
    protected String timestamp;
    
    /** The author. */
	@XmlElement(name="author")
    protected AuthorTO author;
    
    /** The text. */
	@XmlElement(name="text")
    protected String text;
    
    /** The facility. */
	@XmlElement(name="facility")
    protected TaggedText facility;
    
    /** The status. */
	@XmlElement(name="status")
    protected String status;
    
    /** The cpt code. */
	@XmlElement(name="cptCode")
    protected String cptCode;
    
    /** The clinical hx. */
	@XmlElement(name="clinicalHx")
    protected String clinicalHx;
    
    /** The impression. */
	@XmlElement(name="impression")
    protected String impression;
	
	@XmlElement(name="imagingExamTO")
	protected ImagingExamTO imagingExamTO;
	
	/**
	 * Instantiates a new radiology report to.
	 */
	public RadiologyReportTO()
	{
		
	}
	
	/**
	 * Instantiates a new radiology report to.
	 *
	 * @param report the report
	 */
	public RadiologyReportTO(RadiologyReport report) 
	{
		setReport(report); /// moved this code out of constructor so 3rd constructor can use this code
	}

	private void setReport(RadiologyReport report) {
		if ( report.getFault()!=null ){
			this.setFault(new FaultTO(report.getFault()));
		}
		else{
			this.id = report.getId();
            this.title = report.getTitle();
            this.timestamp = report.getTimestamp();           
            this.status = report.getStatus();
            this.cptCode = report.getCptCode();
            this.clinicalHx = report.getClinicalHx();
            this.impression = report.getImpression();
            this.accessionNumber = report.getAccessionNumber();
            this.caseNumber = report.getCaseNumber();
            
            if (report.getAuthor() != null)
            {
                this.author = new AuthorTO(report.getAuthor());
            }
            this.text = report.getText();
            if (report.getFacility() != null)
            {
                this.facility = new TaggedText(report.getFacility().getId(), report.getFacility().getName());
            }   
            if ( report.getExam()!=null)
            {
            	this.imagingExamTO = new ImagingExamTO(report.getExam());
            }
		}
	}
	
	public RadiologyReportTO(Map<String, List<DataBean>> siteDataBeanMapForDao) 
	{
		for ( String site: siteDataBeanMapForDao.keySet())
		{
			for ( DataBean dataBean: siteDataBeanMapForDao.get(site) )
			{
				RadiologyReport report = (RadiologyReport)dataBean;
				setReport(report);
			}			
		}
	}

	/**
	 * Gets the accession number.
	 *
	 * @return the accession number
	 */
	public String getAccessionNumber()
	{
		return accessionNumber;
	}
	
	/**
	 * Sets the accession number.
	 *
	 * @param accessionNumber the new accession number
	 */
	public void setAccessionNumber(String accessionNumber)
	{
		this.accessionNumber = accessionNumber;
	}
	
	/**
	 * Gets the case number.
	 *
	 * @return the case number
	 */
	public String getCaseNumber()
	{
		return caseNumber;
	}
	
	/**
	 * Sets the case number.
	 *
	 * @param caseNumber the new case number
	 */
	public void setCaseNumber(String caseNumber)
	{
		this.caseNumber = caseNumber;
	}
	
	/**
	 * Gets the id.
	 *
	 * @return the id
	 */
	public String getId()
	{
		return id;
	}
	
	/**
	 * Sets the id.
	 *
	 * @param id the new id
	 */
	public void setId(String id)
	{
		this.id = id;
	}
	
	/**
	 * Gets the title.
	 *
	 * @return the title
	 */
	public String getTitle()
	{
		return title;
	}
	
	/**
	 * Sets the title.
	 *
	 * @param title the new title
	 */
	public void setTitle(String title)
	{
		this.title = title;
	}
	
	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public String getTimestamp()
	{
		return timestamp;
	}
	
	/**
	 * Sets the timestamp.
	 *
	 * @param timestamp the new timestamp
	 */
	public void setTimestamp(String timestamp)
	{
		this.timestamp = timestamp;
	}
	
	/**
	 * Gets the author.
	 *
	 * @return the author
	 */
	public AuthorTO getAuthor()
	{
		return author;
	}
	
	/**
	 * Sets the author.
	 *
	 * @param author the new author
	 */
	public void setAuthor(AuthorTO author)
	{
		this.author = author;
	}
	
	/**
	 * Gets the text.
	 *
	 * @return the text
	 */
	public String getText()
	{
		return text;
	}
	
	/**
	 * Sets the text.
	 *
	 * @param text the new text
	 */
	public void setText(String text)
	{
		this.text = text;
	}
	
	/**
	 * Gets the facility.
	 *
	 * @return the facility
	 */
	public TaggedText getFacility()
	{
		return facility;
	}
	
	/**
	 * Sets the facility.
	 *
	 * @param facility the new facility
	 */
	public void setFacility(TaggedText facility)
	{
		this.facility = facility;
	}
	
	/**
	 * Gets the status.
	 *
	 * @return the status
	 */
	public String getStatus()
	{
		return status;
	}
	
	/**
	 * Sets the status.
	 *
	 * @param status the new status
	 */
	public void setStatus(String status)
	{
		this.status = status;
	}
	
	/**
	 * Gets the cpt code.
	 *
	 * @return the cpt code
	 */
	public String getCptCode()
	{
		return cptCode;
	}
	
	/**
	 * Sets the cpt code.
	 *
	 * @param cptCode the new cpt code
	 */
	public void setCptCode(String cptCode)
	{
		this.cptCode = cptCode;
	}
	
	/**
	 * Gets the clinical hx.
	 *
	 * @return the clinical hx
	 */
	public String getClinicalHx()
	{
		return clinicalHx;
	}
	
	/**
	 * Sets the clinical hx.
	 *
	 * @param clinicalHx the new clinical hx
	 */
	public void setClinicalHx(String clinicalHx)
	{
		this.clinicalHx = clinicalHx;
	}
	
	/**
	 * Gets the impression.
	 *
	 * @return the impression
	 */
	public String getImpression()
	{
		return impression;
	}
	
	/**
	 * Sets the impression.
	 *
	 * @param impression the new impression
	 */
	public void setImpression(String impression)
	{
		this.impression = impression;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "RadiologyReportTO [accessionNumber=" + accessionNumber
				+ ", caseNumber=" + caseNumber + ", id=" + id + ", title="
				+ title + ", timestamp=" + timestamp + ", author=" + author
				+ ", text=" + text + ", facility=" + facility + ", status="
				+ status + ", cptCode=" + cptCode + ", clinicalHx="
				+ clinicalHx + ", impression=" + impression + ", toString()="
				+ super.toString() + "]";
	}
}
