package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.Provider;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

// TODO: Auto-generated Javadoc
/**
 * The Class TaggedPatientArray.
 * 
 * @author DOMAINROMERJ
 */
@XmlRootElement(name="TaggedPatientArray")
@XmlAccessorType(XmlAccessType.NONE)
public class TaggedPatientArray
    extends AbstractTaggedArrayTO
{
	/** The patients. */
	@XmlElementWrapper(name="patients")
	@XmlElement(name="patientTO")	
    protected List<PatientTO> patients;

	/**
	 * Instantiates a new tagged patient array.
	 */
	public TaggedPatientArray()
	{	
	}
	
	public TaggedPatientArray(Map<String, List<DataBean>> daoSiteDataBeans) 
	{		
		patients = new ArrayList<PatientTO>();
		
		Set<String> databeanSiteKeys = daoSiteDataBeans.keySet();
		for(String siteKey : databeanSiteKeys)
		{
			boolean foundFault = false;
			for(DataBean dataBean: daoSiteDataBeans.get(siteKey))
			{
				if(dataBean instanceof Fault){
					this.setFault(new FaultTO((Fault)dataBean));
					foundFault = true;
					break;
				}
				patients.add(new PatientTO((Patient)dataBean));
			}
			
			setTag(siteKey);
			if(!foundFault)
			{
				this.setPatients(patients);
				setCount(patients.size());
			}			
		}
	}
	
    /**
     * Instantiates a new tagged patient array.
     *
     * @param patientBeans the patient beans
     */
    public TaggedPatientArray(Patient[] patientBeans)
    {
    	if (patientBeans == null || patientBeans.length == 0) return;
    	
    	if (patientBeans.length == 1 && patientBeans[0].hasErrors())
    	{
    		this.fault = new FaultTO(patientBeans[0].getFault());
    	}
    	else
    	{
    		patients = new ArrayList<PatientTO>();
    	
    		for (int i = 0; i < patientBeans.length; i++ )
    		{
    			patients.add(new PatientTO(patientBeans[i]));
    		}
    	}	
        count = patientBeans.length;
    }
 

    public TaggedPatientArray(List<DataBean> dataBeans, String siteCode){
		if (dataBeans== null || dataBeans.size() == 0){
			count=0;
		    return;
		}
		patients = new ArrayList<PatientTO>();   
		boolean foundFault = false;
		for(DataBean dataBean: dataBeans){
			if(dataBean instanceof Fault){
				this.setFault(new FaultTO((Fault)dataBean));
				foundFault = true;
				break;
			}
			patients.add(new PatientTO((Patient)dataBean));
		}
		
		setTag(siteCode);
		if(!foundFault){
			this.setPatients(patients);
			setCount(patients.size());
		}
	}
	
    
	/**
	 * Gets the patients.
	 *
	 * @return the patients
	 */
	public List<PatientTO> getPatients()
	{
		return patients;
	}

	/**
	 * Sets the patients.
	 *
	 * @param patients the new patients
	 */
	public void setPatients(List<PatientTO> patients)
	{
		this.patients = patients;
	}
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.dto.common.AbstractTaggedArrayTO#toString()
	 */
	@Override
	public String toString()
	{
		return "TaggedPatientArray [patients=" + patients + ", toString()="
				+ super.toString() + "]";
	}
    
}