package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.ProstheticClaim;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The Class TaggedProstheticClaimArray.
 */
@XmlRootElement(name = "TaggedProstheticClaimArray")
@XmlAccessorType(XmlAccessType.NONE)
public class TaggedProstheticClaimArray
    extends AbstractTaggedArrayTO
{

    /** The claims. */
	@XmlElementWrapper(name = "claims")
	@XmlElement(name = "prostheticClaimTO")
    protected List<ProstheticClaimTO> claims;

	/**
	 * Instantiates a new tagged prosthetic claim array.
	 */
	public TaggedProstheticClaimArray() {
	}
	/**
	 * Instantiates a new tagged prosthetic claim array with a DataBean list.
	 * @param beans The list of DataBean(s)
	 */
	public TaggedProstheticClaimArray(List<DataBean> beans) {
		if (beans == null || beans.size() == 0) {
			count=0;
		    return;
		}
		claims = new ArrayList<ProstheticClaimTO>();   
		for (int i = 0; i < beans.size(); i++ ){
			claims.add(new ProstheticClaimTO((ProstheticClaim)beans.get(i)));
		}	    		
		count = beans.size();
	}	
	/**
	 * Instantiates a new tagged prosthetic claim array with a DataBean list and a tag.
	 * @param beans The list of DataBean(s)
	 * @param tag The site code
	 */
	public TaggedProstheticClaimArray(List<DataBean> beans, String tag){
		this(beans);
		setTag(tag);
	}	
	/**
	 * Instantiates a new tagged prosthetic claim array.
	 * @param daoSiteDataBeans The Map of daoSiteDataBeans.
	 */
	public TaggedProstheticClaimArray(Map<String, List<DataBean>> daoSiteDataBeans) 
	{
		List<ProstheticClaimTO> curProstheticClaims = new ArrayList<ProstheticClaimTO>();   
		Set<String> databeanSiteKeys = daoSiteDataBeans.keySet();
		boolean foundFault = false;
		String tag="";
		
		for(String siteKey : databeanSiteKeys) {
			
			tag = siteKey;
			List<DataBean> siteDataBeans = daoSiteDataBeans.get(siteKey);
			
			for (int i = 0; i < siteDataBeans.size(); i++ ) {
				DataBean dataBean = siteDataBeans.get(i);
				if(dataBean instanceof Fault) {
					this.setFault(new FaultTO((Fault)dataBean));
					foundFault = true;
					break;
				}
				curProstheticClaims.add(new ProstheticClaimTO((ProstheticClaim)siteDataBeans.get(i)));
			}	
		}
		if(!foundFault) {
			if (curProstheticClaims.size() > 0) {
				setClaims(curProstheticClaims);
				setCount(claims.size());				
			}			
		}	
		setTag(tag);
	}
	
	/**
	 * Gets the claims.
	 *
	 * @return the claims
	 */
	public List<ProstheticClaimTO> getClaims()
	{
		return claims;
	}

	/**
	 * Sets the claims.
	 *
	 * @param claims the new claims
	 */
	public void setClaims(List<ProstheticClaimTO> claims)
	{
		this.claims = claims;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.via.dto.common.AbstractTaggedArrayTO#toString()
	 */
	@Override
	public String toString()
	{
		return "TaggedProstheticClaimArray [claims=" + claims + ", toString()="
				+ super.toString() + "]";
	}
}