package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.RatedDisability;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The Class TaggedRatedDisabilityArray.
 */
@XmlRootElement(name = "TaggedRatedDisabilityArray")
@XmlAccessorType(XmlAccessType.NONE)
public class TaggedRatedDisabilityArray extends AbstractTaggedArrayTO {

	/** The disabilities. */
	@XmlElementWrapper(name = "disabilities")
	@XmlElement(name = "ratedDisabilityTO")
	protected List<RatedDisabilityTO> disabilities;

	/**
	 * Instantiates a new tagged rated disability array.
	 */
	public TaggedRatedDisabilityArray() {
	}
	/**
	 * Instantiates a new tagged rated disability array with a DataBean list.
	 * @param beans The list of DataBean(s)
	 */
	public TaggedRatedDisabilityArray(List<DataBean> beans) {
		if (beans == null || beans.size() == 0) {
			count=0;
		    return;
		}
		disabilities = new ArrayList<RatedDisabilityTO>();   
		for (int i = 0; i < beans.size(); i++ ){
			disabilities.add(new RatedDisabilityTO((RatedDisability)beans.get(i)));
		}	    		
		count = beans.size();
	}	
	/**
	 * Instantiates a new tagged rated disability array with a DataBean list and a tag.
	 * @param beans The list of DataBean(s)
	 * @param tag The site code
	 */
	public TaggedRatedDisabilityArray(List<DataBean> beans, String tag){
		this(beans);
		setTag(tag);
	}	
	/**
	 * Instantiates a new tagged rated disability array.
	 * @param daoSiteDataBeans The Map of daoSiteDataBeans.
	 */
	public TaggedRatedDisabilityArray(Map<String, List<DataBean>> daoSiteDataBeans) 
	{
		List<RatedDisabilityTO> curDisabilities = new ArrayList<RatedDisabilityTO>();   
		Set<String> databeanSiteKeys = daoSiteDataBeans.keySet();
		boolean foundFault = false;
		String tag="";
		
		for(String siteKey : databeanSiteKeys) {
			
			tag = siteKey;
			List<DataBean> siteDataBeans = daoSiteDataBeans.get(siteKey);
			
			for (int i = 0; i < siteDataBeans.size(); i++ ) {
				DataBean dataBean = siteDataBeans.get(i);
				if(dataBean instanceof Fault) {
					this.setFault(new FaultTO((Fault)dataBean));
					foundFault = true;
					break;
				}
				curDisabilities.add(new RatedDisabilityTO((RatedDisability)siteDataBeans.get(i)));
			}	
		}
		if(!foundFault) {
			if ( curDisabilities.size()>0) {
				setDisabilities(curDisabilities);
				setCount(disabilities.size());				
			}			
		}	
		setTag(tag);
	}

	/**
	 * Gets the disabilities.
	 * 
	 * @return the disabilities
	 */
	public List<RatedDisabilityTO> getDisabilities() {
		return disabilities;
	}

	/**
	 * Sets the disabilities.
	 * 
	 * @param disabilities the new disabilities
	 */
	public void setDisabilities(List<RatedDisabilityTO> disabilities) {
		this.disabilities = disabilities;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.va.med.via.dto.common.AbstractTaggedArrayTO#toString()
	 */
	@Override
	public String toString() {
		return "TaggedRatedDisabilityArray [disabilities=" + disabilities
				+ ", toString()=" + super.toString() + "]";
	}
}