package gov.va.med.via.dto.common;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.DictionaryTextArray;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.databeans.Text;
import gov.va.med.via.rpc.KeyValuePair;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlAccessType;

// TODO: Auto-generated Javadoc
/**
 * <p>Java class for TaggedText complex type.
 * 
 * 
 */
@XmlRootElement(name="TaggedText")
@XmlAccessorType(XmlAccessType.NONE)
public class TaggedText
    extends AbstractTO
{

    /** The tag. */
	@XmlElement(name="tag")
    protected String tag;
    
    /** The text. */
	@XmlElement(name="text")
    protected String text;
    
    /** The text array. */
	@XmlElementWrapper(name="textArray")
	@XmlElement(name="textArray")
    protected List<String> textArray;
    
    /** The tagged results. */
	@XmlElementWrapper(name="taggedResults")
	@XmlElement(name="taggedText")
    protected List<TaggedText> taggedResults;

    /**
     * Instantiates a new tagged text.
     */
    public TaggedText()
    {    	
    }
    
    public TaggedText(Map<String, List<DataBean>> daoSiteDataBeans) 
    {		
		taggedResults = new ArrayList<TaggedText>();
		
		Set<String> databeanSiteKeys = daoSiteDataBeans.keySet();
		for(String siteKey : databeanSiteKeys)
		{
			List<DataBean> daoDataBeans = daoSiteDataBeans.get(siteKey);
			if (daoDataBeans.size() == 1)
			{
				taggedResults.add(new TaggedText(daoDataBeans.get(0), siteKey));
			}
			else
			{
				taggedResults = processTaggedTextList(daoSiteDataBeans.get(siteKey), siteKey);
			}
		}
		
		this.setTaggedResults(taggedResults);
	}
    
   
    /**
     * Instantiates a new tagged text when ONE data bean object is received. The data bean can be a Fault, 
     * a DictionaryTextArray or a Text object. 
     *
     * @param dataBean the data bean
     * @param siteKey the site key
     */
    public TaggedText(DataBean dataBean, String siteKey) 
    {	
		if(dataBean instanceof Fault)
		{
			this.setFault(new FaultTO((Fault)dataBean));
		}
		
		if(dataBean instanceof DictionaryTextArray)
		{				
			DictionaryTextArray dictionary = (DictionaryTextArray)dataBean;
			if (dictionary == null || dictionary.getDictionary() == null) 
				return;
	    	
	    	this.tag = siteKey;
	    	taggedResults = new ArrayList<TaggedText>();
	    	
	    	for (Iterator<String> i = dictionary.getDictionary().keySet().iterator(); i.hasNext();)
	    	{
	    		String key = (String)i.next();
	    		List<String> list = dictionary.getDictionary().get(key);
	    		taggedResults.add(new TaggedText(key, list));
	    	}
		}
		
		if(dataBean instanceof Text)
		{
			Text textBean = (Text)dataBean;
			if (textBean.getTag() == null || textBean.getTag().equals(""))
			{
				this.tag = siteKey;
			}
			else
			{
				this.tag = textBean.getTag();
			}
			this.text = textBean.getText();
			if (textBean.getTextArray() != null)
			{
				this.textArray = new ArrayList<String>();
				for (int i =0; i < textBean.getTextArray().length; i++)
				{
	    			this.textArray.add(textBean.getTextArray()[i]);
				}
			}
		}
    }
    
    /**
     * If multiple daoDataBeans are sent, it most be a list of Text objects that need to be processed for the taggedResults attribute
     *
     * @param daoDataBeans the dao data beans
     * @param siteKey the site key
     */
    public List<TaggedText> processTaggedTextList(List<DataBean> daoDataBeans, String siteKey) 
    {	    	
    	this.tag = siteKey;
    	List<TaggedText> results = new ArrayList<TaggedText>();

		for(DataBean dataBean: daoDataBeans)
		{
			if(dataBean instanceof Text)
			{
				Text textBean = (Text)dataBean;				
				results.add(new TaggedText(textBean.getTag(), textBean.getText()));
			}			
		}
		
		return results;
    }
    
    public TaggedText(SiteId siteId)
    {
    	if(siteId != null){
    		this.tag = siteId.getId();
    		this.text = siteId.getName();
    	}
    }
    
    /**
     * Instantiates a new tagged text.
     *
     * @param tag the tag
     * @param text the text
     */
    public TaggedText(String tag, String text)
    {
        this.tag = tag;
        this.text = text;
    }
       
    /**
     * Instantiates a new tagged text.
     *
     * @param tag the tag
     */
    public TaggedText(String tag)
    {
        this.tag = tag;
    }
    
    /**
     * Instantiates a new tagged text.
     *
     * @param tag the tag
     * @param text the text
     */
    public TaggedText(Integer tag, String text)
    {
        this.tag = tag != null? tag.toString() : null;
        this.text = text;
    }
    
    /**
     * Instantiates a new tagged text.
     *
     * @param tag the tag
     * @param list the list
     */
    public TaggedText(String tag, List<String> list)
    {
    	if (list == null || list.isEmpty()) return;
    	
        this.tag = tag;
        this.textArray = list;
    }
    
    /**
     * Instantiates a new tagged text - adds it to an array if one exists in the databean - otherwise keeps the TO singular, outside of a collection.
     *
     * @param text the text
     */
    public TaggedText(Text text)
    {
    	if (text.hasErrors()) 
    		this.fault = new FaultTO(text.getFault());    		
    	else
    	{
    		this.tag = text.getTag();
    		this.text = text.getText();
    		
    		if (text.getTextArray() != null)
    		{
    			this.textArray = new ArrayList<String>();
    			for (int i =0; i < text.getTextArray().length; i++)
    			{
        			this.textArray.add(text.getTextArray()[i]);
    			}
    		}
    	}
    }
    
    /**
     * Instantiates a new tagged text.
     *
     * @param keyValuePair the key value pair
     */
    public TaggedText(KeyValuePair<String, String> keyValuePair)
    {
    	if (keyValuePair == null) return;
    	
    	this.tag = keyValuePair.getKey();
    	this.text = keyValuePair.getValue();
    }
    
    /**
     * Gets the value of the tag property.
     *
     * @return the tag
     * possible object is
     * {@link String }
     */
    public String getTag() 
    {
        return tag;
    }

    /**
     * Sets the value of the tag property.
     * 
     * @param value
     *     allowed object is
     *     {@link String }
     *     
     */
    public void setTag(String value) {
        this.tag = value;
    }

    /**
     * Gets the value of the text property.
     *
     * @return the text
     * possible object is
     * {@link String }
     */
    public String getText() {
        return text;
    }

    /**
     * Sets the value of the text property.
     * 
     * @param value
     *     allowed object is
     *     {@link String }
     *     
     */
    public void setText(String value) {
        this.text = value;
    }

    /**
     * Gets the value of the textArray property.
     *
     * @return the text array
     * possible object is
     * {@link ArrayOfString }
     */
    public List<String> getTextArray() {
        return textArray;
    }

    /**
     * Sets the value of the textArray property.
     * 
     * @param value
     *     allowed object is
     *     {@link ArrayOfString }
     *     
     */
    public void setTextArray(List<String> value) {
        this.textArray = value;
    }

    /**
     * Gets the value of the taggedResults property.
     *
     * @return the tagged results
     * possible object is
     * {@link ArrayOfTaggedText }
     */
    public List<TaggedText> getTaggedResults() {
        return taggedResults;
    }

    /**
     * Sets the value of the taggedResults property.
     * 
     * @param value
     *     allowed object is
     *     {@link ArrayOfTaggedText }
     *     
     */
    public void setTaggedResults(List<TaggedText> value) {
        this.taggedResults = value;
    }

}
