package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.MentalHealthInstrumentResultSetTO;
import gov.va.med.via.dto.common.NoteResultTO;
import gov.va.med.via.dto.common.OrderDialogItemArray;
import gov.va.med.via.dto.common.OrderTO;
import gov.va.med.via.dto.common.PatientArray;
import gov.va.med.via.dto.common.PatientRecordFlagArray;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.PersonsTO;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.RadiologyReportTO;
import gov.va.med.via.dto.common.ReminderReportPatientListTO;
import gov.va.med.via.dto.common.TaggedAllergyArrays;
import gov.va.med.via.dto.common.TaggedAppointmentArrays;
import gov.va.med.via.dto.common.TaggedChemHemRptArrays;
import gov.va.med.via.dto.common.TaggedConsultArrays;
import gov.va.med.via.dto.common.TaggedCytologyRptArrays;
import gov.va.med.via.dto.common.TaggedHospitalLocationArray;
import gov.va.med.via.dto.common.TaggedIcdRptArrays;
import gov.va.med.via.dto.common.TaggedInpatientStayArray;
import gov.va.med.via.dto.common.TaggedLabTestArrays;
import gov.va.med.via.dto.common.TaggedMedicationArrays;
import gov.va.med.via.dto.common.TaggedMentalHealthInstrumentAdministrationArrays;
import gov.va.med.via.dto.common.TaggedMicrobiologyRptArrays;
import gov.va.med.via.dto.common.TaggedNoteArrays;
import gov.va.med.via.dto.common.TaggedOrderArrays;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedPatientAssociateArrays;
import gov.va.med.via.dto.common.TaggedProblemArrays;
import gov.va.med.via.dto.common.TaggedRadiologyReportArrays;
import gov.va.med.via.dto.common.TaggedSurgeryReportArrays;
import gov.va.med.via.dto.common.TaggedSurgicalPathologyRptArrays;
import gov.va.med.via.dto.common.TaggedText;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedVisitArray;
import gov.va.med.via.dto.common.TaggedVitalSignArrays;
import gov.va.med.via.dto.common.TaggedVitalSignSetArrays;
import gov.va.med.via.dto.common.TextArray;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserArray;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

/**
 * TheEmrSrcInterface is the Electronic Medical Record Service.  
 * This interface provides web services used to obtain clinical data for patients. 
 * There are three service methods that allow write functionality
 * 		signNote, writeNote and writeSimpleOrder 
 * all of the other functionality exposed by this service is read only access.
 * 
 * The interface supports multiple ViatA site data access.
 * Users of this interface are intended to be healthcare providers. 
 *
 */
@WebService
public interface EmrSvcInterface extends BaseServiceInterface
{
	
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
    /**
     * Lookup a CPRS-enabled user
     * 
     * @param queryBean the query bean
     * 
     * @return UserArray - list of users matching the target string
     * 
     * @mdws target queryBean.target
     * 
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a user name in LASTNAME, FIRSTNAME format
     */
    @WebMethod(operationName = "cprsUserLookup")
    public @WebResult(name="UserArray") UserArray cprsUserLookup(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

   /* /**
     * User has permission.
     *
     * VIA Increment 1
     *
     * @param uid the uid
     * @param permissionName the permission name
     * @return the gov.va.med.via.service.generated.emrservice. bool to
     *//*
    @WebMethod(operationName = "userHasPermission")
    public BoolTO userHasPermission(
    	@WebParam(name = "uid") String uid,
        @WebParam(name = "permissionName") String permissionName,
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
   /* /**
     * Gets the user security keys.
     *
     * VIA Increment 1
     * 
     * @param uid the uid
     * @return the user security keys
     *//*
    @WebMethod(operationName = "getUserSecurityKeys")
    public UserSecurityKeyArray getUserSecurityKeys(
       	@WebParam(name = "uid") String uid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/

   /* /**
     * Gets the users with option.
     *
     * VIA Increment 1
     * 
     * @param optionName the option name
     * @return the users with option
     *//*
    @WebMethod(operationName = "getUsersWithOption")
    public TaggedTextArray getUsersWithOption(
         @WebParam(name = "optionName") String optionName,
         @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
    /**
     * Select a patient at logged in site. DFN is the IEN of the patient
     * 
     * @param queryBean the query bean
     * 
     * @returns PatientTO - patient record information
     * 
     * @mdws DFN queryBean.patient.localPid
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "select")
    public @WebResult(name="PatientTO") PatientTO select(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
	
    /**
     * Make a patient inquiry call (address, contact numbers, NOK, etc. information)
     * 
     * @param queryBean the query bean
     * 
     * @return TestTO - the inquiry response
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "patientInquiry")
    public @WebResult(name="TextTO") TextTO patientInquiry(
            @WebParam(name = "queryBean") QueryBean queryBean    	
     );

    /**
     * Get patient's appointments.
     * 
     * @param queryBean - the queryBean with patient/provider context (state data)
     * 
     * @return TaggedAppointmentArrays - list of appointments tagged by site
     *  
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAppointments")
    public  @WebResult(name="TaggedAppointmentArrays") TaggedAppointmentArrays getAppointments(
         @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Write a note.
     * 
     * @return NoteResultTO - result of the write note
     * 
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule When set, the prfIEN must be a valid IEN (a numeric or decimal value)
     * @autorule encounterString must be a valid encounter string
     * @autorule queryBean.patient.localPid must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule text cannot be a null value or an empty string
     * @autorule titleIEN must be a valid IEN (a numeric or decimal value)
     */
    @WebMethod(operationName = "writeNote")
    public @WebResult(name="NoteResultTO") NoteResultTO writeNote(
        @WebParam(name = "titleIEN") String titleIEN,
    	@WebParam(name = "encounterString") String encounterString,
        @WebParam(name = "text") String text,
    	@WebParam(name = "authorDUZ") String authorDUZ,
        @WebParam(name = "cosignerDUZ") String cosignerDUZ,        
    	@WebParam(name = "consultIEN") String consultIEN,
        @WebParam(name = "prfIEN") String prfIEN,
        @WebParam(name = "queryBean") QueryBean queryBean        
    );
    
    /**
     * Get future clinic visits from all connected VistAs.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - clinic visit reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId (optional parameter)
     *
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getFutureClinicVisitsReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getFutureClinicVisitsReports(
		@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's adhoc health summary by display name from all connected VistAs.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of Health Summaries
     * 
     * @mdws displayName queryBean.name
     * 
     * @autorule queryBean.name cannot be a null value or an empty string
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAdhocHealthSummaryByDisplayName")
    public @WebResult(name="TaggedTextArray")TaggedTextArray getAdhocHealthSummaryByDisplayName(
		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get signed note metadata from all connected VistAs
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedNoteArrays - list of signed notes
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nNotes queryBean.maxRecords
     *
     * @autorule The target 'queryBean.endDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule The target 'queryBean.startDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getSignedNotes")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getSignedNotes(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get note for appointment.
     * 
     * @param queryBean the query bean
     * 
     * @return TextTO - the appointment text
     * 
	 * @mdws siteId queryBean.recordSiteCode
	 * @mdws aptId queryBean.itemId
     *
     * @autorule queryBean.itemId must be a valid visit string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAppointmentText")
    public @WebResult(name="TextTO") TextTO getAppointmentText(
		@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get past clinic visits from all connected VistAs.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     *
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getPastClinicVisitsReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getPastClinicVisitsReports(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient associates (NOK, caregiver, etc.)
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedPatientAssociateArrays - list of patient associates
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientAssociates")
    public @WebResult(name="TaggedPatientAssociateArrays") TaggedPatientAssociateArrays getPatientAssociates(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets the pcp for patient.
     *
     * @param queryBean the queryBean
     * 
     * @return TaggedText - the Primary Care Physician info
     * 
     * @mdws pid queryBean.patient.localPid
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
   @WebMethod(operationName = "getPcpForPatient")
    public @WebResult(name="TaggedText") TaggedText getPcpForPatient(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get notes with text from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedNoteArrays - list of notes with text
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.maxRecords
     *
     * @autorule queryBean.maxRecords value, if set, must be an integer greater than zero
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate and queryBean.endDate must be a valid date range. Accepted values are '0' (for unlimited) or a date in 'yyyyMMdd' format
     */   
    @WebMethod(operationName = "getNotesWithText")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getNotesWithText(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Sign a note.
     * 
     * @autorule esig cannot be a null value or an empty string
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "signNote")
    public @WebResult(name="TextTO") TextTO signNote(
    	@WebParam(name = "noteIEN") String noteIEN,
    	@WebParam(name = "userDUZ") String userDUZ, 
    	@WebParam(name = "esig") String esig,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
       
    /**
     * Get the list of teams
     * 
     * @param queryBean the queryBean - list of teams
     * 
     * @return TaggedText
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getTeams")
    public @WebResult(name="TaggedText") TaggedText getTeams(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
            
    /**
     * Get unit dose medications from all connected VistAs
     *
     * @param queryBean the queryBean
     * 
     * @return TaggedMedicationArrays
     *  
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getUnitDoseMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getUnitDoseMeds(
       	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get radiology reports from all connected VistAs
     *  
     * @param queryBean the queryBean
     * 
     * @return TaggedRadiologyReportArrays - list of reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getRadiologyReports")
    public  @WebResult(name="TaggedRadiologyReportArrays") TaggedRadiologyReportArrays getRadiologyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get surgery reports from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedSurgeryReportArrays
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getSurgeryReports")
    public @WebResult(name="TaggedSurgeryReportArrays") TaggedSurgeryReportArrays getSurgeryReports(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get allergies from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedAllergyArrays - list of allergies
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAllergies")
    public  @WebResult(name="TaggedAllergyArrays")TaggedAllergyArrays getAllergies(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );       

    /**
     * Get selected patient's admissions
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedInpatientStayArray - List of Admissions
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "getAdmissions")
    public @WebResult(name="TaggedInpatientStayArray") TaggedInpatientStayArray getAdmissions(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's cytopathology reports.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the report
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getCytopathologyReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getCytopathologyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's vital signs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedVitalSignSetArrays - list of vital signs
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getVitalSigns")
    public @WebResult(name="TaggedVitalSignSetArrays") TaggedVitalSignSetArrays getVitalSigns(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
        
    /**
     * Get non-VA medications from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedMedicationArrays
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getOtherMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getOtherMeds(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's electron microscopy reports.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getElectronMicroscopyReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getElectronMicroscopyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get IV medications from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedMedicationArrays - the Iv meds
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getIvMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getIvMeds(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );   

    /**
     * Get outpatient medications from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedMedicationArrays - the out patient meds
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getOutpatientMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getOutpatientMeds(
        @WebParam(name = "queryBean") QueryBean queryBean
     );
    
    /**
     * Get patient's outpatient prescription profile.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the rx profile
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getOutpatientRxProfile")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getOutpatientRxProfile(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get problem list from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedProblemArrays - list of problems
     * 
     * @mdws type queryBean.type
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.type must be ACTIVE, INACTIVE or ALL.
     */
    @WebMethod(operationName = "getProblemList")
    public @WebResult(name="TaggedProblemArrays") TaggedProblemArrays getProblemList(
        @WebParam(name = "queryBean") QueryBean queryBean    
     );
    
    /**
     * Get patient confidentiality from all connected sites.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - list of confidentialties
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConfidentiality")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getConfidentiality(
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
   /* //**
     * Use when switching patient lookup sites.
     * 
     * VIA Increment 4
     * 
     *//*
    @WebMethod(operationName = "visit")
    public TaggedTextArray visit(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "sitelist") String sitelist,
    	@WebParam(name = "userSiteCode") String userSiteCode,
        @WebParam(name = "userName") String userName,        
    	@WebParam(name = "duz") String duz,
        @WebParam(name = "context") String context,
        @WebParam(name = "queryBean") QueryBean queryBean    
    );   */

    /**
     * Get selected patient's visits
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedVisitArray - list of visit events
     *
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getVisits")
    public @WebResult(name="TaggedVisitArray") TaggedVisitArray getVisits(
       	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a PRF note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - response indicating postive or negative
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isPrfNote")
    public @WebResult(name="TextTO") TextTO isPrfNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a VistA's hospital locations (clinics, etc.).
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedHospitalLocationArray - list of locations
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     * 
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned hospital locations.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of matching hospital locations.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getLocations")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getLocations(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get admission summaries from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getAdmissionsReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getAdmissionsReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's blood availability reports.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getBloodAvailabilityReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getBloodAvailabilityReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's blood bank reports.
     * 
     *  @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getBloodBankReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getBloodBankReports(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's blood transfusion reports.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getBloodTransfusionReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getBloodTransfusionReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get care team from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getCareTeamReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getCareTeamReports(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's crisis notes.
     * 
     *  @param queryBean the queryBean
     * 
     * @return TaggedTextArray - list of the crisis notes
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getCrisisNotes")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getCrisisNotes(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get ADT reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getExpandedAdtReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getExpandedAdtReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get discharge diagnosis from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getDischargeDiagnosisReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getDischargeDiagnosisReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get discharge reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getDischargesReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getDischargesReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get discharge summaries from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedNoteArrays - list of Discharge summaries
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getDischargeSummaries")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getDischargeSummaries(
        @WebParam(name = "queryBean") QueryBean queryBean
    );    

    /**
     * Get patient's clinical warnings.
     * 
     * VIA Increment 5
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getClinicalWarnings")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getClinicalWarnings(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get clinics by name
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws target queryBean.target
     * 
     */
    @WebMethod(operationName = "getClinicsByName")
    public TaggedTextArray getClinicsByName(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get C&P reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getCompAndPenReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getCompAndPenReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's immunizations.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the list of immunizations
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getImmunizations")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getImmunizations(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get lab test description text
     * 
     * @param queryBean  the queryBean
     * 
     * @return TaggedTextArray - the test descriptions
     * 
     * @mdws identifierString queryBean.target
     * 
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target needs to be numeric
     */
    @WebMethod(operationName = "getLabTestDescription")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getLabTestDescription(        
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's advance directives.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getAdvanceDirectives")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getAdvanceDirectives(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a patient's orders
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the orders
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAllOrders")
    public @WebResult(name="TaggedOrderArrays") TaggedOrderArrays getAllOrders(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's autopsy reports.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAutopsyReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getAutopsyReports(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

   /**
     * Get patient's IDs from the session's base connection (i.e. from your local site/authenticated site)
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the reports
     * 
     * @mdws sitecode queryBean.provider.loginSiteCode
     * @mdws patientId queryBean.patient.localPid
     * @mdws patientId queryBean.patient.mpiPid
     * @mdws idType queryBean.type
     * 
     * NOTE: If queryBean.type == "DFN" then queryBean.patient.localPid will be required and populated
     * else if queryBean.type == "ICN" then queryBean.patient.mpiPid will be required and populated
     * 
     * @autorule When queryBean.type equals 'DFN' then queryBean.patient.localPid is required else if queryBean.type equals 'ICN' then queryBean.patient.mpiPid is required.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.type must be DFN or ICN.
     */
    @WebMethod(operationName = "getCorrespondingIds")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getCorrespondingIds(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get hospital locations.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedHospitalLocationArray - the locations
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     * 
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned hospital locations.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of matching hospital locations.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getHospitalLocations")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getHospitalLocations(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get ICD procedure reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedIcdRptArrays - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getIcdProceduresReports")
    public @WebResult(name="TaggedIcdRptArrays") TaggedIcdRptArrays getIcdProceduresReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get ICD surgery reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedIcdRptArrays - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getIcdSurgeryReports")
    public @WebResult(name="TaggedIcdRptArrays") TaggedIcdRptArrays getIcdSurgeryReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get imaging report.  The accession number field .01 (day-case#) contains the date and case number of the imaging exam associated with this report. 
     * The accessionNumber value is captured in Field .01 of File 74 (RAD/NUM MED REPORTS).  The system fills in this field with information obtained from 
     * the 'RAD/NUC MED PATIENT' file (#70) according to the case number selected by the transcriptionist.
     * 
     * @param queryBean the queryBean
     * 
     * @return RadiologyReportTO - the imaging report
     * 
     * @mdws ssn queryBean.patient.ssn
     * @mdws accessionNumber queryBean.criteria
     * 
     * @autorule queryBean.criteria (AccessionNumber) does not match 'MMddyy-<case#>' (e.g. 042070-123) format.
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.patient.ssn must be a valid Social Security Number
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getImagingReport")
    @WebResult(name="RadiologyReportTO")
    public RadiologyReportTO getImagingReport(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Is given note a surgery note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedText - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @mdws noteTitle queryBean.target
     * @mdws visitString queryBean.criteria
     * 
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isOneVisitNote")
    public @WebResult(name="TaggedText") TaggedText isOneVisitNote(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's current consults.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedConsultArrays - list of consults
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConsultsForPatient")
    public @WebResult(name="TaggedConsultArrays") TaggedConsultArrays getConsultsForPatient(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get note titles.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - list of note titles
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     * 
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned note titles.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of up to 44 matching note titles.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getNoteTitles")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getNoteTitles(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a surgery note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedText - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isSurgeryNote")
    public @WebResult(name="TaggedText") TaggedText isSurgeryNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a consult note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isConsultNote")
    public @WebResult(name="TextTO") TextTO isConsultNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is cosigner required?
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @mdws authorDUZ queryBean.criteria
     *
     * @autorule The optional queryBean.criteria value must be the valid, numeric DUZ of the provider in question.  When queryBean.criteria value is not defined, the logged in provider's user id will be used
     * @autorule The queryBean.itemId needs to be a numeric note definition ien (note definition id)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isCosignerRequired")
    public @WebResult(name="TextTO") TextTO isCosignerRequired(      
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get Inpatient for Outpatient medications from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedMedicationArrays - list of meds
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getImoMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getImoMeds(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get a list of lab tests for subsequent call to get test description
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedLabTestArrays - list of lab tests
     * 
     * @mdws target queryBean.target
     * 
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getLabTests")
    public @WebResult(name="TaggedLabTestArrays") TaggedLabTestArrays getLabTests(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get latest vitals from all connected VistAs
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedVitalSignArrays - list of vital signs
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getLatestVitalSigns")
    @WebResult(name="TaggedVitalSignArrays")
    public TaggedVitalSignArrays getLatestVitalSigns(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get medication detail from a single connected VistA.
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - the medication detail
     * 
     * @mdws medId queryBean.itemId
     * @mdws siteId queryBean.recordSiteCode
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getMedicationDetail")
    public  @WebResult(name="TextTO") TextTO getMedicationDetail(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get NHIN Data.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - text array - with the NHIN data response in embedded XML
     * 
     * the type parameter is optional - if left empty all NHIN Data for all domains supported will be returned - not advised for large datasets.
     * 
     * @mdws type queryBean.type
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getNhinData")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getNhinData(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get order dialog items.
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET 
     * 
     */
    @WebMethod(operationName = "getOrderDialogItems")
    public OrderDialogItemArray getOrderDialogItems(
        @WebParam(name = "dialogId") String dialogId,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get order dialogs for display group.
     * 
     * VIA Increment 6 -  NOT IMPLEMENTED YET 
     * 
     */
    @WebMethod(operationName = "getOrderDialogsForDisplayGroup")
    public TaggedTextArray getOrderDialogsForDisplayGroup(
       @WebParam(name = "displayGroupId") String displayGroupId,
       @WebParam(name = "queryBean") QueryBean queryBean
     );

    /**
     * Get order status for patient.
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET 
     * 
     */
    @WebMethod(operationName = "getOrderStatusForPatient")
    public TextTO getOrderStatusForPatient(
       @WebParam(name = "pid") String pid,
       @WebParam(name = "orderableItemId") String orderableItemId,
       @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get a list of patients by team
     *
     * @param queryBean the queryBean
     * 
     * @return TaggedPatientArray - list of patients
     * 
     *@mdws wardId queryBean.teamId
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientsByTeam")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByTeam(       
       @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get a list of patients by ward
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedPatientArray - list of patients
     * 
     * @mdws wardId queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientsByWard")
    public  @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByWard(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient list for reminder report.
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET
     * 
     */
    @WebMethod(operationName = "getPatientListForReminderReport")
    public ReminderReportPatientListTO getPatientListForReminderReport(
        @WebParam(name = "rptId") String rptId,
        @WebParam(name = "queryBean") QueryBean queryBean
     );

    /**
     * Get active reminder reports.
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET
     * 
     */
    @WebMethod(operationName = "getActiveReminderReports")
    public TaggedTextArray getActiveReminderReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's meds administation history.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the meds
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getMedsAdminHx")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getMedsAdminHx(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's meds administation log.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - the admit logs
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getMedsAdminLog")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getMedsAdminLog(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get transfer reports from all connected VistAs.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getTransfersReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getTransfersReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get uncosigned note metadata from all connected VistAs
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedNoteArrays - the uncosigned notes
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nNotes queryBean.maxRecords
     * 
     * @autorule The target 'queryBean.endDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule The target 'queryBean.startDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getUncosignedNotes")
    @WebResult(name="TaggedNoteArrays")
    public TaggedNoteArrays getUncosignedNotes(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Issue patient confidentiality bulletin to all connected sites.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray 
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "issueConfidentialityBulletin")
    @WebResult(name="TaggedTextArray")
    public TaggedTextArray issueConfidentialityBulletin(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a list of patients by provider
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray - the patients
     * 
     * @mdws duz queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientsByProvider")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByProvider(
        @WebParam(name = "queryBean") QueryBean queryBean);
    
    /**
     * Get a list of patients by specialty
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedPatientArray - the patients
     * 
     * @mdws specialityId queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientsBySpecialty")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsBySpecialty(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Validate electronic signature
     * 
     * @param queryBean the queryBean
     *  
     * @return TextTO - positive or negative response
     * 
     * @mdws esig queryBean.criteria
     *
     * @autorule queryBean.criteria must be a valid E-Signature.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isValidEsig")
    @WebResult(name="TextTO")
    public TextTO isValidEsig(
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get orderable items by name
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET
     * 
     */
    @WebMethod(operationName = "getOrderableItemsByName")
    public TaggedTextArray getOrderableItemsByName(
        @WebParam(name = "name") String name,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get sites for station
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET
     * 
     */
    @WebMethod(operationName = "getSitesForStation")
    public TaggedTextArray getSitesForStation(
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a list of patients by clinic
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedPatientArray - list of patients
     * 
     * @mdws clinicId queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPatientsByClinic")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByClinic(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get the list of specialties
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedText - list of patients
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getSpecialties")
    public @WebResult(name="TaggedText") TaggedText getSpecialties(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Retrieves the results of a Mental Health Instrument given an administrationId
     * 
     * @param queryBean the queryBean
     *  
     * @return MentalHealthInstrumentResultSetTO - result
     * 
     * @mdws administrationId queryBean.itemId
     * @mdws siteCode queryBean.recordSiteCode
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getMentalHealthInstrumentResultSet")
    public @WebResult(name="MentalHealthInstrumentResultSetTO") MentalHealthInstrumentResultSetTO getMentalHealthInstrumentResultSet(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Retrieves all Mental Health Instruments For a Patient given a Survey acronym
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedMentalHealthInstrumentAdministrationArrays - result
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getMentalHealthInstrumentsForPatient")
    public @WebResult(name="TaggedMentalHealthInstrumentAdministrationArrays") TaggedMentalHealthInstrumentAdministrationArrays getMentalHealthInstrumentsForPatient(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's microbiology lab results.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedMicrobiologyRptArrays - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.maxRecords
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getMicrobiologyReports")
    public @WebResult(name="TaggedMicrobiologyRptArrays") TaggedMicrobiologyRptArrays getMicrobiologyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get a note from a single connected VistA.
     * 
     * VIA Increment 6
     * 
     * @param queryBean the query bean.
     * 
     * @return TextTO - containing the note for the patient from the specified site(recordSiteCode) received in 
     * the query bean, if successful.
     * 
     * @mdws noteId queryBean.itemId
     * @mdws siteId queryBean.recordSiteCode
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getNote")
    public @WebResult(name="TextTO") TextTO getNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get reminder report templates
     * 
     * VIA Increment 6 - NOT IMPLEMENTED YET
     * 
     */
    @WebMethod(operationName = "getReminderReportTemplates")
    public TextArray getReminderReportTemplates(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get surgery reports from all connected VistAs
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedSurgeryReportArrays - the reports
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getSurgeryReportsWithText")
    public @WebResult(name="TaggedSurgeryReportArrays") TaggedSurgeryReportArrays getSurgeryReportsWithText(
        	@WebParam(name = "queryBean") QueryBean queryBean    
    );

    /**
     * Get text for a certain surgery report
     * 
     * @param queryBean the queryBean
     *  
     * @return TextTO - the report
     * 
     * @mdws rptId queryBean.itemId
     * @mdws siteId queryBean.recordSiteCode
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getSurgeryReportText")
    public @WebResult(name="TextTO") TextTO getSurgeryReportText(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /**
     * Get patient's surgical pathology lab results.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedSurgicalPathologyRptArrays - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getSurgicalPathologyReports")
    public @WebResult(name="TaggedSurgicalPathologyRptArrays") TaggedSurgicalPathologyRptArrays getSurgicalPathologyReports(
       	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get unsigned note metadata from all connected VistAs
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedNoteArrays - the notes
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nNotes queryBean.maxRecords
     * 
     * @autorule The target 'queryBean.endDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule The target 'queryBean.startDate' value must be null,-1 or have the format 'YYYYMMDD'
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getUnsignedNotes")
    @WebResult(name="TaggedNoteArrays")
    public TaggedNoteArrays getUnsignedNotes(
       	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get outpatient encounters from all connected VistAs.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.itemId
     *
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getOutpatientEncounterReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getOutpatientEncounterReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get treating specialty from all connected VistAs.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray - the reports
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getTreatingSpecialtyReports")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getTreatingSpecialtyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient record flag actions.
     * 
     * @param queryBean the queryBean
     *  
     * @return PatientRecordFlagArray - the record flags
     * 
     * @mdws noteDefintionIEN queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPrfNoteActions")
    @WebResult(name="PatientRecordFlagArray")
    public PatientRecordFlagArray getPrfNoteActions(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a list of hospital wards
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedHospitalLocationArray - the wards
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getWards")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getWards(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Close a note.
     * 
     * @param noteIEN - note identifier
     * @param consultIEN - consult identifier
     * @param queryBean - patient and provider context
     * 
     * @return TextTO - result of action
     * 
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "closeNote")
    public @WebResult(name="TextTO") TextTO closeNote(        
    	@WebParam(name = "noteIEN") String noteIEN,
    	@WebParam(name = "consultIEN") String consultIEN, 
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Call when launching from CPRS Tools menu: connects, authenticates, selects patient.
     * 
     * @param queryBean - patient and provider context
     * 
     * @return PersonsTO - patient and provider information
     * 
     * @mdws pwd queryBean.criteria
     * @mdws sitecode queryBean.provider.loginSiteCode
     * @mdws duz queryBean.provider.userId
     * @mdws dfn queryBean.patient.localPid
     * 
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     */
    @WebMethod(operationName = "cprsLaunch")
    public @WebResult(name="PersonsTO") PersonsTO cprsLaunch(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get all medications from all connected VistAs
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedMedicationArrays - the meds
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAllMeds")
    public  @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getAllMeds(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Write a simple, by policy order to a single VistA
     * 
     * VIA Increment 7 
     * 
     * @param providerDUZ 
     * @param esig
     * @param locationIEN
     * @param orderIEN
     * @param startDate - start date
     * @param queryBean the query bean.
     * 
     * @mdws providerDUZ queryBean.provider.userId
     * @mdws esig queryBean.target
     * @mdws locationIEN queryBean.criteria
     * @mdws orderIEN queryBean.itemId
     * @mdws startDate queryBean.fromDate
     * 
     * @return OrderTO - the order created if successful 
     *
     *
     * @autorule queryBean.criteria must be a Location IEN.
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.age needs to be numeric
     * @autorule queryBean.patient.gender must be M or F.
     * @autorule queryBean.patient.inPatient must 1 or 0. 1 for true and 0 for false
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.scPercentage needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.target must be a valid E-Signature.
     */
    @WebMethod(operationName = "writeSimpleOrderByPolicy")
    public @WebResult(name="OrderTO") OrderTO writeSimpleOrderByPolicy(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /*/**
     * Setup patient's remote sites for querying.
     * 
     * VIA Increment 7
     * 
     *//*
    public SiteArray setupMultiSiteQuery(
       	@WebParam(name = "appPwd") String appPwd
    );*/
    
    /**
     * Get patient's chem/hem lab results for the date range provided.
     * 
     * @param queryBean the query bean.
     * 
     * @return TaggedChemHemRptArrays - list of patient's chem/hem lab results for the date range provided if successful.
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.maxRecords
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.maxRecords value, if set, must be an integer greater than zero
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getChemHemReports")
    public @WebResult(name="TaggedChemHemRptArrays") TaggedChemHemRptArrays getChemHemReports(
        @WebParam(name = "queryBean") QueryBean queryBean  
    );
    
    /**
     * Get patient's chem/hem lab results. Use 0 for number of results to retrieve all results for the time period.
     * 
     * @param queryBean the query bean.
     * 
     * @return TaggedChemHemRptArrays 
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nrpts queryBean.maxRecords
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getChemHemReportsSimple")
    public @WebResult(name="TaggedChemHemRptArrays") TaggedChemHemRptArrays getChemHemReportsSimple(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get a list of hospital clinics
     * 
     * @param queryBean the query bean.
     * 
     * @return TaggedHospitalLocationArray - this clinics
     * 
     * @mdws target queryBean.target
     * 
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getClinics")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getClinics(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's cytology reports from VistA sources.
     * 
     @param queryBean the queryBean
     *    
     * @return TaggedCytologyRptArrays - list of all the patient's cytology reports from VistA sources if successful.
     *
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws nprts queryBean.itemId
     * 
     * @autorule maxRecords needs to be a valid integer value.
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     *     */
    @WebMethod(operationName = "getCytologyReports")
    public @WebResult(name="TaggedCytologyRptArrays") TaggedCytologyRptArrays getCytologyReports(
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /**
     * Gets a patient's demographics.
     * 
     * @param queryBean the query bean
     * 
     * @return PatientTO - the patient found in VistA with demographics data if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getDemographics")
    public  @WebResult(name="PatientTO") PatientTO getDemographics(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Match patients at logged in site.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArrays - a list of all the patients found that match the target received if successful.
     * 
     * @mdws target queryBean.target
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target target should be SSN or LAST,FIRST or LXXXX(first digit of last name followed by last four digits of SSN) pattern
     */
    @WebMethod(operationName = "match")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray match(
    	@WebParam(name = "queryBean") QueryBean queryBean        
    );
    
    /**
     * Lookup a patient from MPI.  The former MDWS ssn argument has been replaced by queryBean.criteria
     * 
     * @mdws ssn queryBean.criteria
     * 
     * @param queryBean the query bean
     * 
     * @return PatientArray -a patient array is returned containng a list of Patients found in MPI if successful.
     *
     *
     * @autorule queryBean.criteria must be a valid Social Security Number
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
 	@WebMethod(operationName = "mpiLookup")
    public @WebResult(name="PatientArray") PatientArray mpiLookup(
     	@WebParam(name = "queryBean") QueryBean queryBean
    );
 	
    
    /**
     * Lookup a patient from NPT.
     * 
     * VIA Increment 8
     * 
     * @param ssn
     * @param queryBean the query bean
     * 
     * @return PatientArray -a patient array is returned containing a list of Patients found in NPT if successful.
     */
 	@WebMethod(operationName = "nptLookup")
    public PatientArray nptLookup(
    	@WebParam(name = "ssn") String ssn,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
 	
 	@WebMethod(operationName="getVersion")
 	public String getVersion();
}