package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PatientAppointments;
import gov.va.med.via.databeans.PatientDemographics;
import gov.va.med.via.databeans.PatientDemographicsDetail;
import gov.va.med.via.databeans.ProgressNote;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.dto.common.NoteResultTO;
import gov.va.med.via.dto.common.PatientMedicalRecordTO;
import gov.va.med.via.dto.common.PatientRecordFlagArray;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.TaggedAllergyArrays;
import gov.va.med.via.dto.common.TaggedAppointmentArray;
import gov.va.med.via.dto.common.TaggedAppointmentArrays;
import gov.va.med.via.dto.common.TaggedConsultArrays;
import gov.va.med.via.dto.common.TaggedLabTestArrays;
import gov.va.med.via.dto.common.TaggedMedicationArrays;
import gov.va.med.via.dto.common.TaggedNoteArrays;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedPatientArrays;
import gov.va.med.via.dto.common.TaggedPatientAssociateArrays;
import gov.va.med.via.dto.common.TaggedProblemArrays;
import gov.va.med.via.dto.common.TaggedRadiologyReportArrays;
import gov.va.med.via.dto.common.TaggedText;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedVisitArray;
import gov.va.med.via.dto.common.TaggedVitalSignSetArrays;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

// TODO: Auto-generated Javadoc
/**
 * The Interface FixThePhonesServiceInterface.
 */
//@WebService
public interface FixThePhonesServiceInterface extends BaseServiceInterface
{
	
	// TODO - do we need a lookupPatient method?
	
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * VIA Increment 1
     * 
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
	/**
	 * Match
	 * 
	 * VIA Increment 1
	 * @param queryBean the query bean
	 * 
	 * @return TaggedPatientArrays patient arrays if successful.
     *
     * @autorule target needs to be of pattern '(\d{9})|([a-zA-Z]\S*\, *[a-zA-Z]\S*)|([a-zA-Z]\d{4})'
	 */
	@WebMethod(operationName = "match")
	public @WebResult(name="TaggedPatientArray") TaggedPatientArray match(
		@WebParam(name = "queryBean") QueryBean queryBean
	);
	
	 /**
     * Match by name, city, state at logged in site.
     * 
     * VIA Increment 1
     * 
     * @param name
     * @param city
     * @param stateAbbr
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArray patient array if successful.
     */
	@WebMethod(operationName = "matchByNameCityState")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray matchByNameCityState(
    	@WebParam(name = "name") String name,
    	@WebParam(name = "city") String city,
    	@WebParam(name = "stateAbbr") String stateAbbr,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
	/**
     * Select a patient at logged in site. DFN is the IEN of the patient
     * 
     *  VIA Increment 1
     *  
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "select")
    public @WebResult(name="PatientTO") PatientTO select(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
	/**
	 * Write note.
	 * 
	 * VIA Increment 2
	 *
	 * @param titleIEN the title ien
	 * @param encounterString the encounter string
	 * @param text the text
	 * @param authorDUZ the author duz
	 * @param cosignerDUZ the cosigner duz
	 * @param consultIEN the consult ien
	 * @param prfIEN the prf ien
	 * @param queryBean the query bean
	 * 
	 * @return NoteResultTO the note result to if successful.
     *
     *
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule When set, the prfIEN must be a valid IEN (a numeric or decimal value)
     * @autorule encounterString must be a valid encounter string
     * @autorule queryBean.patient.localPid must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule text cannot be a null value or an empty string
     * @autorule titleIEN must be a valid IEN (a numeric or decimal value)
	 */
	@WebMethod(operationName = "writeNote")
    public @WebResult(name="NoteResultTO") NoteResultTO writeNote(
    	@WebParam(name = "titleIEN") String titleIEN,
        @WebParam(name = "encounterString") String encounterString,
        @WebParam(name = "text") String text,
        @WebParam(name = "authorDUZ") String authorDUZ,
        @WebParam(name = "queryBean") String cosignerDUZ,
        @WebParam(name = "consultIEN") String consultIEN,
        @WebParam(name = "prfIEN") String prfIEN,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
	
	 /**
     * Close a note.
     * 
     * VIA Increment 7
     * 
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "closeNote")
    public @WebResult(name="TextTO") TextTO closeNote(        
    	@WebParam(name = "noteIEN") String noteIEN,
    	@WebParam(name = "consultIEN") String consultIEN, 
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
	/**
	 * Gets the appointments.
	 * 
	 * VIA increment 2
	 *
	 * @param queryBean the query bean
	 * 
	 * @return TaggedAppointmentArrays the appointments if successful.
	 */
	@WebMethod(operationName = "getAppointments")
	public @WebResult(name="TaggedAppointmentArrays") TaggedAppointmentArrays getAppointments(
		@WebParam(name = "queryBean") QueryBean queryBean
	); 

    /**
     * Get patient's appointments from single site.
     * 
     * VIA Increment 2
     * 
     * @param pwd
     * @param sitecode
     * @param mpiPid
     * @param queryBean the query bean
     * 
     * @return TaggedAppointmentArrays the appointments if successful.
     */
	@WebMethod(operationName = "getAppointmentsFromSite")
    public @WebResult(name="TaggedAppointmentArray") TaggedAppointmentArray getAppointmentsFromSite(
    	@WebParam(name = "pwd") String pwd,
        @WebParam(name = "sitecode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
	/**
	 * Get future appointments from site
	 * 
	 * VIA Increment 2
	 * 
	 * @param pwd
     * @param sitecode
     * @param mpiPid
	 * @param queryBean the query bean
     * 
     * @return TaggedAppointmentArrays the appointments if successful.
	 */
	@WebMethod(operationName = "getFutureAppointmentsFromSite")
    public @WebResult(name="TaggedAppointmentArray") TaggedAppointmentArray getFutureAppointmentsFromSite(
    	@WebParam(name = "pwd") String pwd,
    	@WebParam(name = "sitecode") String sitecode,
    	@WebParam(name = "mpiPid") String mpiPid,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
	/**
	 * Get Future Clinic Visits Reports
	 * 
	 * VIA Increment 2
	 *
	 * @param fromDate
     * @param toDate
     * @param nrpts
	 * @param queryBean the query bean
     * 
     * @return TaggedTextArray the text array if successful.
     *
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode must be present if queryBean.itemId has a value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
	 */
	@WebMethod(operationName = "getFutureClinicVisitsReports")
	public @WebResult(name="TaggedTextArray") TaggedTextArray getFutureClinicVisitsReports(
			@WebParam(name = "queryBean") QueryBean queryBean);
	
	/**
	 * Get Adhoc Health Summary By Display Name
	 * 
	 * VIA Increment 2
	 * @param queryBean the query bean
	 * 
	 * @return TaggedTextArray the text array if successful.
     *
     * @autorule displayName cannot be a null value or an empty string
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
	 */
	@WebMethod(operationName = "getAdhocHealthSummaryByDisplayName")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getAdhocHealthSummaryByDisplayName(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get Detail Health Summary
     * 
     * VIA Increment 2 
     * 
     * @param queryBean the query bean
     *  
	 * @return TaggedTextArray the text array if successful.
     */
	@WebMethod(operationName = "getDetailHealthSummary")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getDetailHealthSummary(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Health Summary
     * 
     * VIA Increment 2 
     * 	
     * @param queryBean the query bean
	 * 
	 * @return TaggedTextArray the text array if successful.
     */
	@WebMethod(operationName = "getHealthSummary")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getHealthSummary(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get notes with text from all connected VistAs.
     * 
     * VIA Increment 2
     * 
	 * @param fromDate
     * @param toDate
     * @param nNotes
	 * @param queryBean the query bean
     * 
     * @return TaggedNoteArrays the note arrays if successful.     
     */
	@WebMethod(operationName = "getNotesWithText")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getNotesWithText(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

	/** 
	 * Get Signed Notes
	 * 
	 * VIA Increment 2
	 * 
	 * @param fromDate
     * @param toDate
     * @param nNotes
	 * @param queryBean the query bean
     * 
     * @return TaggedNoteArrays the note arrays if successful.	 
	 */
	@WebMethod(operationName = "getSignedNotes")
	public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getSignedNotes(
			@WebParam(name = "fromDate") String fromDate, 
			@WebParam(name = "toDate") String toDate,
			@WebParam(name = "nNotes") int nNotes,
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
	/**
	 * Match By Name, City, State MS
	 * 
	 * VIA Increment 2
	 * 
	 * @param name
     * @param city
     * @param stateAbbr
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArray patient array if successful.
	 */
	@WebMethod(operationName = "matchByNameCityStateMS")
	public @WebResult(name="TaggedPatientArrays") TaggedPatientArrays matchByNameCityStateMS(
			@WebParam(name = "name") String name, 
			@WebParam(name = "city") String city,
			@WebParam(name = "stateAbbr") String stateAbbr,
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
	/**
     * Retrieve provider profile that is associated with the provider's user id.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters: {@link Provider}
     *
     * @return Provider with duty phone, gender, pager, permission class, flag, and sig filled in.
	 */
	@WebMethod(operationName = "getProviderProfile")
	public @WebResult(name="Provider") Provider getProviderProfile(
			@WebParam(name = "queryBean") QueryBean queryBean
	); 

    /**
     * Lookup User
     * 
     * VIA Increment 2
     * 
     * @param siteCode  site associated with access and verify credentials.
     * @param lookupstr 
     * @param userId 
     * @param maxReturn maximum number of providers found returned.
     * 
     * @return Provider[] a list of providers found associated to the User criteria received.       
     * 
     */
	@WebMethod(operationName = "lookupUser")
    public @WebResult(name="Provider") Provider[] lookupUser(
   		@WebParam(name = "siteCode") String siteCode, 
   		@WebParam(name = "lookupstr") String lookupstr, 
   		@WebParam(name = "userId") String userId, 
   		@WebParam(name = "maxReturn") int maxReturn
    );
	
	/**
	 * Get Appointment Text
	 * 
	 * VIA Increment 2
	 * 
     * @param siteId
     * @param apptId
     * @param queryBean the query bean
     * 
     * @return TextTO send the appointment text if successful.
     *
     * @autorule queryBean.itemId must be a valid visit string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
	 */
	@WebMethod(operationName = "getAppointmentText")
	public @WebResult(name="TextTO") TextTO getAppointmentText(
			@WebParam(name = "siteId") String siteId, 
			@WebParam(name = "apptId") String apptId,
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
	/**
	 * Get a report of the past clinic visits
	 *  
	 * VIA Increment 3
	 * 
	 * @param fromDate
     * @param toDate
     * @param nrpts
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedTextArray send text for all the poast clinic visists found if successful.
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
	 */
	@WebMethod(operationName = "getPastClinicVisitsReports")
	public @WebResult(name="TaggedTextArray") TaggedTextArray getPastClinicVisitsReports(
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
    /**
     * Get patient record flag actions.
     * 
     * VIA Increment 3
     * 
     * @param noteDefinitionIEN
	 * @param queryBean the query bean	 
	 * 
	 * @return PatientRecordFlagArray the array of patient record flags if successful.     
     */
	@WebMethod(operationName = "getPrfNoteActions")
    public @WebResult(name="PatientRecordFlagArray") PatientRecordFlagArray getPrfNoteActions(
    	@WebParam(name = "noteDefinitionIEN") String noteDefinitionIEN,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Create a new note and immediately signs the note using the provided eSig.
     * 
     * VIA Increment 3
     * 
     * @param queryBean requires itemId, provider and patient. The itemId field
     * will be used to store the electronic signature code.
     * 
     * @param progressNote requires noteTitleId and noteText. The noteTitleId
     * field refers to the IEN of the note title as defined in file 8925.1.
     * The noteText field is a free text field delimited by \n. The \n character
     * creates new lines. To display correctly in CPRS, no note line of text
     * should be greater than 80 characters in width.
     *  
     * @return NoteResultTO
     */
	@WebMethod(operationName = "createNewSignedNote")
    public @WebResult(name="NoteResultTO") NoteResultTO createNewSignedNote(
    	@WebParam(name = "queryBean") QueryBean queryBean,
    	@WebParam(name = "progressNote") ProgressNote progressNote);
    
    /**
     * Retrieves a patient progress note from the provided record location.
     *
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters: {@link Provider} {@link Patient}, recordSiteCode, itemId
     *
     * @return String the patient progress note if successful.
     */
	@WebMethod(operationName = "getProgressNote")
    public @WebResult(name="String") String getProgressNote(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient demographics from available patient locations.
     *
     * VIA Increment 3
     * 
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters: {@link Provider}, {@link Patient}

     * @return PatientDemographics - A list of patient demographics if successful. 
     */
	@WebMethod(operationName = "getPatientDemographics")
    public @WebResult(name="PatientDemographics") PatientDemographics getPatientDemographics(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get Patient Associates
     * 
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * @return TaggedPatientAssociateArrays - A list of patient associates if successful. 
     */
	@WebMethod(operationName = "getPatientAssociates")
    public @WebResult(name="TaggedPatientAssociateArrays") TaggedPatientAssociateArrays getPatientAssociates(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Patient Progress Notes
     * 
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * @return TaggedNoteArrays - a list of the patient progress notes, if successful.
     */
	@WebMethod(operationName = "getPatientProgressNotes")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getPatientProgressNotes(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Medical Record
     * 
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * @return 
     */
	@WebMethod(operationName = "getMedicalRecord")
    public @WebResult(name="PatientMedicalRecordTO") PatientMedicalRecordTO getMedicalRecord(
    	@WebParam(name = "queryBean") QueryBean queryBean    		
    );

    /**
     * Retrieves a code gray message for patient from a given location.
     * 
     * VIA Increment 3 
     *
     * @param queryBean Query parameters.
     * 
     * {@link QueryBean} required parameters: {@link Provider}, {@link Patient}, recordSiteCode.
     *
     * @return String - A patient code gray message.
     * 
     */
	@WebMethod(operationName = "getPatientCodeGreen")
    public @WebResult(name="String") String getPatientCodeGreen(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get allergies from all connected VistAs
     * 
     * VIA Increment 3
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAllergies")
    public  @WebResult(name="TaggedAllergyArrays")TaggedAllergyArrays getAllergies(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );  
    
    /**
     * Get Allergies As XML
     * 
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * @return TextTO - allergies as xml if successful.
     */
	@WebMethod(operationName = "getAllergiesAsXML")
    public @WebResult(name="TextTO") TextTO getAllergiesAsXML(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Pcp For Patient
     * 
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * @return TaggedText - the PCP for the patient, if successful.
     */
	@WebMethod(operationName = "getPcpForPatient")
    public @WebResult(name="TaggedText") TaggedText getPcpForPatient(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Patient Inquiry
     * 
     * VIA Increment 3
     *  
     * @param queryBean Query parameters.
     * 
     * @return 
     */
	@WebMethod(operationName = "patientInquiry")
    public void patientInquiry(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get selected patient's visits
     * 
     * VIA Increment 3
     * @param queryBean the query bean	 
     * 
	 * @return TaggedVisitArray - a list of the patient visits found if successful.
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
	@WebMethod(operationName = "getVisits")
    public @WebResult(name="TaggedVisitArray") TaggedVisitArray getVisits(
    	@WebParam(name = "queryBean")QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient appointments from all available patient locations.
     * 
     * VIA Increment 3 
     *
     * @param queryBean Query parameters
     *
     * {@link QueryBean} required parameters: {@link Provider}, {@link Patient}
     *
     * Optional: startDate (defaults to T-3 months), endDate   (defaults to T + 1 year).
     *
     * @return PatientAppointments[] - a list of patient appointments if successful.
     */
	@WebMethod(operationName = "getPatientAppointments")
    public @WebResult(name="PatientAppointments") PatientAppointments[] getPatientAppointments(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Sign Note.
     * 
     * VIA Increment 3
     * 
   	 * @param noteIEN
     * @param userDUZ
     * @param esig
     * @param queryBean Query parameters
     * 
     * @return TextTO - 
     *
     *
     * @autorule esig cannot be a null value or an empty string
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
	@WebMethod(operationName = "signNote")
	public @WebResult(name="TextTO") TextTO signNote(
			@WebParam(name = "noteIEN") String noteIEN,			
			@WebParam(name = "userDUZ") String userDUZ, 
			@WebParam(name = "esig") String esig,
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
    /**
     * Retrieves the patient demographics.
     * 
     * VIA Increment 3
     *  
	 * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters: {@link Provider}, {@link Patient} ?????
     *
     * @return PatientTO - the patient demographics.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
	@WebMethod(operationName = "getDemographics")
    public @WebResult(name="PatientTO") PatientTO getDemographics(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient demographics details from available patient locations.
     *
     * VIA Increment 3 
     * 
     * @param queryBean Query parameters.
     * 
     * {@link QueryBean} required parameters: {@link Provider}, {@link Patient}
     *
     * @return PatientDemographicsDetail[] - A list of patient demographics details if successful.
     */
	@WebMethod(operationName = "getPatientDemographicsDetail")
    public @WebResult(name="PatientDemographicsDetail") PatientDemographicsDetail[] getPatientDemographicsDetail(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );

	/** 
	 * Get Teams
	 * 
	 * VIA Increment 3
	 * 
	 * @param queryBean Query parameters.
	 * 
	 * @return TaggedText - a list of teams if successful.
	 */
	@WebMethod(operationName = "getTeams")
	public @WebResult(name="TaggedText") TaggedText getTeams(
			@WebParam(name = "queryBean") QueryBean queryBean
	);
	
	 /**
     * Get problem list from all connected VistAs
     * 
     * VIA Increment 4
     * 
     */
    @WebMethod(operationName = "getProblemList")
    public @WebResult(name="TaggedProblemArrays") TaggedProblemArrays getProblemList(
        @WebParam(name = "type") String type,
        @WebParam(name = "queryBean") QueryBean queryBean    
     );

    /**
     * Get patient's immunizations.
     * 
     * VIA Increment 5
     * 
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getImmunizations")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getImmunizations(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get discharge summaries from all connected VistAs.
     * 
     * VIA Increment 5
     * 
     */
    @WebMethod(operationName = "getDischargeSummaries")
    public @WebResult(name="TaggedNoteArrays") TaggedNoteArrays getDischargeSummaries(
        @WebParam(name = "queryBean") QueryBean queryBean
    );    

    /**
     * Get a list of lab tests for subsequent call to get test description
     * 
     * VIA Increment 6
     * 
     */
    @WebMethod(operationName = "getLabTests")
    public @WebResult(name="TaggedLabTestArrays") TaggedLabTestArrays getLabTests(
    	@WebParam(name = "target") String target,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get lab test description text
     * 
     * VIA Increment 5
     * 
     */
    @WebMethod(operationName = "getLabTestDescription")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getLabTestDescription(        
        @WebParam(name = "identifierString") String identifierString,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's current consults.
     * 
     * VIA Increment 5
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConsultsForPatient")
    public @WebResult(name="TaggedConsultArrays") TaggedConsultArrays getConsultsForPatient(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get all medications from all connected VistAs
     * 
     * VIA Increment 7
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAllMeds")
    public @WebResult(name="TaggedMedicationArrays") TaggedMedicationArrays getAllMeds(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get radiology reports from all connected VistAs
     *  
     * VIA Increment 7
     * 
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode must be present if queryBean.itemId has a value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getRadiologyReports")
    public  @WebResult(name="TaggedRadiologyReportArrays") TaggedRadiologyReportArrays getRadiologyReports(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's vital signs.
     * 
     * VIA Increment 4
     * 
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getVitalSigns")
    public @WebResult(name="TaggedVitalSignSetArrays") TaggedVitalSignSetArrays getVitalSigns(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's advance directives.
     * 
     * VIA Increment 5
     * 
     *
     * @autorule queryBean.endDate must be present if queryBean.startDate has a value
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode must be present if queryBean.itemId has a value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate must be present if queryBean.endDate has a value
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getAdvanceDirectives")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getAdvanceDirectives(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
}