package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.AllergyTO;
import gov.va.med.via.dto.common.LabResultTO;
import gov.va.med.via.dto.common.MedicationTO;
import gov.va.med.via.dto.common.SiteArray;
import gov.va.med.via.dto.common.TaggedAppointmentArray;
import gov.va.med.via.dto.common.TaggedAppointmentArrays;
import gov.va.med.via.dto.common.TaggedChemHemRptArray;
import gov.va.med.via.dto.common.TaggedChemHemRptArrays;
import gov.va.med.via.dto.common.TaggedMedicationArrays;
import gov.va.med.via.dto.common.TaggedPatientArrays;
import gov.va.med.via.dto.common.TaggedText;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

/**
 * The Interface MhvServiceInterface.
 */
//@WebService
public interface MhvServiceInterface extends BaseServiceInterface
{
	// Do we need to add lookupPatient() to return a Patient bean?
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * VIA Increment 1
     * 
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
    /**
     * Get patient's appointments from all sites.
     * 
     * VIA Increment 2
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedAppointmentArrays - list of patient's appointments from all sites if successful.
     */
    @WebMethod(operationName = "getAppointments")
    public TaggedAppointmentArrays getAppointments(
    	@WebParam(name = "pwd") String pwd,
    	@WebParam(name = "siteCode") String sitecode,
    	@WebParam(name = "mpiPid") String mpiPid,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    
    /**
     * Get patient's appointments from single site.
     * 
     * VIA Increment 2
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedAppointmentArrays - list of patient's appointments from single site if successful.
     */
    @WebMethod(operationName = "getAppointmentsFromSite")
    public TaggedAppointmentArray getAppointmentsFromSite(
    	@WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's future appointments from all sites.
     * 
     * VIA Increment 2
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedAppointmentArrays - list of patient's future appointments from all sites if successful.
     */
    @WebMethod(operationName = "getFutureAppointments")
    public TaggedAppointmentArrays getFutureAppointments(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's future appointments from single site.
     * 
     * VIA Increment 2
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedAppointmentArrays - list of patient's future appointments from a single site if successful.
     */
    @WebMethod(operationName = "getFutureAppointmentsFromSite")
    public TaggedAppointmentArray getFutureAppointmentsFromSite(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    

    /**
     * Get patient's allergies as an XML payload.
     * 
     * VIA Increment 3
     * 
     * @param pwd password
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TextTO - list of patient's allergies in XML format if successful.
     */
    @WebMethod(operationName = "getAllergiesAsXML")
    public TextTO getAllergiesAsXML(
       @WebParam(name = "pwd") String pwd,
       @WebParam(name = "mpiPid") String mpiPid,
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's summary reminders from single VistA.
     * 
     * VIA Increment 7
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TextTO - patient's summary reminders from single VistA if successful.
     */
    @WebMethod(operationName = "getSummaryRemindersFromSite")
    public TextTO getSummaryRemindersFromSite(
       @WebParam(name = "pwd") String pwd,
       @WebParam(name = "siteCode") String sitecode,
       @WebParam(name = "mpiPid") String mpiPid,
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's detailed reminders from single VistA.
     * 
     * VIA Increment 7
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TextTO - patient's detail reminders from single VistA if successful.
     */
    @WebMethod(operationName = "getDetailedRemindersFromSite")
    public TextTO getDetailedRemindersFromSite(
    	@WebParam(name = "pwd") String pwd,
    	@WebParam(name = "siteCode") String sitecode,
    	@WebParam(name = "mpiPid") String mpiPid,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /**
     * Get the patient's allergies
     * 
     * VIA Increment 7
     * 
     * @param pwd password
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return AllergyTO - list of patient's allergies if successful.
     */
    @WebMethod(operationName = "getAllergies")
    public AllergyTO[] getAllergies(
    	@WebParam(name = "pwd") String pwd,
    	@WebParam(name = "mpiPid") String mpiPid,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get list of patients with updated chem hem reports.
     * 
     * VIA Increment 7
     * 
     * @param username - user name
     * @param pwd - password 
     * @param fromDate - start date
     * @param queryBean the query bean.
     * 
     * @return TaggedPatientArrays - list of patients with updated chem hem reports if successful.
     */
    @WebMethod(operationName = "getPatientsWithUpdatedChemHemReports")
    public TaggedPatientArrays getPatientsWithUpdatedChemHemReports(
       @WebParam(name = "username") String username,
       @WebParam(name = "pwd") String pwd,
       @WebParam(name = "fromDate") String fromDate,
       @WebParam(name = "queryBean") QueryBean queryBean
    );


    /**
     * Get list of patients with future appointments that have been edited since a timestamp.
     * 
     * VIA Increment 7
     * 
     * @param username - user name
     * @param pwd - password 
     * @param fromDate - start date
     * @param queryBean the query bean.
     * 
     * @return TaggedPatientArrays - list of patients future appointments that have been edited since a timestamp if successful.
     */
    @WebMethod(operationName = "getPatientsWithUpdatedFutureAppointments")
    public TaggedPatientArrays getPatientsWithUpdatedFutureAppointments(
        @WebParam(name = "username") String username,
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "fromDate") String fromDate,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    

    /**
     * Get patient's prescriptions via MHV HL7 routine.
     * 
     * VIA Increment 7
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedMedicationArrays - list of patient's prescriptions via MHV HL7 routine if successful.
     */
    @WebMethod(operationName = "getPrescriptionsHL7")
    public TaggedMedicationArrays getPrescriptionsHL7(
    	@WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    

    /**
     * Get patient MOS report.
     * 
     * VIA Increment 7
     * 
	 * @param appPwd - application password?
     * @param ediPi
     * @param queryBean the query bean.
     * 
     * @return TaggedText - patient's MOS report if successful.
     */
    @WebMethod(operationName = "getMOSReport")
    public TaggedText getMOSReport(
       @WebParam(name = "appPwd") String appPwd,
       @WebParam(name = "ediPi") String ediPi,
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Refill patient's prescription
     * 
     * VIA Increment 7
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param rxId - RX Medication Id
     * @param queryBean the query bean.
     * 
     * @return MedicationTO - patient's medicine that will be refilled if successful.
     */
    @WebMethod(operationName = "refillPrescription")
    public MedicationTO refillPrescription(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "rxId") String rxId,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Set the current session's sites file
     * 
     * VIA Increment 7
     * 
     * @param sitesFileName - sites file name
     * @param queryBean the query bean.
     * 
     * @return SiteArray - list of sites if successful.
     */
    @WebMethod(operationName = "setVha")
    public SiteArray setVha(
    	@WebParam(name = "sitesFileName") String sitesFileName,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    
    /**
     * Get patient's chem/hem lab results for the date range provided.
     * 
     * VIA Increment 8
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param fromDate - start date
     * @param toDate - end date
     * @param queryBean the query bean.
     * 
     * @return TaggedChemHemRptArrays - list of patient's chem/hem lab results for the date range provided if successful.
     */
    @WebMethod(operationName = "getChemHemReports")
    public TaggedChemHemRptArrays getChemHemReports(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "fromDate") String fromDate,
        @WebParam(name = "toDate") String toDate,
        @WebParam(name = "queryBean") QueryBean queryBean
    );


    /**
     * Get patient's chem/hem lab results from a site
     * 
     * VIA Increment 8
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param queryBean the query bean.
     * 
     * @return TaggedChemHemRptArrays - list of patient's chem/hem lab results from a site if successful.
     */
    @WebMethod(operationName = "getChemHemReportsForPatientFromSite")
    public TaggedChemHemRptArray getChemHemReportsForPatientFromSite(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "queryBean") QueryBean queryBean
    );


    /**
     * Get patient's chem/hem lab results from a site for the date range provided.
     * 
     * VIA Increment 8
     * 
     * @param pwd password
     * @param siteCode  site associated with access and verify credentials.
     * @param mpiPid
     * @param fromDate - start date
     * @param toDate - end date
     * @param queryBean the query bean.
     * 
     * @return TaggedChemHemRptArrays - list of patient's chem/hem lab results from a site for the date range provided if successful.
     */
    @WebMethod(operationName = "getChemHemReportsFromSite")
    public TaggedChemHemRptArray getChemHemReportsFromSite(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteCode") String sitecode,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "fromDate") String fromDate,
        @WebParam(name = "toDate") String toDate,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get patient's lab reports.
     * 
     * VIA Increment 8
     * 
     * @param pwd password
     * @param mpiPid
     * @param fromDate - start date
     * @param toDate - end date
     * @param queryBean the query bean.
     * 
     * @return LabResultTO - list of patient's lab reports if successful.
     */
    @WebMethod(operationName = "getLabReports")
    public LabResultTO getLabReports(
        @WebParam(name = "pwd") String pwd,
        @WebParam(name = "mpiPid") String mpiPid,
        @WebParam(name = "fromDate") String fromDate,
        @WebParam(name = "toDate") String toDate,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    

    /**
     * Get patient's lab reports as an XML payload for the date range provided.
     * 
     * VIA Increment 9
     * 
     * @param pwd password
     * @param mpiPid
     * @param fromDate - start date
     * @param toDate - end date
     * @param queryBean the query bean.
     * 
     * @return TextTO - list of patient's lab reports in XML format for the date range provided if successful.
     */
    public TextTO getLabReportsAsXML(
       @WebParam(name = "pwd") String pwd,
       @WebParam(name = "mpiPid") String mpiPid,
       @WebParam(name = "fromDate") String fromDate,
       @WebParam(name = "toDate") String toDate,
       @WebParam(name = "queryBean") QueryBean queryBean
    );
    
}