package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.databeans.AdmissionDetail;
import gov.va.med.via.databeans.Alert;
import gov.va.med.via.databeans.Allergy;
import gov.va.med.via.databeans.ClinicalReminder;
import gov.va.med.via.databeans.Consult;
import gov.va.med.via.databeans.ConsultInfoBean;
import gov.va.med.via.databeans.Diagnosis;
import gov.va.med.via.databeans.FeeBasisPatDataBean;
import gov.va.med.via.databeans.FeeBasisBean;
import gov.va.med.via.databeans.FeedBack;
import gov.va.med.via.databeans.Immunization;
import gov.va.med.via.databeans.LabOrder;
import gov.va.med.via.databeans.LabResult;
import gov.va.med.via.databeans.Medication;
import gov.va.med.via.databeans.NHINLabOrder;
import gov.va.med.via.databeans.NHINMedication;
import gov.va.med.via.databeans.NHINProblem;
import gov.va.med.via.databeans.Order;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.PatientAdmission;
import gov.va.med.via.databeans.PatientAppointments;
import gov.va.med.via.databeans.PatientDemographics;
import gov.va.med.via.databeans.PatientDemographicsDetail;
import gov.va.med.via.databeans.Prescription;
import gov.va.med.via.databeans.Problem;
import gov.va.med.via.databeans.ProblemDetail;
import gov.va.med.via.databeans.Procedure;
import gov.va.med.via.databeans.ProgressNote;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.ProviderAppointmentBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.RadiologyReport;
import gov.va.med.via.databeans.ReferralBean;
import gov.va.med.via.databeans.Vitals;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

/**
 * The Interface VistaDataServiceInterface.
 */
//@WebService
public interface VistaDataServiceInterface extends BaseServiceInterface
{
    /**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     * 
     * VIA Increment 1 
     * @throws ValidationException 
     * 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     */
	@WebMethod(operationName = "login")
	public Provider login(
	    String siteCode, 
	    String accessCode, 
	    String verifyCode, 
	    String requestingApp) throws ValidationException;
    
    /**
     * Lookup Patient
     * 
     * VIA Increment 1 
     * 
     */
	@WebMethod(operationName = "lookupPatient")
    public Patient lookupPatient(
    		@WebParam(name = "queryBean") QueryBean queryBean);

	/**
	 * Get Provider Profile
	 * 
	 * VIA Increment 2
	 * 
	 */
    @WebMethod(operationName = "getProviderProfile")
	public Provider getProviderProfile(
			@WebParam(name = "queryBean") QueryBean queryBean); 
	
    /**
     * Lookup User
     * 
     * VIA Increment 2 
     * 
     */
	@WebMethod(operationName = "lookupUser")
    public Provider[] lookupUser(
    		@WebParam(name = "siteCode") String siteCode, 
    		@WebParam(name = "lookupstr") String lookupstr,
    		@WebParam(name = "userId") String userId, 
    		@WebParam(name = "maxReturn") int maxReturn);
	
    /**
     * Create New Signed Note
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "createNewSignedNote")
    public void createNewSignedNote(
    		@WebParam(name = "queryBean") QueryBean queryBean,
    		@WebParam(name = "progressNote") ProgressNote progressNote);
    
    /**
     * Get Progress Note
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "getProgressNote")
    public String getProgressNote(@WebParam(name = "queryBean")
    		QueryBean queryBean);
    
    /**
     * Retrieves a list of patient demographics from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "getPatientDemographics")
    public PatientDemographics[] getPatientDemographics(@WebParam(name = "queryBean")
    		QueryBean queryBean);
    
    /**
     * Get Patient Code Green
     * 
     * VIA Increment 3 
     * 
     */	
    @WebMethod(operationName = "getPatientCodeGreen")
    public String getPatientCodeGreen(@WebParam(name = "queryBean")
    		QueryBean queryBean);
    

    /**
     * Get Patient Progress Notes
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "getPatientProgressNotes")
    public ProgressNote[] getPatientProgressNotes(@WebParam(name = "queryBean")
    		QueryBean queryBean);
    
    /**
     * Retrieves a list of patient appointments from all available patient locations.
     *
     * @param queryBean Query parameters
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "getPatientAppointments")
    public PatientAppointments[] getPatientAppointments(@WebParam(name = "queryBean")
    		QueryBean queryBean);

    /**
     * Retrieves a list of patient demographics details from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 3 
     * 
     */
    @WebMethod(operationName = "getPatientDemographicsDetail")
    public PatientDemographicsDetail[] getPatientDemographicsDetail(@WebParam(name = "queryBean")
    		QueryBean queryBean);

    /**
     * Retrieves a consult report.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 5
     *
     */
    @WebMethod(operationName = "getConsultReport")
    public String[] getConsultReport(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );

    /**
     * Retrieves a clinical reminder detail.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 5
     *
     */
    @WebMethod(operationName = "getClinicalReminderDetail")
    public String getClinicalReminderDetail(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient medications from available patient locations.
     * 
     * VIA Increment 6
     * 
     * */
    public NHINMedication[] getNHINPatientMedications(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient problems from Nhin.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     *
     */
    public NHINProblem[] getNhinPatientProblemList(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );


    /**
     * Retrieves the patient clinical reminders.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     *
     */
    @WebMethod(operationName = "getPatientClinicalReminders")
    public ClinicalReminder[] getPatientClinicalReminders(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves the patient's immunizations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     *
     */
    @WebMethod(operationName = "getPatientImmunizations")
    public Immunization[] getPatientImmunizations(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a list of the NHIN labs.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     *
     */
    @WebMethod(operationName = "getNHINLabs")
    public NHINLabOrder[] getNHINLabs(@WebParam(name = "queryBean")
            QueryBean queryBean
     );
    
    /**
     * Retrieves a list of patient consult requests.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     *
     */
    @WebMethod(operationName = "getPatientConsultRequests")
    public Consult[] getPatientConsultRequests(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient's current vitals from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 6
     * 
     */
    @WebMethod(operationName = "getPatientCurrentVitals")
    public Vitals[] getPatientCurrentVitals(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient admissions.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientAdmissions")
    public PatientAdmission[] getPatientAdmissions(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient allergies.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientAllergies")
    public Allergy[] getPatientAllergies(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
   
    /**
     * Retrieves a list of patient radiologies.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */       
    @WebMethod(operationName = "getPatientRads")
    public RadiologyReport[] getPatientRads(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a radiology report.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getRadiologyReport")
    public RadiologyReport getRadiologyReport(@WebParam(name = "queryBean")
    		QueryBean queryBean
    ); 
    
    /**
     * Retrieves a list of patient medications from available patient locations.
     *
     * VIA Increment 7
     *  
     * Optional:
     * status (possible options: O - outpatient, I - inpatient, defaults to "O"
     * active (defaults to 1)
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return List of patient medications.
     */
    @WebMethod(operationName = "getPatientMedications")
    public Medication[] getPatientMedications(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient problems from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientProblemList")
    public Problem[] getPatientProblemList(@WebParam(name = "queryBean")
    		QueryBean queryBean
    );
    
    /**
     * Retrieves a list of non-VA patient medications from available patient locations.
     * 
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getPatientMedicationsNVA")
    public Medication[] getPatientMedicationsNVA(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient discharge summaries from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientDischargeSummaries")
    public ProgressNote[] getPatientDischargeSummaries(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );

    /**
     * Retrieves a list of patient vitals from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientVitals")
    public Vitals[] getPatientVitals(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves patient admission record details from the provided record location.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getAdmissionDetails")
    public AdmissionDetail getAdmissionDetails(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of admission diagnosis from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     */
    @WebMethod(operationName = "getAdmissionDiagnosis")
    public Diagnosis[] getAdmissionDiagnosis(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of admission procedures from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getAdmissionProcedures")
    public Procedure[] getAdmissionProcedures(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves the information for an authorization consult.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getAuthConsultInfo")
    public ConsultInfoBean getAuthConsultInfo(@WebParam(name = "queryBean") 
            QueryBean queryBean
    );
    
    /**
     * Retrieves the authorization data by date.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getAuthDataByDate")
    public FeeBasisBean[] getAuthDataByDate(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves the authorization data by patient.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getAuthDataByPat")
    public FeeBasisBean[] getAuthDataByPat(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    
    /**
     * Retrieves patient lab orders from all available patient locations.
     *
     * @param queryBean Query parameters
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getPatientLabs")
    public LabOrder[] getPatientLabs(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient lab test results from the provided record location.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientLabTestResults")
    public LabResult[] getPatientLabTestResults(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient orders from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientOrders")
    public Order[] getPatientOrders(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of patient procedures from available patient locations.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getPatientProcedures")
    public Procedure[] getPatientProcedures(@WebParam(name = "queryBean")
        QueryBean queryBean
    );
    
    /**
     * Retrieves patient prescription fills from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getPrescriptionFills")
    public Prescription getPrescriptionFills(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a patient problem list detail from the provided record location.
     *
     * @param queryBean Query parameters.
     * 
     * VIA Increment 7
     *
     */
    @WebMethod(operationName = "getProblemDetail")
    public ProblemDetail getProblemDetail(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Gets a list of patients who have been admitted or are being attended
     * by the provider passed in the queryBean.
     * 
     * @param queryBean requires a provider
     * 
     * VIA Increment 7
     */
    @WebMethod(operationName = "getProviderAdmissions")
    public PatientAdmission[] getProviderAdmissions(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a patient order from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getOrderDetail")
    public String getOrderDetail(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );

    /**
     * Retrieves a pat? authorization
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "getPatAuthorization")
    public FeeBasisPatDataBean getPatAuthorization(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Sends feedback to location.
     * 
     * @param siteCode Site to send feedback.
     * @param feedback Feedback bean.
     *
     * VIA Increment 7
     * 
     */
    @WebMethod(operationName = "sendFeedBack")
    public boolean sendFeedBack(@WebParam(name = "siteCode") 
    		String siteCode, @WebParam(name = "feedback") 
    		FeedBack feedback
    );
    
    
    /**
     * Retrieves a list of DR Referrals
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getDRReferrals")
    public ReferralBean[] getDRReferrals(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves a list of feedback items from VistA cache location.
     *
     * @param queryBean Query parameters
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getFeedBackItems")
    public FeedBack[] getFeedBackItems(@WebParam(name = "queryBean")
            QueryBean queryBean
    );

    /**
     * Gets a list of past and/or future appointments which are assigned to the
     * provider passed in the queryBean.
     * 
     * @param queryBean requires a provider, startDate and endDate. Start and
     * end dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderAppointments")
    public PatientAppointments[] getProviderAppointments(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Gets a list of all consults which a provider must complete. Ie,
     * provider X orders an echocardiogram; provider Y receives the consult
     * to be completed. This method enables a provider (in this example Y) to
     * see what consults need to be completed on a longitudinal patient domain
     * rather than a patient by patient basis.
     * 
     * @param queryBean requires provider, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and
     * stop searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderConsultsReceived")
    public Consult[] getProviderConsultsReceived(@WebParam(name = "queryBean") 
         QueryBean queryBean
    );
    
    /**
     * Gets a list of all consults which have been requested by the provider
     * passed in the queryBean. All statuses will be returned.
     * 
     * @param queryBean requires a provider, startDate and endDate. Start and
     * end dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderConsultsRequested")
    public Consult[] getProviderConsultsRequested(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Gets a list of abnormal lab results which were ordered by the provider
     * passed in the queryBean.
     * 
     * @param queryBean requires provider, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderLabAbnormalResults")
    public LabResult[] getProviderLabAbnormalResults(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Gets the details for the provider's order.
     * 
     * @param queryBean Query parameters.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderOrderDetails")
    public Order getProviderOrderDetails(
            @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets a list of orders pending which have been placed by the provider
     * passed in the queryBean.
     * 
     * @param queryBean requires provider, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderOrdersPending")
    public Order[] getProviderOrdersPending(@WebParam(name = "queryBean") 
            QueryBean queryBean
    );
    
    /**
     * Gets a list of orders which have results which have been placed by the
     * provider passed in the queryBean.
     * 
     * @param queryBean requires provider, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderOrdersResulted")
    public Order[] getProviderOrdersResulted(@WebParam(name = "queryBean") 
    		QueryBean queryBean
    );
    
    /**
     * Gets a list of all unsigned notes which belong to the provider passed
     * in the queryBean.
     * 
     * @param queryBean requires a provider
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderUnsignedNotes")
    public ProgressNote[] getProviderUnsignedNotes(
            @WebParam(name = "queryBean") QueryBean queryBean
    );
 
    /**
     * Gets the workload for the provider.
     * 
     * @param queryBean requires a provider
     * 
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getProviderWorkload")
    public ProviderAppointmentBean[] getProviderWorkload(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves a list of provider alerts associated with the provider's user id.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 8
     * 
     */
    @WebMethod(operationName = "getUserAlerts")
    public Alert[] getUserAlerts(@WebParam(name = "queryBean") 
    	QueryBean queryBean
    );
    
    /**
     * Retrieves a list of lab order results from all available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * VIA Increment 8
     *
     */
    @WebMethod(operationName = "getLabOrderResult")
    public LabResult[] getLabOrderResult(@WebParam(name = "queryBean")
    	QueryBean queryBean
    );
    
    
    /**
     * Retrieves inpatient medication record report from the provided VistA location.
     *
     * VIA Increment 7
     * 
     * @param queryBean Query parameters.
     * 
     * {@link QueryBean} required parameters: {@link gov.va.med.vds.webservice.Provider},
     * {@link gov.va.med.vds.webservice.Patient}, recordSiteCode and itemId
     *
     * @return String containing the inpatient medication record report. 
     */
    @WebMethod(operationName = "getInptMedicationDetail")
    public String getInptMedicationDetail(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Retrieves a list of visit diagnosis.
     * 
     * @param queryBean Query parameters
     * 
     * VIA Increment 9
     * 
     */
    @WebMethod(operationName = "getVisitDiagnosis")
    public Diagnosis[] getVisitDiagnosis(@WebParam(name = "queryBean")
            QueryBean queryBean
    );

    /**
     * Retrieves a list of visit notes.
     * 
     * @param queryBean Query parameters
     * 
     * VIA Increment 9
     * 
     */
    @WebMethod(operationName = "getVisitNotes")
    public String getVisitNotes(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Retrieves a list of visit procedures.
     * 
     * @param queryBean Query parameters
     * 
     * VIA Increment 9
     * 
     */
    @WebMethod(operationName = "getVisitProcedures")
    public Procedure[] getVisitProcedures(@WebParam(name = "queryBean")
            QueryBean queryBean
    );
    
    /**
     * Gets a list of patients who are admitted in the specified ward(s). This
     * method <b>only</b> queries the local VistA system.
     * 
     * @param queryBean requires itemId. itemId is a ^ delimited ward ien
     * string, ie "24^47" for two wards or just "24" for a single ward.
     * 
     * VIA Increment 9
     * 
     */
    @WebMethod(operationName = "getWardAdmissions")
    public PatientAdmission[] getWardAdmissions(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

}