

package gov.va.med.via.log4j;


import org.apache.log4j.spi.LoggingEvent;
import org.apache.log4j.spi.ThrowableInformation;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.Scanner;


/**
 *
 */
public class ApplicationLogEvent
{
    private Long id;

    private String category;

    private String requestingApp;

    private String message;

    private Date localTime;

    private String severity;

    private static final String CATEGORY = "<<category>>";
    private static final String REQUESTING_APP = "<<requesting app>>";
    private static final String MESSAGE_DELIMITER = "#";


    public ApplicationLogEvent( )
    {
    }


    /**
     * Use Log4J log event to populate the VIA application log event.
     * @param logEvent 
     */
    public ApplicationLogEvent( LoggingEvent logEvent )
    {
        String message = ( String )logEvent.getMessage();
        if ( null != message && 0 != message.length() )
        {
            extractData( message );

            //Append throwable information if exists
            ThrowableInformation throwableInfo = logEvent.getThrowableInformation();
            StringBuilder tempMessage = new StringBuilder();

            if ( throwableInfo != null && throwableInfo.getThrowableStrRep().length > 0 )
            {
                String[] stringReps = throwableInfo.getThrowableStrRep();
                tempMessage.append( getMessage() );
                tempMessage.append( "Appending Throwable information - " );
                for ( String rep : stringReps )
                {
                    tempMessage.append( rep );
                }
                setMessage( tempMessage.toString() );
            }

            //setMessage(myString);
            setLocalTime( new Date() );
            setSeverity( logEvent.getLevel().toString() );
        }
    }


    private void extractData( String message )
    {
        if ( !message.contains( MESSAGE_DELIMITER ) )
        {
            this.setCategory( CATEGORY );
            this.setLocalTime(new Date());
            this.setRequestingApp( REQUESTING_APP );
            this.setMessage( message );

        }
        else
        {
            Scanner scanner = new Scanner( message ).useDelimiter( MESSAGE_DELIMITER );
            String value = null;
            
            // 1st position - Category
            if ( scanner.hasNext() )
            {
                value = scanner.next();
                if ( !StringUtils.hasText( value ) )
                {
                    this.setCategory( CATEGORY );
                }
                else
                {
                    this.setCategory( value );
                }
            }

            // 2nd position - RequestingApp
            if ( scanner.hasNext() )
            {
                value = scanner.next();
                if ( !StringUtils.hasText( value ) )
                {
                    this.setRequestingApp( REQUESTING_APP );
                }
                else
                {
                    this.setRequestingApp( value );
                }

            }

            // 3rd position - Exception/Error Message
            if ( scanner.hasNext() )
            {
                this.setMessage( scanner.next() );
            }
        }
    }


    public String getCategory( )
    {
        return category;
    }


    public void setCategory( String category )
    {
        this.category = category;
    }


    public String getRequestingApp( )
    {
        return requestingApp;
    }


    public void setRequestingApp( String requestingApp )
    {
        this.requestingApp = requestingApp;
    }


    public String getMessage( )
    {
        return message;
    }


    public void setMessage( String message )
    {
        this.message = message;
    }


    public Date getLocalTime( )
    {
        return localTime;
    }


    public void setLocalTime( Date localTime )
    {
        this.localTime = localTime;
    }


    public String getSeverity( )
    {
        return severity;
    }


    public void setSeverity( String severity )
    {
        this.severity = severity;
    }

    public Long getId( )
    {
        return id;
    }
}
