

package gov.va.med.via.loggerAudit;


import gov.va.med.via.audit.AuditRequest;
import gov.va.med.via.audit.persistence.AuditLogPersistenceManager;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author ZZZZZZTALBOM
 * 
 * AuditRequestMessageDrivenBean receives JMS Messages on AuditQueue, and stores those messages
 * in the database. The asynchronous handling of Requests audit information is expected to improve overall performance of the system.
 * 
 */
@MessageDriven(mappedName= "jms/via/AuditQueue", activationConfig =
{ @ActivationConfigProperty(propertyName="destinationType", propertyValue="javax.jms.Queue"),
		@ActivationConfigProperty(propertyName="destination", propertyValue="jms/via/AuditQueue"),
			@ActivationConfigProperty(propertyName="connectionFactoryJndiName", propertyValue="jms/via/LoggerAuditConnectionFactory"),
				@ActivationConfigProperty(propertyName="subscriptionDurability", propertyValue="Durable")
})
public class AuditRequestMessageDrivenBean implements MessageListener
{
    
    private AuditLogPersistenceManager hibernateAuditLogPersistenceManager;
    protected final Log logger = LogFactory.getLog(getClass());
    @Resource
	private MessageDrivenContext messageDrivenContext;
    
    public AuditRequestMessageDrivenBean() {
	}


    @PostConstruct
    public void setAppContext( )
    {
    	BeanFactory beanFactory = new ClassPathXmlApplicationContext("classpath:gov/va/med/via/loggerAudit/config/weblogicLoggerContext.xml");
        this.hibernateAuditLogPersistenceManager = ( AuditLogPersistenceManager ) beanFactory.getBean( "hibernateAuditLogPersistenceManager" );
    }
 

    /***
     * onMessage receives JMS Messages from AuditQueue, and writes audit information to database 
     * @param requestAuditInformation is the audit information created from a request
     */
    @TransactionAttribute(value=TransactionAttributeType.REQUIRED)
    public void onMessage( Message requestAuditInformation )
    {
        String requestingApp="Uncaptured-App-Name";
        String category="Uncaptured-Category";
    	try
        {
            ObjectMessage objectMessage = ( ( ObjectMessage )requestAuditInformation );
            AuditRequest auditRequest = (AuditRequest)objectMessage.getObject();
            requestingApp = auditRequest.getRequestingApp();
            category = auditRequest.getCategory();
            hibernateAuditLogPersistenceManager.storeAuditLog( auditRequest );
        }
        catch ( Exception e )
        {
            logger.error( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( category, requestingApp,
                            "Error auditing request message. ", e ), e );
		}
	}
}