

package gov.va.med.via.loggerAudit;


import gov.va.med.via.log4j.ApplicationLogEvent;
import gov.va.med.via.log4j.persistence.LoggerPersistenceManager;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.spi.LoggingEvent;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author ZZZZZZTALBOM
 * 
 *  LoggerMessageDrivenBean receives JMS Messages on LoggerQueue, and stores those messages
 *  in the database. The asynchronous handling of logging information is expected to improve overall performance of the system.
 * 
 */
@MessageDriven( mappedName= "jms/via/LoggerQueue", activationConfig =
{ @ActivationConfigProperty(propertyName="destinationType", propertyValue="javax.jms.Queue"), 
		@ActivationConfigProperty(propertyName="destination", propertyValue="jms/via/LoggerQueue"),
			@ActivationConfigProperty(propertyName="connectionFactoryJndiName", propertyValue="jms/via/LoggerAuditConnectionFactory"),
				@ActivationConfigProperty(propertyName="subscriptionDurability", propertyValue="Durable")
})
public class LoggerMessageDrivenBean implements MessageListener   
{
   
	protected final Log logger = LogFactory.getLog(getClass());
	private LoggerPersistenceManager loggerPersistenceManager;
	@Resource
	private MessageDrivenContext messageDrivenContext;
	   	
    public LoggerMessageDrivenBean() {
	}
        
    @PostConstruct
    public void setAppContext( )
    {
    	BeanFactory beanFactory = new ClassPathXmlApplicationContext("classpath:gov/va/med/via/loggerAudit/config/weblogicLoggerContext.xml");
        this.loggerPersistenceManager = ( LoggerPersistenceManager ) beanFactory.getBean( "loggerPersistenceManager" );
    }

    @TransactionAttribute(value=TransactionAttributeType.REQUIRED)
    public void onMessage( Message message )
    {
        ApplicationLogEvent appLogEvent = null;
        try
        {
            ObjectMessage objectMessage = ( ( ObjectMessage )message );
            LoggingEvent logEvent = ( LoggingEvent )objectMessage.getObject();

            appLogEvent = new ApplicationLogEvent( logEvent );
            loggerPersistenceManager.persistApplicationLogEvent( appLogEvent );
        }
        catch ( Exception e )
        {
            // make sure that any logs coming from the logging sub-system only go
            // to the file or stdout.
            logger.error( gov.va.med.via.exceptionframework.LogMessageUtil.buildMessage( appLogEvent.getCategory(), appLogEvent.getRequestingApp(), "Error proccessing JMS Appender log event. ", e ), e );
         }
    }

}
