/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.util.Calendar;

/**
 *
 * @author vhahonbrunos
 */
public class DateUtils
{
    public static boolean isLeapYear(int year)
    {
        return ((year > 0) && (year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0)));
    }

    public static boolean isValidMonth(int month)
    {
        return (month > 0 && month < 13);
    }

    public static boolean isValidDay(int year, int month, int dy)
    {
        if (dy < 1 || dy > 31)
            return false;
        if (is30DayMonth(month) && dy > 30)
            return false;
        if (month != 2)
            return true;
        if (dy > 29)
            return false;
        if (!isLeapYear(year) && dy > 28)
            return false;
        return true;
    }

    public static boolean is30DayMonth(int month)
    {
        return (month == 4 || month == 6 || month == 9 || month == 11);
    }

    /// <summary>
    /// Changes yyyyMMdd.HHmmss formatted string to date.
    /// </summary>
    /// <remarks>pads missing least significant digits (e.g. seconds or milliseconds)
    /// with 0s.</remarks>
    /// <param name="dateString"></param>
    /// <returns></returns>
    public static Calendar IsoDateStringendDateTime(String dateString)
    {
        Calendar retTime = Calendar.getInstance();
        
        // Add time to date received if missing
        if (dateString.indexOf('.') == -1)
        {
        	dateString += ".000000";
        } 
        else
        {
        	if (dateString.length() < 15)
        	{
            	//pad right dateString with 15 zeros
            	for(int i = 0; i < 15; i++)
            	{
                	dateString += "0";
            	}
        	}
        }
        
        Integer hour = new Integer(0);
        Integer minute = new Integer(0);
        Integer second = new Integer(0);
        
        if (dateString.length() >= 11)	
        {
            hour = new Integer(Integer.parseInt(RPCStringUtils.CSubstring(dateString, 9, 2)));
        }

        if (dateString.length() >= 13)
        {
            minute = new Integer(Integer.parseInt(RPCStringUtils.CSubstring(dateString, 11, 2)));
        }
        
        if (dateString.length() >= 15)
        {
            second = new Integer(Integer.parseInt(RPCStringUtils.CSubstring(dateString, 13, 2)));
        }
                
        Integer year = Integer.parseInt(RPCStringUtils.CSubstring(dateString, 0, 4));
        Integer month = Integer.parseInt(RPCStringUtils.CSubstring(dateString, 4, 2));
        Integer date = Integer.parseInt(RPCStringUtils.CSubstring(dateString, 6, 2));
        
        retTime.set(Integer.parseInt(RPCStringUtils.CSubstring(dateString, 0, 4)),
                    Integer.parseInt(RPCStringUtils.CSubstring(dateString, 4, 2)) - 1,
                    Integer.parseInt(RPCStringUtils.CSubstring(dateString, 6, 2)),
                    hour.intValue(), minute.intValue(), second.intValue());

        return retTime;
    }

    public static boolean canParseInt(String intStr, Integer result)
    {
        try
        {
            int num = Integer.parseInt(intStr);
            result = num;
            return true;
        }
        catch(Exception e)
        {
            return false;
        }
    }

    public static String trimSeconds(String timestamp)
    {
        int p = timestamp.indexOf(".");
        if (p == -1)
        {
            return timestamp;
        }
        String[] parts = RPCStringUtils.split(timestamp, RPCStringUtils.PERIOD);
        if (parts[1].length() < 5)
        {
            return timestamp;
        }
        parts[1] = RPCStringUtils.CSubstring(parts[1], 0, 4);
        return parts[0] + '.' + parts[1];
    }

    public static String zeroSeconds(String timestamp)
    {
        int p = timestamp.indexOf(".");
        if (p == -1)
        {
            return timestamp + ".000000";
        }
        String[] parts = RPCStringUtils.split(timestamp, RPCStringUtils.PERIOD);
        if (parts[1].length() < 5)
        {
            return RPCStringUtils.CSubstring(timestamp + "000000", 0,6-parts[1].length());
        }
        parts[1] = RPCStringUtils.CSubstring(parts[1], 0, 4)  + "00";
        return parts[0] + '.' + parts[1];
    }

    public static String trimTime(String timestamp)
    {
        int p = timestamp.indexOf(".");
        if (p == -1)
        {
            return timestamp;
        }
        String[] parts = RPCStringUtils.split(timestamp, RPCStringUtils.PERIOD);
        return parts[0];
    }

    public static boolean isValidDateRange(String startDate, String endDate)
    {
        if (startDate.equals(endDate))
        {
            return false;
        }
        try
        {
            Calendar fromDT = IsoDateStringendDateTime(startDate);
            Calendar toDT = IsoDateStringendDateTime(endDate);
            
            if (fromDT.compareTo(toDT) > 0 || fromDT.compareTo(Calendar.getInstance()) > 0)
            {
                return false;
            }
        }
        catch (Exception e)
        {
            return false;
        }
        return true;
    }

    String subtractSecond(String timestamp)
    {
        String[] parts = RPCStringUtils.split(timestamp, RPCStringUtils.PERIOD);

        // Only do this if there is a time part that ends with 0.
        if (parts.length < 2 || !parts[1].endsWith("0"))
        {
            return timestamp;
        }

        return "";
    }
}
