/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
 *
 * @author vhahonbrunos
 */
public class DdrLister extends DdrQuery
{
    private String file = "";
    private String iens = "";
    private String[] requestedFields;       // Holds user's requested order
    private LinkedHashMap requestedFieldsTbl;   // Needed to fetch by FileMan field #, and to hold if External or not
    private ArrayList ienLst;               // Needed to hold record IENs across methods.
    private String flags = "";
    private String max = "";
    private String from = "";
    private String part = "";
    private String xref = "";
    private String screen = "";
    private String id = "";
    private String options = "";
    private String moreFrom = "";
    private String moreIens = "";

    public boolean more = false;

    public DdrLister(RPCConnection cxn)
    {
        super(cxn);
    }

    protected String buildRequest() throws RPCException
    {
        if (file.equals(""))
        {
            throw new RPCException("Must have a file!");
        }
        VistaQuery vq = new VistaQuery("DDR LISTER");
        DictionaryHashList paramLst = new DictionaryHashList();
        paramLst.add("\"FILE\"", file);
        if (!iens.equals(""))
        {
            paramLst.add("\"IENS\"", iens);
        }
        if (requestedFields.length > 0)
        {
            paramLst.add("\"FIELDS\"", getFieldsArg());
        }
        if (!flags.equals(""))
        {
            paramLst.add("\"FLAGS\"", flags);
        }
        if (!max.equals(""))
        {
            paramLst.add("\"MAX\"", max);
        }
        if (!from.equals(""))
        {
            paramLst.add("\"FROM\"", from);
        }
        if (!part.equals(""))
        {
            paramLst.add("\"PART\"", part);
        }
        if (!xref.equals(""))
        {
            paramLst.add("\"XREF\"", xref);
        }
        if (!screen.equals(""))
        {
            paramLst.add("\"SCREEN\"", screen);
        }
        if (!id.equals(""))
        {
            paramLst.add("\"ID\"", id);
        }
        if (!options.equals(""))
        {
            paramLst.add("\"OPTIONS\"", options);
        }
        if (!moreFrom.equals(""))
        {
            paramLst.add("\"FROM\"", moreFrom);
        }
        if (!moreIens.equals(""))
        {
            paramLst.add("\"IENS\"", moreIens);
        }
        vq.addParameter(vq.LIST, paramLst);
        return vq.buildMessage();
    }

    public String[] execute() throws RPCException
    {
        String request = buildRequest();
        String response = this.execute(request);
        return buildResult(response.replace('|','^'));
    }

    public static DdrLister buildIenNameQuery(RPCConnection cxn, String fileNum)
    {
        DdrLister query = new DdrLister(cxn);
        query.setFile(fileNum);
        query.setFields(".01");
        query.setFlags("IP");
        query.setXref("#");
        return query;
    }

    public static DdrLister buildFileQuery(RPCConnection cxn, VistaFile file)
    {
        DdrLister query = new DdrLister(cxn);
        query.setFile(file.getFileNum());
        query.setFields(file.getFieldString());
        query.setFlags("IP");
        query.setXref("#");
        return query;
    }

    private String getFieldsArg()
    {
        String result = "@";
        for (int i = 0; i < requestedFields.length; i++)
        {
            if (requestedFields[i].equals("@"))
            {
                continue;
            }
            result += ';' + requestedFields[i];
        }
        return result;
    }

    private void setMoreParams(String line) throws RPCException
    {
        String[] flds = RPCStringUtils.split(line, RPCStringUtils.CARET);
        if (!flds[0].equals("MORE"))
        {
            throw new RPCException("Invalid return data: expected 'MORE', got " + flds[0]);
        }
        moreFrom = flds[1];
        moreIens = flds[2];
        more = true;
    }

    public String[] buildResult(String rtn) throws RPCException
    {
        String[] lines = RPCStringUtils.split(rtn, RPCStringUtils.CRLF);
        lines = RPCStringUtils.trimArray(lines);
        int i = 0;

        if (lines[i].equals(VistaConstants.MISC))
        {
            if (!lines[++i].startsWith("MORE"))
            {
                throw new RPCException("Error packing LISTER return; expected 'MORE...', got " + lines[i]);
            }
            setMoreParams(lines[i]);
        }

        if (this.flags.indexOf("P") != -1)
        {
            return parsePackedResult(lines);
        }
        else
        {
            return packResult(lines);
        }
    }

    private String[] parsePackedResult(String[] lines) throws RPCException
    {
        int idx = RPCStringUtils.getIdx(lines, VistaConstants.BEGIN_ERRS, 0);
        if (idx != -1)
        {
            throw new RPCException(getErrMsg(lines,idx));
        }

        idx = RPCStringUtils.getIdx(lines, VistaConstants.BEGIN_DATA, 0);
        if (idx == -1)
        {
            throw new RPCException("Error parsing packed result: expected " + VistaConstants.BEGIN_DATA + ", found none.");
        }

        ArrayList<String> lst = new ArrayList<String>();
        idx++;
        while (idx < lines.length && !lines[idx].equals(VistaConstants.END_DATA))
        {
            lst.add(lines[idx++]);
        }
        return (String[])lst.toArray(new String[0]);
    }

    private String[] packResult(String[] lines) throws RPCException
    {
        int idx = 0;

        if (lines[idx].equals(VistaConstants.UNPACKED_NO_RESULTS))
        {
            return new String[] { };
        }

        LinkedHashMap rs = new LinkedHashMap();

        if ((idx = RPCStringUtils.getIdx(lines, VistaConstants.BEGIN_ERRS, 0)) != -1)
        {
            throw new RPCException(getErrMsg(lines,idx));
        }

        if ((idx = RPCStringUtils.getIdx(lines, VistaConstants.BEGIN_DATA, 0)) != -1)
        {
            ienLst = new ArrayList();
            if (!lines[++idx].equals(VistaConstants.BEGIN_IENS))
            {
                throw new RPCException("Incorrectly formatted return data");
            }
            idx++;
            while (!lines[idx].equals(VistaConstants.END_IENS))
            {
                ienLst.add(lines[idx++]);
            }

            idx++;
            if (!lines[idx].equals(VistaConstants.BEGIN_IDVALS))
            {
                throw new RPCException("Incorrectly formatted return data");
            }
            String[] flds = RPCStringUtils.split(lines[++idx], RPCStringUtils.SEMICOLON);

            int recIdx = 0;
            idx++;
            while (!lines[idx].equals(VistaConstants.END_IDVALS))
            {
                // the last field in flds is the field count, not a field
                LinkedHashMap rec = new LinkedHashMap();
                for (int fldIdx = 0; fldIdx < flds.length-1; fldIdx++)
                {
                    DdrField f = new DdrField();
                    f.setFmNumber(flds[fldIdx]);
                    String requestedOptions = (String)requestedFieldsTbl.get(f.getFmNumber());
                    f.setHasExternal(requestedOptions.indexOf('E') != -1);
                    if (f.isHasExternal())
                    {
                        f.setExternalVal(lines[idx++]);
                    }
                    if (requestedOptions.indexOf('I') != -1)
                    {
                        f.setVal(lines[idx++]);
                    }
                    rec.put(f.getFmNumber(), f);
                }
                rs.put((String)ienLst.get(recIdx++), rec);
            }
            // at this point line should be VistaConstants.END_DATA
            // unless more functionality is added.
        }
        return toStringArray(rs);
    }

    private String getErrMsg(String[] lines, int idx)
    {
        String msg = lines[idx + 3];
        int endIdx = RPCStringUtils.getIdx(lines, VistaConstants.END_ERRS, 0);
        for (int i = idx + 4; i < endIdx; i++)
        {
            if (msg.charAt(msg.length() - 1) != '.')
            {
                msg += ". ";
            }
            msg += lines[i];
        }
        return msg;
    }

    private String[] toStringArray(LinkedHashMap hashedRex)
    {
        ArrayList<String> lst = new ArrayList<String>();
        for (int recnum = 0; recnum < ienLst.size(); recnum++)
        {
            String s = (String)ienLst.get(recnum);
            LinkedHashMap hashedFlds = (LinkedHashMap)hashedRex.get(ienLst.get(recnum));
            for (int fldnum = 0; fldnum < requestedFields.length; fldnum++)
            {
                String fmNum = requestedFields[fldnum];
                boolean external = false;
                if (fmNum.indexOf('E') != -1)
                {
                    fmNum = RPCStringUtils.CSubstring(fmNum, 0, fmNum.length() - 1);
                    external = true;
                }
                DdrField fld = (DdrField)hashedFlds.get(fmNum);
                if (external)
                {
                    s += '^' + fld.getExternalVal();
                }
                else
                {
                    s += '^' + fld.getVal();
                }
            }
            lst.add(s);
        }
        return (String[])lst.toArray(new String[0]);
    }

    public String getFile() {
        return file;
    }

    public void setFile(String file) {
        this.file = file;
    }

    public String getFlags() {
        return flags;
    }

    public void setFlags(String flags) {
        this.flags = flags;
    }

    public String getFrom() {
        return from;
    }

    public void setFrom(String from) {
        this.from = from;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getMax() {
        return max;
    }

    public void setMax(String max) {
        this.max = max;
    }

    public String getMoreFrom() {
        return moreFrom;
    }

    public void setMoreFrom(String moreFrom) {
        this.moreFrom = moreFrom;
    }

    public String getMoreIens() {
        return moreIens;
    }

    public void setMoreIens(String moreIens) {
        this.moreIens = moreIens;
    }

    public String getOptions() {
        return options;
    }

    public void setOptions(String options) {
        this.options = options;
    }

    public String getPart() {
        return part;
    }

    public void setPart(String part) {
        this.part = part;
    }

    public String getScreen() {
        return screen;
    }

    public void setScreen(String screen) {
        this.screen = screen;
    }

    public String getXref() {
        return xref;
    }

    public void setXref(String xref) {
        this.xref = xref;
    }

    public void setFields(String value)
    {
        String s = value;
        requestedFields = RPCStringUtils.split(s, RPCStringUtils.SEMICOLON);
        requestedFieldsTbl = new LinkedHashMap(requestedFields.length);
        for (int i = 0; i < requestedFields.length; i++)
        {
            if (requestedFields[i].equals(""))
            {
                continue;
            }
            String fldnum = requestedFields[i];
            String option = "I";
            if (fldnum.indexOf('E') != -1)
            {
                fldnum = RPCStringUtils.CSubstring(fldnum, 0, fldnum.length() - 1);
                option = "E";
            }
            if (!requestedFieldsTbl.containsKey(fldnum))
            {
                requestedFieldsTbl.put(fldnum, option);
            }
            else
            {
                String fieldOption = (String)requestedFieldsTbl.get(fldnum);
                fieldOption += option;
                requestedFieldsTbl.put(fldnum, fieldOption);
            }
        }
    }

    public String getIens() {
        return iens;
    }

    public void setIens(String iens) {
        this.iens = iens;
    }
}
