/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.lang.reflect.Method;

/**
 *
 * @author vhahonbrunos
 */
public class QueryThread implements Runnable {

    private Object classObj;
    private String methodName;
    private Object[] args;
    private Object result;

    public QueryThread(Object dao, String methodName, Object[] args)
    {
        this.classObj = dao;
        this.methodName = methodName;
        this.args = args;
    }

    public Object getResult()
    {
        return result;
    }

    public boolean isExceptionResult()
    {
        return (result != null && result.getClass().getName().endsWith("Exception"));
    }

    public void run()
    {
        if (classObj == null)
        {
            return;
        }

        try
        {
            Class cls = classObj.getClass();
            Class[] paramTypes = new Class[args.length];
            for(int i = 0; i < args.length; i++)
            {
                paramTypes[i] = args[i].getClass();
            }

            Method method = getMethod(cls, methodName, paramTypes);
            result = method.invoke(classObj, args);
        }
        catch(Exception e)
        {
            result = e;
        }       
    }
    
    /**
     * Searches for and returns method within given class that matches 
     * name and parameter type list. This method is used in lieu of
     * Class.getMethod() because it matches parameter types based on
     * assignability instead of type name.
     * (i.e. can pass parameters that are a subclass of those
     * defined in method signature).
     * 
     * @param theClass Class to get method from.
     * @param methodName Name of method to get.
     * @param paramTypes List of parameter types.
     * @return Method if found.
     * @throws NoSuchMethodException If no method is found.
     */
    private Method getMethod(Class theClass, String methodName, Class[] paramTypes)
            throws NoSuchMethodException
    {
        Method[] methods = theClass.getMethods();

        for(Method method : methods)
        {
            if (method.getName().equals(methodName))
            {
                Class[] methodParamTypes = method.getParameterTypes();
                if (methodParamTypes.length != paramTypes.length)
                    continue;

                boolean isParamMatch = true;
                for(int i = 0; i < paramTypes.length; i++)
                {
                    Class param = methodParamTypes[i];
                    if (!param.isAssignableFrom(paramTypes[i]))
                    {
                        isParamMatch = false;
                    }
                }

                if (isParamMatch)
                {
                    return method;
                }
            }
        }

        throw new NoSuchMethodException();
    }
}
