/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

/**
 *
 * @author vhahonbrunos
 */
public class RPCStringUtils
{
	public static final String CR = "\n";
    public static final String CRLF = "\r\n";
    public static final String CARET = "^";
    public static final String STICK = "|";
    public static final String COLON = ":";
    public static final String SEMICOLON = ";";
    public static final String COMMA = ",";
    public static final String PERIOD = ".";
    public static final String SLASH = "/";
    public static final String SPACE = " ";
    public static final String EQUALS = "=";
    public static final String AMPERSAND = "&";
    public static final String ATSIGN = "@";
    public static final String DASH = "-";
    public static final String TILDE = "~";
    public final static String RECORD_SECTION_FIELD_SEPARATOR = "&#94;";
    
    public static boolean isNullOrEmpty(String s)
    {
        return s == null || s.equals("");
    }

    public static String[] split(String s, char delimiter)
    {
        return split(s, "" + delimiter);
    }

    public static String[] split(String s, String delimiter)
    {
        /*calling String.split with -1 to ensure we get all the empty index elements to the end of the string*/
    	return s.split(escapeRegExMetaChars(delimiter), -1);
    }

    private static String escapeRegExMetaChars(String str)
    {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < str.length(); i++)
        {
            if (isRegExMetaChar(str.charAt(i)))
            {
                sb.append("\\" + str.charAt(i));
            }
            else sb.append(str.charAt(i));
        }

        return sb.toString();
    }

    private static boolean isRegExMetaChar(char c)
    {
        switch(c)
        {
            case '^': return true;
            case '.': return true;
            case '$': return true;
            case '|': return true;
            case '*': return true;
            case '+': return true;
            case '?': return true;
            case '-': return true;
            default: return false;
        }
    }

    public static String[] trimArray(String[] s)
    {
        int i;
        for (i = s.length - 1; i >= 0; i--)
        {
            if (!s[i].equals(""))
            {
                break;
            }
        }
        String[] result = new String[i + 1];
        int len = i+1;
        for(int j = 0; j < len; j++)
        {
            result[j] = s[j];
        }

        return result;
    }

    public static boolean isNumeric(String s)
        {
        if (s.equals(""))
        {
            return false;
        }
        for (int i=0; i<s.length(); i++)
        {
            if (s.charAt(i) < '0' || s.charAt(i) > '9')
            {
                return false;
            }
        }
        return true;
    }

    public static boolean isAlpha(String s)
    {
        if (s.equals(""))
        {
            return false;
        }
        for (int i = 0; i<s.length(); i++)
        {
            if (!isAlphaChar(s.charAt(i)))
            {
                return false;
            }
        }
        return true;
    }

    public static boolean isNumericChar(char c)
    {
        return (c >= '0' && c <= '9');
    }

    public static boolean isAlphaChar(char c)
    {
        return ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'));
    }

    public static boolean isAlphaNumericChar(char c)
    {
        return (isAlphaChar(c) || isNumericChar(c));
    }

    public static boolean isWhiteSpace(char c)
    {
        return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
    }

    public static String removeNonNumericChars(String s)
    {
        if (s == null)
        {
            return null;
        }
        String rtn = "";
        for (int i=0; i<s.length(); i++)
        {
            if (s.charAt(i) >= '0' && s.charAt(i) <= '9')
            {
                rtn += s.charAt(i);
            }
        }
        return rtn;
    }

    public static String piece(String s, String delimiter, int pieceNum)
    {
        String[] flds = split(s,delimiter);
        if (pieceNum > flds.length)
        {
            return null;
        }
        return flds[pieceNum-1];
    }

    public static String piece(String s, int pieceNum)
    {
        return piece(s, "^", pieceNum);
    }

    public static boolean isEmpty(String s)
    {
        return s == null || s.length() == 0;
    }

    public static String strPack(String s, int n)
    {
        String result = ""+s.length();
        while (result.length() < n)
        {
                result = '0' + result;
        }
        return result + s;
    }

    public static int getFirstWhiteSpaceAfter(String s, int idx)
    {
        int i = idx;
        while (i >= 0 && !isWhiteSpace(s.charAt(i)))
        {
            i--;
        }
        return i;
    }

    public static String filteredString(String s)
    {
        String result = "";
        for (int i=0; i<s.length(); i++)
        {
            int c = asciiAt(s,i);
            if (c == 9)
            {
                    result += "        ";
            }
            else if ((c >= 32 && c <=127) || (c >= 161 && c <=255))
            {
                    result += s.charAt(i);
            }
            else if (c >= 128 && c <= 159)
            {
                    result += "?";
            }
            else if (c == 10 || c == 13 || c == 160)
            {
                    result += " ";
            }
        }
        return result;
    }

    public static int asciiAt(String s, int idx)
    {
        char c = s.charAt(idx);
        return (int)c;
    }

    public static String prependChars(String s, char c, int sLen)
    {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < sLen; i++)
        {
            sb.append(c);
        }

        String chars = sb.toString();
        return CSubstring(chars, 0, sLen - s.length()) + s;
    }

    public static int getIdx(String[] lines, String target, int startingIdx)
    {
        for (int i = startingIdx; i < lines.length; i++)
        {
            if (lines[i].startsWith(target))
            {
                return i;
            }
        }
        return -1;
    }

    public static String trimTrailingZeroes(String s)
    {
        int i = s.length() - 1;
        while (s.charAt(i) == '0' && i > 0)
        {
            i--;
        }

        return CSubstring(s, 0, i+1);
    }

    public static String reverse(String s)
    {
        StringBuilder result = new StringBuilder("");
        for (int i = s.length() - 1; i >= 0; i--)
        {
            result.append(s.charAt(i));
        }

        return result.toString();
    }

    /**
     * Return a string with all invalid XML 1.0 characters
     * (per W3C standard: http://www.w3.org/TR/REC-xml/#charsets) removed
     */
    public static String stripInvalidXmlCharacters(String s)
    {
        if (s == null || s.equals(""))
        {
            return "";
        }

        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++)
        {
            char current = s.charAt(i);
            if (current == 0x9 || current == 0xA || current == 0xD ||
                ((current >= 0x20) && (current <= 0xD7FF)) ||
                ((current >= 0xE000) && (current <= 0xFFFD)) ||
                ((current >= 0x10000) && (current <= 0x10FFFF)))
            {
                sb.append(current);
            }
        }
        return sb.toString();
    }

    /**
     * Returns a new string that is a substring of str.
     * Function is a C# porting utility.
     * @param str Source string
     * @param beginIndex The beginning index, inclusive.
     * @param length The number of characters in the substring. 
     * @return a new string that is a substring of str.
     */
    public static String CSubstring(String str, int beginIndex, int length)
    {
        return str.substring(beginIndex, beginIndex+length);
    }
    
    /**
     * Check the response string and standardized EOF characters. 
	 * The response string may have come from different environments, this method
	 * uses a pattern to replace a variety of EOF possibilities with a standard EOF
	 * that subsequent parsing code can use to determine line separation.
	 * 
	 * @param response - an RPC response string
	 * @return a new string with end of line characters (generally '\n' or '\r' or '\r\n') replaced with a linux standard '\r\n'
	 */
    public static String replaceEndofLineCharacters(String response) {
		
	    return response.replaceAll("[\r\n]", RPCStringUtils.CRLF);
	    
	}
    
    
    //brought over from MDWS - from VistaUtils
    public static String adjustForNameSearch(String target)
    {
        int lth = target.length();
        if (lth == 0)
        {
            return "";
        }   
        String rtn = target.substring(0, lth - 1);
        char c = target.charAt(lth - 1);
        int asciiCode = (byte)c - 1;
        c = (char)asciiCode;
        rtn = rtn + c + '~';
        return rtn;
    }
}
