/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.via.rpc;

import java.util.LinkedHashMap;

import javax.xml.bind.ValidationException;

/**
 *
 * @author vhahonbrunos
 */
public class RPCUtils
{		
	public static String adjustForNameSearch(String target)
	{
		int lth = target.length();
		if (lth == 0)
		{
			return "";
		}
		//target = target.ToUpper();
		String rtn = RPCStringUtils.CSubstring(target, 0, lth - 1);
		char c = target.charAt(lth - 1);
		int asciiCode = (byte)c - 1;
		c = (char)asciiCode;
		rtn = rtn + c + '~';
		return rtn;
	}

	/// <summary>
	/// </summary>
	/// <param name="target"></param>
	/// <returns></returns>
	public static String adjustForNumericSearch(String target)
	{
		Long iTarget = Long.parseLong(target);
		return "" + (iTarget - 1);
	}

	public static String setDirectionParam(String direction)
	{
		if (direction == null || direction.equals(""))
		{
			return "1";
		}
		if (!direction.equals("1") && !direction.equals("-1"))
		{
			throw new IllegalArgumentException("Invalid direction.  Must be 1 or -1.");
		}
		return direction;
	}

	//    // DoD User
	//    public synchronized static Provider getDodUser()
	//    {
	//        Provider p = new Provider();
	//        p.setLogonSite(new KeyValuePair("200", "DOD"));
	//        p.setLogonSiteUserId("31066");
	//        p.setName("DEPARTMENT OF DEFENSE,USER");
	//        p.setPermissionString(VistaConstants.CAPRI_CONTEXT);
	//        p.setSSN("123456789");
	//        return p;
	//    }

	public static String buildReportTextRequest(
			String dfn, String startDate, String endDate, int nrpts, String arg)
	{
		VistaQuery vq = new VistaQuery("ORWRP REPORT TEXT");
		vq.addParameter(vq.LITERAL, dfn);
		if (nrpts != 0)
		{
			arg += ("" + nrpts);
		}
		vq.addParameter(vq.LITERAL, arg);
		vq.addParameter(vq.LITERAL, "");
		vq.addParameter(vq.LITERAL, "");
		vq.addParameter(vq.LITERAL, "");
		if (startDate.equals(""))
		{
			startDate = "0";
		}
		else
		{
			startDate = Timestamp.fromUtcString(startDate);
		}
		vq.addParameter(vq.LITERAL, startDate);
		if (endDate.equals(""))
		{
			endDate = "0";
		}
		else
		{
			endDate = Timestamp.fromUtcString(endDate);
		}
		vq.addParameter(vq.LITERAL, endDate);
		return vq.buildMessage();
	}

	public static String buildReportTextRequest_AllResults(String dfn, String arg)
	{
		VistaQuery vq = new VistaQuery("ORWRP REPORT TEXT");
		vq.addParameter(vq.LITERAL, dfn);
		vq.addParameter(vq.LITERAL, arg + '0');
		vq.addParameter(vq.LITERAL, "");
		vq.addParameter(vq.LITERAL, "00000");
		vq.addParameter(vq.LITERAL, "");
		vq.addParameter(vq.LITERAL, "0");
		vq.addParameter(vq.LITERAL, "0");
		return vq.buildMessage();
	}

	public static String responseOrOk(String response)
	{
		if (!response.equals(""))
		{
			return response;
		}
		return "OK";
	}

	public static String errMsgOrOK(String response)
	{
		String[] flds = RPCStringUtils.split(response, RPCStringUtils.CARET);
		if (!flds[0].equals("1"))
		{
			return flds[1];
		}
		return "OK";
	}

	/**
	 * 
	 * @param response
	 * @return "OK" if first part of caret-separated response string is zero, otherwise returns the second part of response string as error message
	 */
	public static String errMsgOrZero(String response)
	{
		String[] flds = RPCStringUtils.split(response, RPCStringUtils.CARET);
		if (!flds[0].equals("0"))
		{
			return flds[1];
		}
		return "OK";
	}

	public static String errMsgOrIen(String response) throws RPCException
	{
		if (!response.equals("") && !RPCStringUtils.isNumeric(response))
		{
			throw new RPCException("Non-numeric IEN");
		}
		return response;
	}

	public static String removeCtlChars(String s)
	{
		if (s == null || s.equals(""))
		{
			return "";
		}
		String result = "";
		int i = 0;
		while (i < s.length())
		{

			byte c = ("" + s.charAt(i)).getBytes()[0];
			if (c == 9 || c == 10 || c == 13 || (c > 31 && c < 127))
			{
				result += s.charAt(i);
			}
			i++;
		}
		return result;
	}

	public static String getVistaName(String s)
	{
		/*if (!PersonName.isValid(s))
        {
            return "";
        }*/
		String[] flds = RPCStringUtils.split(s, RPCStringUtils.COMMA);
		if (flds.length != 2)
		{
			return "";
		}
		String result = flds[0] + ',' + flds[1];
		return result.toUpperCase();
	}

	public static String getVistaGender(String s)
	{
		if (RPCStringUtils.isEmpty(s))
		{
			return "";
		}
		String result = RPCStringUtils.CSubstring(s, 0, 1).toUpperCase();
		if (!result.equals("M") && !result.equals("F"))
		{
			return "";
		}
		return result;
	}

	public static LinkedHashMap<String, String> toStringDictionary(String[] response)
	{
		if (response == null || response.length == 0)
		{
			return null;
		}
		LinkedHashMap<String, String> result = new LinkedHashMap<String, String>();
		for (int i = 0; i < response.length; i++)
		{
			String[] flds = RPCStringUtils.split(response[i], RPCStringUtils.CARET);
			result.put(flds[0], flds[1]);
		}
		return result;
	}

	/** CANNOT USE XWB GET VARIABLE VALUE
    public static String getVariableValue(RPCConnection cxn, String arg) throws RPCException
	{
		try
		{
			VistaQuery vq = new VistaQuery("XWB GET VARIABLE VALUE");
			vq.addParameter(vq.REFERENCE, arg);
			return (String) cxn.query(vq.buildMessage());
		}
		catch (Exception ex)
		{
			throw new RPCException(ex);
		}
	}**/

	public static String buildFromendDateScreenParam(String startDate, String endDate, int node, int pieceNum)
	{
		if (startDate.equals(""))
		{
			return "";
		}

		// Need test for valid dates.
		startDate = Timestamp.fromUtcFromDate(startDate);
		endDate = Timestamp.fromUtcString(endDate);
		return "S FD=" + startDate + ",TD=" + endDate + ",CDT=$P(^(" + node + "),U," + pieceNum + ") I CDT>=FD,CDT<TD";
	}

	public static String buildFromendDateScreenParam(String startDate, String endDate, int pieceNum)
	{
		if (startDate.equals(""))
		{
			return "";
		}

		// Need test for valid dates.
		startDate = Timestamp.fromUtcFromDate(startDate);
		endDate = Timestamp.fromUtcString(endDate);
		return "S FD=" + startDate + ",TD=" + endDate + ",CDT=$P(^(0),U," + pieceNum + ") I CDT>=FD,CDT<TD";
	}


}
