/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.lang.reflect.Field;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 *
 * @author vhahonbrunos
 */
public class VistaFile
{
    private String fileName;
    private String fileNum;
    private String global;
    private String mdoName;
    DictionaryHashList fields;

    public VistaFile() 
    {
    }

    public DictionaryHashList getFields() {
        return fields;
    }

    public void setFields(DictionaryHashList fields) {
        this.fields = fields;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public String getFileNum() {
        return fileNum;
    }

    public void setFileNum(String fileNum) {
        this.fileNum = fileNum;
    }

    public String getGlobal() {
        return global;
    }

    public void setGlobal(String global) {
        this.global = global;
    }

    public String getMdoName() {
        return mdoName;
    }

    public void setMdoName(String mdoName) {
        this.mdoName = mdoName;
    }

    public String getFieldString()
    {
        String result = "";
        for (int i = 0; i < fields.size(); i++)
        {
            KeyValuePair e = fields.get(i);
            result += ((VistaField)e.getValue()).getVistaNum() + ';';
        }
        if (!result.equals(""))
        {
            result = RPCStringUtils.CSubstring(result, 0, result.length() - 1);
        }
        return result;
    }

    public LinkedHashMap<String, Object> toMdo(RPCConnection cxn,
            String[] response, String mdoName) throws ClassNotFoundException, InstantiationException, IllegalAccessException, NoSuchFieldException, Exception
    {
        if (response == null || response.length == 0)
        {
            return null;
        }
        LinkedHashMap<String, Object> result = new LinkedHashMap<String, Object>(response.length);
        for (int lineIdx = 0; lineIdx < response.length; lineIdx++)
        {
            //Need to instantiate the mdo here
            Object theMdo = Class.forName(mdoName, true, Thread.currentThread().getContextClassLoader()).newInstance();
            //Object theMdo = Activator.CreateInstance(Type.GetType(mdoName), new object[] { });
            //Type theClass = theMdo.GetType();
            Class theClass = theMdo.getClass();
            String[] flds = RPCStringUtils.split(response[lineIdx], RPCStringUtils.CARET);
            for (int fldIdx = 0; fldIdx < flds.length; fldIdx++)
            {
                VistaField vf = (VistaField)((KeyValuePair)fields.get(fldIdx)).getValue();

                Field theField = theClass.getField(vf.getMdoName());

                //FieldInfo theField = theClass.GetField(vf.MdoName,BindingFlags.NonPublic | BindingFlags.Instance);

                if (vf.getMdoType().equals("string"))
                {
                    if (vf.getMapping() != null && vf.getMapping().getType().equals("decode"))
                    {
                        if (vf.getMapping().getDecodeMap().containsKey(flds[fldIdx]))
                        {
                            theField.set(theMdo,  vf.getMapping().getDecodeMap().get(flds[fldIdx]));
                        }
                        else
                        {
                            theField.set(theMdo, flds[fldIdx]);
                        }
                    }
                    else
                    {
                        theField.set(theMdo, flds[fldIdx]);
                    }
                }
                else if (vf.getMapping().getType().equals("kvp"))
                {
                    String key = flds[fldIdx];
                    String value = "";
                    if (vf.getMapping() != null && vf.getMapping().getType().equals("decode"))
                    {
                        value = vf.getMapping().getDecodeMap().get(key);
                    }
                    else
                    {
                        Map<String, String> lookupTbl = cxn.getLookupTable(vf.getMapping().getVistaFileNum());
                        if (lookupTbl.containsKey(key))
                        {
                            value = lookupTbl.get(key);
                        }
                    }
                    theField.set(theMdo, new KeyValuePair(key, value));
                    //KeyValuePair<string, string> kvp = new KeyValuePair<string, string>(key, value);
                }
            }
            result.put(flds[0], theMdo);
        }
        return result;
    }
}
