/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.util.ArrayList;
import java.util.Random;

/**
 *
 * @author vhahonbrunos
 */
public class VistaQuery
{
    private String rpcName;
    private ArrayList<Parameter> parameters;

    public int LITERAL = 1;
    public int REFERENCE = 2;
    public int LIST = 3;
    public int WORDPROC = 4;

    public static final String concatStr = "_\"^\"_";

    public VistaQuery() 
    {
        this.parameters = new ArrayList<Parameter>();
    }

    public VistaQuery(String rpcName)
    {
        if (rpcName != null)
        {
            this.rpcName = rpcName;
        }
        
        this.parameters = new ArrayList<Parameter>();
    }

    @Override
    public String toString()
    {
        return buildMessage();
    }

    public ArrayList getParameters() {
        return parameters;
    }

    public void setParameters(ArrayList parameters) {
        this.parameters = parameters;
    }

    public String getRpcName() {
        return rpcName;
    }

    public void setRpcName(String rpcName) {
        this.rpcName = rpcName;
    }

    public void addParameter(int type, String value)
    {
        Parameter vp = new Parameter(type,value);
        parameters.add(vp);
    }

    public void addParameter(int type, String value, String text)
    {
        Parameter vp = new Parameter(type,value,text);
        parameters.add(vp);
    }

    public void addParameter(int type, DictionaryHashList lst)
    {
        Parameter vp = new Parameter(type, lst);
        parameters.add(vp);
    }

    public void addEncryptedParameter(int type, String value)
    {
        Parameter vp = new Parameter(type, encrypt(value));
        parameters.add(vp);
    }

    private String buildApi(String rpcName, String parameters, String fText)
    {
        String sParams = RPCStringUtils.strPack(parameters,5);
        return RPCStringUtils.strPack(fText + rpcName + '^' + sParams,5);
    }

    public String buildMessage()
    {
        final String PREFIX = "{XWB}";
        final String HDR = "007XWB;;;;";

        String sParams = "";
        DictionaryHashList lst = null;
        String text = null;
        for (int i=0; i<parameters.size(); i++)
        {
            Parameter vp = (Parameter)parameters.get(i);
            int pType = vp.getType();
            if (pType == LITERAL)
            {
                sParams += RPCStringUtils.strPack('0' + vp.getValue(),3);
            }
            else if (pType == REFERENCE)
            {
                sParams += RPCStringUtils.strPack('1' + vp.getValue(),3);
            }
            else if (pType == LIST)
            {
                sParams += RPCStringUtils.strPack('2' + vp.getValue(),3);
                lst = vp.getList();
            }
            else if (pType == WORDPROC)
            {
                sParams += RPCStringUtils.strPack('2' + vp.getValue(),3);
                text = vp.getText();
            }
        }
        String msg = "";
        if (lst == null && text == null)
        {
            msg = RPCStringUtils.strPack(HDR + buildApi(rpcName,sParams,"0"),5);
        }
        else if (lst != null)
        {
            msg = RPCStringUtils.strPack(HDR + buildApi(rpcName,sParams,"1"),5);
            for (int j = 0; j < lst.size(); j++)
            {
                KeyValuePair entry = lst.get(j);
                String key = (String)entry.getKey();
                String value = (String)entry.getValue();
                if (value == null || value.equals(""))
                {
                    value = "\u0001";
                }
                msg += RPCStringUtils.strPack(key, 3) + RPCStringUtils.strPack(value, 3);
            }
            msg += "000";
        }
        else
        {
            msg = RPCStringUtils.strPack(HDR + buildApi(rpcName,sParams,"1"),5);
            String[] lines = RPCStringUtils.split(text,RPCStringUtils.CRLF);
            for (int i=0; i<lines.length; i++)
            {
                msg += RPCStringUtils.strPack(""+i,3) +
                RPCStringUtils.strPack(lines[i] + RPCStringUtils.CRLF,3);
            }
            msg += "000";
        }
        
        msg = getVersion() + msg;
        msg = PREFIX + RPCStringUtils.strPack(msg,5);
        return msg;
    }

    protected String getVersion()
    {
        byte[] num = {1};
        String numChar = new String(num);
        return "|" + numChar + "1";
    }

    public static String encrypt(String inString)
    {
        final int MAXKEY = 19;
        String[] cipherPad =
        {
            "wkEo-ZJt!dG)49K{nX1BS$vH<&:Myf*>Ae0jQW=;|#PsO`\'%+rmb[gpqN,l6/hFC@DcUa ]z~R}\"V\\iIxu?872.(TYL5_3",
            "rKv`R;M/9BqAF%&tSs#Vh)dO1DZP> *fX\'u[.4lY=-mg_ci802N7LTG<]!CWo:3?{+,5Q}(@jaExn$~p\\IyHwzU\"|k6Jeb",
            "\\pV(ZJk\"WQmCn!Y,y@1d+~8s?[lNMxgHEt=uw|X:qSLjAI*}6zoF{T3#;ca)/h5%`P4$r]G\'9e2if_>UDKb7<v0&- RBO.",
            "depjt3g4W)qD0V~NJar\\B \"?OYhcu[<Ms%Z`RIL_6:]AX-zG.#}$@vk7/5x&*m;(yb2Fn+l\'PwUof1K{9,|EQi>H=CT8S!",
            "NZW:1}K$byP;jk)7\'`x90B|cq@iSsEnu,(l-hf.&Y_?J#R]+voQXU8mrV[!p4tg~OMez CAaGFD6H53%L/dT2<*>\"{\\wI=",
            "vCiJ<oZ9|phXVNn)m K`t/SI%]A5qOWe\\&?;jT~M!fz1l>[D_0xR32c*4.P\"G{r7}E8wUgyudF+6-:B=$(sY,LkbHa#\'@Q",
            "hvMX,\'4Ty;[a8/{6l~F_V\"}qLI\\!@x(D7bRmUH]W15J%N0BYPkrs&9:$)Zj>u|zwQ=ieC-oGA.#?tfdcO3gp`S+En K2*<",
            "jd!W5[];4\'<C$/&x|rZ(k{>?ghBzIFN}fAK\"#`p_TqtD*1E37XGVs@0nmSe+Y6Qyo-aUu%i8c=H2vJ\\) R:MLb.9,wlO~P",
            "2ThtjEM+!=xXb)7,ZV{*ci3\"8@_l-HS69L>]\\AUF/Q%:qD?1~m(yvO0e\'<#o$p4dnIzKP|`NrkaGg.ufCRB[; sJYwW}5&",
            "vB\\5/zl-9y:Pj|=(R\'7QJI *&CTX\"p0]_3.idcuOefVU#omwNZ`$Fs?L+1Sk<,b)hM4A6[Y%aDrg@~KqEW8t>H};n!2xG{",
            "sFz0Bo@_HfnK>LR}qWXV+D6`Y28=4Cm~G/7-5A\\b9!a#rP.l&M$hc3ijQk;),TvUd<[:I\"u1\'NZSOw]*gxtE{eJp|y (?%",
            "M@,D}|LJyGO8`$*ZqH .j>c~h<d=fimszv[#-53F!+a;NC\'6T91IV?(0x&/{B)w\"]Q\\YUWprk4:ol%g2nE7teRKbAPuS_X",
            ".mjY#_0*H<B=Q+FML6]s;r2:e8R}[ic&KA 1w{)vV5d,$u\"~xD/Pg?IyfthO@CzWp%!`N4Z\'3-(o|J9XUE7k\\TlqSb>anG",
            "xVa1\']_GU<X`|\\NgM?LS9{\"jT%s$}y[nvtlefB2RKJW~(/cIDCPow4,>#zm+:5b@06O3Ap8=*7ZFY!H-uEQk; .q)i&rhd",
            "I]Jz7AG@QX.\"%3Lq>METUo{Pp_ |a6<0dYVSv8:b)~W9NK`(r\'4fs&wim\\kReC2hg=HOj$1B*/nxt,;c#y+![?lFuZ-5D}",
            "Rr(Ge6F Hx>q$m&C%M~Tn,:\"o\'tX/*yP.{lZ!YkiVhuw_<KE5a[;}W0gjsz3]@7cI2\\QN?f#4p|vb1OUBD9)=-LJA+d`S8",
            "I~k>y|m};d)-7DZ\"Fe/Y<B:xwojR,Vh]O0Sc[`$sg8GXE!1&Qrzp._W%TNK(=J 3i*2abuHA4C\'?Mv\\Pq{n#56LftUl@9+",
            "~A*>9 WidFN,1KsmwQ)GJM{I4:C%}#Ep(?HB/r;t.&U8o|l[\'Lg\"2hRDyZ5`nbf]qjc0!zS-TkYO<_=76a\\X@$Pe3+xVvu",
            "yYgjf\"5VdHc#uA,W1i+v\'6|@pr{n;DJ!8(btPGaQM.LT3oe?NB/&9>Z`-}02*%x<7lsqz4OS ~E$\\R]KI[:UwC_=h)kXmF",
            "5:iar.{YU7mBZR@-K|2 \"+~`M%8sq4JhPo<_X\\Sg3WC;Tuxz,fvEQ1p9=w}FAI&j/keD0c?)LN6OHV]lGy\'$*>nd[(tb!#"
        };
        Random r = new Random();
        int associatorIndex = r.nextInt(MAXKEY);
        int identifierIndex = r.nextInt(MAXKEY);
        while (associatorIndex == identifierIndex)
        {
            identifierIndex = r.nextInt(MAXKEY);
        }
        String xlatedString = "";
        for (int i=0; i < inString.length(); i++)
        {
            char inChar = inString.charAt(i);
            int pos = cipherPad[associatorIndex].indexOf(inChar);
            if (pos == -1)
            {
                xlatedString += inChar;
            }
            else
            {
                xlatedString += cipherPad[identifierIndex].charAt(pos);
            }
        }
        return (char)(associatorIndex + 32) + xlatedString +
                    (char)(identifierIndex + 32);
    }

    public class Parameter
    {
        int type;
        String value;
        String text;
        DictionaryHashList lst;

        public Parameter()
        {
            this.type = -1;
            this.value = "";
        }

        public Parameter(int type, String value)
        {
            this.type = type;
            this.value = value;
        }

        public Parameter(int type, String value, String text)
        {
            this.type = type;
            this.value = value;
            this.text = text;
        }

        public Parameter(int type, DictionaryHashList lst)
        {
            this.type = type;
            this.value = ".x";
            this.lst = lst;
        }

        public void setType(int type)
        {
            this.type = type;
        }

        public int getType()
        {
            return this.type;
        }

        public void setValue(String value)
        {
            this.value = value;
        }

        public String getValue()
        {
            return this.value;
        }

        public void setText(String text)
        {
            this.text = text;
        }

        public String getText()
        {
            return this.text;
        }

        public void setList(DictionaryHashList lst)
        {
            this.lst = lst;
        }

        public DictionaryHashList getList()
        {
            return lst;
        }
    }
}
