/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 *
 * @author vhahonuyehac
 */
public class VistaResponse implements Iterator<String>
{
    private String rawResponse;

    public VistaResponse(String rawResponse)
    {
        this.rawResponse = rawResponse;
    }

    /**
     * Gets the raw string response of the executed vista query.
     * @return string response
     */
    public String getRawResponse()
    {
        return rawResponse;
    }

    /**
     * Returns an array of strings using the ^ as the default delimiter
     * @return a zero based array of pieces
     */
    public String[] getArrayOfPieces()
    {
        return getArrayOfPieces(RPCStringUtils.CARET);
    }

    /**
     * Returns an array of strings using the given delimiter
     * @param delimiter the delimiter to be used to piece (split) the string
     * @return a zero based array of pieces
     */
    public String[] getArrayOfPieces(String delimiter)
    {
        return RPCStringUtils.split(rawResponse, delimiter);
    }

    /**
     * Gets a specific string of text located in the given piece #; note to
     * retrieve the first piece, the pieceNumber should be 1 and not zero.
     * @param pieceNumber the piece # to return
     * @return string located at the given piece #
     */
    public String getPiece(int pieceNumber)
    {
        return RPCStringUtils.piece(rawResponse, RPCStringUtils.CARET, pieceNumber);
    }

    public String getPiece(String delimiter, int pieceNumber)
    {
        return RPCStringUtils.piece(rawResponse, delimiter, pieceNumber);
    }

    @Override
    public String toString()
    {
        return rawResponse;
    }

    /** Helper variable used for the Iterator **/
    private int next = -1;

    public boolean hasNext()
    {
        if(rawResponse.length() > 0)
        {
            if(next < 0)
            {
                return true;
            }
            else
            {
                if(rawResponse.indexOf("\r\n", next) > next)
                {
                    return true;
                }
                if(rawResponse.length() > next)
                {
                    return true;
                }
            }
        }
        return false;
    }

    public String next()
    {
        String result = null;
        int temp = 0;
        if(next < 0)
        {
            temp = rawResponse.indexOf("\r\n");
            next = temp + 2;
            return rawResponse.substring(0, temp);
        }
        else if(next < rawResponse.length())
        {
            temp = rawResponse.indexOf("\r\n", next);
            if(temp == -1)
            {
                result = rawResponse.substring(next);
                next = rawResponse.length();
                return result;
            }
            else
            {
                result = rawResponse.substring(next, temp);
                next = temp + 2;
                return result;
            }
        }
        else
        {
            throw new NoSuchElementException("Iteration has no more elements");
        }
    }

    public void remove()
    {
        throw new UnsupportedOperationException("Not supported.");
    }

}
